# 1 "modgengrid.F90"
! modules that can access any available grid for conversion

MODULE modgengrid

USE modgridm

# 9



IMPLICIT NONE

CONTAINS

!=======================================================================
SUBROUTINE xy2ij(xlon,ylat,ix,jy)
!=======================================================================
 INTEGER, INTENT(OUT) :: ix, jy
 REAL, INTENT(IN) :: xlon, ylat

 IF(IgridType(kiceS)==2) THEN !PS grid
# 28

 ELSE
   ix=NINT((xlon-xcglonLLmi(kiceS))/dphid)
   jy=NINT((ycglatLLma(kiceS) - ylat)/dthed)
 ENDIF


 IF (ix<0) THEN
   CALL outerrI('ix is <0 in xy2ij',ix)
 ELSEIF (ix>nip) THEN
   CALL outerrI('ix is >nip in xy2ij',ix)
 ENDIF
 IF (jy<0) THEN
   CALL outerrI('jy is <0 in xy2ij',jy)
 ELSEIF (jy>njp) THEN
   CALL outerrI('jy is >njp in xy2ij',jy)
 ENDIF



 RETURN
END SUBROUTINE xy2ij
!=======================================================================


!=======================================================================
SUBROUTINE xy2ijB(xlon,ylat,ix,jy) !bounded
!=======================================================================
 INTEGER, INTENT(OUT) :: ix, jy
 REAL, INTENT(IN) :: xlon, ylat

 IF(IgridType(kiceS)==2) THEN !PS grid
# 65

 ELSE
   ix=MAX(1,MIN(nLLi,NINT((xlon-xcglonLLmi(kiceS))/dphid)))
   jy=MAX(1,MIN(nLLj,NINT((ycglatLLma(kiceS) - ylat)/dthed)))
 ENDIF

 RETURN
END SUBROUTINE xy2ijB
!=======================================================================


!=======================================================================
SUBROUTINE ij2xy(ix,jy,xlon,ylat)
!=======================================================================
 INTEGER, INTENT(IN) :: ix, jy
 REAL, INTENT(out) :: xlon, ylat

 
 IF(IgridType(kiceS)==2) THEN !PS grid
# 86

 ELSE
   ylat=ylatma(kiceS) + dthed*(0.5 -REAL(jy))
   xlon=xlonmi(kiceS) + dphid*(REAL(ix) - 0.5)
 ENDIF

 RETURN
END SUBROUTINE ij2xy
!=======================================================================

!=======================================================================
REAL FUNCTION finterp(f,xlon,ylat,ki)
!=======================================================================

# 103


REAL, INTENT(IN)                     :: f(0:mNAp,0:mNBp)
REAL, INTENT(IN)                     :: xlon
REAL, INTENT(IN)                     :: ylat
INTEGER, INTENT(IN)                  :: ki

REAL :: ax, ay, wx, wy
INTEGER :: ix, jy, ixp, jyp

ax = (xlon-xlonmi(ki) + dphid/2.)/dphid
ay = (ylatma(ki) + dthed/2. - ylat)/dthed

# 122


ix=INT(ax)
jy=INT(ay)
ixp=1+ix
jyp=1+jy

wx=ax-REAL(ix)
wy=ay-REAL(jy)
!t      write(*,*)"finterp x,y,ix,jy,wx,wy: ",x,y,ix,jy,wx,wy
IF(ix<0 .OR. jy<0 .OR. wx>1 .OR. wx<0 .OR. wy>1 .OR. wy<0) THEN
  WRITE(*,*)"finterp error x,y,ix,jy,wx,wy: ",xlon,ylat,ix,jy,wx,wy
END IF
finterp=wx*wy*f(ixp,jyp) +(1.-wx)*wy*f(ix,jyp)  &
    +wx*(1.-wy)*f(ixp,jy) +(1.-wx)*(1.-wy)*f(ix,jy)

IF((finterp>f(ixp,jyp) .AND. finterp>f(ixp,jy) .AND.  &
      finterp>f(ix,jyp) .AND. finterp>f(ix,jy)) .OR.  &
      (finterp<f(ixp,jyp) .AND. finterp<f(ixp,jy) .AND.  &
      finterp<f(ix,jyp) .AND. finterp<f(ix,jy))) THEN
  WRITE(*,*)"finterp error2 ",finterp,f(ix,jyp) ,f(ixp,jyp),f(ixp,jy),f(ix,jy)
END IF

RETURN
END FUNCTION finterp
!=======================================================================

!=======================================================================
REAL FUNCTION finterpNG(f,xlon,ylat,ki)
!=======================================================================

# 156


REAL, INTENT(IN)                     :: f(mNA,mNB)
REAL, INTENT(IN)                     :: xlon
REAL, INTENT(IN)                     :: ylat
INTEGER, INTENT(IN)                  :: ki

REAL :: ax, ay, wx, wy
INTEGER :: ix, jy, ixp, jyp

ax = (xlon-xlonmi(ki) + dphid/2.)/dphid
ay = (ylatma(ki) + dthed/2. - ylat)/dthed

# 175


ix=INT(ax)
jy=INT(ay)
ixp=1+ix
jyp=1+jy

wx=ax-REAL(ix)
wy=ay-REAL(jy)
!t      write(*,*)"finterpNG x,y,ix,jy,wx,wy: ",x,y,ix,jy,wx,wy
IF(ix<0 .OR. jy<0 .OR. wx>1 .OR. wx<0 .OR. wy>1 .OR. wy<0) THEN
  WRITE(*,*)"finterpNG error x,y,ix,jy,wx,wy: ",xlon,ylat,ix,jy,wx,wy
END IF
finterpNG=wx*wy*f(ixp,jyp) +(1.-wx)*wy*f(ix,jyp)  &
    +wx*(1.-wy)*f(ixp,jy) +(1.-wx)*(1.-wy)*f(ix,jy)

IF((finterpNG>f(ixp,jyp) .AND. finterpNG>f(ixp,jy) .AND.  &
      finterpNG>f(ix,jyp) .AND. finterpNG>f(ix,jy)) .OR.  &
      (finterpNG<f(ixp,jyp) .AND. finterpNG<f(ixp,jy) .AND.  &
      finterpNG<f(ix,jyp) .AND. finterpNG<f(ix,jy))) THEN
  WRITE(*,*)"finterpNG error2 ",finterpNG,f(ix,jyp) ,f(ixp,jyp),f(ixp,jy),f(ix,jy)
END IF

RETURN
END FUNCTION finterpNG
!=======================================================================


END MODULE modgengrid
