MODULE modinparm

IMPLICIT NONE
SAVE

INTEGER, PARAMETER :: IFMWPext=0, kANT=0, kNA=0, kEA=0, kTIB=0, kGRonly=0, kGR=0 !if not present, set k??=0
REAL :: rmu, fslid, fnflow, fnpre, fnPdexp, &
    fnTdfscale, fcalvin, fcalvin2, fc3, pfactdwcrack, cfacemelt

REAL :: rmevap, rlapselgm
REAL :: fbedpow, dsb

REAL :: rhetillmin, wGF1, GfHbay !needed for USE  statement or argument in other routine, even if not set

REAL :: zpinFncoef=1.0

REAL :: sealflot, sealmin

! softHEINO parameters
REAL :: rLapseRate, rPDDmelt, hpre, precRef, rTnorth, rTgrad 

REAL :: hwbCrit, rBedDrainRate, rNeffFact ! Neff0D basal hydrol params

INTEGER, PARAMETER :: IUNN=101

! need IFCoulomb = 0 in Makefile to activate
REAL :: rCfrict = 1.0 ! friction coefficient Pattyn, 2017 inverted rCfrict spatially within range 0.01:5

! sub ensembles
REAL :: subEnszTrampF, subEnsWTbMN, subEnsCRHNOSb, subEnsdykm, subEnsExp

CONTAINS

SUBROUTINE readparms(notes,icalv)
USE modflags
USE modbeddrag, ONLY : POWb

CHARACTER (LEN=*), INTENT(IN OUT)        :: notes
INTEGER, INTENT(INOUT)                  :: icalv

INTEGER, PARAMETER :: Nparmnc=10 !# of params before EOFs
INTEGER, PARAMETER :: Nparm=18, Nparmin=18
REAL :: parm(Nparm), parmin(Nparmin)
INTEGER :: k, ki, idnnformat=1, idnncod2=Nparm
INTEGER :: krun

fc3=0.
fslid=0.7
fnflow=5.0
fcalvin2=0.3
dsb=2.5

READ(*,*)

IF(ifHEAD==1) WRITE(*,*)"fnflow,fcalvin2"
READ(*,*) fnflow,fcalvin2
IF(ifHEAD==1) WRITE(*,*)fnflow,fcalvin2

PRINT *,"NOTE, parms file now read with NO header"

CALL askofil(37,'ruNparm list')
READ(*,*)krun
parm=-100.

IF(krun>1) THEN
  DO k=1,(krun-1)
    READ(37,*)
  END DO
END IF
READ(37,*)(parmin(ki),ki=1,Nparmin)
CLOSE(37)

parm=parmin(1:Nparm)
                          !below are input ranges for ensemble vector file
rmu=parm(1)               !0.01:2 soft bed sliding parameter, 0.5
fslid=parm(2)            ! 0:3 hard bed sliding parameter, 0.5
rLapseRate=-1.e-2*parm(3) !pd lapse rate 0.5:1 K/hm : 5-> 10 K/km, 0.8
rPDDmelt=1.e-2*parm(4)    ! 0.5:1.2 cm/PDD  0.83
hpre=0.1*parm(5)          ! 0.4:2  1.0   precip = precRef*exp(hpre*Tsurf) 
precRef= parm(6)          !1:3 m/yr  2.5 
rTnorth= -10.*parm(7)     ! 2:4 C  3.2
rTgrad=0.01*parm(8)      ! 0.5:1.2 C/100km 0.9
POWb=NINT(10.*parm(9)) ! 0.1:0.7
parm(9)=0.1*REAL(POWb)
rCfrict=parm(10)         !0.1:2 Coulomb friction coefficient

! sub ensembles
subEnszTrampF=parm(11)
subEnsWTbMN=parm(12)
subEnsCRHNOSb=parm(13)
subEnsdykm=parm(14)
subEnsExp=10.*parm(15)

hwbCrit= 0.01 *10.**(2*parm(16)) ! : 0:1 ; effective bed roughness scale m -> 0.01:1 m
rBedDrainRate= 1.e-3 * 10.**parm(17) !: 0:1 ! constant bed drainage rate  (m/yr) -> 1:10 mm/yr
!rNeffFact= 2.e4* 10.**(1.699*parm(18)) !0:1 Basal drag factor for 1/Neff term, 1.699=log10(50) so gives 2e4:1e6 range
rNeffFact= 2.e4* 10.**parm(18) !0:1 Basal drag factor for 1/Neff term, gives 2e4:2e5 Pa range

IF(rmu*rNeffFact/2.e4 > 20.0) THEN !impose Ub(Taub=30kPa)<=60 km/yr
  PRINT *,"rNeffFact is too high:",rNeffFact
  rNeffFact= 40.e4/rmu
  PRINT *,"WARNING: reducing rNeffFact to preserve rmu * rNeffFact/2e4 < 20, rNeffFact now:",rNeffFact
ENDIF

!fcalvin=parm(6)           ! 0.1:1.0 Uc = 10.*fcalvin km/yr
!pfactdwCrack=100.*parm(7) ! 0.5:3 ! calv dependence on meltwater
!CfaceMELT=10.*parm(8)     ! 5:20 m/yr on face/ ! face/SSM
!dsb=parm(8)              ! 0:

sealflot=0.               !0->20
sealmin=20.               !20->50


IF(ifHEAD==1) THEN
WRITE(IUNN,120)notes
120  FORMAT('#',a)
WRITE(IUNN,121)idnnformat, idnncod2,Nparmin
121  FORMAT(i1,1X,i2,1X,i2,3(1X,f7.5))
WRITE(IUNN,'(8(1x,f7.5))') 
WRITE(IUNN,122)parm(1),fslid,fnflow
WRITE(IUNN,124)icalv,sealflot*0.1,sealmin*0.1, fcalvin
122  FORMAT(7(f7.5,1X))
124  FORMAT(i1,7(1X,f7.5))

WRITE(IUNN,'(33(1x,f7.5))')parm
CALL FLUSH(IUNN)


PRINT *, "rmu : ",rmu
PRINT *, "fslid ", fslid
PRINT *,"rLapseRate K/m: ",rLapseRate
PRINT *,"rPDDmelt m/PDD: ",rPDDmelt
PRINT *,"hpre: ",hpre
PRINT *,"precRef m/yr:",precRef
PRINT *,"rTnorth K:",rTnorth
PRINT *,"rTgrad K/km:",rTgrad
PRINT *,"POWb:",POWb

! sub ensembles
PRINT *,"subEnszTrampF:",subEnszTrampF
PRINT *,"subEnsWTbMN:",subEnsWTbMN
PRINT *,"subEnsCRHNOSb:",subEnsCRHNOSb
PRINT *,"subEnsdykm:",subEnsdykm
PRINT *,"subEnsExp:",subEnsExp

PRINT *,"hwbCrit, rBedDrainRate :",hwbCrit, rBedDrainRate
PRINT *,"rNeffFact :",rNeffFact

!PRINT *, "fnpre, fnTdfscale: ",fnpre, fnTdfscale

!PRINT *, "fcalvin, pfactdwcrack:",fcalvin, pfactdwcrack
!PRINT *, "cfacemelt: ",cfacemelt

ENDIF

RETURN
END SUBROUTINE readparms

END MODULE modinparm

