# 1 "icedyn.F90"
!-----------------------------------------------------------------------
SUBROUTINE icedyn(h, hs, ua, va, ub, vb, uia, via, uit, vit, hb, sealev, budgall, &
   timeice, dt, ifConvFail, Icheck)

! L  ifrest -> ifRestart, restart now for ifRestart>0 (before was >-1)
! L  iffirsr -> ifStartIceDyn (following GSM istart nomenclature)

!SUBROUTINE icedyn (h, hs, hb, hbsd, hw, sedim, t,  &
!    arhap, s1a0, s1b0, s2a0, s2b0, heath, heatb,  &
!    budgall, budgneg, cliffmelt, calvice, crevliq, ifnexo, fedge, maskh, visc,  &
!    sih, siu, siv, dfu, dfv, dbu, dbv,  &
!    ua, va, uia, via, ub, vb, uit, vit, uadv, vadv,  &
!    hu, hv, uw, vw, masku, maskv, crhu, crhv, fsedu, fsedv,  &
!    fracgu, fracgv, muind, mvind, thetau, thetav,  &
!    hgu, hgv, qgu, qgv, uschoof, vschoof, fschoofu, fschoofv,  &
!    uagrid, vagrid, fluxgrdu, fluxgrdv, fluxschu, fluxschv,  &
!    angnormu, angnormv, numh, nuvtot, itera, iterc, totflow,  &
!    maskwater, indlake, npoilake, nlake, sealev,  &
!    timeice, dt, ifRestart, iffirse, ifStartIceDyn)

!     Steps combined ice sheet-shelf dynamics.
!     Shelf dyanimcs (SSA) solves for ua,va (dotria, dosparse).
!     Sheet dynamics (SIA) solves for uia,via, and also time-implicitly
!     steps ice thickness h including SSA ua,va (icestep).
!     ua,va,uia,via are passed, saved (in main prog) between iterations.

!     SSA solutions really solve for ua = ub+uia, given uia:
!     LHS(ua) = driving stress + C*(ua-uia).
!     ub are generally just diagnostic, set to ua-uia.
!     Where the SSA solution itself is needed (schoofgl, icestep,
!     ua-uia or uagrid-uia is used).

!     Imposed Schoof fluxes and velocities are assumed (imagined) to
!     include both stretching and internal deformation across g.l.,
!     i.e., uschoof are equiv. to ua, not ub, and are imposed on
!     the ua solution in dotria/dosparse.

!     See notes 1/28/12 for flow diagram.

! BSL: track BSL to find changes (exception: NX,NY)
! removed zero,izero,reseti,resetr,scopy_i,scopy
! sealev -> sealev(i,j) definition sealev(0:NXp,0:NYp)
! NX,NY -> ni,nj (non definitions cases)
! removed save statements with implicit saves
! PARAMETERS are capitalized
! replace GOTO statements with EXIT or CYCLE where possible
! basal heating due to sliding: heatb [J/m2/y]
! vertical shear contribution to internal deformation work heating: heati [J/m3/y] ~ dbu dbv sih
! heat horizontal shear and stretching contributions to deformation work heating: heath [J/m3/y]
! convergence failure state added convfail
! velocity fields (0:NXp,0:NYp)->(0:NX,0:NY) for ua ub uia uit uadv uagrid uaprev
! separate power law for hard bed and till FLAG dualpowb

! uia=0 for SSA cells unless -DUIACROSS, uia is nonzero for SIA cells
! independent of compflag (after feb10/19change)

! ^^^

 USE modconstants
 USE modPHOMgrid
! USE modPHOMsparse
 USE modPHOMvardyn
 USE modPHOMvartherm
 USE modPHOMvarhydrol
 USE modPHOMvarocean
 USE modgridm, ONLY : ni, nj, darea, dxg, dyg, dxu, dyu, dxv, dyv, dxc, dyc, kiceS
 USE modIU, ONLY : IUconvg, IOTERM
# 70

 USE modbeddrag, ONLY : POWb

 USE modflags, ONLY : ifHEAD
 USE modinparm, ONLY : fslid, kNA, kANT, kEA
# 77




 IMPLICIT NONE 

!     passed:

 REAL, INTENT(IN OUT) :: h(0:NXp,0:NYp) !L changed to ghosted range
 REAL, INTENT(IN OUT) :: hs(0:NXp,0:NYp)!L changed to ghosted range
 REAL, INTENT(IN)     :: hb(0:NXp,0:NYp)
 REAL, INTENT(IN OUT) :: ua(0:NX,0:NY)
 REAL, INTENT(IN OUT) :: va(0:NX,0:NY)
 REAL, INTENT(OUT)    :: ub(0:NX,0:NY)
 REAL, INTENT(OUT)    :: vb(0:NX,0:NY)
 REAL, INTENT(IN OUT) :: uia(0:NX,0:NY), via(0:NX,0:NY), uit(0:NX,0:NY), vit(0:NX,0:NY)
 REAL, INTENT(IN)     :: budgall(NX,NY)
 REAL, INTENT(IN)     :: sealev(0:NXp,0:NYp)
 REAL, INTENT(IN)     :: timeice
 REAL, INTENT(IN)     :: dt
 INTEGER, INTENT(IN)  :: Icheck

 REAL :: & 
    s1a0(NX,NY),           s1b0(NX,NY), &
    s2a0(NX,NY),           s2b0(NX,NY), &
    s1a(NX,NY,0:NLEVp),    s2a(NX,NY,0:NLEVp),    s3a(NX,NY,NLEV), &
    s1b(NX,NY,0:NLEVp),    s2b(NX,NY,0:NLEVp),    s3b(NX,NY,NLEV), &
    thetau(NX,NY),         thetav(NX,NY), &
    qgu(0:NX,0:NY),        qgv(0:NX,0:NY), &
    fschoofu(0:NX,0:NY),   fschoofv(0:NX,0:NY), &
    budgneg(NX,NY), &
    uadv(0:NX,0:NY),       vadv(0:NX,0:NY), &
    hu(0:NX,0:NY),         hv(0:NX,0:NY), &
    uagrid(0:NX,0:NY),     vagrid(0:NX,0:NY), &
    dfu(0:NX,0:NY),        dfv(0:NX,0:NY), &
    totflow
!    angnormu(0:NX,0:NY),   angnormv(0:NX,0:NY), &

 REAL ::  uschoof(0:NX,0:NY),    vschoof(0:NX,0:NY), &
    fluxgrdu(0:NX,0:NY),   fluxgrdv(0:NX,0:NY), &
    fluxschu(0:NX,0:NY),   fluxschv(0:NX,0:NY)


 INTEGER :: & 
    maskh(NX,NY), &
    numh,   nuvtot, &
    itera,  iterc

 REAL :: vmax, delmaxa, delmaxc, delmaxaOLD, delmaxcOLD !, crhuTillcrit, crhuSlidcrit


 REAL, PARAMETER :: SSACRITU= 30.0 !m/yr
 REAL, PARAMETER :: SSAhardCRITU= 200.0 !m/yr, test ensemble showed 200 better than 100 and 500
# 138


 REAL :: CRHUcrit=0

! say 200 m/yr edge, 200m thick, = 4e4m/yr, so 20 km at 2 m in 1 year
! mar3/22 changed to 1m/yr given Frobisher Bay hole results
 REAL, PARAMETER :: HthinFloatingCut=0.5 ! m for thin floating ice, unfortunately not tracked in budget calcs

 INTEGER :: i, j, ii, jj, look, iterasch, istart=1

!     local:

 REAL :: &
    h0(0:NXp,0:NYp),       h1(0:NXp,0:NYp),  &
    coef1(NX,NY), &          
    dhsu(0:NX,0:NY),       dhsv(0:NX,0:NY),  &
    coefbu(0:NX,0:NY),     coefbv(0:NX,0:NY),  &
    lhsu(0:NX,0:NY),       lhsv(0:NX,0:NY), &
    hu0(0:NX,0:NY),        hv0(0:NX,0:NY),  &
    dhsu0(0:NX,0:NY),      dhsv0(0:NX,0:NY),  &
    dfu0(0:NX,0:NY),       dfv0(0:NX,0:NY)
 INTEGER :: medgeu(0:NX,0:NY), medgev(0:NX,0:NY)

!     for basal heating calculations:
 REAL :: heatbu(0:NX,0:NY),   heatbv(0:NX,0:NY)

!     cross grid:
 REAL :: coef2(0:NX,0:NY)
 INTEGER :: maskc(0:NX,0:NY)

!     for iteration (a,c) convergence criteria:
 REAL :: uaprev (0:NX,0:NY),  vaprev (0:NX,0:NY), &    ! loop a
    h1prev(0:NXp,0:NYp)                           ! loop c

! convfail
 INTEGER, INTENT(OUT) :: ifConvFail
!INTEGER, INTENT(IN    ) :: idieA,idieS,idieC,iletfail
 INTEGER :: icntAfail=0, icntSfail=0, icntCfail=0, iconvSfail,icntTT=1
!REAL, PARAMETER :: Hmin=90.
! ^^^

!     for Hindmarsh and Payne (1996) convergence, loop c:
!     dimension delh1(NX,NY), delh1prev(NX,NY)

! convfail
! briggs PHOM
!REAL, parameter :: delcritab = 30.0   ! m/a  ! .01,.03
!REAL, parameter :: delavgcrita = 0.1   ! m    ! 1.e-6
!REAL, parameter :: delcrita = 2.0   ! m/a  ! .01,.03
!REAL, parameter :: delcritc = 2.0   ! m    ! .01,.03
! REAL, PARAMETER :: delcrita = 5.0   ! m/a  ! .01,.03
! REAL, PARAMETER :: delcritc = 5.0   ! m    ! .01,.03
 REAL, PARAMETER :: delcrita = 10.0   ! m/a  ! .01,.03
 REAL, PARAMETER :: delcritc = 5.0   ! m    ! .01,.03
!REAL, parameter :: delcrita = 0.5   ! m/a  ! .01,.03
!REAL, parameter :: delcritc = 0.5   ! m    ! .01,.03
! newest values in PHOM
!REAL, PARAMETER :: DELCRITA = .01   ! m/a  ! .01,.03
!REAL, PARAMETER :: DELCRITC = .01   ! m    ! .01,.03
! ^^^

# 202

# 205

 INTEGER, PARAMETER :: NITERA = 2

# 213

 INTEGER, PARAMETER :: NITERC = 2



 INTEGER, PARAMETER :: NITERASCH = NITERA
! INTEGER, PARAMETER :: NITERASCH = 2


!#if defined (1) || ! defined (NOSUBGRIDEDGESHELF)

 INTEGER, PARAMETER :: NLOOK = 4
! BSLnew
 INTEGER, PARAMETER :: ISH(NLOOK) = (/-1, 1,  0, 0 /)
 INTEGER, PARAMETER :: JSH(NLOOK) = (/ 0, 0, -1, 1 /)
! old
!INTEGER ISH(NLOOK), JSH(NLOOK)
!SAVE ISH, JSH
!DATA ISH /-1, 1,  0, 0 /
!DATA JSH / 0, 0, -1, 1 /
! ^^^


# 245

 INTEGER, PARAMETER :: MxSpeedLIMIT=40000

! INTEGER  :: NITERASCH, NITERA
 REAL :: ifDOsor

# 253

 INTEGER, PARAMETER :: ifassimPD=0


# 260


 INTEGER :: iflcnt

# 266

 REAL, PARAMETER  :: DYNMASKHcut=0.


! REAL, PARAMETER  :: DYNMASKHcut=0.1

!-------------------------
! Start of executable code
!-------------------------

! BSLoutput transect CALL jouttran(41,coefbu(0:nx,0:ny),coefbv(0:nx,0:ny),0.,0.,94,1)

!i=31
!j=31

# 283


! convfail
 ifConvFail=0
 iconvSfail=0
 if (mod(nint(100.*abs(timeice)),5000)==0) then
   write(*,'("icntAfail, icntSfail, icntCfail: ",3(1x,i8))'),  &
      icntAfail, icntSfail, icntCfail
 endif
! ^^^
# 295


# 305


 IF(Icheck /= 111) CALL outerrI("interface error on icedyn call, Icheck:",Icheck)

!       Set basal coefficient cutoff to select points for inclusion
!       in elliptic SSA (shelf) eqns, or sheet only (m[u,v]ind below)

!#if defined (CRHUCRIT)
!crhucrit = crhucrit * (1.e5**2)/(1.e5**POWb)
!crhucrit = crhucrit
!#elif defined (EISLINE) || defined (TEST2D) || defined (TEST2DAXI)
!crhucrit = 0.       ! all points included
!#else
! dualpowb crhucrit might need revision with POWb=3/1 for hardb and till
!CRHUtillCrit = 20.e-4 ! 20 m/yr / 10 kPa
!CRHUslidCrit=CRHUtillCrit*(10e5**(POWbtill-POWbhardb)) !1.e-14
!crhucrit = 1.e-4 ! 10 m/yr for 100kPa linear only
!crhucrit = 1.e-5 ! 10 m/yr for 100kPa linear
!crhucrit = 1.e50    ! no grounded points included - no SSA
!     crhucrit = 1.e-50    ! all grounded points included - all SSA
!crhucrit = 1.e-8 * (1.e5**2)/(1.e5**POWbtill) ! old SHELFCRHU8
! old
!crhucrit = 1.e-8    * (1.e5**2)/(1.e5**POWb) ! old SHELFCRHU8
! ^^^
!     1.e-6 is ~stream for POWb=2 (see basecoef, 1.e5 is "taunorm"):
!#endif

!       Set |ua|,|va| speed limit (used after dosparse and in schoofgl)

# 342

!vmax = 0.75 * dd0 / dt
!vmax = 0.1 * dd0 / dt
 vmax = MxSpeedLIMIT


 IF(istart==1) THEN
# 353

     CRHUcrit=30.0/(30.e3**POWb)

   IF(ifHEAD==1) THEN
# 359

     PRINT *,"icedyn version mar18/22, hu/v set to avg(Heff,Hgrnd) for reversed slope GL, using maskwater GL"

# 364

     PRINT *,"icedyn version mar16/22, thetacalc fix"! and 0 u/vschoof set for reverse hs slope at GL"

     PRINT *,"using -DSUBGRIDEDGE"

# 371

# 374

# 377

     PRINT *,"icedyn version sep24/20, fixed heath"

     PRINT *,"threshold for ice dynamics calc: DYNMASKHcut=",DYNMASKHcut," m"
     PRINT *,"fpin(Gaussian CDF(hw/hbstd)) to 0.295 then (1-(hw/2hbstd))**6"
!  PRINT *,"2.*arh(,,NLEV-1:NLEV):bottom 3% and  0.5*arh(,,1:NLEVo2)"
!  PRINT *,"2.*arh(,,NLEV) and for glacIndex<0.5 0.5*arh(,,1:NLEVo2)"
# 386

     PRINT *,"POWb:",POWb

# 391

# 408

     WRITE(*,'("icedyn start: SSA: -DSSAcritU: ",(f5.1,1x),"m/yr")') SSACRITU !no -Ddualpowb

     PRINT *,"SIA imposed Max |Vb| is 10 km/yr" ! VbMaxSIA=10.e3 !m/yr
     WRITE(*,'("delcrita, delcritc:",2(x,f5.2))') delcrita,delcritc
     WRITE(*,'("NITERA, NITERC, vmax:",2(x,I3),1x,f7.0)') NITERA,NITERC,vmax
     PRINT *,"HthinFloatingCut=",HthinFloatingCut,"/dt"
!     PRINT *,"HthinFloatingCut=",HthinFloatingCut
   ENDIF
   istart=0
 ENDIF

!       Set dynamic mask for no flow *out* of grid boxes with fedge < 1.
!       fedge has been set for grounded pts in climate_all, and for
!       floating in subr ocean. Implemented here just in the advection
!       of h, in icestep.

! BSLnew
 medgeu=1
 medgev=1
! old
!CALL reseti (medgeu, (NXp+1)*(NYp+1), 1)
!CALL reseti (medgev, (NXp+1)*(NYp+1), 1)
! ^^^

!#if defined (1) || ! defined (NOSUBGRIDEDGESHELF)

 DO j = 1,nj
   DO i = 1,ni
     IF (fedge(i,j) < 1.) THEN
       DO look=1,4
         ii = MAX (1, MIN (ni, i + ISH(look)))
         jj = MAX (1, MIN (nj, j + JSH(look)))
         IF ( .NOT. ( fedge(ii,jj) == 1. .AND. h(ii,jj) > 0.  &
            .AND. hs(ii,jj) > hs(i,j) ) ) THEN
           IF (look == 1) medgeu(i-1,j  ) = 0
           IF (look == 2) medgeu(i  ,j  ) = 0
           IF (look == 3) medgev(i  ,j-1) = 0
           IF (look == 4) medgev(i  ,j  ) = 0
         ENDIF
       ENDDO
     ENDIF
   ENDDO
 ENDDO


 nuvtot = 0
 iterasch = 0
 itera = 0
 iterc = 0

 heath=0.0
 siu=0.0
 siv=0.0
 sih=0.0


! BSLtherm from icetherm
!Calculate s1a, s2a, s3a (vertical integral terms) for
!computation of u,uh,v,vh,heati
 CALL satable_dyn (s1a, s2a, s3a, s1b, s2b, s3b, arh) !needs arh(Tice)
 s1a0(1:ni,1:nj)=s1a(1:ni,1:nj,0)
 s1b0(1:ni,1:nj)=s1b(1:ni,1:nj,0)
 s2a0(1:ni,1:nj)=s2a(1:ni,1:nj,0)
 s2b0(1:ni,1:nj)=s2b(1:ni,1:nj,0)

!        Impose boundary constraints on h for various expts

!.......................................
# 509

!.......................................................................
   

IF (ni > 1) THEN
 h(1, 1:nj) = 0.
 h(ni,1:nj) = 0.
ENDIF
IF (nj > 1) THEN
 h(1:ni, 1) = 0.
 h(1:ni,nj) = 0.
ENDIF


!.....................
# 530

!.....

!        Set masks (maskh, masku, maskv). All masks are set
!        outside iteration loops, ignoring changes of non-zero <->
!        zero ice (h) between iterations.

!        Set maskh (0/1 if no ice/ice, h grid)

maskh=0

numh = 0
DO j = 1,nj
 DO i = 1,ni
   IF (h(i,j) > DYNMASKHcut) THEN
     maskh(i,j) = 1
     numh = numh + 1
   ENDIF
 ENDDO
ENDDO

!        Set maskc (cross grid)

maskc=0

DO j = 1,nj-1
 DO i = 1,ni-1
   maskc(i,j)=maskh(i,j)*maskh(i+1,j)*maskh(i,j+1)*maskh(i+1,j+1)
 ENDDO
ENDDO

!        Set masku,v (1 to allow flow, 0 for no flow if no ice to
!        either side, or lower ice than non-ice surface) (u,v grids).
!        (nb: outside loop C, so ignores hs changes within iters).

masku=0

DO j = 1,nj
 DO i = 1,ni-1
   IF ( maskh(i,j) == 1 .AND. maskh(i+1,j) == 1 ) THEN
     masku(i,j) = 1
   ELSE IF ( maskh(i,j) == 1 .AND. maskh(i+1,j) == 0 .AND.  &
      hs(i,j) > hs(i+1,j) ) THEN
     masku(i,j) = 1
   ELSE IF ( maskh(i,j) == 0 .AND. maskh(i+1,j) == 1 .AND.  &
      hs(i,j) < hs(i+1,j) ) THEN
     masku(i,j) = 1
   ENDIF
# 581

 ENDDO
ENDDO

maskv=0

DO j = 1,nj-1
 DO i = 1,ni
   IF ( maskh(i,j) == 1 .AND. maskh(i,j+1) == 1 ) THEN
     maskv(i,j) = 1
   ELSE IF ( maskh(i,j) == 1 .AND. maskh(i,j+1) == 0 .AND.  &
      hs(i,j) > hs(i,j+1) ) THEN
     maskv(i,j) = 1
   ELSE IF (maskh(i,j) == 0 .AND. maskh(i,j+1) == 1 .AND.  &
      hs(i,j) < hs(i,j+1) ) THEN
     maskv(i,j) = 1
   ENDIF
# 601

 ENDDO
ENDDO

!        Zero diagnostic quantities (some necessary for finite diffs
!        near ice edges)

hu=0.
hv=0.
dhsu=0.
dhsv=0.
uadv=0.
vadv=0.
lhsu=0.
lhsv=0.
dfu=0. ! -1*driving stress_x (Pa)
dfv=0. ! "" _y
dbu=0. ! basal (drag) stress (Pa) on ice
dbv=0. ! "" _y


DO j = 1,nj
 DO i = 1,ni
   IF (masku(i,j) == 0) THEN
     ua(i,j)  = 0.
     uia(i,j) = 0.
     uit(i,j) = 0.
     ub(i,j)  = 0.
     uagrid(i,j) = 0.
!!$   ELSE IF( maskwater(i,j)==1 .AND. maskwater(MIN(i+1,ni),j)==1 &
!!$      .AND. ( (h(i,j)>1.0 .AND. h(MIN(i+1,ni),j)<1.0 .AND. ua(i,j)<0) .OR. &
!!$      (h(i,j)<1.0 .AND. h(MIN(i+1,ni),j)>1.0 .AND. ua(i,j)>0) ) )  THEN
!!$     ua(i,j)  = 0.
!!$     uia(i,j) = 0.
!!$     uit(i,j) = 0.
!!$     ub(i,j)  = 0.
!!$     uagrid(i,j) = 0.
   ENDIF
   IF (maskv(i,j) == 0) THEN
     va(i,j)  = 0.
     via(i,j) = 0.
     vit(i,j) = 0.
     vb(i,j)  = 0.
     vagrid(i,j) = 0.
!!$   ELSE IF( maskwater(i,j)==1 .AND. maskwater(i,MIN(j+1,nj))==1 &
!!$      .AND. ( (h(i,j)>1.0 .AND. h(i,MIN(j+1,nj))<1.0 .AND. va(i,j)<0) .OR. &
!!$      (h(i,j)<1.0 .AND. h(i,MIN(j+1,nj))>1.0 .AND. va(i,j)>0) ) ) THEN
!!$     va(i,j)  = 0.
!!$     via(i,j) = 0.
!!$     vit(i,j) = 0.
!!$     vb(i,j)  = 0.
!!$     vagrid(i,j) = 0.
   ENDIF
 ENDDO
ENDDO


!        If first timestep of run, zero all velocities. Else, keep
!        for next timestep's initial guess (u,v grids, all passed)
!        (some necessary for finite diffs near ice edges)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~
!       ua,va are on restart file, so only set here if cold start:
# 699


uagrid=ua
vagrid=va

# 711


!        Test for no ice. If so, just apply local forcing
!        and skip all velocity calcs

IF (numh == 0) THEN ! no ice, just add SMB and exit
 totflow = 0.
 DO j=1,nj
   DO i=1,ni
     h(i,j) = h(i,j) + budgall(i,j)*dt
     IF (h(i,j) < 0.) THEN
       budgneg(i,j) = -h(i,j)/dt
       h(i,j) = 0.
     ELSE
       budgneg(i,j) = 0.
     ENDIF
     cliffmelt(i,j) = 0.
   ENDDO
 ENDDO
 GOTO 3500
ENDIF


CALL findwater (hw, maskwater, indlake, npoilake, nlake, h, hb, sealev, Icheck)


!============================
!     Top of loop C (~Picard)
DO iterc = 1,NITERC
!============================
  
!        Adjust hw and hs (for change in h):


!     don't call findwater here: causes blowups at 10 km (8/15/13)
!     call findwater (maskwater, indlake, npoilake, nlake,
!    *                h, hb, hw, sealev)
! LT test mar/22, findwater still unstable, suspect oscillations block convergence
!CALL findwater (hw, maskwater, indlake, npoilake, nlake, h, hb, sealev, Icheck)

 CALL adjustpres (maskwater, indlake, npoilake, nlake, h, hb, hw, hs, sealev, Icheck)

!       Set hu, hv, dhsu, dhsv, dfu, dfv  (u,v grids)
!       Do inside loop C to capture hs changes within iters.

 DO j = 1,nj
   DO i = 1,ni-1
     IF (masku(i,j) == 1) THEN
       hu(i,j) = 0.5*(h(i,j) + h(i+1,j))
# 762

       IF(hs(i+1,j) > hs(i,j) .AND. maskwater(i,j) == 0 .AND. maskwater(i+1,j) /=0) THEN ! fix for reverse slope GL

!         hu(i,j) = MIN( hu(i,j) ,(hs(i+1,j)-hb(i,j)) )
         hu(i,j) = 0.5*( h(i,j) + (hs(i+1,j)-hb(i,j)) )
# 769

       ELSEIF(hs(i+1,j) < hs(i,j) .AND. maskwater(i,j) /=0 .AND. maskwater(i+1,j) == 0) THEN

         hu(i,j) = 0.5*( h(i+1,j) + (hs(i,j)-hb(i+1,j)) )
       ENDIF
       dhsu(i,j) = (hs(i+1,j) - hs(i,j)) / dxu(i,j)
       dfu(i,j)  = RHOIc*gEARTH*hu(i,j)*dhsu(i,j) !  - driving stress
     ENDIF
   ENDDO
 ENDDO

 DO j = 1,nj-1
   DO i = 1,ni
     IF (maskv(i,j) == 1) THEN
       hv(i,j) = 0.5*(h(i,j) + h(i,j+1))
# 786

       IF(hs(i,j+1) > hs(i,j) .AND. maskwater(i,j) == 0 .AND. maskwater(i,j+1) /= 0) THEN ! fix for reverse slope GL

!         hv(i,j) = MIN( hv(i,j) ,(hs(i,j+1)-hb(i,j)) )
         hv(i,j) = 0.5*( h(i,j) + (hs(i,j+1)-hb(i,j)) )
# 793

       ELSEIF(hs(i,j+1) < hs(i,j) .AND. maskwater(i,j) /= 0 .AND. maskwater(i,j+1) == 0) THEN

         hv(i,j) = 0.5*( h(i,j+1) + (hs(i,j)-hb(i,j+1)) )
       ENDIF
       dhsv(i,j) = (hs(i,j+1) - hs(i,j)) / dyv(i,j)
       dfv(i,j)  = RHOIc*gEARTH*hv(i,j)*dhsv(i,j)
     ENDIF
   ENDDO
 ENDDO

# 806



!     store original h (for whole time-step) in h0, etc,
!     (and initialize h1):
 IF (iterc == 1) THEN
   h0=h
   h1=h
   hu0=hu
   hv0=hv
   dhsu0=dhsu
   dhsv0=dhsv
   dfu0=dfu
   dfv0=dfv
 ENDIF

 h1prev=h1

!        Impose boundary constraints,etc, on velocs for various expts

!.......................................
# 858

!.....

!        Remove very thin floating ice BSLschoofdebug
 DO j=1,nj
   DO i=1,ni
     IF (maskwater(i,j) == 1 .AND. h(i,j) < HthinFloatingCut/dt) h(i,j)=0.0
!     IF (maskwater(i,j) == 1 .AND. h(i,j) < HthinFloatingCut) h(i,j)=0.0
   ENDDO
 ENDDO
!# if defined (ISMIP6GR)
! CALL forceMarineH(H,budgall,timeice)
!#endif


!        Set nuvtot and muind,mvind, indices from 2-D arrays into
!        linear sequence of selected points for tridia or sparse solns
!        below. Cutoff basal coefficient crhucrit set above.
!        (Can do outside A loop, depends only on mask[u,v],hw).

 muind=0
 mvind=0
 iflcnt=0
 nuvtot = 0
!----------------------

!----------------------
 DO j = 1,nj
   DO i = 1,ni
     IF (masku(i,j) == 1) THEN
!           include if both adjacent h-points have water,
!           or if this u-point is slippery enough (crhucrit set above):
# 894


! issue: maskwater is only set for beginning of iterations.
! If maskwater =0, hw is kept at 0. If maskwater==1, hw can go to 0 from adjustpres.
! So can get non-SSA with maskwater=1
!       IF ( (maskwater(i,j) > 0 .AND. maskwater(MIN(i+1,ni),j) > 0)  &
       IF ( (hw(i,j) > HWcut .AND. hw(MIN(i+1,ni),j) > HWcut)  & ! can't use maskwater since not updated for iterations
# 921


         .OR. (crhu(i,j)*ABS(dfu(i,j))**POWb > SSACRITU)) THEN
# 926


# 931

         nuvtot = nuvtot + 1
         muind(i,j) = nuvtot
# 936

       ENDIF
     ENDIF

     IF (maskv(i,j) == 1) THEN
# 945


!       IF ( (maskwater(i,j) > 0 .AND. maskwater(i,MIN(j+1,nj)) > 0)  & maskwater can be <0 for lakes
       IF ( (hw(i,j) > HWcut .AND. hw(i,MIN(j+1,nj)) > HWcut)  &
# 969


         .OR. (crhv(i,j)*ABS(dfv(i,j))**POWb > SSACRITU)) THEN
# 974


# 979

         nuvtot = nuvtot + 1
         mvind(i,j) = nuvtot
       ENDIF
     ENDIF
   ENDDO
 ENDDO
!-----

!-----

# 1003



# 1011


!@@@@@@@@@@@@@@@@@@@@@

!@@@@@@@@@@@@@@@@@@@@@

 uschoof=0.
 vschoof=0.
 fschoofu=0.
 fschoofv=0.
 thetau=-1.
 thetav=-1.

!        Extra elliptic calculation for velocities [u,v]bgrid without
!        anj Schoof imposed conditions, used (i) to compare fluxes
!        at g.l. with Schoof fluxes, for ad-hoc rule in schoofgl,
!        and (ii) to set buttressing factor in thetacalc.
!        n.b.: outside A loop, inside C loop.

 DO j=1,nj
   DO i=1,ni-1
     IF (muind(i,j) == 0) uagrid(i,j) = ua(i,j)
   ENDDO
 ENDDO
 DO j=1,nj-1
   DO i=1,ni
     IF (mvind(i,j) == 0) vagrid(i,j) = va(i,j)
   ENDDO
 ENDDO

!&&&&&&&&&&&&&&&&&&&&&&&&&&
 IF (nuvtot > 0) THEN
!&&&&&&&&&&&&&&&&&&&&&&&&&&
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!     Top of loop ASCH (shelf flow)
   DO iterasch = 1,NITERASCH
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
# 1050

      
! BSLnew
     uaprev=uagrid
     vaprev=vagrid
! old
!    CALL scopy ((nip+1)*(njp+1), uagrid, 1, uaprev, 1)
!    CALL scopy ((nip+1)*(njp+1), vagrid, 1, vaprev, 1)
! ^^^

!  Set various coefficients (coef1, coef2, coefb[u,v], heath, sih, siu, and siv), rest are INTENT(IN)
     CALL setcoefs (coef1, coef2, coefbu, coefbv, heath,  &
        maskh, arhap, h, hbsd, hw,  &
        masku, maskv, uagrid,vagrid,uia,via, fracgu,fracgv,  &
        uit, vit, sih, siu, siv, hu, hv, crhu, crhv,  &
        muind, mvind, maskc, iterasch, iterc)

!        Tridia or Sparse matrix solution of ice-shelf eqns
!        (for uagrid,vagrid)

!----------------------
     IF (nj == 1) THEN
!----------------------
       CALL dotridia  (uagrid, uschoof, fschoofu,  &
          hu, dhsu, hw, coef1, coefbu, uia, uw, muind, nuvtot)
!---------------------------
     ELSE IF (ni == 1) THEN
!---------------------------
       CALL dotridia2 (vagrid, vschoof, fschoofv,  &
          hv, dhsv, hw, coef1, coefbv, via, vw, mvind, nuvtot)
!---------
     ELSE
!---------

! INTENT(INOUT) : ua,va, and iconvSfail, rest are INTENT(IN)
       CALL dosparse (uagrid,vagrid, uschoof,vschoof,fschoofu,fschoofv,  &
          hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia,via, uw,vw,  &
          muind, mvind, nuvtot, hw, timeice, iterasch, iterc, 0, iconvSfail)
       IF(iconvSfail==1) THEN !  allow 1 more try via iconvSfail=2
         CALL dosparse (uagrid,vagrid, uschoof,vschoof,fschoofu,fschoofv,  &
          hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia,via, uw,vw,  &
          muind, mvind, nuvtot, hw, timeice, iterasch, iterc, 0, iconvSfail)
         ifDOsor=0
         IF(iconvSfail==1) THEN !  allow 1 more try via iconvSfail=2
           ifDOsor=1
           iconvSfail=2
         ENDIF
       ELSEIF(iconvSfail==2) THEN ! dosparse already ran with ifGMRESfail==1
           ifDOsor=1
       ELSE ! converged
         ifDOsor=0
       ENDIF

       If(ifDOsor==1) CALL dosparse_sor (uagrid,vagrid, uschoof,vschoof,fschoofu,fschoofv,  &
          hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia,via, uw,vw,  &
          muind, mvind, h, hw, timeice, iterasch, iterc, 0, iconvSfail)
!----------
     ENDIF
!----------

! convfail
     IF (iconvSfail==1) THEN
       icntSfail=icntSfail+1
       IF(idieS==1 .AND. iletfail==1) THEN
         ifConvFail=1
         WRITE(ioterm,*) "sparse 1 ITERASCH fail"
         RETURN
       ENDIF
       iconvSfail=2 ! keeps low rFacSOR
     ENDIF
! ^^^

!     speed limit:
     DO j=1,nj
       DO i=1,ni
         uagrid(i,j) = MAX (-vmax, MIN (vmax, uagrid(i,j)))
         vagrid(i,j) = MAX (-vmax, MIN (vmax, vagrid(i,j)))
       ENDDO
     ENDDO

!        Test for loop ASCH convergence (on velocs uagrid,vagrid)

! convfail
! briggs PHOM
!  delmaxa = 0.
!  idcnt=0.
!  difAavg=0.
!  do j=1,njp
!    do i=1,nip
!      if(h(i,j)>Hmin)  then
!        ddif = max (abs(uagrid(i,j)-uaprev(i,j)), &
!               abs(vagrid(i,j)-vaprev(i,j)))
!        delmaxa = max(delmaxa,ddif)
!        difAavg=difAavg+ddif
!        idcnt=idcnt+1
!      endif
!    enddo
!  enddo
!  difAavg=difAavg/real(idcnt)
!  if ( (difAavg<delavgcrita .and. iterasch>=niterag ) .or. &
!      delmaxa<delcrita .or. (delmaxa<delcritab .and. &
!      iterasch>=niterab) )  go to 500
! newest PHOM
     delmaxa = 0.
     DO j=0,nj
       DO i=0,ni
         delmaxa = MAX (delmaxa, ABS(uagrid(i,j)-uaprev(i,j)),  &
            ABS(vagrid(i,j)-vaprev(i,j)))
       ENDDO
     ENDDO
     IF ( delmaxa < DELCRITA .OR. (iterasch > 2 .AND. delmaxa>(DELCRITA+delmaxaOLD)) ) GOTO 500
     delmaxaOLD=delmaxa
! ^^^

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!     End loop ASCH (iterasch)
   ENDDO
   iterasch = NITERASCH
500 CONTINUE
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!&&&&&&&&&&
 ENDIF ! nuvtot > 0 test
!&&&&&&&&&&

!        Sets Schoof-imposed velocities ([u,v]schoof) and weightings
!        for shelf and sheet dynamics (fschoof[u,v]), on u,v grids.
!        Uses [u,v]bgrid from above, and theta[u,v] (buttressing frac)
!        from thetacalc. Also returns various diagnostics:
!        hg[u,v], qg[u,v], at sub-grid g.l. location,
!        and flux* shifted to grid points.

 CALL schoofgl(uschoof, vschoof, fschoofu, fschoofv, thetau, thetav, & !INTENT(INOUT)
    hgu, hgv, qgu, qgv, fluxgrdu, fluxgrdv, fluxschu, fluxschv, & ! angnormu, angnormv,  & ! INTENT(OUT)
    h, hs, hb, hw, fedge, maskwater,  & ! rest INTENT(IN)
    arhap, s2a0, s2b0, siu, siv, lhsu, lhsv, budgall,  &
    crhu, crhv, dfu, dfv, hu, hv, dhsu, dhsv, masku, maskv, muind, mvind, fracgu, fracgv,  &
    ua, va, uagrid, vagrid, uia, via, sealev, timeice, vmax)

!@@@@@

!@@@@@

!==========================
 IF (nuvtot > 0) THEN ! SSA calcs
!==========================
!===============================
!     Top of loop A (SSA flow)
   DO itera = 1,NITERA
!===============================
# 1201


     uaprev=ua
     vaprev=va

!  Set various coefficients (coef1, coef2, coefb[u,v], heath, sih, siu, and siv), rest are INTENT(IN)
     CALL setcoefs (coef1, coef2, coefbu, coefbv, heath,  &
        maskh, arhap, h, hbsd, hw,  &
        masku, maskv, ua, va, uia, via, fracgu, fracgv,  &
        uit, vit, sih, siu, siv, hu, hv, crhu, crhv,  &
        muind, mvind, maskc, itera, iterc)

!        Tridia or Sparse matrix solution of ice-shelf eqns (for ua,va)

!----------------------
     IF (nj == 1) THEN
!----------------------
       CALL dotridia  (ua, uschoof, fschoofu,  &
          hu, dhsu, hw, coef1, coefbu, uia, uw, muind, nuvtot)
!---------------------------
     ELSE IF (ni == 1) THEN
!---------------------------
       CALL dotridia2 (va, vschoof, fschoofv,  &
          hv, dhsv, hw, coef1, coefbv, via, vw, mvind, nuvtot)
!---------
     ELSE
!---------

! INTENT(INOUT) : ua,va, and iconvSfail, rest are INTENT(IN)
       CALL dosparse (ua,va, uschoof,vschoof, fschoofu,fschoofv,  &
          hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia, via, uw, vw,  &
          muind, mvind, nuvtot, hw, timeice, itera, iterc, 1, iconvSfail)
       IF(iconvSfail==1) THEN
         CALL dosparse (ua,va, uschoof,vschoof, fschoofu,fschoofv,  &
            hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia, via, uw, vw,  &
            muind, mvind, nuvtot, hw, timeice, itera, iterc, 1, iconvSfail)
         ifDOsor=0
         IF(iconvSfail==1) THEN !  allow 1 more try via iconvSfail=2
           ifDOsor=1
           iconvSfail=2
         ENDIF
       ELSEIF(iconvSfail==2) THEN ! dosparse already ran with ifGMRESfail==1
           ifDOsor=1
       ELSE
         ifDOsor=0
       ENDIF


       If(ifDOsor==1) CALL dosparse_sor (ua,va, uschoof,vschoof, fschoofu,fschoofv,  &
          hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia, via, uw, vw,  &
          muind, mvind, h, hw, timeice, itera, iterc, 1, iconvSfail)

!----------
     ENDIF
!----------

! convfail
     IF (iconvSfail==1) THEN
       icntSfail=icntSfail+1
         IF(idieS==1 .AND. iletfail==1) THEN
           ifConvFail=1
           WRITE(ioterm,*) "sparse 2 ITERA fail"
           RETURN
         ENDIF
         iconvSfail=2 ! keeps low rFacSOR
     ENDIF
! ^^^

!     speed limit:
     DO j=1,nj
       DO i=1,ni
         ua(i,j) = MAX (-vmax, MIN (vmax, ua(i,j)))
         va(i,j) = MAX (-vmax, MIN (vmax, va(i,j)))
       ENDDO
     ENDDO

!        Test for loop A convergence (on velocs ua,va)

! convfail
! briggs PHOM
!  delmaxa = 0.
!  idcnt=0.
!  difAavg=0.
!  DO j=0,njp
!    DO i=0,nip
!      if(h(i,j)>Hmin)  then
!        ddif = max (abs(ua(i,j)-uaprev(i,j)), &
!               abs(va(i,j)-vaprev(i,j)))
!        delmaxa = MAX (delmaxa, ddif)
!        difAavg=difAavg+ddif
!        idcnt=idcnt+1.
!      endif
!    END DO
!  END DO
!  difAavg=difAavg/real(idcnt)
!  if ( (difAavg<delavgcrita .and. itera>=niterag ) .or.  &
!        delmaxa<delcrita .or. (delmaxa<delcritab .and.  &
!        itera>=niterab) )  go to 1000
! newest PHOM
     delmaxa = 0.
     DO j=0,nj
       DO i=0,ni
         delmaxa = MAX (delmaxa, ABS(ua(i,j)-uaprev(i,j)),  &
            ABS(va(i,j)-vaprev(i,j)))
       ENDDO
     ENDDO
     IF(itera > 2 .AND. delmaxa>(DELCRITA+delmaxaOLD)) THEN
       icntAfail=icntAfail+1
       GOTO 1000
     ELSEIF (delmaxa < DELCRITA) THEN
# 1313

       GOTO 1000
     ENDIF
     delmaxaOLD=delmaxa
! ^^^

!=======================
!     End loop A (itera)
   ENDDO
   itera = NITERA

! convfail
   icntAfail=icntAfail+1
   IF(idieA==1 .AND. iletfail==1) THEN
     ifConvFail=1
!!  write(*,'("convA: ",f7.2,2(1x,f9.3),2(1x,e10.3),2(1x,i2))')  &
!!         ,timeice*0.001,delmaxa,delmaxc,difAavg,difCavg,itera,iterc
     WRITE(*,'("convA: ",f7.2,2(1x,f9.3),2(1x,i2))')  &
        ,timeice*0.001,delmaxa,delmaxc,itera,iterc
     RETURN
   ENDIF
! ^^^

1000 CONTINUE
!=======================
!==========
 ENDIF ! nuvtot > 0 test
!==========

!        Set lhsu,lhsv for icestep. Also set dbu,dbv (diagnostic except
!        used for heatb[u,v] after C loop).

 DO j = 1,nj
   DO i = 1,ni-1
     IF (muind(i,j) >= 1) THEN
# 1354

       dbu(i,j) = -coefbu(i,j) * (ua(i,j) - uia(i,j) - uw(i,j)) ! Weertman basal drag, last term in eq 2a


       lhsu(i,j) = dfu(i,j) - dbu(i,j) ! RHS SSA eq 2a, dfu= - driving stress

     ELSE IF (masku(i,j) == 1) THEN ! SIA only
       dbu(i,j)  = dfu(i,j)
       lhsu(i,j) = 0.
     ELSE
       dbu(i,j)  = 0.
       lhsu(i,j) = 0.
     ENDIF
   ENDDO
 ENDDO

 DO j = 1,nj-1
   DO i = 1,ni
     IF (mvind(i,j) >= 1) THEN
# 1379

       dbv(i,j) = -coefbv(i,j) * (va(i,j) - via(i,j) - vw(i,j))


       lhsv(i,j) = dfv(i,j) - dbv(i,j)

     ELSE IF (maskv(i,j) == 1) THEN
       dbv(i,j)  = dfv(i,j)
       lhsv(i,j) = 0.
     ELSE
       dbv(i,j)  = 0.
       lhsv(i,j) = 0.
     ENDIF
   ENDDO
 ENDDO

!        Step through one timestep to get h1, with time-implicit
!        internal flow, and implicit h * explicit ub .
!        Also return internal flow quantities uia, via, uit, vit

!m    call mascalc (h0, ztoth0, ztota0)
# 1456



! INTENT(OUT): h1, INTENT(INOUT): uia, via, uit, vit, ub, vb, ua, va, rest INTENT(IN)
 CALL icestep(h0, h1, hw, maskwater, hu0, hv0, dhsu0, dhsv0, dfu0, dfv0,  &
    lhsu, lhsv, s1a0, s1b0, s2a0, s2b0, siu, siv,  &
    crhu, crhv, uia, via, uit, vit, ub, vb, ua, va,  &
    uschoof, vschoof, fschoofu, fschoofv,  &
    masku, maskv, muind, mvind, medgeu, medgev, dt, vmax)



!        Set uadv,vadv fluxes for cliffcalc, and diagnostics in iceshow
!        (do before h is updated below)

 DO j = 1,nj
   DO i = 1,ni-1
     IF (masku(i,j) == 1) THEN
!           centered for sia uia and sia ub, upstream for ssa ub:
       uadv(i,j) = uia(i,j)*hu(i,j)*dyu(i,j)
       IF (muind(i,j) == 0) THEN
         uadv(i,j) = uadv(i,j) + ub(i,j)*hu(i,j)*dyu(i,j)
       ELSE
         uadv(i,j) = uadv(i,j) +  &
            (  MAX (ub(i,j),0.)*h(i,j) + MIN (ub(i,j),0.)*h(i+1,j) ) * dyu(i,j)
       ENDIF
     ENDIF
   ENDDO
 ENDDO

 DO j = 1,nj-1
   DO i = 1,ni
     IF (maskv(i,j) == 1) THEN
!           centered for sia via and sia vb, upstream for ssa vb:
       vadv(i,j) = via(i,j)*hv(i,j)*dxv(i,j)
       IF (mvind(i,j) == 0) THEN
         vadv(i,j) = vadv(i,j) + vb(i,j)*hv(i,j)*dxv(i,j)
       ELSE
         vadv(i,j) = vadv(i,j) +  &
            (  MAX (vb(i,j),0.)*h(i,j) + MIN (vb(i,j),0.)*h(i,j+1) ) * dxv(i,j)
       ENDIF
     ENDIF
   ENDDO
 ENDDO

!        Calculate cliffmelt (depends on ua,va at grounding lines)

# 1507

 cliffmelt=0.


!        Explicitly apply surface mass balance to ice thickness.
!        cliffmelt and calvice are not yet included in budgall
!        (set/modified in cliffcalc), so appear explicitly here.

 IF(ifassimPD==1) THEN
   h1(1:ni,1:nj)=h0(1:ni,1:nj)
! skip C loop convergence check (effective NITERC=1)
   GOTO 3000
 ENDIF
 
# 1524

     DO j=1,nj
       DO i=1,ni

!    h1(i,j) = h1(i,j) + (budgall(i,j) - cliffmelt(i,j) - calvice(i,j))*dt
         h1(i,j) = h1(i,j) + budgall(i,j)*dt
       ENDDO
     ENDDO

!m    call mascalc (h1, ztoth1a, ztota1a)

!        Correct for negative h1
     DO j=1,nj
       DO i=1,ni
         IF (h1(i,j) < 0.) THEN
           budgneg(i,j) = -h1(i,j)/dt
           h1(i,j) = 0.
         ELSE
           budgneg(i,j) = 0.
         ENDIF
       ENDDO
     ENDDO

!        Re-impose boundary constraints,etc, on h for various expts
     totflow = 0.
!....................................................................
# 1567

!.......................................................................

     IF (ni > 1) THEN
       DO j=1,nj
         totflow = totflow - h1(1,j) *darea(1,j)
         totflow = totflow - h1(ni,j)*darea(ni,j)
         h1(1, j) = 0.
         h1(ni,j) = 0.
       ENDDO
     ENDIF
     IF (nj > 1) THEN
       DO i=1,ni
         totflow = totflow - h1(i,1) *darea(i,1)
         totflow = totflow - h1(i,nj)*darea(i,nj)
         h1(i, 1) = 0.
         h1(i,nj) = 0.
       ENDDO
     ENDIF
# 1603


!.....................
# 1613

!.....

!m    call mascalc (h1, ztoth1b, ztota1b)
!m    ztotneg = 0.
!m    do j=1,nj
!m      do i=1,ni
!m        ztotneg = ztotneg + budgneg(i,j)*darea(ni,j)*dt
!m      enddo
!m    enddo
!m    write (6,'(a,4f20.14)')
!m   *    '   toth1b, 1b-1a, totflow, totneg:',
!m   *     ztoth1b/totarea, (ztoth1b-ztoth1a)/totarea,
!m   *     totflow/totarea, ztotneg/totarea

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     Hindmarsh and Payne(1996) convergence:
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     do j=1,nj
!       do i=1,ni
!         delh1(i,j) = h1(i,j) - h1prev(i,j)
!       enddo
!     enddo

!     zcos = 999.
!     zthed= 999.
!     zrat = 999.
!     if (iterc.ge.2) then
!       znorma = 0.
!       znormb = 0.
!       znormc = 0.
!       zdot   = 0.
!       do j=1,nj
!         do i=1,ni
!           if (h1(i,j).gt.0.) then
!             znorma = znorma + delh1prev(i,j)**2
!             znormb = znormb + delh1(i,j)**2
!             znormc = znormc + (delh1(i,j)-delh1prev(i,j))**2
!             zdot   = zdot   + delh1(i,j)*delh1prev(i,j)
!           endif
!         enddo
!       enddo
!       znorma = sqrt(znorma)
!       znormb = sqrt(znormb)
!       znormc = sqrt(znormc)
!       if (znorma.gt.0. .and. znormb.gt.0. .and. znormc.gt.0.) then
!         zcos = zdot/(znorma*znormb)
!         if (abs(zcos).le.1.) then
!           zthe = acos (zcos)
!           zthed= zthe*180./pi
!           if (abs(zthed).lt.30. .or. abs(zthed).gt.180.-30.) then
!c          if (abs(zthed).lt.45. .or. abs(zthed).gt.180.-45.) then
!             zrat = znorma/znormc
!             do j=1,nj
!               do i=1,ni
!                 h1(i,j) = h1prev(i,j) + delh1(i,j)*zrat
!               enddo
!             enddo
!           endif
!         endif
!       endif
!     endif

!     call scopy (ni*nj, delh1, 1, delh1prev, 1)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     End of Hindmarsh and Payne convergence
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!        Update h at t+0.5*dt to be used for next iter's setcoefs

     DO j=1,nj
       DO i=1,ni
         h(i,j) = 0.5*(h1(i,j) + h0(i,j))
       ENDDO
     ENDDO

!        Test for loop C convergence (on ice thickness h1)

! convfail
! briggs PHOM
!delmaxc = 0.
!difCavg=0.
!DO j=1,NY
!  DO i=1,NX
!    IF(h(i,j)>Hmin)  then
!      ddif=ABS(h1(i,j)-h1prev(i,j))
!      delmaxc = MAX (delmaxc, ddif)
!      difCavg=difCavg+ddif
!    ENDIF
!  END DO
!END DO
!difCavg=difCavg/real(idcnt)
!IF (delmaxc < delcritc) GO TO 3000
! newest PHOM
     delmaxc = 0.
     DO j=1,nj
       DO i=1,ni
         delmaxc = MAX (delmaxc, ABS(h1(i,j)-h1prev(i,j)))
       ENDDO
     ENDDO
     IF(iterc > 2 .AND. delmaxc>(DELCRITC+delmaxcOLD)) THEN
       icntCfail=icntCfail+1
       GOTO 3000
     ELSEIF (delmaxc < DELCRITC)  THEN
       GOTO 3000
     ENDIF
     delmaxcOLD=delmaxc

! ^^^

!=======================
!     End loop C (iterc)
   ENDDO
   iterc = NITERC

! convfail
   icntCfail=icntCfail+1
   IF(idieC==1 .AND. iletfail==1) THEN
     ifConvFail=1
!  write(IUconvg,'("convC: ",f7.2,2(1x,f9.4),2(1x,e10.3),2(1x,i2))') &
!       ,timeice*0.001,delmaxa,delmaxc,difAavg,difCavg,itera,iterc
     WRITE(*,'("convC: ",f7.2,2(1x,f9.4),2(1x,i2))') &
        ,timeice*0.001,delmaxa,delmaxc,itera,iterc
     RETURN
   ENDIF
! ^^^

3000 CONTINUE
!=======================

!        Set new h for this timestep (at t+dt)
   h=h1

!        Calculate basal heating (heatb, J/m2/y) due to basal sliding,
!        equal to taub_u*ub + taub_v*vb. First on u-grid (heatbu)
!        and v-grid (heatbv), then transfer to h-grid (heatv).
!        heatb is passed to icetherm.

!        If defined sediment, heatb has been calculated in sedflow
!        (using last time step quantities), allowing for tau-split
!        between sediment and bare rock.


!     u-direction:
   heatbu=0.

   DO j = 1,nj
     DO i = 1,ni-1
       IF ( masku(i,j) == 1  &
!    *         .and. fracgu(i,j).ne.0.    ! but ignores sub-grid pinning
          .AND..NOT. (hw(i,j) > HWcut .AND. hw(i+1,j) > HWcut) ) THEN
!         heatbu(i,j) = -(1.-fsedu(i,j)) * dbu(i,j) * (ua(i,j)-uia(i,j)-uw(i,j))
!         heatbu(i,j) = -(1.-fsedu(i,j)) * dbu(i,j) * ub(i,j)
!         heatbu(i,j) = ABS( -(1.-fsedu(i,j)) * dbu(i,j) * (ua(i,j)-uia(i,j)-uw(i,j)) ) ! ABS needed as dfu*dfu0 can be <0
!         heatbu(i,j) = ABS( -(1.-fsedu(i,j)) * dbu(i,j) * (ua(i,j)-uia(i,j)) ) ! ABS needed as dfu*dfu0 can be <0
         heatbu(i,j) = ABS( -(1.-fsedu(i,j)) * dbu(i,j) * ub(i,j) ) ! ABS needed as dfu*dfu0 can be <0
# 1790

       ENDIF
     ENDDO
   ENDDO

!     v-direction:
   heatbv=0.

   DO j = 1,nj-1
     DO i = 1,ni
       IF ( maskv(i,j) == 1  &
!    *         .and. fracgv(i,j).ne.0.
          .AND..NOT. (hw(i,j) > HWcut .AND. hw(i,j+1) > HWcut) ) THEN
!         heatbv(i,j) = -(1.-fsedv(i,j)) * dbv(i,j)*(va(i,j)-via(i,j)-vw(i,j))
!         heatbv(i,j) = -(1.-fsedv(i,j)) * dbv(i,j) * vb(i,j)
!         heatbv(i,j) = ABS( -(1.-fsedv(i,j)) * dbv(i,j) * (va(i,j)-via(i,j)-vw(i,j))  ) ! ABS needed as dfv*dfv0 can be <0
!         heatbv(i,j) = ABS( -(1.-fsedv(i,j)) * dbv(i,j) * (va(i,j)-via(i,j))  ) ! ABS needed as dfv*dfv0 can be <0
         heatbv(i,j) = ABS( -(1.-fsedv(i,j)) * dbv(i,j) * vb(i,j) ) ! ABS needed as dfv*dfv0 can be <0
# 1822

       ENDIF
     ENDDO
   ENDDO

!     transfer to h-grid:
   heatb=0.
   DO j = 1,nj
     DO i = 1,ni
       IF (maskh(i,j) == 1) THEN
         heatb(i,j) = 0.5*(heatbu(i-1,j) + heatbu(i,j))  &
            + 0.5*(heatbv(i,j-1) + heatbv(i,j))
!           heatb(i,j) = 0.
! BSLheatb
!      heatb(i,j)=1.0*heatb(i,j)
! ^^^
       ENDIF
     ENDDO
   ENDDO


!        Skip point for no ice (numh=0, top of loop C)

3500 CONTINUE

!        Zero very small shelf amounts (h ~1.e-20) created by icedyn

   DO j=1,nj
     DO i=1,ni
       IF (h(i,j) < 1.e-10) h(i,j) = 0.
     ENDDO
   ENDDO

!          Final adjust of maskwater, hw and hs (for change in h):

   CALL findwater (hw, maskwater, indlake, npoilake, nlake, h, hb, sealev, Icheck)

   CALL adjustpres (maskwater, indlake, npoilake, nlake, h, hb, hw, hs, sealev, Icheck)

! Remove very thin floating ice and set mass-conserved budget budgCons
   DO j=1,nj
     DO i=1,ni
       IF (maskwater(i,j) == 1 .AND. h(i,j) < HthinFloatingCut/dt) h(i,j)=0.0
!       IF (maskwater(i,j) == 1 .AND. h(i,j) < HthinFloatingCut) h(i,j)=0.0
       budgCons(i,j)=budgall(i,j)+budgneg(i,j)
     ENDDO
   ENDDO
!# if defined (ISMIP6GR)
! CALL forceMarineH(H,budgall,timeice)
!#endif


!!$WRITE(241,*)"hs,H,hb,sealev,hw,budgall,maskwater",ni,nj
!!$WRITE(242,*)"crhu, crhv, ua, va, ub, vb, dxg, dyg, darea, dxu, dyv, dxc, dyc, dxv, dyu",ni,nj
!!$ DO j=nj,1,-1
!!$   DO i=1,ni
!!$     WRITE(241,142) hs(i,j),H(i,j),hb(i,j), &
!!$        sealev(i,j),hw(i,j),budgall(i,j),maskwater(i,j)
!!$        142  FORMAT(3(f7.1,x),3(f6.1,x),3(i1,x))
!!$     WRITE(242,'(18(e9.3,1x))') crhu(i,j),crhv(i,j),ua(i,j),va(i,j),ub(i,j),vb(i,j),&
!!$        dxg(i,j), dyg(i,j), darea(i,j), dxu(i,j), dyv(i,j), dxc(i,j), dyc(i,j), dxv(i,j), dyu(i,j)
!!$   ENDDO
!!$ ENDDO
!!$ STOP

!#if defined (phomDIAGNO)
!   if(nuvtot>0) PRINT *,"PHOM nuvtot: ",nuvtot
!#endif

# 1910



# 1925

   RETURN

# 1974

  END SUBROUTINE icedyn

!-----------------------------------------------------------------------

  SUBROUTINE setcoefs (coef1, coef2, coefbu, coefbv, heath,  &
     maskh, arhap, h, hbsd, hw, masku, maskv, ua,va, uia, via, fracgu, fracgv,  &
     uit, vit, sih, siu, siv, hu, hv, crhu, crhv,  &
     muind, mvind, maskc, itera, iterc)

!        Set various coefficients, etc, at top of A loop:
!        coef[1,2], coefb[u,v], heath
!        KEY: all for SSA except for sih(i,j) used for heati(i,j,kz)= 2. * zdb2 * (zdb2*s3a(i,j,k) + sih(i,j)*s3b(i,j,k))

   USE modconstants
   USE modPHOMgrid
# 1992

   USE modPHOMvardyn, ONLY : ubmin
   USE modbeddrag, ONLY : POWb

   USE modgridm, ONLY : ni, nj, dxg, dyg, dxu, dyu, dxv, dyv, dxc, dyc
   USE modicerheology, ONLY : POWih, POWiv
# 2000


   IMPLICIT NONE

!     passed:

   REAL, INTENT(IN) ::  h(0:NXp,0:NYp), fracgu(0:NX,0:NY), fracgv(0:NX,0:NY) ! IN
   REAL, INTENT(IN) ::  arhap(NX,NY), & ! IN
      hbsd(NX,NY),       hw(NX,NY), &
      ua(0:NX,0:NY),     va(0:NX,0:NY), &
      uia(0:NX,0:NY),    via(0:NX,0:NY), &
      uit(0:NX,0:NY),    vit(0:NX,0:NY), &
      hu(0:NX,0:NY),     hv(0:NX,0:NY), &
      crhu(0:NX,0:NY),   crhv(0:NX,0:NY)

   REAL, INTENT(IN OUT) :: & ! OUT, IN only to retain previous 0 setting for itera == 1
      coef1(NX,NY),  coef2(0:NX,0:NY), &
      sih(NX,NY),  siu(0:NX,0:NY),    siv(0:NX,0:NY)
   
   REAL, INTENT(OUT)  ::  coefbu(0:NX,0:NY), coefbv(0:NX,0:NY), heath(NX,NY)

   INTEGER, INTENT(IN) :: maskh(NX,NY), & ! IN
      masku(0:NX,0:NY), maskv(0:NX,0:NY), &
      muind(0:NX,0:NY), mvind(0:NX,0:NY), &
      maskc(0:NX,0:NY), itera, iterc

! Avoid singular matrix if floating areas.
! Also used for water friction (coefbsu*(ub-uw),coefbsv*(vb-vw))
! in icedyn, with uw,vw set in movewater, to mimic ice-dam
! breaking in catastrophic floods.
   REAL, PARAMETER :: COEFbWATER    = .001  ! Pa/(m/a)
!REAL, PARAMETER :: COEFbWATER    = .100  ! Pa/(m/a)



   REAL :: ZPOW2 = 0.
# 2038


# 2043

   
!     local:
   REAL :: g1(NX,NY), g2(NX,NY), g3(NX,NY),  & ! save
      g2c(0:NX,0:NY),  &                  ! cross grid
      duiz(0:NX,0:NY), dviz(0:NX,0:NY)
   REAL :: visc(NX,NY)
! BSLsave not nec SAVE g1, g2, g3, g2c, duiz, dviz
! ^^^

   REAL :: zdudx,zdvdy,zdudy,zdvdx,zhw,zhbsd,zfpin,zc,zpow,zu,zv,zcoef,sedw,zc1,zc2,powblend
   INTEGER :: i,j

!     Statement function (fpin) for fraction of u,v grid box with
!     sub-grid bedrock mounds scraping ice bottom:

!      USE modPHOMpin

!        Zero initially (some necessary for finite diffs near ice edges)


   IF (itera == 1) THEN   ! don't mix between calls for uagrid vs. ua
# 2067

! BSLnew
       coef1=0.
       coef2=0.
       visc=0.
       g1=0.
       g2=0.
       g3=0.
       duiz=0.
       dviz=0.
       g2c=0.
       sih=0.
       siu=0.
       siv=0.

       ZPOW2 = -0.5*(1. - 1./POWb)

     ENDIF

!        Set g1 ((du/dxg)**2 + (dv/dyg)**2 + (du/dxg)*(dv/dyg)) (h grid)

     DO j = 1,nj
       DO i = 1,ni
         IF (maskh(i,j) == 1) THEN
           zdudx = (ua(i,j)-ua(i-1,j))/dxg(i,j)
           zdvdy = (va(i,j)-va(i,j-1))/dyg(i,j)
           g1(i,j) = zdudx**2 + zdvdy**2 + zdudx*zdvdy
         ENDIF
       ENDDO
     ENDDO

!        Set g2c (cross grid), then g2 = [du/dyg+dv/dxg]**2}/4 (h grid) (pollardC.modelDescript.gmd12.pdf eq 7 has error)

     DO j = 1,nj-1
       DO i = 1,ni-1
         IF (maskc(i,j) /= 0) THEN
           zdudy = (ua(i,j+1)-ua(i,j))/dyc(i,j)
           zdvdx = (va(i+1,j)-va(i,j))/dxc(i,j)
           g2c(i,j) = 0.25*((zdudy + zdvdx)**2)
         ENDIF
       ENDDO
     ENDDO

     DO j = 1,nj
       DO i = 1,ni
         IF (maskh(i,j) == 1) THEN
           g2(i,j) = 0.25*(g2c(i-1,j-1)+g2c(i,j-1)+g2c(i-1,j)+g2c(i,j))
         ENDIF
       ENDDO
     ENDDO

!       Set internal(d/dz,sheet)-stress contrib to shelf rheology (g3)

!       Set duiz,dviz (u,v grids)

     DO j = 1,nj
       DO i = 1,ni-1
         IF (masku(i,j) == 1) THEN
           duiz(i,j) = uit(i,j)/hu(i,j)
         ENDIF
       ENDDO
     ENDDO

     DO j = 1,nj-1
       DO i = 1,ni
         IF (maskv(i,j) == 1) THEN
           dviz(i,j) = vit(i,j)/hv(i,j)
         ENDIF
       ENDDO
     ENDDO

!       Set g3 ([duiz**2 + dviz**2]/4) (h grid)

     DO j = 1,nj
       DO i = 1,ni
         IF (maskh(i,j) == 1) THEN
           g3(i,j) = 0.25 * (   (0.5*(duiz(i-1,j)+duiz(i,j)))**2  &
              + (0.5*(dviz(i,j-1)+dviz(i,j)))**2 )
         ENDIF
       ENDDO
     ENDDO

!        Set visc (h grid)

     DO j = 1,nj
       DO i = 1,ni
         IF (maskh(i,j) == 1) THEN
           visc(i,j) = 0.5 / (MAX(g1(i,j)+g2(i,j)+g3(i,j), gmin) ** POWiv) ! pollardC.modelDescript.gmd12.pdf eq4,6
# 2158

           heath(i,j) = 2.*((g1(i,j)+g2(i,j))**POWih) / arhap(i,j) ! cf GSMcodemods.org /energy balance check

         ELSE
           heath(i,j) = 0.
         ENDIF
       ENDDO
     ENDDO

!        Set coef1 (2*visc*h/(A**1/n)) for ice-shelf eqns (h grid)

     DO j = 1,nj
       DO i = 1,ni
         IF (maskh(i,j) == 1) THEN
!           coef1(i,j) = 2.*visc(i,j)*h(i,j)          / arhap(i,j)
           coef1(i,j) = 2.0*visc(i,j)*MAX(h(i,j),0.01) / arhap(i,j)
         ENDIF
       ENDDO
     ENDDO

!        Set coef2 (visc*h/(A**1/n)) for ice-shelf eqns (cross grid)

     DO j = 1,nj-1
       DO i = 1,ni-1
         IF (maskc(i,j) /= 0) THEN
           coef2(i,j) = 0.5 * 0.25 * (   coef1(i,j)   + coef1(i+1,j)  &
              + coef1(i,j+1) + coef1(i+1,j+1)) ! at i+,j+ corner, so top right
         ENDIF
       ENDDO
     ENDDO

!        Set linearized basal sliding coeffs coefb[u,v] for shelf eqns
!        = (crhu**(-1/m)) (ub2**((1-m)/2m)) (u,v grids)

     DO j = 1,nj
       DO i = 1,ni-1
         IF (masku(i,j) == 1.) THEN

!           calculate zfpin, fraction of u-grid cell actually in
!           contact with bottom, scraped by sub-grid bedrock mounds
!           (stmt fnct fpin):
           zhw  = 0.5*(hw(i,j) + hw(i+1,j))
           zhbsd  = 0.5*(hbsd(i,j) + hbsd(i+1,j))
! BSLmod
           CALL fracpin(zfpin, zhw, zhbsd)
!            zfpin = fpin(zhw,zhbsd)
! ^^^

!           calculate coefbu (N/m2 per m/y), weighting by water vs.
!           non-water contact fractions:
           IF (fracgu(i,j) == 0. .AND. zfpin == 0.) THEN
             coefbu(i,j) = COEFbWATER
           ELSE
             zu = ua(i,j) - uia(i,j)
             zv = 0.25*( va(i,j) + va(i+1,j) + va(i,j-1) + va(i+1,j-1)  &
                -via(i,j)-via(i+1,j) -via(i,j-1) -via(i+1,j-1))
# 2239

             zc = crhu(i,j) ** (-1./POWb)
             zcoef = zc * ( MAX(zu**2+zv**2, ubmin**2)**ZPOW2)

             zcoef = MAX (zcoef, COEFbWATER)
             IF (fracgu(i,j) == 0.) THEN ! shelf, not at grounding line
               coefbu(i,j) = (1.-zfpin) * COEFbWATER +     zfpin  * zcoef 
             ELSE
               coefbu(i,j) = zcoef ! GL
             ENDIF
           ENDIF
         ELSE
           coefbu(i,j) = 0. ! not used
         ENDIF
       ENDDO
     ENDDO

     DO j = 1,nj-1
       DO i = 1,ni
         IF (maskv(i,j) == 1) THEN

           zhw   = 0.5*(hw(i,j) + hw(i,j+1))
           zhbsd  = 0.5*(hbsd(i,j) + hbsd(i,j+1))
! BSLmod
           CALL fracpin(zfpin, zhw, zhbsd)
!            zfpin = fpin(zhw,zhbsd)
! ^^^

           IF (fracgv(i,j) == 0. .AND. zfpin == 0.) THEN
             coefbv(i,j) = COEFbWATER
           ELSE
             zu = 0.25*( ua(i,j) + ua(i,j+1) + ua(i-1,j) + ua(i-1,j+1)  &
                -uia(i,j)-uia(i,j+1) -uia(i-1,j) -uia(i-1,j+1))
             zv = va(i,j) - via(i,j)
# 2298

             zc = crhv(i,j) ** (-1./POWb)
             zcoef = zc * ( MAX(zu**2+zv**2, ubmin**2)**ZPOW2)

             zcoef = MAX (zcoef, COEFbWATER)
             IF (fracgv(i,j) == 0.) THEN ! shelf, not at grounding line
               coefbv(i,j) = (1.-zfpin) * COEFbWATER +     zfpin  * zcoef
             ELSE
               coefbv(i,j) = zcoef ! GL
             ENDIF
           ENDIF
         ELSE
           coefbv(i,j) = 0. ! not used
         ENDIF
       ENDDO
     ENDDO

!        Set sih on h grid, then siu, siv on u,v grids (shelf-stress
!        contrib to sheet rheol, used in icestep,icetherm).
!        Don't need to do within ASCH,A loops, only after A loop,
!        but "belongs" in setcoefs (and uses visc, g1, g2).


     DO j = 1,nj
       DO i = 1,ni
         IF (maskh(i,j) == 1) THEN
           sih(i,j)= ((2.*visc(i,j)/arhap(i,j))**2) * (g1(i,j)+g2(i,j)) ! horizontal components of Sigma^2
         ENDIF
       ENDDO
     ENDDO

     DO j = 1,nj
       DO i = 1,ni-1
         IF (muind(i,j) >= 1) THEN
           siu(i,j)  = 0.5 * (sih(i,j) + sih(i+1,j))
         ELSE
           siu(i,j) = 0.
         ENDIF
       ENDDO
     ENDDO

     DO j = 1,nj-1
       DO i = 1,ni
         IF (mvind(i,j) >= 1) THEN
           siv(i,j)  = 0.5 * (sih(i,j) + sih(i,j+1))
         ELSE
           siv(i,j) = 0.
         ENDIF
       ENDDO
     ENDDO


     RETURN
    END SUBROUTINE setcoefs

!-----------------------------------------------------------------------

    SUBROUTINE dotridia (ua, uschoof, fschoofu,  &
       hu, dhsu, hw, coef1, coefbu, uia, uw, muind,nuvtot)

!     Assembles *tri arrays for 1-D tridiagonal solution.
!     Calls tridia_i, sets new velocities ua.
!     Only for 1-D in x with NY=1 (use dotridia2 for 1-D in y, NX=1).
!     Weights imposed Schoof velocities (uschoof) into model eqation,
!     with weights fschoofu...which are only potentially non-zero
!     for 2 u-grid points closest and downstream of grounding line.
!     Do this weighting as follows:
!        (1-fschoofu)*(orig tridiag eqn) + fschoofu*atri*(u=uschoof),
!     i.e., multiply the imposed equation by atri from the orig eqn,
!     so they're both the same order of magn. for intermediate fschoofs.

     USE modconstants
     USE modPHOMgrid
     USE modgridm
# 2374



     IMPLICIT NONE

     REAL, INTENT(INOUT) :: ua(0:NX,0:NY)
     REAL, INTENT(IN) :: uschoof(0:NX,0:NY), fschoofu(0:NX,0:NY), &
        hu(0:NX,0:NY), dhsu(0:NX,0:NY), hw(NX,NY), &
        coef1(NX,NY), coefbu(0:NX,0:NY), &
        uia(0:NX,0:NY), uw(0:NX,0:NY)
     INTEGER, INTENT(IN) :: muind(0:NX,0:NY)
     INTEGER, INTENT(IN) :: nuvtot

     REAL :: atri(NX), btri(NX), ctri(NX), rtri(NX), vtri(NX)

     REAL :: z1,z2,zwei
     INTEGER :: i,j,n1

!        Tridiagonal solution for ua in 1-D problems (in x, with nj=1).
!        Relies on muind running left to right (gaps in ice are ok).

     j = 1

     atri=0.
     btri=0.
     ctri=0.
     rtri=0.
     vtri=0.

     DO i=1,ni-1
       n1 = muind(i,j)

       IF (n1 >= 1) THEN

         z1 = 2.*coef1(i+1,j) / (dxg(i+1,j)*dxu(i,j))  ! ua(i+1,j)
         z2 = 2.*coef1(i  ,j) / (dxg(i,j  )*dxu(i,j))  ! ua(i-1,j)

# 2421

         rtri(n1) = RHOIc*gEARTH*hu(i,j)*dhsu(i,j) - coefbu(i,j)*(uia(i,j)+uw(i,j))
!         diagonal:
         atri(n1) = - (z1 + z2) - coefbu(i,j)


# 2430


!         superdiagonal:
         IF (muind(i+1,j) >= 1) THEN
           btri(n1) = z1
         ELSE
           rtri(n1) = rtri(n1) - z1*ua(i+1,j)
         ENDIF

!         subdiagonal:
         IF (muind(i-1,j) >= 1) THEN
           ctri(n1) = z2
         ELSE
           rtri(n1) = rtri(n1) - z2*ua(i-1,j)
         ENDIF

!         impose Schoof velocity:
         zwei = fschoofu(i,j)
!         atri(n1) = (1.-zwei)*atri(n1) + zwei*atri(n1) ! atri unchanged
         btri(n1) = (1.-zwei)*btri(n1)
         ctri(n1) = (1.-zwei)*ctri(n1)
         rtri(n1) = (1.-zwei)*rtri(n1) + zwei*atri(n1)*uschoof(i,j)

       ENDIF

     ENDDO

     CALL tridia_i (atri, btri, ctri, vtri, rtri, nuvtot, 0)

!        Set new ua

     DO i=1,ni
       n1 = muind(i,j)
       IF (n1 >= 1) THEN
         ua(i,j) = vtri(n1)
       ENDIF
     ENDDO

     RETURN
    END SUBROUTINE dotridia
!-----------------------------------------------------------------------

    SUBROUTINE dotridia2 (va, vschoof, fschoofv,  &
       hv, dhsv, hw, coef1, coefbv, via, vw, mvind, nuvtot)

!     Assembles *tri arrays for 1-D tridiagonal solution.
!     Calls tridia_i, sets new velocities va.
!     Only for 1-D in y with NX=1. (Use dotridia for 1-D in x, NY=1).
!     See comments in dotridia.

     USE modconstants
     USE modPHOMgrid
     USE modgridm
# 2485


     IMPLICIT NONE

     REAL, INTENT(IN OUT) :: va(0:NX,0:NY)
     REAL, INTENT(IN    ) :: vschoof(0:NX,0:NY), fschoofv(0:NX,0:NY), &
        hv(0:NX,0:NY), dhsv(0:NX,0:NY), hw(NX,NY), &
        coef1(NX,NY), coefbv(0:NX,0:NY), &
        via(0:NX,0:NY), vw(0:NX,0:NY)
     INTEGER, INTENT(IN) :: mvind(0:NX,0:NY)
     INTEGER, INTENT(IN) :: nuvtot

     REAL :: atri(NY), btri(NY), ctri(NY), rtri(NY), vtri(NY)

     REAL :: z1,z2,zwei
     INTEGER :: i,j,n1

!        Tridiagonal solution for va in 1-D problems (in y, with ni=1).
!        Relies on mvind running "bottom to top" (gaps in ice are ok).

     i = 1

     atri=0.
     btri=0.
     ctri=0.
     rtri=0.
     vtri=0.

     DO j=1,nj-1
       n1 = mvind(i,j)

       IF (n1 >= 1) THEN

         z1 = 2.*coef1(i,j+1) / (dyg(i,j+1)*dyv(i,j))  ! va(i,j+1)
         z2 = 2.*coef1(i,j  ) / (dyg(i,j  )*dyv(i,j))  ! va(i,j-1)

# 2531

         rtri(n1) = RHOIc*gEARTH*hv(i,j)*dhsv(i,j) - coefbv(i,j)*(via(i,j)+vw(i,j))
!         diagonal:
         atri(n1) = - (z1 + z2) - coefbv(i,j)


# 2540


!         superdiagonal:
         IF (mvind(i,j+1) >= 1) THEN
           btri(n1) = z1
         ELSE
           rtri(n1) = rtri(n1) - z1*va(i,j+1)
         ENDIF

!         subdiagonal:
         IF (mvind(i,j-1) >= 1) THEN
           ctri(n1) = z2
         ELSE
           rtri(n1) = rtri(n1) - z2*va(i,j-1)
         ENDIF

!         impose Schoof velocity:
         zwei = fschoofv(i,j)
!         atri(n1) = (1.-zwei)*atri(n1) + zwei*atri(n1) ! atri unchanged
         btri(n1) = (1.-zwei)*btri(n1)
         ctri(n1) = (1.-zwei)*ctri(n1)
         rtri(n1) = (1.-zwei)*rtri(n1) + zwei*atri(n1)*vschoof(i,j)

       ENDIF

     ENDDO

     CALL tridia_i (atri, btri, ctri, vtri, rtri, nuvtot, 0)

!        Set new va

     DO j=1,nj
       n1 = mvind(i,j)
       IF (n1 >= 1) THEN
         va(i,j) = vtri(n1)
       ENDIF
     ENDDO

     RETURN
    END SUBROUTINE dotridia2

!-----------------------------------------------------------------------

!|||||||||||||||||||||

!|||||||||||||||||||||

    SUBROUTINE dosparse (ua, va, uschoof, vschoof, fschoofu, fschoofv,  &
       hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia, via, uw, vw,  &
       muind, mvind, nuvtot, hw, timeice, itera, iterc, iflag, iconvSfail)

!     Solves elliptic shelf equations for ua,va,
!     just for points with m[u,v]ind >= 1.
!     Solves using choice of sparse solvers.
!     Assembles elspa, ijspa, rhs arrays for sparse solver,
!     calls sparse solver, sets new velocities ua,va
!     Weights imposed Schoof velocities (u,vschoof) into model eqation,
!     with weights fschoofu,v - only potentially non-zero for
!     2 u,v-grid points closest and downstream of grounding line.
! LT verified shelf edge stress BCs respected since coef2=0 at margin interface
!    and coef1 = 0 where maskh=0

     USE modconstants
     USE modPHOMgrid
     USE modPHOMvardyn, ONLY : ifSPSdiagSUMout
# 2607

!     USE modPHOMsparse
     USE modIU, ONLY : IOTERM
     USE modgridm

     IMPLICIT NONE

     REAL, INTENT(IN OUT) :: ua(0:NX,0:NY), va(0:NX,0:NY)
     REAL, INTENT(IN    ) :: uschoof(0:NX,0:NY), vschoof(0:NX,0:NY), &
        fschoofu(0:NX,0:NY), fschoofv(0:NX,0:NY), hu(0:NX,0:NY), hv(0:NX,0:NY), &
        dhsu(0:NX,0:NY), dhsv(0:NX,0:NY), coef1(NX,NY), coef2(0:NX,0:NY), &
        coefbu(0:NX,0:NY), coefbv(0:NX,0:NY), hw(NX,NY), &
        uia(0:NX,0:NY), via(0:NX,0:NY), &
        uw(0:NX,0:NY), vw(0:NX,0:NY), timeice

     INTEGER, INTENT(IN) :: nuvtot, itera, iterc, iflag
     INTEGER, INTENT(IN) :: muind(0:NX,0:NY), mvind(0:NX,0:NY)
     INTEGER, INTENT(INOUT) :: iconvSfail
     
     REAL :: z1,z2,z3,z4,za,zb,zc,zd,zwei
     INTEGER :: i,j,kof,kofprev,kofsch,n1,n2,itol,itmax,itlin,info, istart=1

!     elspa, ijspa, in comicesparse.h, shared with solver.
!     vecu passed back so can be on restart file.
     
!************NSPCG DEF *********************
!     REAL, PARAMETER :: gtol=1.e-4
# 2642

     INTEGER, PARAMETER :: NNDIM=NX*NY*13/30
# 2646

     INTEGER, PARAMETER :: MDIM=9, NMDIM=NNDIM*MDIM
!     INTEGER, PARAMETER :: NNDIM=NX*NY/2, MDIM=9, NMDIM=NNDIM*MDIM
!      parameter(NW=13*NNDIM,INW=20) !no propA size
!     INTEGER, PARAMETER :: NW=15*NNDIM, INW=20 ! bcgs
     INTEGER, PARAMETER :: NW=28*NNDIM+25800, INW=20 ! GMRES, with NS1=IPARM(9)= 10, ie  (2*NS1+8)*NNDIM=28 * NNDIM
!     INTEGER, PARAMETER :: NW=20*NNDIM, INW=20000
!      parameter(NW=13*NNDIM,INW=2200)

     REAL :: coef(NNDIM,MDIM)=0., rhs(NNDIM), uns(NNDIM), wksp(NW), rparm(30)=0.
     REAL :: ubar(1), vecu(NNDIM), gtol
! NSPCG tolerance is metric on whole resid metric
# 2660

     REAL, PARAMETER :: TOLwide=4.e-5, TOLfin=1.e-5, TOLm=2.e-5 ! SOR now uses 0.1,0.01,0.02 max relative diff from previous iterate

!     REAL, PARAMETER :: TOLwide=5.e-5, TOLfin=1.e-5, TOLm=2.e-5 ! SOR now uses 0.1,0.01,0.02 max relative diff from previous iterate
!     REAL, PARAMETER :: TOLwide=2.e-5, TOLfin=0.5e-5, TOLm=1.e-5 ! SOR now uses 0.1,0.01,0.02 max relative diff from previous iterate
!     REAL, PARAMETER :: TOLwide=1.e-5, TOLfin=0.1e-5, TOLm=0.3e-5 ! SOR now uses 0.1,0.01,0.02 max relative diff from previous iterate
     REAL, PARAMETER :: TOLpoor=5.e-5, OMEGAfail=1.37
# 2672

     REAL, PARAMETER ::  OMEGAlow=1.45, OMEGAhi=1.55 !SSOR over-relaxation factors

     INTEGER, PARAMETER :: MaxITERLowOmega=32
     INTEGER :: jcoef(NNDIM,MDIM)=0, iwksp(INW), iparm(30)=0, pns(1), ipns(1)
     INTEGER ::  maxnz, irNW, itNW, ier, ifLowOmegaCnt=1
     INTEGER :: icntSPSfail=0, icntSPS=1, icntSPSiter=0, ifGMRESfail=0, njold=0, ifGMRES, itimestep=0


!      external cg, mic1
!      external bcgs, mic1
!      external bcgs, lssor1,omin
!EXTERNAL sor, sor1
     EXTERNAL bcgs, ssor1,gmres
!      external bcgs, rich1
!     EXTERNAL sor, ssor1
!     EXTERNAL omin, ssor1
     
!*********************************
     
     
     
!        Set sparse arrays and rhs's needed to solve system of eqns.
!        (using elspa,ijspa,rhs). Can't parallelize for
!        sparse method, since off-diag terms in ijspa,elspa are
!        assembled using incremented index kof. Must loop through
!        i,j,muind,mvind in same sequence as muind,mvind were assembled
!        above, so that diagonal elements are in same order as
!        off-diagonal column groups in elspa,ijspa.
     
!        If the neighbor's u or v are not being computed (n2 = 0), add
!        their (explicit) contributions to rhs(n1).
     
     jcoef=0
     coef=0.
     rhs=0.
     IF(njold /= nj) THEN !catch change of ice sheet ki
       ifGMRESfail=0 
       njold=nj
       ifLowOmegaCnt=MaxITERLowOmega
     ENDIF

# 2717


     kof = nuvtot + 1

     IF(nuvtot>NNDIM) CALL outerrI('increase NNDIM in SUBROUTINE dosparse to ',nuvtot+1)

# 2725


     DO j=1,nj
       DO i=1,ni

!         u velocities:

         n1 = muind(i,j)
!<<<<<<<<<<<<<<<<<<<<<<<<<<
         IF (n1 > 0) THEN
!<<<<<<<<<<<<<<<<<<<<<<<<<<
           kofprev = kof

!              rhs for ua(i,j):

# 2746

           rhs(n1) = RHOIc*gEARTH*hu(i,j)*dhsu(i,j) - coefbu(i,j)*(uia(i,j)+uw(i,j))


           z1 = 2.*coef1(i+1,j) / (dxg(i+1,j )*dxu(i,j))  ! ua(i+1,j)
           z2 = 2.*coef1(i  ,j) / (dxg(i,j   )*dxu(i,j))  ! ua(i-1,j)
           z3 =    coef2(i  ,j) / (dyc(i,j  )*dyu(i,j))  ! ua(i,j+1)
           z4 =    coef2(i,j-1) / (dyc(i,j-1)*dyu(i,j))  ! ua(i,j-1)

!                diagonal for ua(i,j):

# 2763

           coef(n1,1) =  -(z1 + z2 + z3 + z4) - coefbu(i,j)

           jcoef(n1,1)=n1
           kof = kof + 1
# 2771



!                u-neighbors for ua(i,j):

           n2 = muind(i+1,j)
           IF (n2 > 0) THEN
             coef(n1,2) = z1
             jcoef(n1,2)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) - z1*ua(i+1,j)
           ENDIF

           n2 = muind(i-1,j)
           IF (n2 > 0) THEN
             coef(n1,3) =  z2
             jcoef(n1,3)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) - z2*ua(i-1,j)
           ENDIF

           n2 = muind(i,j+1)
           IF (n2 > 0) THEN
             coef(n1,4) = z3
             jcoef(n1,4)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) - z3*ua(i,j+1)
           ENDIF

           n2 = muind(i,j-1)
           IF (n2 > 0) THEN
             coef(n1,5) = z4
             jcoef(n1,5)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) - z4*ua(i,j-1)
           ENDIF

!                v-neighbors for ua(i,j):

           za = coef1(i+1,j) / (dyg(i+1,j )*dxu(i,j)) ! i+1,j; i+1,j-1
           zb = coef1(i  ,j) / (dyg(i,j   )*dxu(i,j)) ! i,j  ; i,j-1
           zc = coef2(i  ,j) / (dxc(i,j  )*dyu(i,j)) ! i+1,j; i,j
           zd = coef2(i,j-1) / (dxc(i,j-1)*dyu(i,j)) ! i,j-1; i+1,j-1

           n2 = mvind(i+1,j)
           IF (n2 > 0) THEN
             coef(n1,6) = za + zc
             jcoef(n1,6)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) - (za+zc)*va(i+1,j)
           ENDIF

           n2 = mvind(i+1,j-1)
           IF (n2 > 0) THEN
             coef(n1,7) = -za - zd
             jcoef(n1,7)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) + (za+zd)*va(i+1,j-1)
           ENDIF

           n2 = mvind(i,j)
           IF (n2 > 0) THEN
             coef(n1,8) = -zb - zc
             jcoef(n1,8)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) + (zb+zc)*va(i,j)
           ENDIF

           n2 = mvind(i,j-1)
           IF (n2 > 0) THEN
             coef(n1,9) = zb + zd
             jcoef(n1,9)=n2
             kof = kof + 1
           ELSE
             rhs(n1) = rhs(n1) - (zb+zd)*va(i,j-1)
           ENDIF

!           endif  ! uag != 0

           IF (fschoofu(i,j) > 0.) THEN
             zwei = fschoofu(i,j)
!             coef(n1,1) = (1.-zwei)*coef(n1,1) + zwei*coef(n1,1) ! #@ ???
             rhs(n1) = (1.-zwei)*rhs(n1) + zwei*coef(n1,1)*uschoof(i,j)
             IF (kof >= kofprev+1) THEN
               DO kofsch = 2,9
                 IF(jcoef(n1,kofsch) /=0) coef(n1,kofsch) = (1.-zwei)*coef(n1,kofsch)
               ENDDO
             ENDIF
           ENDIF

!<<<<<<<<<<<<<<
         ENDIF    ! n1 >= 1
!<<<<<<<<<<<<<<

!         v velocities:

         n1 = mvind(i,j)
!^^^^^^^^^^^^^^^^^^^^^^^^^^
         IF (n1 > 0) THEN
!^^^^^^^^^^^^^^^^^^^^^^^^^^
           kofprev = kof

!                rhs for va(i,j):

# 2888

           rhs(n1) = RHOIc*gEARTH*hv(i,j)*dhsv(i,j) - coefbv(i,j)*(via(i,j)+vw(i,j))


           z1 = 2.*coef1(i,j+1) / (dyg(i,j+1 )*dyv(i,j))  ! va(i,j+1)
           z2 = 2.*coef1(i  ,j) / (dyg(i,j   )*dyv(i,j))  ! va(i,j-1)
           z3 =    coef2(i  ,j) / (dxc(i,j  )*dxv(i,j))  ! va(i+1,j)
           z4 =    coef2(i-1,j) / (dxc(i-1,j)*dxv(i,j))  ! va(i-1,j)

!                diagonal for va(i,j):
# 2904

           coef(n1,1) =  -(z1 + z2 + z3 + z4) - coefbv(i,j)


           jcoef(n1,1)=n1
# 2912


!                v-neighbors for va(i,j):

           n2 = mvind(i,j+1)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,2)= z1
             jcoef(n1,2)=n2
           ELSE
             rhs(n1) = rhs(n1) - z1*va(i,j+1)
           ENDIF

           n2 = mvind(i,j-1)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,3)= z2
             jcoef(n1,3)=n2
           ELSE
             rhs(n1) = rhs(n1) - z2*va(i,j-1)
           ENDIF

           n2 = mvind(i+1,j)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,4)= z3
             jcoef(n1,4)=n2
           ELSE
             rhs(n1) = rhs(n1) - z3*va(i+1,j)
           ENDIF

           n2 = mvind(i-1,j)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,5)= z4
             jcoef(n1,5)=n2
           ELSE
             rhs(n1) = rhs(n1) - z4*va(i-1,j)
           ENDIF

!                u-neighbors for va(i,j):

           za = coef1(i,j+1) / (dxg(i,j+1 )*dyv(i,j)) ! i,j+1; i-1,j+1
           zb = coef1(i  ,j) / (dxg(i,j   )*dyv(i,j)) ! i,j  ; i-1,j
           zc = coef2(i  ,j) / (dyc(i,j  )*dxv(i,j)) ! i,j+1; i,j
           zd = coef2(i-1,j) / (dyc(i-1,j)*dxv(i,j)) ! i-1,j; i-1,j+1

           n2 = muind(i,j+1)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,6)= za + zc
             jcoef(n1,6)=n2
           ELSE
             rhs(n1) = rhs(n1) - (za+zc)*ua(i,j+1)
           ENDIF

           n2 = muind(i-1,j+1)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,7)= -za - zd
             jcoef(n1,7)=n2
           ELSE
             rhs(n1) = rhs(n1) + (za+zd)*ua(i-1,j+1)
           ENDIF

           n2 = muind(i,j)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,8)= -zb - zc
             jcoef(n1,8)=n2
           ELSE
             rhs(n1) = rhs(n1) + (zb+zc)*ua(i,j)
           ENDIF

           n2 = muind(i-1,j)
           IF (n2 > 0) THEN
             kof = kof + 1
             coef(n1,9)= zb + zd
             jcoef(n1,9)=n2
           ELSE
             rhs(n1) = rhs(n1) - (zb+zd)*ua(i-1,j)
           ENDIF

!           endif ! vag != 0

           IF (fschoofv(i,j) > 0.) THEN
             zwei = fschoofv(i,j)
!             coef(n1,1) = (1.-zwei)*coef(n1,1) + zwei*coef(n1,1)
             rhs(n1) = (1.-zwei)*rhs(n1) + zwei*coef(n1,1)*vschoof(i,j)
             IF (kof >= kofprev+1) THEN
               DO kofsch = 2,9
                 IF(jcoef(n1,kofsch) /=0) coef(n1,kofsch) = (1.-zwei)*coef(n1,kofsch)
               ENDDO
             ENDIF
           ENDIF

!^^^^^^^^^^^^^^
         ENDIF   ! n1 >= 1
!^^^^^^^^^^^^^^

       ENDDO
     ENDDO

!        Solve linear system arr*vec = rhs for vec.
!        If number of pts <= nuvsmall, use gaussian elimination
!        (avoids sparse-solution problems for nuvtot <= ~2)

!+++++++++++++++++++++++++++++++++
!     IF (nuvtot > nuvsmall) THEN
!+++++++++++++++++++++++++++++++++

       maxnz=9

       IF(istart==1) THEN
         PRINT *,"init of NSPCG solver"
         PRINT *,"NSPCG: TOLwide=",TOLwide,", TOLm=",TOLm,", TOLfin=",TOLfin, "TOLpoor=",TOLpoor
         PRINT *,"MaxITERLowOmega: ",MaxITERLowOmega, OMEGAlow, OMEGAhi
         PRINT *,"NSCPG to low SORfactor  if > 100 iterations"
         CALL dfault (iparm,rparm)
         iparm(1) = 10       !stopping test, cheapest is supposed to be 10, ! for r=b=Au 5: resid/b, 10: QIL*resid/(QIL*b), QiL is inverse of left preconditioner
         iparm(4) = 6       !ioutunit, default=6
         iparm(12) = 1       !storage mode, primary model
!              iparm(11) = 40            !orthomin  accelerator only
!     iparm(15) = 1            ! do factorization
         iparm(18) = 0       !0 no Property A, 2 : check
         iparm(23) = 1       ! if assymetric =1, 2 for check
         iparm(3) = 0        !output level, default=0: error only, 1 warnings, 2 summary
     
         iparm(24)=1  !IELIM is a switch for effectively removing rows and columns when the diagonal
!ENTRY is extremely large compared to the nonzero non-diagonal entries in that row.
         
         RPARM(9) = 1.35 ! (OMEGA): OMEGA serves two purposes:
!1. It is the overrelaxation parameter ! for the SOR and SSOR
!methods. OMEGA contains on output a nal adapted value if IOMGAD = 1
!is specied and the acceleration allows estimation of ! (SOR, SRCG,
!and SRSI only). Otherwise, OMEGA is not changed and the xed value of
!OMEGA is used throughout the iterations.

         iparm(11)=0 ! Used only in ORTHOMIN, NS3 denotes the size of
! the largest Hessenberg matrix to be used to
! estimate the eigenvalues; it should be set to
! some VALUE such as 40. [Default: 0]
         
         istart=0
       ENDIF

       vecu=0.
       DO j=1,nj
         DO i=1,ni
           n1 = muind(i,j)
           IF (n1 > 0) THEN
             vecu(n1) = ua(i,j)
           ENDIF
           n2 = mvind(i,j)
           IF (n2 > 0) THEN
             vecu(n2) = va(i,j)
           ENDIF
         ENDDO
       ENDDO

       itNW=INW
       irNW=NW
!       PRINT *,"nuvtot before nspcg",nuvtot,NNDIM
!       iparm(2) = 100             !max iterations
       iparm(2) = 160             !max iterations
!       rparm(1) = gtol   !tolerance, default 1e-6

       IF (iterc <2 .AND. itera<2) THEN 
         rparm(1) =TOLwide ! wide
       ELSEIF ( iterc==2  .AND. itera>1) THEN
         rparm(1) =TOLfin ! high res final
       ELSE
         rparm(1) =TOLm ! mod res
       ENDIF

       iparm(9)=5 !NS1 is the number of old vectors to be saved for the truncated accel methods. [Default: 5]

       IF(ifLowOmegaCnt<MaxITERLowOmega) THEN
         IF(RPARM(9)==OMEGAfail) iparm(9)=10
         RPARM(9) = OMEGAlow !SSOR over-relaxation factor
       ELSE
         RPARM(9) = OMEGAhi
       ENDIF
       IF(ifGMRESfail==1) THEN
         RPARM(9) = OMEGAfail
         iparm(9)=10
       ENDIF
!       iparm(9)=5

       IF(iconvSfail==1 .OR. ifLowOmegaCnt<MaxITERLowOmega) THEN 
         CALL nspcg (ssor1,gmres,NNDIM,MDIM,nuvtot,maxnz,coef,jcoef,pns,ipns,  &
           vecu(1:nuvtot),ubar,rhs(1:nuvtot),wksp,iwksp,irNW,itNW,iparm,rparm,ier)
         ifGMRES=1
       ELSE
        CALL nspcg (ssor1,bcgs,NNDIM,MDIM,nuvtot,maxnz,coef,jcoef,pns,ipns,  &
           vecu(1:nuvtot),ubar,rhs(1:nuvtot),wksp,iwksp,irNW,itNW,iparm,rparm,ier)
        ifGMRES=0
      ENDIF
# 3113


     IF(ifSPSdiagSUMout==1) THEN 
       PRINT *,"SPS avg iter, avg totIter, totFail",&
          icntSPSiter/icntSPS,(icntSPSiter+200*icntSPSfail)/icntSPS,icntSPSfail
       icntSPSiter=0
       icntSPS=0
       icntSPSfail=0
       ifSPSdiagSUMout=0
     ENDIF
     icntSPS=icntSPS+1
    
   IF(ier == 1) THEN
!     PRINT *, 'NSPCG convergence failure: itera,c,iflag:', itera, iterc, iflag
!     PRINT *,"SSORomega:",RPARM(9)
!     rparm(1) = gtol        !tolerance
     ifLowOmegaCnt=0
     IF(RPARM(1)>TOLpoor) THEN
       IF(ifGMRESfail==1) THEN
         iconvSfail=2 ! go straight to SOR sparse
       ELSE
         iconvSfail=1
       ENDIF
       icntSPSfail=icntSPSfail+1
       IF(ifGMRES==1) ifGMRESfail=1
# 3141

       RETURN
     ENDIF
# 3147

   ELSE IF(ier<0) THEN
     PRINT *, 'NSPCG error: ',ier
     PRINT *, 'real,Int workspace sizes ',irNW,itNW
     PRINT *, 'jcoef',jcoef
     CALL outerr('pGSMicedynSGhyps/icedynGSM, matrix solver failure ')
   ELSE IF(ier>0) THEN
!     rparm(1) = gtol        !tolerance
     PRINT *, 'real,Int workspace sizes ',irNW,itNW
   END IF
!   PRINT *, 'real,Int workspace sizes ',irNW,itNW, NW,INW
!+++++++++
!     ELSE
!+++++++++
!       CALL gaussdo (nuvtot, rhs, vecu, info)
!       IF (info /= 0) WRITE (IOTERM,*)'*** Warning gauss(u):info=',info
!++++++++++
! ENDIF
!++++++++++

!        Set new ua,va
   
   DO j=1,nj
     DO i=1,ni
       n1 = muind(i,j)
       IF (n1 > 0) THEN
         ua(i,j) = vecu(n1)
       ENDIF

       n2 = mvind(i,j)
       IF (n2 > 0) THEN
         va(i,j) = vecu(n2)
       ENDIF
     ENDDO
   ENDDO
   iconvSfail=0
   ifGMRESfail=0

   ifLowOmegaCnt=1+ifLowOmegaCnt
!     IF(iparm(2)>80) ifLowOmegaCnt=1 ! trigger low omega value for > 80 NSPCG iterations
   IF(iparm(2)>100) ifLowOmegaCnt=1 ! trigger low omega value for > 100 NSPCG iterations
# 3190

   icntSPSiter=icntSPSiter+iparm(2)

   RETURN
  END SUBROUTINE dosparse

!|||||

!|||||

!-----------------------------------------------------------------------

    SUBROUTINE dosparse_sor (ua, va, uschoof, vschoof, fschoofu, fschoofv,  &
       hu, hv, dhsu, dhsv, coef1, coef2, coefbu, coefbv, uia,via, uw,vw,  &
       muind, mvind, h, hw, timeice, itera, iterc, iflag, iconvSfail)

!     Solevs elliptic shelf equations for ua,va, using SOR (Successive
!     Over Relaxation), just for points with m[u,v]ind >= 1.
!     Weights imposed Schoof velocities (u,vschoof) into model eqation,
!     with weights fschoofu,v - only potentially non-zero for
!     2 u,v-grid points closest and downstream of grounding line.

     USE modPHOMgrid
     USE modconstants
!     USE modPHOMsparse
     USE modgridm, ONLY : ni, nj, darea, dxg, dyg, dxu, dyu, dxv, dyv, dxc, dyc
# 3218


     IMPLICIT NONE

     REAL, INTENT(INOUT) :: ua(0:NX,0:NY),     va(0:NX,0:NY)
     INTEGER, INTENT(INOUT) :: iconvSfail
     REAL, INTENT(IN) :: &
        uschoof(0:NX,0:NY),    vschoof(0:NX,0:NY),  &
        fschoofu(0:NX,0:NY),   fschoofv(0:NX,0:NY),  &
        hu(0:NX,0:NY),         hv(0:NX,0:NY),  &
        dhsu(0:NX,0:NY),       dhsv(0:NX,0:NY),  &
        coef1(NX,NY),          coef2(0:NX,0:NY),  &
        coefbu(0:NX,0:NY),     coefbv(0:NX,0:NY),  &
        uia(0:NX,0:NY),        via(0:NX,0:NY),  &
        uw(0:NX,0:NY),         vw(0:NX,0:NY),  &
        h(0:NXp,0:NYp),        hw(NX,NY)
     REAL, INTENT(IN) :: timeice
     INTEGER, INTENT(IN) :: muind(0:NX,0:NY), mvind(0:NX,0:NY)
     INTEGER, INTENT(IN) :: itera, iterc, iflag 


     REAL :: zuav,zdumax,z1,z2,z3,z4,za,zb,zc,zd, &
        zforcu,zforcv,zclhsu,zclhsv,zs,zf,uaold,vaold,zdu,zdv,zdumax0, zdumax2, zdumax3
     INTEGER :: i,j,itersur,icheck,ip1,jp1,ndav
     INTEGER :: istart=1, iconvWarn=10

# 3248

!     parameter (NITERSURMAX =  500, NITERSURMIN = 20, FACSOR=0.9)
!     parameter (NITERSURMAX =  100, NITERSURMIN = 20, FACSOR=1.2) ! std
! convfail
!INTEGER, PARAMETER :: NITERSURMAX =   20, NITERSURMIN = 10
!INTEGER, PARAMETER :: NITERSURMAX =   120, NITERSURMIN = 10
!     INTEGER, PARAMETER :: NITERSURMAX =   500, NITERSURMIN = 10
!     INTEGER, PARAMETER :: NITERSURMAX =   600, NITERSURMIN = 10, NITERSURMINpoor=100 !, NITERSURboost=5
     INTEGER, PARAMETER :: NITERSURMAX =   800, NITERSURMIN = 10, NITERSURMINpoor=90 !, NITERSURboost=5
!     INTEGER, PARAMETER :: NITERSURMAX =   200, NITERSURMIN = 10
!     REAL, PARAMETER :: FACSOR=1.6 ! new
!!INTEGER, PARAMETER (nitersurmax = 100, nitersurmin = 20, facsor=1.2) ! old
!!INTEGER, PARAMETER :: NITERSURMAX = 500, NITERSURMIN = 10
!!REAL, PARAMETER :: FACSOR=1.2 ! mod
! ^^^
# 3268

     REAL, PARAMETER :: DUCRITfin   = 0.010
     REAL, PARAMETER :: DUCRIT      = 0.020
     REAL, PARAMETER :: DUCRITpoor   = 0.040
     REAL, PARAMETER :: DUCRITw      = 0.10




     REAL :: rFacSOR


!     for checkerboard sor:
     INTEGER :: mcheck(NX, NX) = 0


!     ifPoorCONV=0

     IF(iconvSfail==2) THEN
# 3295

       rFacSOR=1.4 !SOR convergence failure this icedyn call
     ELSE
       rFacSOR=1.55

     ENDIF

     IF(istart==1) THEN
       PRINT *,"dosparse_sor first call: rfacSOR:",rFacSOR,",NITERSURMAX:",NITERSURMAX
       WRITE(*,'("dosparse_sor DUCRITw, DUCRIT, DUCRITfin, DUCRITpoor:",4(1x,f5.3))'),DUCRITw, DUCRIT, DUCRITfin, DUCRITpoor
       istart=0

       DO j=1,nj
         DO i=1,ni
           mcheck(i,j) = MOD (i+j,2)
         ENDDO
       ENDDO

     ENDIF


!     call scopy ((nip+1)*(njp+1), ua, 1, uaorig, 1)
!     call scopy ((nip+1)*(njp+1), va, 1, vaorig, 1)

!        Calculate mean veloc (was used to test iter skip, now diag)

     zuav = 0.
     ndav = 0.
     DO j=1,nj
       DO i=1,ni
         IF (muind(i,j) > 0) THEN
           zuav = zuav + ua(i,j)**2
           ndav = ndav + 1
         ENDIF
         IF (mvind(i,j) > 0) THEN
           zuav = zuav + va(i,j)**2
           ndav = ndav + 1
         ENDIF
       ENDDO
     ENDDO
     zuav = SQRT (zuav/ MAX(ndav,1))

     zdumax0 =  0.
     zdumax2 = 0.
     zdumax3 = 0.
!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     DO itersur = 1,NITERSURMAX
!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       zdumax =  0.

!       IF(itersur==NITERSURboost) rFacSOR=rFacSOR+0.05



       DO icheck = 0,1    ! for checkerboard sor updating
         DO j=1,nj
           DO i=1,ni-1
             IF (muind(i,j) > 0 .AND. mcheck(i,j) == icheck) THEN
               ip1 = i+1
# 3359


             jp1 = MIN (j+1,nj)

!              u-neighbors for ua(i,j):
             z1 = 2.*coef1(ip1,j) / (dxg(ip1,j )*dxu(i,j))  ! i+1,j; i,j
             z2 = 2.*coef1(i  ,j) / (dxg(i,j   )*dxu(i,j))  ! i,j  ; i-1,j
             z3 =    coef2(i  ,j) / (dyc(i,j  )*dyu(i,j))  ! i,j+1; i,j
             z4 =    coef2(i,j-1) / (dyc(i,j-1)*dyu(i,j))  ! i,j  ; i,j-1

!              v-neighbors for ua(i,j):
             za = coef1(ip1,j) / (dyg(ip1,j )*dxu(i,j)) ! i+1,j; i+1,j-1
             zb = coef1(i  ,j) / (dyg(i,j   )*dxu(i,j)) ! i,j  ; i,j-1
             zc = coef2(i  ,j) / (dxc(i,j  )*dyu(i,j)) ! i+1,j; i,j
             zd = coef2(i,j-1) / (dxc(i,j-1)*dyu(i,j)) ! i,j-1; i+1,j-1

# 3395

             zforcu = z1*(ua(ip1,j)        )     - z2*(       -ua(i-1,j))  &
                + z3*(ua(i,jp1)        )     - z4*(       -ua(i,j-1)) &
                + za*(va(ip1,j)-va(ip1,j-1)) - zb*(va(i,j)-va(i,j-1))  &
                + zc*(va(ip1,j)-va(i,j))     - zd*(va(ip1,j-1)-va(i,j-1)) &
                - RHOIc*gEARTH*hu(i,j)*dhsu(i,j)  &
                + coefbu(i,j)*(uia(i,j)+uw(i,j))

             zclhsu = z1 + z2 + z3 + z4 + coefbu(i,j)



# 3410


!@@@ std:
!           zdu = zforcu/zclhsu - ua(i,j)
!           ua(i,j) = rFacSOR*zforcu/zclhsu + (1.-rFacSOR)*ua(i,j)
!           impose (weighted) Schoof velocities:
!           zwei = fschoofu(i,j)
!           ua(i,j) = (1.-zwei)*ua(i,j) + zwei*uschoof(i,j)
!           if (zwei==0.) zdumax = max (zdumax, abs(zdu))
!@@@ new:
!           Set new ua (zforcu/zclhsu or uschoof, weighted by fschoofu).
!           Modify by rFacSOR (std sor if fschoofu=0), but rFacSOR -> 1
!           as fschoofu ->1, so ua=uschoof exactly for fschoofu=1.
             zs = fschoofu(i,j)
             zf = (1.-zs)*rFacSOR + zs*1.
             uaold = ua(i,j)
             ua(i,j) = zf * ((1.-zs)*(zforcu/zclhsu) + zs*uschoof(i,j))  &
                + (1.-zf) * ua(i,j)
!           for test to skip out of iteration:
             zdu = ABS(ua(i,j)-uaold) / MAX (ABS(uaold),1.)
             zdumax = MAX (zdumax, ABS(zdu))
!@@@

           ENDIF

         ENDDO
       ENDDO
     ENDDO

     DO icheck = 0,1    ! for checkerboard sor updating
       DO j=1,nj-1
         DO i=1,ni

           IF (mvind(i,j) > 0 .AND. mcheck(i,j) == icheck) THEN
             jp1 = j+1

# 3449


               ip1 = MIN (i+1,ni)

!              v-neighbors for va(i,j):
               z1 = 2.*coef1(i,jp1) / (dyg(i,jp1 )*dyv(i,j))  ! va(i,j+1)
               z2 = 2.*coef1(i  ,j) / (dyg(i,j   )*dyv(i,j))  ! va(i,j-1)
               z3 =    coef2(i  ,j) / (dxc(i,j  )*dxv(i,j))  ! va(i+1,j)
               z4 =    coef2(i-1,j) / (dxc(i-1,j)*dxv(i,j))  ! va(i-1,j)

!              u-neighbors for va(i,j):
               za = coef1(i,jp1) / (dxg(i,jp1 )*dyv(i,j)) ! i,j+1; i-1,j+1
               zb = coef1(i  ,j) / (dxg(i,j   )*dyv(i,j)) ! i,j  ; i-1,j
               zc = coef2(i  ,j) / (dyc(i,j  )*dxv(i,j)) ! i,j+1; i,j
               zd = coef2(i-1,j) / (dyc(i-1,j)*dxv(i,j)) ! i-1,j; i-1,j+1

# 3485

               zforcv = z1*(va(i,jp1)        )     - z2*(       -va(i,j-1))  &
                  + z3*(va(ip1,j)        )     - z4*(       -va(i-1,j))  &
                  + za*(ua(i,jp1)-ua(i-1,jp1)) - zb*(ua(i,j)-ua(i-1,j))  &
                  + zc*(ua(i,jp1)-ua(i,j))     - zd*(ua(i-1,jp1)-ua(i-1,j))  &
                  - RHOIc*gEARTH*hv(i,j)*dhsv(i,j)  &
                  + coefbv(i,j)*(via(i,j)+vw(i,j))

               zclhsv = z1 + z2 + z3 + z4 + coefbv(i,j)



# 3500


!@@@ std:
!           zdv = zforcv/zclhsv - va(i,j)
!           va(i,j) = rFacSOR*zforcv/zclhsv + (1.-rFacSOR)*va(i,j)
!           impose (weighted) Schoof velocities:
!           zwei = fschoofv(i,j)
!           va(i,j) = (1.-zwei)*va(i,j) + zwei*vschoof(i,j)
!           if (zwei==0.) zdumax = max (zdumax, abs(zdv))
!@@@ new (see comments above for u):
               zs = fschoofv(i,j)
               zf = (1.-zs)*rFacSOR + zs*1.
               vaold = va(i,j)
               va(i,j) = zf * ((1.-zs)*(zforcv/zclhsv) + zs*vschoof(i,j))  &
                  + (1.-zf) * va(i,j)
               zdv = ABS(va(i,j)-vaold) / MAX (ABS(vaold),1.)
               zdumax = MAX (zdumax, ABS(zdv))
!@@@

             ENDIF
           ENDDO
         ENDDO

       ENDDO


!        Print out sor diagnostics

!     if (.true.) then
       IF (.FALSE.) THEN
         IF (iterc == 1 .AND. itera == 1 .AND. itersur == 1 )  &
            WRITE (6,'(/25("=")/a,i10/25("="))') 'dosparse_sor: time=', nint(timeice)

         IF (itersur == 1) WRITE (6,'(/a,3i8,f12.3/a)')  &
            'iterc, itera, ndav, zuav=', iterc, itera, ndav, zuav,  &
            '  itersur      zdumax      DUCRIT'

!       if (itersur==1 .or. itersur==NITERSURMAX .or.
!    *     (zdumax.lt.DUCRIT .and. itersur.ge.NITERSURMIN)) then
         WRITE (6,'(i9, f12.3, 2f6.3, i9)') itersur, zdumax, DUCRIT, DUCRITw, iflag
!       endif
       ENDIF

!       IF (itersur == (NITERSURMAX-1)) zdumax0 = zdumax
!               iconvSfail=0
!     skip out of sor iteration if max change is small enough:
!               IF (iterc==1 .AND. itera==2) THEN ! use strong DUCRIT1 to force early sparse failure for reloop trigger
!                 IF(zdumax < DUCRIT1 .AND. itersur >= NITERSURMIN) RETURN
!               ELSE

       IF (iterc <2 .AND. itera<2 .AND. zdumax < DUCRITw .AND. itersur >= NITERSURMIN) RETURN
       IF ( iterc==2 .AND. itera>1 ) THEN
         IF (zdumax < DUCRITfin .AND. itersur >= NITERSURMIN) RETURN
       ELSE
         IF (zdumax < DUCRIT .AND. itersur >= NITERSURMIN) RETURN 
       ENDIF

       IF (zdumax <DUCRITpoor .AND. itersur >= NITERSURMINpoor) THEN
# 3560

         IF ( iterc<2 .OR. itera<2 ) RETURN
         iconvWarn=iconvWarn+1
         IF(iconvWarn==20) THEN
           WRITE(*,'("doSPARSE poor convergence: zdumax, itersur:",f5.3,1x,i3)') zdumax, itersur
           iconvWarn=0
         ENDIF
         RETURN

       ENDIF


! check for  2 sequential divergences
!       IF(zdumax3 < zdumax2 .AND. zdumax2<zdumax0 .AND. zdumax0<zdumax .AND. (itersur >= NITERSURMIN)) THEN
!         IF (ifPoorCONV==0) THEN
!           ifPoorCONV=1
!           rFacSOR=1.5
!         ENDIF
!       ENDIF

!>>>>>>>>>>
       zdumax3=zdumax2
       zdumax2=zdumax0
       zdumax0=zdumax
     ENDDO ! itersur
!>>>>>>>>>>

     IF(iterc>1) &
        WRITE(*,'("sparse_sor fail: DUCRIT,zdumax2, zdumax0, zdumax,rFacSOR ",4(g11.4,1x),f4.2)'),&
        DUCRIT,zdumax2,zdumax0,zdumax,rFacSOR

! convfail
     IF(iconvSfail==2) THEN
       iconvSfail=1 ! restep icedynGSM with dt/2
     ELSE
       iconvSfail=2 ! allows 1 reloop retry with smaller omega if only sor
     ENDIF

     RETURN
    END SUBROUTINE dosparse_sor

!-----------------------------------------------------------------------
      
!)))))))))))))))))))))

!)))))))))))))))))))))
    
    SUBROUTINE schoofgl(uschoof, vschoof, fschoofu, fschoofv, thetau, thetav, & !INTENT(INOUT)
    hgu, hgv, qgu, qgv, fluxgrdu, fluxgrdv, fluxschu, fluxschv, & ! angnormu, angnormv,  & ! INTENT(OUT)
    h, hs, hb, hw, fedge, maskwater,  & ! rest INTENT(IN)
    arhap, s2a0, s2b0, siu, siv, lhsu, lhsv, budgall,  &
    crhu, crhv, dfu, dfv, hu, hv, dhsu, dhsv, masku, maskv, muind, mvind, fracgu, fracgv,  &
    ua, va, uagrid, vagrid, uia, via, sealev, timeice, vmax)

!     Does Schoof-based calculations for grounding lines (Schoof, JGR,
!     2007, see 9/27/07 and 11/10/07 notes).
!     (1) Calculates hg[u,v], qg[u,v] at sub-grid g.l. location
!         (given by fracg[u,v], passed). Schoof flux qg[u,v (in calcgl)
!         is modified by buttressing fraction theta[u,v] (thetacalc).
!     (2) Sets [u,v]schoof velocs, and weightings fschoof[u,v],
!         for [u,v]-grid point at g.l. or 1 pt downstream,
!         depending on Schoof vs. non-Schoof flux at former pt.
!         (Non-Schoof flux is explicit SIA calced here for m[u,v]ind=0,
!         or from [u,v]bgrid (passed) for m[u,v]ind > 0).
!         These schoof velocs and weightings will be used in next
!         dotria/dosparse, and icestep.
!     (3) Also returns various diagnostics:
!         hg[u,v], qg[u,v], at sub-grid g.l. location,
!         and flux* shifted to grid points.

     USE modPHOMgrid
     USE modconstants
     USE modgridm, ONLY : ni, nj, darea, dxg, dyg, dxu, dyu, dxv, dyv
# 3638

     USE modbeddrag, ONLY : POWb

     USE modicerheology, ONLY : POWi, POWiv

     IMPLICIT NONE

     REAL, INTENT(INOUT) :: &
        uschoof(0:NX,0:NY),    vschoof(0:NX,0:NY),  &
        fschoofu(0:NX,0:NY),   fschoofv(0:NX,0:NY),  &
        thetau(NX,NY),         thetav(NX,NY)
     REAL, INTENT(OUT) :: &
        hgu(0:NX,0:NY),        hgv(0:NX,0:NY),  &
        qgu(0:NX,0:NY),        qgv(0:NX,0:NY),  &
        fluxgrdu(0:NX,0:NY),   fluxgrdv(0:NX,0:NY),  &
        fluxschu(0:NX,0:NY),   fluxschv(0:NX,0:NY)
!        angnormu(0:NX,0:NY),   angnormv(0:NX,0:NY)

     REAL, INTENT(IN) :: &
        h(0:NXp,0:NYp),        hs(0:NXp,0:NYp),  &
        hb(0:NXp,0:NYp),       hw(NX,NY),  &
        fedge(NX,NY),          arhap(NX,NY),  &
        s2a0(NX,NY),           s2b0(NX,NY),  &
        siu(0:NX,0:NY),        siv(0:NX,0:NY),  &
        lhsu(0:NX,0:NY),       lhsv(0:NX,0:NY), budgall(NX,NY),  &
        crhu(0:NX,0:NY),       crhv(0:NX,0:NY),  &
        dfu(0:NX,0:NY),        dfv(0:NX,0:NY),  &
        hu(0:NX,0:NY),         hv(0:NX,0:NY),  &
        dhsu(0:NX,0:NY),       dhsv(0:NX,0:NY),  &
        fracgu(0:NX,0:NY),     fracgv(0:NX,0:NY),  &
        ua(0:NX,0:NY),         va(0:NX,0:NY),  &
        uagrid(0:NX,0:NY),     vagrid(0:NX,0:NY),  &
        uia(0:NX,0:NY),        via(0:NX,0:NY)

     INTEGER, INTENT(IN) :: maskwater(NX,NY), masku(0:NX,0:NY), maskv(0:NX,0:NY), &
        muind(0:NX,0:NY), mvind(0:NX,0:NY)

     REAL, INTENT(IN) :: sealev(0:NXp,0:NYp),timeice,vmax

     REAL :: weischoof,fluxdif,zpowi,zpowb, &
        ztheta,zasch,zbsch,zfrac,zdx,zhg,zfg,angnorm, &
        zfluxsa,zfluxsb,zdfu,zdfv,zdf2,zhu,zs2a,zs2b, &
        zbra,zfluxg,zcrh,zfluxdif,dxgconst,dygconst,zdar,zhv, &
        sedw,zbra1,zbra2,zdfda11,zdfda12,zdfda21,zdfda22, & 
        zbsch1,zbsch2, Heff
     INTEGER :: i,j,ip3,im2,ia,ib,iau,ibu,idir,jp3,jm2,ja,jb,jau,jbu

!     local:
     INTEGER :: ifglu(0:NX), ifglv(0:NY)

!     weischoof is statement fn for Schoof weighting, dep on Schoof-"free" g.l. flux:
!     weischoof (fluxdif) = max (0., min (1., fluxdif/1.e4 ))  ! m^2 /yr
!     2/13: try to reduce butterfly-effect differences in totice(t)
!     between machines:
!     weischoof (fluxdif) = max (0., min (1.,  fluxdif/1.e2 + 0.5 ))
!!! BSL function change weischoof(fluxdif) = MAX (0., MIN (1.,  fluxdif/1.e5 + 0.5 ))
! BSLnew
     uschoof=0.
     vschoof=0.
     fschoofu=0.
     fschoofv=0.
     hgu=0.
     hgv=0.
     qgu=0.
     qgv=0.
     fluxgrdu=0.
     fluxgrdv=0.
     fluxschu=0.
     fluxschv=0.
!     angnormu=-999.
!     angnormv=-999.

!     PRINT *,"schooftest",timeice ! to test call interface
!        Set thetau,v = 1 - fraction of g.l. longitudinal stress
!        buttressed, used below (i) to decide whether to impose
!        schoofgl velocities (schoofgl), and (ii) to affect the
!        imposed velocity itself (calcgl). Use non-Schoof "grid"
!        velocities, calculated in ASCH loop above.

     CALL thetacalc (h, fedge, maskwater, arhap,  &
        uagrid, vagrid, thetau, thetav) ! from non-Schoof
!    *                ua,     va,     thetau, thetav) ! from prev C-iter

     IF (POWi == 3) THEN
       zpowi = 1.
     ELSE
       zpowi = (POWi-1)/2.
     ENDIF


     IF (POWb == 3) THEN
       zpowb = 1.
     ELSE
       zpowb = (POWb-1)/2.
     ENDIF


!uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuu
!        u grid:
!uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuu

!==============
     DO j=1,nj
!==============

!          Use ifglu to require at least 3 h-grid floating/water pts
!          next to g.l. If only 2, nearby g.l. calcs with opposite flow
!          direction could impose fluxes twice at same u-pt (if both
!          in advancing case).

!          If advancing, procedure imposes veloc (ua,va) at 1st u-pt
!          *downstream* of g.l. u-pt. So to test on buttressing,
!          use theta[u,v] from closest floating h cell to g.l.
!          depending on whether u-pts were included in prev
!          elliptic soln or not (muind, mvind).

       ifglu=0

       DO i=2,ni-2
         ip3 = MIN (i+3,ni)
         im2 = MAX (i-2,1)
         IF (masku(i,j) == 1) THEN
           IF      ( h(i  ,j) > 0. .AND. maskwater(i  ,j) == 0  &
              .AND. maskwater(i+1,j) /= 0 .AND. maskwater(i+2,j) /= 0  &
              .AND. maskwater(ip3,j) /= 0 ) THEN
             ifglu(i) = 1
           ELSE IF ( h(i+1,j) > 0. .AND. maskwater(i+1,j) == 0  &
              .AND. maskwater(i  ,j) /= 0 .AND. maskwater(i-1,j) /= 0  &
              .AND. maskwater(im2,j) /= 0 ) THEN
             ifglu(i) = -1
           ENDIF
         ENDIF
       ENDDO

!>>>>>>>>>>>>>>>>>>
       DO i=2,ni-2
!>>>>>>>>>>>>>>>>>>
         IF (IABS(ifglu(i)) == 1) THEN !GL

!              Set indices for points around g.l. On h grid,
!              (ia,j) is grounded with ice, (ib,j) is downstream
!              ocean or *open* lake. On u grid, (iau,j) contains g.l.,
!              and (ibu,j) is next downstream pt.

           IF (ifglu(i) == 1) THEN
             ia  = i
             ib  = i+1
             iau = i
             ibu = i+1
             idir = 1
           ELSE
             ia  = i+1
             ib  = i
             iau = i
             ibu = i-1
             idir = -1
           ENDIF
           ztheta = thetau(ib,j)

!              Only do if marine ice is not abutting land,
!              and g.l. is  not completely buttressed

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!           if (hs(ia,j).gt.hs(ib,j) .and. ztheta.gt.0.) then  ! was 0.3
           IF ( hs(ia,j) > hs(ib,j) ) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!                Set needed Schoof-like values for calcgl:

             zasch = 0.5*(arhap(ia,j)**POWi+arhap(ib,j)**POWi) ! arhap(i,j)=(fEnhSSA*A(t))**(1/POWi)
             zfrac = fracgu(iau,j)
             zdx = 0.5*(dxg(ia,j) + dxg(ib,j))
# 3829

             zbsch = (1./crhu(iau,j))**(1./POWb)
!                Calculates hg, ug (g.l. ice thickness, schoof veloc)
             CALL calcgl ( hb(ia,j), hb(ib,j),  &
                zasch, zbsch, zfrac, zdx,  &
                sealev(ib,j), ztheta, zhg, zfg, POWb)
!            (1-zfrac)*sealev(ia,j)+zfrac*sealev(ib,j), ztheta, zhg, zfg, POWb)


!                Calculate angle between actual normal to grounding line
!                and x axis (angnorm), and reduce flux accordingly

# 3844


!                If requested, correct Schoof flux for fraction of
!                budgall on first-floating h-cell between sub-grid g.l.
!                location (fracg[u,v]) and u-grid interface where flux
!                is applied (iau or ibu, zfluxsa or zfluxsb). This
!                assumes that dynamic flux variations are close to
!                equilibrium  with local budget on small (sub-grid)
!                scales. It yields equilibrated grounding-line locations
!                closer to the analytic Schoof solns in idealized tests
!                (MISMIP, Docquier/Pattyn; see notes 12/8/10, 12/12/10).
!                nb: Not valid for huge unrealistic budgall values
!                (MAXSHELFMELT, NOSHELF).

# 3868

             zfluxsa = zfg
             zfluxsb = zfg


!                Calculate Non-schoof "grid" flux at closest
!                u-grid pt to g.l. (iau), like explicit zf's in
!                icestep. zfluxg has model's i-index sign
!                (+ve if flowing to "right", as idir).

             zdfu  = dfu(iau,j) - lhsu(iau,j)
             zdfv  = 0.25 * (  dfv(ia,j) +dfv(ib,j) +dfv(ia,j-1) +dfv(ib,j-1)  &
                -lhsv(ia,j)-lhsv(ib,j)-lhsv(ia,j-1)-lhsv(ib,j-1))
             zdf2  = MAX (zdfu**2 + zdfv**2, .001)

!             SIA internal deformation:
             zhu    = hu(iau,j)
             zs2a   = 0.5*(s2a0(ia,j) + s2a0(ib,j))
             zs2b   = 0.5*(s2b0(ia,j) + s2b0(ib,j))
             zbra   = zs2a*(zdf2**zpowi) + zs2b*siu(iau,j) ! zpowi=1 for n=3
             zfluxg = 2. * zbra * (-zdfu) * zhu**2
             IF (muind(iau,j) == 0) THEN
!               plus SIA basal sliding:
               zcrh   = crhu(iau,j)
# 3900

               zbra   = zcrh * (zdf2**zpowb)

               zfluxg = zfluxg + zbra * (-zdfu) *  zhu
             ELSE
!               plus shelf elliptic soln (minus uia), with upstream h:
               zfluxg = zfluxg + (uagrid(iau,j)-uia(iau,j))*h(ia,j)
             ENDIF

!                Impose Schoof velocity, weighted at closest g.l.
!                u-grid pt (iau) or 1 point downstream (ibu),
!                as function (weischoof) of difference between
!                Schoof |flux| at iau vs. Schoof-free model |flux|
!                at iau (zfluxdif). More weighting for iau if Schoof
!                |flux| > model |flux| (g.l. retreat), more for ibu
!                if <  (g.l. advance). Sum of the two weights = 1.

             zfluxdif = zfluxsa - ABS(zfluxg)
! BSL function change
!fschoofu(iau,j) = weischoof(zfluxdif) ! weischoof is statement fn
!fschoofu(ibu,j) = 1. - weischoof(zfluxdif)
!weischoof(fluxdif) = MAX (0., MIN (1.,  fluxdif/1.e5 + 0.5 ))
             fschoofu(iau,j) = MAX (0., MIN (1.,  zfluxdif/1.e5 + 0.5 ))
             fschoofu(ibu,j) = 1. - fschoofu(iau,j)
! ^^^

!                Convert Schoof flux to veloc using upstream h-grid h,
!                to match upstream advection for it in icestep.
!                Limit |uschoof| for stability, in case h is v. small.

! BSLdiag
!             IF(maskcoulbu(iau,j)==1) PRINT *,"uschoof iau ibu ",zfluxsa/MAX(h(ia,j),1.),zfluxsb/MAX(h(ib,j),1.)
! ^^^

! BSLschoofgl
!    uschoof(iau,j) = idir * MIN (vmax, zfluxsa/MAX(h(ia,j),.01))
!    uschoof(ibu,j) = idir * MIN (vmax, zfluxsb/MAX(h(ib,j),.01))
             uschoof(iau,j) = idir * MIN (vmax, zfluxsa/MAX(h(ia,j),1.))
             uschoof(ibu,j) = idir * MIN (vmax, zfluxsb/MAX(h(ib,j),1.))

!                Set diagnostic g.l. quantities, returned:

             hgu(iau,j) = zhg
             qgu(iau,j) = zfg
             fluxschu(iau,j) = idir*zfluxsa
             fluxgrdu(iau,j) = zfluxg
!             angnormu(iau,j) = angnorm*180./PI
!~~~~~~~~~~~~~~~~
# 3952

           ENDIF
!~~~~~~~~~~~~~~~~
         ENDIF
!>>>>>>>>>>>>
       ENDDO
!>>>>>>>>>>>>
!==========
     ENDDO
!==========

!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
!        v grid (same comments as above for u grid):
!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv

!==============
     DO i=1,ni
!==============

       ifglv=0

       DO j=2,nj-2
         jp3 = MIN (j+3,nj)
         jm2 = MAX (j-2,1)
         IF (maskv(i,j) == 1) THEN
           IF      ( h(i  ,j) > 0. .AND. maskwater(i,j  ) == 0  &
              .AND. maskwater(i,j+1) /= 0 .AND. maskwater(i,j+2) /= 0  &
              .AND. maskwater(i,jp3) /= 0 ) THEN
             ifglv(j) = 1
           ELSE IF ( h(i,j+1) > 0. .AND. maskwater(i,j+1) == 0  &
              .AND. maskwater(i,j  ) /= 0 .AND. maskwater(i,j-1) /= 0  &
              .AND. maskwater(i,jm2) /= 0 ) THEN
             ifglv(j) = -1
           ENDIF
         ENDIF
       ENDDO

!<<<<<<<<<<<<<<<<<<
       DO j=2,nj-2
!<<<<<<<<<<<<<<<<<<
         IF (IABS(ifglv(j)) == 1) THEN

           IF (ifglv(j) == 1) THEN
             ja  = j
             jb  = j+1
             jau = j
             jbu = j+1
             idir = 1
           ELSE
             ja  = j+1
             jb  = j
             jau = j
             jbu = j-1
             idir = -1
           ENDIF
           ztheta = thetav(i,jb)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!           if (hs(i,ja).gt.hs(i,jb) .and. ztheta.gt.0.) then  ! was 0.3
           IF (hs(i,ja) > hs(i,jb)                   ) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
             zasch = 0.5*(arhap(i,ja)**POWi+arhap(i,jb)**POWi)
             zfrac = fracgv(i,jau)
             zdx = 0.5*(dyg(i,ja) + dyg(i,jb))
# 4034

             zbsch = (1./crhv(i,jau))**(1./POWb)
             CALL calcgl (hb(i,ja), hb(i,jb),  &
                zasch, zbsch, zfrac, zdx,  &
                sealev(i,jb), ztheta, zhg, zfg, POWb) !sedwj
!            (1-zfrac)*sealev(i,ja)+zfrac*sealev(i,jb), ztheta, zhg, zfg, POWb) !sedwj


# 4045


# 4056

             zfluxsa = zfg
             zfluxsb = zfg


             zdfu  = 0.25 * ( dfu(i,ja) +dfu(i,jb) +dfu(i-1,ja) +dfu(i-1,jb)  &
                -lhsu(i,ja)-lhsu(i,jb)-lhsu(i-1,ja)-lhsu(i-1,jb))
             zdfv  = dfv(i,jau) - lhsv(i,jau)
             zdf2  = MAX (zdfu**2 + zdfv**2, .001)

!             SIA internal deformation:
             zhv    = hv(i,jau)
             zs2a   = 0.5*(s2a0(i,ja) + s2a0(i,jb))
             zs2b   = 0.5*(s2b0(i,ja) + s2b0(i,jb))
             zbra   = zs2a*(zdf2**zpowi) + zs2b*siv(i,jau)
             zfluxg = 2. * zbra * (-zdfv) * zhv**2
             IF (mvind(i,jau) == 0) THEN
!               plus SIA basal sliding:
               zcrh   = crhv(i,jau)
# 4083

               zbra   = zcrh * (zdf2**zpowb)

               zfluxg = zfluxg + zbra * (-zdfv) * zhv
             ELSE
!               plus shelf elliptic soln (minus via), with upstream h:
! bug fix 07/17/15:
!               zfluxg = zfluxg + (vagrid(i,jau)-via(i,jau))*h(ia,j)!bug
               zfluxg = zfluxg + (vagrid(i,jau)-via(i,jau))*h(i,ja) !ok
             ENDIF

             zfluxdif = zfluxsa - ABS(zfluxg)
! BSL function change
!fschoofv(i,jau) = weischoof(zfluxdif) ! weischoof is statement fn
!fschoofv(i,jbu) = 1. - weischoof(zfluxdif)
!weischoof(fluxdif) = MAX (0., MIN (1.,  fluxdif/1.e5 + 0.5 ))
             fschoofv(i,jau) = MAX (0., MIN (1.,  zfluxdif/1.e5 + 0.5 ))
             fschoofv(i,jbu) = 1. - fschoofv(i,jau)
! ^^^

! BSLdiag
!             IF(maskcoulbv(i,jau)==1) PRINT *,"vschoof iau ibu ",zfluxsa/MAX(h(i,ja),1.),zfluxsb/MAX(h(i,jb),1.)
! ^^^

! BSLschoofgl
!    vschoof(i,jau) = idir * MIN (vmax, zfluxsa/MAX(h(i,ja),.01))
!    vschoof(i,jbu) = idir * MIN (vmax, zfluxsb/MAX(h(i,jb),.01))
             vschoof(i,jau) = idir * MIN (vmax, zfluxsa/MAX(h(i,ja),1.))
             vschoof(i,jbu) = idir * MIN (vmax, zfluxsb/MAX(h(i,jb),1.))

             hgv(i,jau) = zhg
             qgv(i,jau) = zfg
             fluxschv(i,jau) = idir*zfluxsa
             fluxgrdv(i,jau) = zfluxg
!             angnormv(i,jau) = angnorm*180./PI
!~~~~~~~~~~~~~~~~
# 4123

           ENDIF
         ENDIF
!<<<<<<<<<<<<
         ENDDO
!<<<<<<<<<<<<
!==========
     ENDDO
!==========

     RETURN
    END SUBROUTINE schoofgl
    
!-----------------------------------------------------------------------
    
    SUBROUTINE normcalc (iuc, juc, maskwater, angnorm, ifu)

!     Calculates angle (angnorm) between actual normal to grounding
!     line and the x axis, for a single u-grid (ifu=1) or v-grid
!     (ifu=0) point given by iuc,juc. The actual normal is estimated
!     using Ben P.'s algorithm! (Boulder, 2/12): averages the vectors
!     (zdx,zdy) from the point to the h-grid centers of nearby
!     ocean points (maskwater ne 0), looking only in a box of
!     half width nwin (few tens of km), and weighted by 1/dist.
!     Then the normal is just the arc tangent of the average (zdx,zdy).

!     Called from subr schoofgl. The u-grid Schoof flux is reduced there
!     by abs(cos(angnorm)), and the v-grid flux by abs(sin(angnorm))
!     (if defined ANGNORM).
!     For a long straight grounding line with (shallow) angle A to
!     x axis, pixelated by cells that step one grid box perpendicularly
!     for maNY cells along, the resulting mean flux across an actual
!     length L of grounding line calculated by this method is:
!       (L*cos(A)/dxg)*F*cos(A)*dxg + (L*sin(A)/dxg)*F*sin(A)*dxg = L*F
!     which is exactly what it should be for the actual segment L.

     USE modPHOMgrid
     USE modgridm, ONLY : ni, nj
     USE modconstants

     IMPLICIT NONE

     REAL, INTENT(OUT) :: angnorm
     INTEGER, INTENT(IN) :: maskwater(NX,NY)
     INTEGER, INTENT(IN) :: iuc, juc, ifu
     REAL :: zic,zjc,zavx,zavy,zava,zdx,zdy,zwei
     INTEGER :: i,j,nwin,ia,ib,ii,jj,ja,jb

     nwin = MAX (3, NINT(15.e3/dd0))         ! half-width of window box

!        Set (zic,zjc) = h-grid coords of u- or v-grid point (iuc,juc).
!        Also set (ia,ib,ja,jb) =  window box limits, h grid.

     IF (ifu == 1) THEN
       zic = iuc + 0.5
       zjc = juc
       ia = iuc   - (nwin-1)
       ib = iuc+1 + (nwin-1)
       ja = juc - nwin
       jb = juc + nwin
     ELSE
       zic = iuc
       zjc = juc + 0.5
       ia = iuc - nwin
       ib = iuc + nwin
       ja = juc   - (nwin-1)
       jb = juc+1 + (nwin-1)
     ENDIF

!       Average vectors (zdx,zdy) from u/v point to ocean (not land)
!       h-grid centers within window box, weighted by 1/distance
!       (1/zwei). If at edge of domain, assume land-ocean map is
!       extended from last edge row or column.

     zavx = 0.
     zavy = 0.
     zava = 0.
     DO j=ja,jb
       jj = MAX (1, MIN (nj, j))
       DO i=ia,ib
         ii = MAX (1, MIN (ni, i))
         IF (maskwater(ii,jj) /= 0) THEN
           zdx = (i - zic) * dd0
           zdy = (j - zjc) * dd0
           zwei = SQRT(zdx**2 + zdy**2)
           zavx = zavx + zdx / zwei
           zavy = zavy + zdy / zwei
           zava = zava + 1.  / zwei
         ENDIF
       ENDDO
     ENDDO
     IF (zava == 0.) THEN
       WRITE (6,'(a,2i6)') 'Error (normcalc): zava=0. iuc,juc=',iuc,juc
       STOP
     ENDIF
     zavx = zavx/zava
     zavy = zavy/zava

!       Set angle (counterclockwise) from x-axis to the estimated normal

     angnorm = ATAN2(zavy,zavx)

     RETURN
    END SUBROUTINE normcalc

!-----------------------------------------------------------------------

    SUBROUTINE calcgl ( hb1, hb2,  &
       acoef, bcoef, frac, dxr, sealevCell, theta, hg, fg, powbind)

!     Calculates hg (g.l. depth) and fg (Schoof flux = hg*ug).
!     h1,hs1,hb1 are upstream, and h2,hs2,hb2 are downstream (all supp),
!     regardless of actual grid direction. Accounts for buttressing
!     fraction theta.
!     acoef: Glenn flow Law A
!    ifTsai: bcoef: coulomb friction coefficient f, where Taub=f*Neff

     USE modconstants
     USE modicerheology, ONLY : POWi
# 4244


     IMPLICIT NONE

     REAL, INTENT(OUT) :: hg, fg
     REAL, INTENT(IN) :: hb1, hb2, acoef, bcoef, frac, dxr,  &
        sealevCell, theta

     INTEGER, INTENT(IN) :: powbind
     REAL :: fresol,x,hbg,hsg,ug,sedw,ug1

!     statement function for grid resol/non-resol term, to
!     eliminate explicit stretching perturbation if grid resolves
!     g.l. (see notes 9/27/07).
!     fresol(x) = 1.-exp(-(x**2))
! fresol(x) = 1.   ! or not

!     calc g.l. depth (linear interp to u-grid of bed elev, and s.l.)
!     hbg =       0.5*hb1 +  0.5*hb2
     hbg = (1.-frac)*hb1 + frac*hb2      ! (a bit better 100 vs 300 ka)
     hg = MAX ((sealevCell-hbg)/RHOIovS, 0.)
     hsg = hbg + hg

!     1/e-folding length of upstream perturbation (stretching) veloc
!     zlen = ( acoef * (0.5*bcoef)**powbind  * hg**(POWi - 2*powbind)  &
!        * (0.25*RHOIp*gEARTH*theta)**(POWi-powbind) ) ** (1./(powbind+1.))

!     perturbation (stretching) veloc
!     ug = (0.25*RHOIp*gEARTH*hg*theta)**(POWb*(POWi+1.)/(POWb+1.))
!     bug fix, POWer of theta:
# 4297

! Schoof, 2007 eq.16, Note Schoof uses m=1/powb, => 1/(m+1)=powb/(powb+1)
! bcoef matches Schoof C, RHOIp = (1.-RHOIc/RHOWSc)*RHOIc
! correspondence to eq16 has been repeatedly verified
! weertman power law basal drag rheology
!     ug = ( (0.25*RHOIp*gEARTH)**(POWi+1.) * (hg*theta)**POWi  &
!          * (4.*(acoef/bcoef)/(1-RHOIovS)) *hg**2 )**(powbind/(powbind+1.)) ! newest splitting, tested but different answers

     ug = ( (0.25*RHOIp*gEARTH*hg)**(POWi+1.) * (theta)**POWi  &
        * (4.*hg*(acoef/bcoef)/(1-RHOIovS)) )**(powbind/(powbind+1.)) ! winter/21 splitting

!       ug = ( (0.25*RHOIp*gEARTH*hg)**(POWi+1.) * (theta)**POWi  &
!          * (4.*hg*acoef/bcoef)/(1-RHOIovS) )**(powbind/(powbind+1.)) ! test variant

!     ug = (0.25*RHOIp*gEARTH*hg)**(powbind*(POWi+1.)/(powbind+1.))  &
!        *            (theta)**(powbind*(POWi   )/(powbind+1.))  &
!        * (2.*hg*acoef/bcoef)**(powbind/(powbind+1.))  &
!        / (0.5*(1.-RHOIovS))**(powbind/(powbind+1.)) ! ~Sch eq.16


!ug = ug * fresol(zlen*dxr)

# 4321


     fg = hg*ug

     RETURN
    END SUBROUTINE calcgl

!-----------------------------------------------------------------------

    SUBROUTINE thetacalc (h, fedge, maskwater, arhap, uagrid, vagrid, thetau, thetav)

!     Sets theta[u,v] = 1 - fraction of g.l. longitudinal stress
!     buttressed, used in schoofgl and calcgl.

!     Nb: theta[u,v] are on h-grid, for E-W and N-S directions
!     respectively, and ice thickness is h, so that if all shelf is
!     free-floating, theta=1 exactly (property of elliptic solution).
!     (And current h is that used in previous ASCH-loop, because
!     it's inside the C-loop).


     USE modPHOMgrid
     USE modgridm, ONLY : ni, nj, dxg, dyg
     USE modconstants
     USE modicerheology, ONLY : POWiv
     IMPLICIT NONE

     REAL, INTENT(OUT) :: thetau(NX,NY),         thetav(NX,NY)
     REAL, INTENT(IN) :: h(0:NXp,0:NYp), fedge(NX,NY),  &
        arhap(NX,NY),  &
        uagrid(0:NX,0:NY), vagrid(0:NX,0:NY)
     INTEGER, INTENT(IN) :: maskwater(NX,NY)

     REAL :: zdudx,zdvdy,zg1,zvisc,zfed
     INTEGER :: i,j,im1,ip1,jm1,jp1

     REAL , PARAMETER :: HTHETMIN = .01   ! have tried 0.1, 1., 10.

     DO j=1,nj
       DO i=1,ni
         IF (maskwater(i,j) /= 0) THEN
# 4365

           IF (h(i,j) < HTHETMIN) THEN
             thetau(i,j) = 1.
             thetav(i,j) = 1.
           ELSE
             zdudx = (uagrid(i,j)-uagrid(i-1,j)) / dxg(i,j)
             zdvdy = (vagrid(i,j)-vagrid(i,j-1)) / dyg(i,j)

!             zdiv = max (0., zdudx+zdvdy)
!             thetau(i,j) = ( zdiv**(1./POWi) / arhap(i,j) )
!    *                      * 4. / (RHOIp*gEARTH*h(i,j))

             zg1 =  MAX (zdudx**2 + zdvdy**2 + zdudx*zdvdy, gmin)
             zvisc = 0.5 / (zg1**POWiv)
!               thetau(i,j)  = ( 4.*zvisc*2.*zdudx/arhap(i,j) ) / (RHOIp*gEARTH*h(i,j))
!               thetav(i,j)  = ( 4.*zvisc*2.*zdvdy/arhap(i,j) ) / (RHOIp*gEARTH*h(i,j))
             thetau(i,j)  = ( 2.*zvisc*ABS(2.*zdudx+zdvdy)/arhap(i,j))/(0.5*RHOIp*gEARTH*h(i,j))
             thetav(i,j)  = ( 2.*zvisc*ABS(2.*zdvdy+zdudx)/arhap(i,j))/(0.5*RHOIp*gEARTH*h(i,j))

             thetau(i,j) = MAX (0., MIN (1., thetau(i,j)))
             thetav(i,j) = MAX (0., MIN (1., thetav(i,j)))

             im1 = MAX (i-1,1)
             ip1 = MIN (i+1,ni)
             zfed = fedge(i,j)
             IF (maskwater(im1,j) /= 0) zfed = MIN(zfed,fedge(im1,j))
             IF (maskwater(ip1,j) /= 0) zfed = MIN(zfed,fedge(ip1,j))
             thetau(i,j) = (1.-zfed)*1. + zfed*thetau(i,j)

             jm1 = MAX (j-1,1)
             jp1 = MIN (j+1,nj)
             zfed = fedge(i,j)
             IF (maskwater(i,jm1) /= 0) zfed = MIN(zfed,fedge(i,jm1))
             IF (maskwater(i,jp1) /= 0) zfed = MIN(zfed,fedge(i,jp1))
             thetav(i,j) = (1.-zfed)*1. + zfed*thetav(i,j)
           ENDIF

         ELSE
           thetau(i,j) = -1.
           thetav(i,j) = -1.
         ENDIF
       ENDDO
     ENDDO

     RETURN
    END SUBROUTINE thetacalc

!)))))

!)))))

!-----------------------------------------------------------------------
    
!((((((((((((((((((((((
# 4621

!(((((

!-----------------------------------------------------------------------

! INTENT(OUT): hnew, INTENT(INOUT): uia, via, uit, vit, ub, vb, ua, va, rest INTENT(IN)
      SUBROUTINE icestep(hold, hnew, hw, maskwater,  &
         hu,  hv,  dhsu,  dhsv, dfu, dfv, lhsu, lhsv,  &
         s1a0, s1b0, s2a0, s2b0, siu, siv, crhu, crhv,  &
         uia, via, uit, vit, ub, vb, ua, va, &
         uschoof, vschoof, fschoofu, fschoofv,  &
         masku, maskv, muind, mvind, medgeu, medgev, dt, vmax)

!        Step ice thickness through one timestep, from hold (beginning
!        of timestep) to hnew (end of timestep, returned).
!        For all pts:
!          Does SIA internal deformation advection (fully time-implicit
!          Newton-Raphson ADI + tridia, see notes 2/9/11). Sets uia,via
!          for SSA solns, uit,vit for cross-softening in shelf coeffs.
!        For non-SSA (non-elliptic eqn) pts, i.e., m[u,v]ind = 0:
!          Adds SIA basal sliding (fully time-implicit in h advection).
!          Sets diagnostic ub,vb.
!        For SSA (elliptic eqn) pts, i.e., m[u,v]ind >= 1:
!          Adds prev elliptic ua,va with upstream (but time-implicit) h,
!          minus uia,via to represent basal sliding.
!          Sets diagnostic ub,vb.

!        For closest pt and 1 pt downstream of grounding line (where
!        fschoof[u,v] are non zero), weights bewtween the above
!        solutions, and [u,v]schoof with upstream (but time-implicit)
!        h. This assumes Schoof(2007) analysis yields g.l. flux and
!        velocity that includes internal deformation, although it
!        isn't in the Schoof model. (So for idealized 1-D Mismip-like
!        tests, need to set enhancesheet=0).

!        Sheet-pts ADI method operates on each horizontal direction
!        in turn, calling subr ice to step the 1-D ice flow equation.
!        Units are meters and years. The 1-D strip arrays *(N1D) are
!        used both for both directions in the domain.

!        For ua advection (sliding/sed def/shelf),
!        use an upstream scheme for h interp (or could use cubic SLT?),
!        since any centered-interp scheme gives spurious buildup
!        at the last non-zero grid box of a flat uniform ice shelf.

       USE modPHOMgrid
       USE modconstants
       USE modgridm, ONLY : ni, nj, darea, dxu, dyu, dxv, dyv, kiceS
# 4672

       USE modbeddrag, ONLY : POWb

       USE modicerheology, ONLY : POWi

# 4680

       IMPLICIT NONE

!        hold  = initial thickness (m) (h-grid, supplied)
!        h     = mid-timestep ice-sheet thickness (m) (supplied)
!        hnew  = new ice-sheet thickness (m) (h-grid, returned)
!        hw    = liquid water thickness (m) (h-grid, supplied
!        uia = depth-averaged internal-shear u veloc (u-grid, returned)
!        via = depth-averaged internal-shear v veloc (v-grid, returned)
!        dt = ice-model timestep (yr) (supplied)

       REAL, INTENT(IN) :: hold(0:NXp,0:NYp)
       REAL, INTENT(OUT) :: hnew(0:NXp,0:NYp)
       REAL, INTENT(IN OUT) ::  &
          uia(0:NX,0:NY),        via(0:NX,0:NY),  &
          uit(0:NX,0:NY),        vit(0:NX,0:NY),  &
          ub(0:NX,0:NY),         vb(0:NX,0:NY),  &
          ua(0:NX,0:NY),         va(0:NX,0:NY)
       REAL, INTENT(IN) ::  &
          hw(NX,NY),             hu(0:NX,0:NY),       hv(0:NX,0:NY),  &
          dhsu(0:NX,0:NY),       dhsv(0:NX,0:NY),  &
          dfu(0:NX,0:NY),        dfv(0:NX,0:NY),  &
          lhsu(0:NX,0:NY),       lhsv(0:NX,0:NY),  &
          s1a0(NX,NY),           s1b0(NX,NY),  &
          s2a0(NX,NY),           s2b0(NX,NY),  &
          siu(0:NX,0:NY),        siv(0:NX,0:NY),  &
          crhu(0:NX,0:NY),       crhv(0:NX,0:NY),  &
          uschoof(0:NX,0:NY),    vschoof(0:NX,0:NY),  &
          fschoofu(0:NX,0:NY),   fschoofv(0:NX,0:NY), &
          dt, vmax

       INTEGER, INTENT(IN) :: maskwater(NX,NY), &
          masku(0:NX,0:NY),    maskv(0:NX,0:NY),  &
          muind(0:NX,0:NY),    mvind(0:NX,0:NY), &
          medgeu(0:NX,0:NY),   medgev(0:NX,0:NY)

!     local:
       REAL :: pconv(NX,NY)     ! tendency from first ADI stage (m/yr) (h-grid)
       REAL :: timeice,zpowi,zpowim1,zpowb,zpowbm1,zdfu,zdfv,zdf2,zdhsu,zdhsv, &
          zhu,zdh,zs1a,zs1b,zs2a,zs2b,zbra,zf,zdfdh,zdfda1,zdfda2,zcrh,zhv, &
          sedw,zbra1,zbra2,zdfda11,zdfda12,zdfda21,zdfda22,zuia,zvia
       INTEGER :: i,j,n1,n2,ig,ip,jg,jp

!        1-D strip arrays:

       INTEGER, PARAMETER :: N1D=NX+NY
       INTEGER, PARAMETER :: VbMaxSIA=10.e3 !m/yr
       REAL :: hold1(N1D),   hnew1(N1D),   darea1(N1D),  &
          pconv1(N1D),  facfloat(N1D),  &
          e(0:N1D),     f(0:N1D),     g(0:N1D),      flux(0:N1D),  &
          ubsp(0:N1D),  ubsm(0:N1D),  fluxub(0:N1D)

       hnew=hold

# 4751


       uia=0.
       via=0.
       uit=0.
       vit=0.
       ub=0.
       vb=0.



       IF (POWi == 3) THEN
         zpowi = 1.
         zpowim1 = 0.
       ELSE
         zpowi = (POWi-1)/2.
         zpowim1 = zpowi - 1.
       ENDIF


       IF (POWb == 3) THEN
         zpowb = 1.
         zpowbm1 = 0.
       ELSE
         zpowb = (POWb-1)/2.
         zpowbm1 = zpowb - 1.
       ENDIF


!        First ADI pass, doing flow for all E-W strips.
!        Calculate tendency (pconv) due to E-W flow,
!        don't change 2-D ice thickness yet.

       pconv=0.

!~~~~~~~~~~~~~~~~~~~~~~
       IF (ni > 1) THEN
!~~~~~~~~~~~~~~~~~~~~~~

# 4806


!>>>>>>>>>>>>>>
# 4811

           DO j=1,nj

!>>>>>>>>>>>>>>

!        Find low-index edge of ice sheet (n1 is adjacent box with h=0).
!        If non-zero ice exists in first box, set n1=1, and b.c. will
!        implicitly be no flow out of left-hand edge of box.

             n1 = -1
             DO i = 1,ni
               IF (hold(i,j) + dt*MAX(pconv(i,j),0.) > 0.) THEN
                 n1 = MAX (i-1, 1)
                 GOTO 10
               ENDIF
             ENDDO
10           IF (n1 == -1) n1 = 1

!        Find high-index edge of ice sheet(n2 is adjacent box with h=0).
!        If non-zero ice exists in last box, set n2=ni, and b.c. will
!        implicitly be no flow out of right-hand edge of box.

             n2 = -1
             DO i = ni,1,-1
               IF (hold(i,j) + dt*MAX(pconv(i,j),0.) > 0.) THEN
                 n2 = MIN (i+1, ni)
                 GOTO 20
               ENDIF
             ENDDO
20           IF (n2 == -1) n2 = 1
# 4844


             DO i=1,ni
               hold1(i) = hold(i,j)
               pconv1(i)= pconv(i,j)
               darea1(i)= darea(i,j)
               facfloat(i) = 1.
               IF (maskwater(i,j) /= 0) facfloat(i) = 1.-RHOIovS
             ENDDO

! BSLnew
             e=0.
             f=0.
             g=0.
             flux=0.
             ubsp=0.
             ubsm=0.
             fluxub=0.

!*************************
             IF (n2 > n1) THEN
!*************************

!          Set 1-D strips (E-W)

               DO i=n1-1,n2
                 ig = MAX(i,1)
                 ip = MIN(i+1,ni)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                 IF (masku(i,j) == 1) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!              Use quantities at beginning of timestep for time-implicit
!              tridiag calcs (hu0, dhsu0, dfu0 passed to icestep):

! longitudinal and horizontal shear gradient force (LHS of SSA equations): lhs ; = 0 for pure SIA

                   zdfu  = dfu(i,j) - lhsu(i,j) ! central term in eq 3, ie sigma_xz at bed
                   zdfv  = 0.25 * ( dfv(i,j) + dfv(i+1,j) + dfv(i,j-1) + dfv(i+1,j-1)  &
                      -lhsv(i,j) -lhsv(i+1,j) -lhsv(i,j-1) -lhsv(i+1,j-1))
                   zdf2  = MAX (zdfu**2 + zdfv**2, .001)
                   zdhsu = dhsu(i,j)
                   zdhsv = 0.25 * (  dhsv(i,j)   + dhsv(i+1,j)  &
                      + dhsv(i,j-1) + dhsv(i+1,j-1) )
                   zhu   = hu(i,j)
                   zdh   = (facfloat(ip)*hold(ip,j) - facfloat(ig)*hold(ig,j)) / dxu(i,j)

!              Internal deformation (see notes 2/9/11):

                   zs1a = 0.5*(s1a0(ig,j) + s1a0(ip,j))
                   zs1b = 0.5*(s1b0(ig,j) + s1b0(ip,j))
                   zs2a = 0.5*(s2a0(ig,j) + s2a0(ip,j))
                   zs2b = 0.5*(s2b0(ig,j) + s2b0(ip,j))

                   zbra   = zs2a*(zdf2**zpowi) + zs2b*siu(i,j) ! Sigma**(n-1)
                   zf     = 2. * zbra * zdfu * zhu**2 ! DP12 eq 1 *H**2 ie : H**2 * dUi/dz
                   zdfdh  = 4. * zbra * zdfu * zhu
                   zdfda1 = 2.*zbra*zhu**2  &
                      + 4.*zs2a*zpowi*(zdfu**2)  *(zhu**2)*(zdf2**zpowim1)
                   zdfda2 = 4.*zs2a*zpowi*(zdfu*zdfv)*(zhu**2)*(zdf2**zpowim1)

                   e(i)    = (1.-fschoofu(i,j)) * ( - zdfdh*zhu  &
                      - zdfda1*RHOIc*gEARTH*zhu*(zdhsu+zdh) - zdfda2*RHOIc*gEARTH*zhu*zdhsv  &
                      ) * dyu(i,j)
                   f(i)    = (1.-fschoofu(i,j)) * (   zdfdh  &
                      + zdfda1*RHOIc*gEARTH*zdhsu + zdfda2*RHOIc*gEARTH*zdhsv  &
                      ) *dyu(i,j)
                   g(i)    = (1.-fschoofu(i,j)) * ( zdfda1*RHOIc*gEARTH*zhu/dxu(i,j)  &
                      ) * dyu(i,j)
                   flux(i) = (1.-fschoofu(i,j)) * ( -zf  &
                      ) * dyu(i,j)

!           set uia,uit, used in icedyn and setcoefs (softening)
                   zuia = (1.-fschoofu(i,j))*(-zf)/MAX(zhu,1.e-20)
                   zuia = MAX (-vmax, MIN (vmax, zuia)) ! 2/2021 /LTmar22
# 4923


                   uit(i,j) = -2.*zdfu*zhu * (zdf2*zs1a + siu(i,j)*zs1b)


!              Plus basal sliding or elliptic (see notes 2/9/11):

!------------------------------------
!           sheet pts, not shelf:
                   IF (muind(i,j) == 0) THEN
!------------------------------------

                     zcrh = crhu(i,j)
# 4951

                     zbra = zcrh * (zdf2**zpowb)
                     zdfda1 = zbra*zhu + 2.*zcrh*zpowb*(zdfu**2)  *zhu*(zdf2**zpowbm1)
                     zdfda2 = 2.*zcrh*zpowb*(zdfu*zdfv)*zhu*(zdf2**zpowbm1)

                     zf   = zbra * zdfu * zhu
                     zdfdh = zbra * zdfu

                     e(i)    = e(i) + (1.-fschoofu(i,j))  &
                        * ( - zdfdh*zhu - zdfda1*RHOIc*gEARTH*zhu*(zdhsu+zdh)  &
                        - zdfda2*RHOIc*gEARTH*zhu*zdhsv ) * dyu(i,j)
                     f(i)    = f(i) + (1.-fschoofu(i,j))  &
                        * (   zdfdh + zdfda1*RHOIc*gEARTH*zdhsu  &
                        + zdfda2*RHOIc*gEARTH*zdhsv ) *dyu(i,j)
                     g(i)    = g(i) + (1.-fschoofu(i,j))  &
                        * ( zdfda1*RHOIc*gEARTH*zhu/dxu(i,j) ) * dyu(i,j)
                     flux(i) = flux(i) + (1.-fschoofu(i,j))  &
                        * ( -zf ) * dyu(i,j)

                     fluxub(i) = fschoofu(i,j) * (  MAX(uschoof(i,j),0.)*hold(ig,j)  &
                        + MIN(uschoof(i,j),0.)*hold(ip,j) ) * dyu(i,j)
                     ubsp(i) = fschoofu(i,j) * MAX(uschoof(i,j),0.) * dyu(i,j)
                     ubsm(i) = fschoofu(i,j) * MIN(uschoof(i,j),0.) * dyu(i,j)

!             diagnostic:
                     ub(i,j) = (1.-fschoofu(i,j)) *(-zf)/MAX(hu(i,j),1.e-20)  &
                        +     fschoofu(i,j)  * uschoof(i,j)
# 4984

!feb10/19change                        ua(i,j) = uia(i,j) + ub(i,j)

                     uia(i,j) = zuia          ! DP Nov 2019, needed for heatbu for sheet pts

                     ua(i,j) = zuia + ub(i,j)

!-----------------------------------------
!           elliptic pts:
                   ELSE IF (muind(i,j) > 0) THEN
!-----------------------------------------

!             nb: if fschoofu > 0, Schoof velocities are already
!             imposed in elliptic soln ua.
                     ub(i,j) = ua(i,j) - uia(i,j)
                     ubsp(i) = MAX(ub(i,j),0.) * dyu(i,j)
                     ubsm(i) = MIN(ub(i,j),0.) * dyu(i,j)
                     fluxub(i) = ubsp(i)*hold(ig,j) + ubsm(i)*hold(ip,j) ! upwinding as per pollardC.modelDescript.gmd12.pdf

!----------------
                   ENDIF
!----------------

!#if defined (1) || ! defined (NOSUBGRIDEDGESHELF)

!           no flow *out* of boxes with fedge < 1 (medgeu set above):
                   IF (medgeu(i,j) == 0) THEN
                     e(i) = 0.
                     f(i) = 0.
                     g(i) = 0.
                     flux(i) = 0.
                     ubsp(i) = 0.
                     ubsm(i) = 0.
                     fluxub(i) = 0.
                     ua(i,j) = 0.
                     ub(i,j) = 0.
                     uia(i,j) = 0.
                     uit(i,j) = 0.
                   ENDIF


!~~~~~~~~~~~~~~
                 ENDIF
!~~~~~~~~~~~~~~

               ENDDO

! Don't do following if nesting (bug fix 5/25/10)

               e(n1-1) = 0.
               f(n1-1) = 0.
               g(n1-1) = 0.
               flux(n1-1) = 0.
               ubsp  (n1-1) = 0.
               ubsm  (n1-1) = 0.
               fluxub(n1-1) = 0.
               e(n2) = 0.
               f(n2) = 0.
               g(n2) = 0.
               flux(n2) = 0.
               ubsp  (n2) = 0.
               ubsm  (n2) = 0.
               fluxub(n2) = 0.


!          Call iceflow to do E-W flow

# 5056

             CALL iceflow (hold1, hnew1, facfloat,  &
                e, f, g, flux, ubsp, ubsm, fluxub, pconv1, darea1, n1, n2, ni, dt, j, 0)

!          Store tendency (dh/dt) due to E-W flow only.
!          But if nj=1 (no N-S flow), won't do N-S loop below,
!          so store final result in hnew.

             IF (nj > 1) THEN
               DO i=n1,n2
                 pconv(i,j) = (hnew1(i)-hold1(i)) / dt
               ENDDO
             ELSE
               DO i=n1,n2
                 hnew(i,j) = hnew1(i)
               ENDDO
             ENDIF

!************
           ENDIF   ! n2 > n1
!************
!>>>>>>>>>>
         ENDDO     ! j loop
!>>>>>>>>>>
!~~~~~~~~~~
       ENDIF     ! ni > 1
!~~~~~~~~~~

!        Second ADI pass, doing flow for all N-S strips.
!        Use tendency (pconv) saved from first ADI pass.
!        Set new ice thickness (hnew).

!~~~~~~~~~~~~~~~~~~~~~~
       IF (nj > 1) THEN
!~~~~~~~~~~~~~~~~~~~~~~

# 5108


!<<<<<<<<<<<<<<
# 5113

           DO i=1,ni

!<<<<<<<<<<<<<<

!        Find low-index edge of ice sheet (n1 is adjacent box with h=0).
!        If non-zero ice exists in first box, set n1=1, and b.c. will
!        implicitly be no flow out of bottom edge of box.

             n1 = -1
             DO j = 1,nj
               IF (hold(i,j) + dt*MAX(pconv(i,j),0.) > 0.) THEN
                 n1 = MAX (j-1, 1)
                 EXIT
               ENDIF
             ENDDO
             IF (n1 == -1) n1 = 1

!        Find high-index edge of ice sheet(n2 is adjacent box with h=0).
!        If non-zero ice exists in last box, set n2=nj, and b.c. will
!        implicitly be no flow out of top edge of box.

             n2 = -1
             DO j = nj,1,-1
               IF (hold(i,j) + dt*MAX(pconv(i,j),0.) > 0.) THEN
                 n2 = MIN (j+1, nj)
                 EXIT
               ENDIF
             ENDDO
             IF (n2 == -1) n2 = 1
# 5146


             DO j=1,nj
               hold1(j) = hold(i,j)
               pconv1(j)= pconv(i,j)
               darea1(j)= darea(i,j)
               facfloat(j) = 1.
               IF (maskwater(i,j) /= 0) facfloat(j) = 1.-RHOIovS
             ENDDO

! BSLnew
             e=0.
             f=0.
             g=0.
             flux=0.
             ubsp=0.
             ubsm=0.
             fluxub=0.

!*************************
             IF (n2 > n1) THEN
!*************************

!          Set 1-D strips (N-S)

               DO j=n1-1,n2
                 jg = MAX(j,1)
                 jp = MIN(j+1,nj)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                 IF (maskv(i,j) == 1) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!              Use quantities at beginning of timestep for time-implicit
!              tridiag calcs (hv0, dhsv0, dfv0 passed to icestep):

!           longitudinal-stress force (lhs), strain softening (beta2),

                   zdfv   = dfv(i,j) - lhsv(i,j) ! DP12 eq 3 zdfv is sigma_yz at bed
                   zdfu   = 0.25 * (dfu(i,j) + dfu(i,j+1) + dfu(i-1,j) + dfu(i-1,j+1)  &
                      -lhsu(i,j) -lhsu(i,j+1) -lhsu(i-1,j) -lhsu(i-1,j+1))
                   zdf2   = MAX (zdfu**2 + zdfv**2, .001)
                   zdhsv = dhsv(i,j)
                   zdhsu = 0.25 * (  dhsu(i,j)   + dhsu(i,j+1)  &
                      + dhsu(i-1,j) + dhsu(i-1,j+1) )
                   zhv   = hv(i,j)
                   zdh   = (facfloat(jp)*hold(i,jp) - facfloat(jg)*hold(i,jg)) / dyv(i,j)

!              Internal deformation (see notes 2/9/11):

                   zs1a = 0.5*(s1a0(i,jg) + s1a0(i,jp))
                   zs1b = 0.5*(s1b0(i,jg) + s1b0(i,jp))
                   zs2a = 0.5*(s2a0(i,jg) + s2a0(i,jp))
                   zs2b = 0.5*(s2b0(i,jg) + s2b0(i,jp))

                   zbra   = zs2a*(zdf2**zpowi) + zs2b*siv(i,j) 
                   zf     = 2. * zbra * zdfv * zhv**2 ! DP12 eq 1 *H**2 ie : H**2 * dUi/dz
                   zdfdh  = 4. * zbra * zdfv * zhv
                   zdfda1 = 2.*zbra*zhv**2  &
                      + 4.*zs2a*zpowi*(zdfv**2)  *(zhv**2)*(zdf2**zpowim1)
                   zdfda2 = 4.*zs2a*zpowi*(zdfv*zdfu)*(zhv**2)*(zdf2**zpowim1)

                   e(j)   = (1.-fschoofv(i,j)) * ( - zdfdh*zhv  &
                      - zdfda1*RHOIc*gEARTH*zhv*(zdhsv+zdh) - zdfda2*RHOIc*gEARTH*zhv*zdhsu  &
                      ) * dxv(i,j)
                   f(j)    = (1.-fschoofv(i,j)) * (   zdfdh  &
                      + zdfda1*RHOIc*gEARTH*zdhsv + zdfda2*RHOIc*gEARTH*zdhsu  &
                      ) *dxv(i,j)
                   g(j)    = (1.-fschoofv(i,j)) * ( zdfda1*RHOIc*gEARTH*zhv/dyv(i,j)  &
                      ) * dxv(i,j)
                   flux(j) = (1.-fschoofv(i,j)) * ( -zf  &
                      ) * dxv(i,j)

!           set via,vit, used in icedyn and setcoefs (softening)
                   zvia = (1.-fschoofv(i,j))*(-zf)/max(zhv,1.e-20)
                   zvia = MAX (-vmax, MIN (vmax, zvia)) ! 2/2021/LTmar22
# 5225


                   vit(i,j) = -2.*zdfv*zhv * (zdf2*zs1a + siv(i,j)*zs1b)


!              Plus basal sliding or elliptic (see notes 2/9/11):

!------------------------------------
!           sheet pts, not shelf:
                   IF (mvind(i,j) == 0) THEN
!------------------------------------

                     zcrh = crhv(i,j)
# 5253

                     zbra = zcrh * (zdf2**zpowb)
                     zdfda1 = zbra*zhv + 2.*zcrh*zpowb*(zdfv**2)  *zhv*(zdf2**zpowbm1)
                     zdfda2 = 2.*zcrh*zpowb*(zdfv*zdfu)*zhv*(zdf2**zpowbm1)

                     zf   = zbra * zdfv * zhv
                     zdfdh = zbra * zdfv

                     e(j)    = e(j) + (1.-fschoofv(i,j))  &
                        * ( - zdfdh*zhv - zdfda1*RHOIc*gEARTH*zhv*(zdhsv+zdh)  &
                        - zdfda2*RHOIc*gEARTH*zhv*zdhsu ) * dxv(i,j)
                     f(j)    = f(j) + (1.-fschoofv(i,j))  &
                        * (   zdfdh + zdfda1*RHOIc*gEARTH*zdhsv  &
                        + zdfda2*RHOIc*gEARTH*zdhsu ) *dxv(i,j)
                     g(j)    = g(j) + (1.-fschoofv(i,j))  &
                        * ( zdfda1*RHOIc*gEARTH*zhv/dyv(i,j) ) * dxv(i,j)
                     flux(j) = flux(j) + (1.-fschoofv(i,j))  &
                        * ( -zf ) * dxv(i,j)

                     fluxub(j) = fschoofv(i,j) * (  MAX(vschoof(i,j),0.)*hold(i,jg)  &
                        + MIN(vschoof(i,j),0.)*hold(i,jp) ) * dxv(i,j)
                     ubsp(j) = fschoofv(i,j) * MAX(vschoof(i,j),0.) * dxv(i,j)
                     ubsm(j) = fschoofv(i,j) * MIN(vschoof(i,j),0.) * dxv(i,j)

!             diagnostic:
                     vb(i,j) = (1.-fschoofv(i,j)) *(-zf)/MAX(hv(i,j),1.e-20)  &
                        +     fschoofv(i,j)  * vschoof(i,j)
# 5286

!feb10/19change                       va(i,j) = via(i,j) + vb(i,j)

                     via(i,j) = zvia          ! DP Nov 2019

                     va(i,j) = zvia + vb(i,j)

!-----------------------------------------
!           elliptic pts:
                   ELSE IF (mvind(i,j) > 0) THEN
!-----------------------------------------

!             nb: if fschoofv > 0, Schoof velocities are already
!             imposed in elliptic soln va.
                     vb(i,j) = va(i,j) - via(i,j)
                     ubsp(j) = MAX(vb(i,j),0.)*dxv(i,j)
                     ubsm(j) = MIN(vb(i,j),0.)*dxv(i,j)
                     fluxub(j) = ubsp(j)*hold(i,jg) + ubsm(j)*hold(i,jp)

!----------------
                   ENDIF
!----------------

!#if defined (1) || ! defined (NOSUBGRIDEDGESHELF)

!           no flow *out* of boxes with fedge < 1 (medgev set above):
                   IF (medgev(i,j) == 0) THEN
                     e(j) = 0.
                     f(j) = 0.
                     g(j) = 0.
                     f(j) = 0.
                     flux(j) = 0.
                     fluxub(j) = 0.
                     ubsp(j) = 0.
                     ubsm(j) = 0.
                     va(i,j) = 0.
                     vb(i,j) = 0.
                     via(i,j) = 0.
                     vit(i,j) = 0.
                   ENDIF


!~~~~~~~~~~~~~~
                 ENDIF
!~~~~~~~~~~~~~~

               ENDDO

! Don't do following if nesting (bug fix 5/25/10)

               e(n1-1) = 0.
               f(n1-1) = 0.
               g(n1-1) = 0.
               flux(n1-1) = 0.
               ubsp  (n1-1) = 0.
               ubsm  (n1-1) = 0.
               fluxub(n1-1) = 0.
               e(n2) = 0.
               f(n2) = 0.
               g(n2) = 0.
               flux(n2) = 0.
               ubsp  (n2) = 0.
               ubsm  (n2) = 0.
               fluxub(n2) = 0.


!          Call iceflow to do N-S flow

# 5359

             CALL iceflow (hold1, hnew1, facfloat,  &
                e, f, g, flux, ubsp, ubsm, fluxub, pconv1, darea1, n1, n2, nj, dt, j, 1)

!          Store new 2-D ice thickness

             DO j=n1,n2
               hnew(i,j) = hnew1(j)
             ENDDO

!************
           ENDIF   ! n2 > n1
!************
!<<<<<<<<<<
         ENDDO     ! i loop
!<<<<<<<<<<
!~~~~~~~~~~
       ENDIF     ! NY > 1
!~~~~~~~~~~

RETURN
END SUBROUTINE icestep

!-----------------------------------------------------------------------

SUBROUTINE iceflow (h0, h1, facfloat,  &
    e, f, g, flux, ubsp, ubsm, fluxub, yconv, dar, n1, n2, NXory, dt, j, ifv)

!        Steps ice-sheet 1-D flow equation through one time step.
!        Time-implicit contributions lead to a tridiagonal matrix
!        equation. Units are meters and years.

!        The current direction is called the "x-direction" (index i)
!        here, and the perpendicular direction is the "y-direction",
!        accounted for either by yconv.
!        Only the x-direction has time-implicit terms.
!        The correspondence with real E-W or N-S directions depends
!        on the current ADI stage in the calling program.

!        Ice-sheet thickness are defined on h-grid (h).
!        Other terms (...u) are defined on a staggered u-grid, with
!        ...u(i) at the interface between h-grid boxes i and i+1.

!        h0   = actual ice thickness at start of time step (m)(supplied)
!        h1   = actual ice thickness at end of time step (m) (returned)
!        yconv= convergence rate due to perpendicular flow (m/y) (supp)
!        dar  = grid-box area (h-grid) (m**2) (supplied)
!        dt   = timestep (yr) (supplied)

 USE modPHOMgrid
 USE modconstants

 IMPLICIT NONE
 
 INTEGER :: i,im,ip
 INTEGER, PARAMETER :: N1D=NX+NY

 INTEGER, INTENT(IN) :: n1, n2, NXory, j, ifv
 REAL, INTENT(OUT) :: h1(N1D)
 REAL, INTENT(IN) :: h0(N1D),  &
    facfloat(N1D),  &
    e(0:N1D), f(0:N1D), g(0:N1D), flux(0:N1D),  &
    ubsp(0:N1D), ubsm(0:N1D), fluxub(0:N1D), yconv(N1D), dar(N1D)
 REAL, INTENT(IN) :: dt

 REAL :: a(N1D), b(N1D), c(N1D), forc(N1D)
 REAL :: dti,dte

 REAL, PARAMETER :: RIMP = 1.         ! time-implicit vs explicit fraction
!     parameter (RIMP = 0.5)        ! time-implicit vs explicit fraction

!        Set implicit vs explicit timesteps

 dti =     RIMP *dt
 dte = (1.-RIMP)*dt

!       Set the 3 diagonals of the tridiagonal matrix, and the rhs

 DO i = n1,n2
   im = MAX(i-1,1)
   ip = MIN(i+1,NXory)

!       main diagonal:
   a(i) = 1.+ dti*(   (-0.5*f(i)   + g(i)*facfloat(i))  &
      + ( 0.5*f(i-1) + g(i-1)*facfloat(i))  &
      + ( ubsp (i)   - ubsm (i-1))  &  ! upstream
!    *                   + ( 0.5*( -ubsp(i-1) + ubsp(i)  ! centered
!    *                             -ubsm(i-1) + ubsm(i)))! centered
      ) / dar(i)

!       superdiagonal:
   b(i) =     dti*(   (-0.5*f(i)   - g(i)*facfloat(ip))  &
      + ubsm (i)  &              ! upstream
!    *                              + 0.5*(ubsp(i) + ubsm(i)) ! centered
      ) / dar(i)

!       subdiagonal:
   c(i) =     dti*(   (0.5*f(i-1) - g(i-1)*facfloat(im))  &
      - ubsp (i-1)  &               !upstream
!    *                              - 0.5*(ubsp(i-1)+ubsm(i-1))!centered
      ) / dar(i)

!       rhs:
   forc(i)= h0(i) + dti*(  e(i)      - e(i-1)      ) / dar(i)  &
      + dt *( -flux(i)   + flux(i-1)   ) / dar(i)  &
      + dte*( -fluxub(i) + fluxub(i-1) ) / dar(i) + dt * yconv(i)

# 5477


 ENDDO

# 5489


!        Solve tridiagonal system (steps ice thickness from h0 to h1)

 CALL tridia_i (a(n1), b(n1), c(n1), h1(n1), forc(n1), n2-n1+1, 0)

 RETURN
END SUBROUTINE iceflow

!-----------------------------------------------------------------------

SUBROUTINE tridia_i (a, b, c, h, forc, n, ktrid)

!        Solves tridiagonal matrix equation.
!        a,b,c are diagonals of the matrix.
!        a is main diagonal (1:n)
!        b is superdiagonal (1:n-1),
!        c is subdiagonal (2:n),
!        forc is right-hand side,
!        h is vector to be solved for.
!        n is the size of the matrix and vectors.
!        If ktrid is non-zero, uses condition that resulting h(i)
!        must be >=  0.

 IMPLICIT NONE

INTEGER, INTENT(IN) :: n,ktrid
REAL, INTENT(OUT) :: h(n)
REAL, INTENT(IN OUT) :: a(n)
REAL, INTENT(IN) :: b(n), c(n), forc(n)

!     parameter (NMAX = 25000)
INTEGER, PARAMETER  :: NMAX = 50000
REAL :: r(NMAX), s(NMAX), y(NMAX)
REAL :: amin
INTEGER :: i,j

!        Check that local arrays are big enough

IF (n > NMAX) THEN
  WRITE(6,*)'*** Error in tridia_i: Too many points.',  &
      '   NMAX=',NMAX,'   n=',n
  STOP
ENDIF

!     amin = 1.e-3
amin = 1.e-6
DO i = 1,n
  IF (ABS(a(i)) < amin) THEN
    WRITE (6,9000) i,a(i)
9000 FORMAT(' tridia_i warning: a(',i3,')=',f12.8)
    IF (a(i) >= 0.) a(i) = amin
    IF (a(i) < 0.) a(i) = -amin
  ENDIF
ENDDO

s(1) = a(1)
r(1) = b(1)/s(1)
IF (n > 2) THEN
  DO i = 2,n-1
    s(i) = a(i)-c(i)*r(i-1)
    r(i) = b(i)/s(i)
  ENDDO
ENDIF
IF (n >= 2) THEN
  s(n) = a(n)-c(n)*r(n-1)
ENDIF

y(1) = forc(1)/s(1)
IF (n >= 2) THEN
  DO i = 2,n
    y(i) = (forc(i)-c(i)*y(i-1)) / s(i)
  ENDDO
ENDIF

IF (ktrid == 0) THEN
  h(n) = y(n)
  IF (n >= 2) THEN
    DO i = n-1,1,-1
      h(i) = y(i)-r(i)*h(i+1)
    ENDDO
  ENDIF
ELSE
  h(n) = MAX (0., y(n))
  IF (n >= 2) THEN
    DO i = n-1,1,-1
      h(i) = MAX (0., y(i)-r(i)*h(i+1))
    ENDDO
  ENDIF
ENDIF

RETURN
END SUBROUTINE tridia_i

!-----------------------------------------------------------------------

SUBROUTINE fracpin(fpin, zhw, zhbsd)

!     Statement function (fpin) for fraction of u,v grid box with
!     sub-grid bedrock mounds scraping ice bottom:

 IMPLICIT NONE

 REAL, INTENT(OUT) :: fpin
 REAL, INTENT(IN) :: zhw, zhbsd

!-----------------------
# 5601

!----
# 5607

!fpin = 0.7* MAX (0., MIN (1., 1. - zhw/(MAX(2.*zhbsd,1.)) ))
!fpin = 0.5 * MAX (0., MIN (1., 1. - zhw/(MAX(2.*zhbsd,1.)) ))
IF(zhw<=0) THEN
  fpin=1.0
ELSEIF(zhw<0.295*zhbsd) THEN
! 1./(1. + exp(-0.07056*x**3 -1.5976*x))
  fpin = (MAX (0., MIN (1., 1. - zhw/(2.*zhbsd) )))**6 !
ELSEIF(zhw<3.0*zhbsd) THEN
 fpin= MAX(0.,MIN(1.,1.- 1./(1. + EXP(-0.07056*(zhw/zhbsd)**3 -1.5976*(zhw/zhbsd))))) ! Gaussian CDF approx: max error of 0.00014 at |x|=3
ELSE
fpin=0.
ENDIF
!fpin = (MAX (0., MIN (1., 1. - zhw/(MAX(3.*zhbsd,1.)) )))**3 ! starts at zhw<3*zhbsd, = 0.29630 when zhw=zhbsd
! until nov 1/18 fpin = (MAX (0., MIN (1., 1. - zhw/(MAX(2.*zhbsd,1.)) )))**2 ! starts at zhw<2*zhbsd, = 0.25 when zhw=zhbsd


!-----

!-----

RETURN
END SUBROUTINE fracpin

!-----------------------------------------------------------------------
!

SUBROUTINE satable_dyn (s1a, s2a, s3a, s1b, s2b, s3b, arh)

!Computes various vertical integrals needed for vertical
!dependence of internal velocities (horiz:sa, vert:s2a) and,
!vertical shear contribution to internal frictional heating (s3a).
!Uses simple finite difference approx. for dep on T (T constant
!within each layer, between interfaces zetah), but within
!each layer, does integrals w.r.t. zeta exactly.

!All these integrals are on the h-grid horizontally and the
!zeta grid vertically. In therm, s*a are interpolated to the
!u,v-grids for horizontal velocity calcs.
!Extra k=0 and NLEVp indices for s1a are at surface and base.

!Vertical integrals for u, uh, and heati (involving arh) are
!done following Ritz et al.(1997), using:
!s1a (at mid levels, for u,v)
!s2a (at mid levels, for uh,vh and hence w)
!s3a (integral over each layer, for heati)

!Similarly for shelf-softening and effect on driving stress.
!Nb: treatment of shelf softening only valid for POWi=3!

 USE modconstants
 USE modPHOMgrid
 USE modgridm, ONLY : ni, nj
 USE modicerheology, ONLY : POWi


 IMPLICIT NONE

 REAL, INTENT(IN)                         :: arh(NX,NY,NLEV)
 REAL, INTENT(OUT)                        :: s1a(NX,NY,0:NLEVp)
 REAL, INTENT(OUT)                        :: s2a(NX,NY,0:NLEVp)
 REAL, INTENT(OUT)                        :: s3a(NX,NY,NLEV)
 REAL, INTENT(OUT)                        :: s1b(NX,NY,0:NLEVp)
 REAL, INTENT(OUT)                        :: s2b(NX,NY,0:NLEVp)
 REAL, INTENT(OUT)                        :: s3b(NX,NY,NLEV)

 REAL, SAVE :: dzpow1a(NLEV,2), dzpow2a(NLEV,2), dzpow3a(NLEV), &
    dzpow1b(NLEV,2), dzpow2b(NLEV,2), dzpow3b(NLEV)
 INTEGER :: i,j,k, istart=1

!if first call, compute vertical integral zeta deps (saved)
 IF (istart==1) THEN
   DO k=1,NLEV
!---
!for s1a (u,v):
     dzpow1a(k,1) = ( zeta(k)**(POWi+1) - zetah(k-1)**(POWi+1)) / (POWi+1.)
     dzpow1a(k,2) = (zetah(k)**(POWi+1) - zeta(k)**(POWi+1)   ) / (POWi+1.)

!for s2a (uh,vh):
     dzpow2a(k,1) = ((zeta(k)**(POWi+1))*(zeta(k)-zetah(k-1))  &
        - (zeta(k)**(POWi+2)-zetah(k-1)**(POWi+2)) / (POWi+2.)  &
        ) / (POWi+1.)
     dzpow2a(k,2) = (- (zeta(k)**(POWi+1))*(zetah(k)-zeta(k)) &
        + (zetah(k)**(POWi+2)-zeta(k)**(POWi+2)) / (POWi+2.) &
        ) / (POWi+1.)

!for s3a (heati):
     dzpow3a(k)   = (zetah(k)**(POWi+2) - zetah(k-1)**(POWi+2)) / (POWi+2.)
!---

!for s1b (u,v):
     dzpow1b(k,1) = (zeta(k)**2 - zetah(k-1)**2) / 2.
     dzpow1b(k,2) = (zetah(k)**2 - zeta(k)**2) / 2.

!for s2b (uh,vh):
     dzpow2b(k,1) = (  (zeta(k)**2)*(zeta(k)-zetah(k-1)) &
        - (zeta(k)**3 - zetah(k-1)**3) / 3. &
        ) / 2.
     dzpow2b(k,2) = (- (zeta(k)**2)*(zetah(k)-zeta(k)) &
        + (zetah(k)**3 - zeta(k)**3) / 3. &
        ) / 2.

!for s3b (heati):
     dzpow3b(k)   = (zetah(k)**3 - zetah(k-1)**3) / 3.
!---
   ENDDO
   istart=0
 ENDIF


!Do vertical integrations for s1a,s2a,s3a
 DO j=1,nj
   DO i=1,ni
!at layer mid points, plus top and bottom (for s1a,s2a):
     s1a(i,j,NLEVp) = 0.
     s2a(i,j,NLEVp) = 0.
     s1a(i,j,NLEV)  = arh(i,j,NLEV)*dzpow1a(NLEV,2)
     s2a(i,j,NLEV)  = arh(i,j,NLEV)*dzpow2a(NLEV,2)
     s3a(i,j,NLEV)  = arh(i,j,NLEV)*dzpow3a(NLEV)/dzeta(NLEV)
     DO k=NLEV-1,1,-1
       s1a(i,j,k)= s1a(i,j,k+1) + arh(i,j,k+1)* dzpow1a(k+1,1) &
          + arh(i,j,k)  * dzpow1a(k,  2)

       s2a(i,j,k)= s2a(i,j,k+1) + s1a(i,j,k+1)*(zeta(k+1)-zetah(k)) &
          + arh(i,j,k+1)* dzpow2a(k+1,1) + s1a(i,j,k  )*(zetah(k)-zeta(k)) &
          - arh(i,j,k  )* dzpow2a(k,  2)

       s3a(i,j,k)   =             arh(i,j,k)*dzpow3a(k)/dzeta(k)
     ENDDO
     s1a(i,j,0) = s1a(i,j,1) + arh(i,j,1)*dzpow1a(1,1)
     s2a(i,j,0) = s2a(i,j,1) + s1a(i,j,1)*(zeta(1)-zetah(0)) &
        + arh(i,j,1)* dzpow2a(1,1)

!at layer mid points, plus top and bottom (for s1b,s2b):
     s1b(i,j,NLEVp) = 0.
     s2b(i,j,NLEVp) = 0.
     s1b(i,j,NLEV)  = arh(i,j,NLEV)*dzpow1b(NLEV,2)
     s2b(i,j,NLEV)  = arh(i,j,NLEV)*dzpow2b(NLEV,2)
     s3b(i,j,NLEV)  = arh(i,j,NLEV)*dzpow3b(NLEV)/dzeta(NLEV)
     DO k=NLEV-1,1,-1
       s1b(i,j,k)= s1b(i,j,k+1) + arh(i,j,k+1)* dzpow1b(k+1,1) &
          + arh(i,j,k)  * dzpow1b(k,  2)

       s2b(i,j,k)= s2b(i,j,k+1) + s1b(i,j,k+1)*(zeta(k+1)-zetah(k)) &
          + arh(i,j,k+1)* dzpow2b(k+1,1) + s1b(i,j,k  )*(zetah(k)-zeta(k)) &
          - arh(i,j,k  )* dzpow2b(k,  2)

       s3b(i,j,k)   =             arh(i,j,k)*dzpow3b(k)/dzeta(k)
     ENDDO
     s1b(i,j,0) = s1b(i,j,1) + arh(i,j,1)*dzpow1b(1,1)
     s2b(i,j,0) = s2b(i,j,1) + s1b(i,j,1)*(zeta(1)-zetah(0)) &
        + arh(i,j,1)* dzpow2b(1,1)

   ENDDO
 ENDDO

 RETURN
END SUBROUTINE satable_dyn

!-----------------------------------------------------------------------

SUBROUTINE findwater (hw, maskwater, indlake, npoilake, nlake,  &
    h, hb, sealev, Icheck)

!        Locates oceans from scratch, resets maskwater and hw
!        for ocean and adjacent land. Resets lakes
!        (maskwater,indlake,npoilake,nlake)
!        depending on existing hw.

!     maskwater = 0=grounded ice or ice-free land, (based on gridcell average)
!                 1=ocean,
!                 +/- 2,3,4,...=lake number (+ open, - subice) (ret)
!                 Here, lakes all +ve, set to + or - in adjustpres
!     indlake   = (packed) i,j of each lake's points (returned)
!     npoilake  = number of points in each lake      (returned)
!     nlake     = # of lakes (including ocean)        (returned)
!     h         = ice thickness                       (supplied)
!     hb        = bed+sed elevation                   (supplied)
!     hw        = water thickness           (modified for ocean)
!     sealev    = sea level                           (supplied)

 
 USE modconstants
 USE modPHOMgrid
 USE modgridm, ONLY : ni, nj
 USE modPHOMvarhydrol, ONLY : NPOIMAX,NlakeMAX
 USE modIU, ONLY : IOTERM

 IMPLICIT NONE

 REAL, INTENT(OUT)                        :: hw(NX,NY)
 INTEGER, INTENT(OUT)                     :: maskwater(NX,NY)
 INTEGER, INTENT(OUT)                     :: indlake(NPOIMAX,NlakeMAX)
 INTEGER, INTENT(OUT)                     :: npoilake(NlakeMAX)
 INTEGER, INTENT(OUT)                     :: nlake
 REAL, INTENT(IN)                         :: h(0:NXp,0:NYp)
 REAL, INTENT(IN)                         :: hb(0:NXp,0:NYp)
 REAL, INTENT(IN)                         :: sealev(0:NXp,0:NYp)
 INTEGER, INTENT(IN)      :: Icheck

!     local:
 INTEGER ::  list(NX*NY),   listprev(NX*NY)
 INTEGER, PARAMETER :: NLOOK1 = 8

 INTEGER :: i, ia, ifopen, ii, ilake, iskip, j, ja, jj
 INTEGER :: look, n, nlist, nlistprev, iter, npoi

! BSLnew
 INTEGER, PARAMETER :: ISH1(NLOOK1) = (/-1, 1,  0, 0, -1,  1, -1, 1/)
 INTEGER, PARAMETER :: JSH1(NLOOK1) = (/0, 0, -1, 1, -1, -1,  1, 1/)
! old
!DIMENSION ISH(8), JSH(8)
!INTEGER, PARAMETER :: NLOOK = 8
!SAVE ISH, JSH
!     First 4 are E-W/N-S, last 4 are diagonals:
!DATA ISH /-1, 1,  0, 0, -1,  1, -1, 1/
!DATA JSH / 0, 0, -1, 1, -1, -1,  1, 1/
! ^^^


IF(Icheck /= 111) CALL outerrI("interface error on findwater call, Icheck:",Icheck)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# 5893

!~~~~~

 maskwater=0

!        Perform search for oceans (submerged points that are in
!        contact with any submerged points at domain edges),
!        setting maskwater to 1 and setting hw for ocean pts,
!        and also setting hw to 0 for non-floating ocean-adjacent pts.

!----------------------
 DO iter = 1,ni*nj
!----------------------

   nlist = 0

   IF (iter == 1) THEN

     DO j=1,nj  ! scan non-ghost grid edge
       iskip = ni-1
       IF (j == 1.OR.j == nj) iskip = 1
       DO i=1,ni,iskip
         IF (sealev(i,j)-hb(i,j)-RHOIovS*h(i,j) > HWcut) THEN
           maskwater(i,j) = 1
           nlist = nlist + 1
           list(nlist) = i + (j-1)*ni
           hw(i,j) = sealev(i,j) - hb(i,j) - RHOIovS*h(i,j)
         ELSE
           hw(i,j) = 0.
         ENDIF
       ENDDO
     ENDDO

   ELSE

!         normal iteration, look at adjacent points for new ocean pts:
     DO n=1,nlistprev
       j = (listprev(n)-1) / ni + 1
       i =  listprev(n) - (j-1)*ni
       DO look=1,NLOOK1
         ii = MAX (1, MIN (ni, i + ISH1(look)))
         jj = MAX (1, MIN (nj, j + JSH1(look)))
!         IF(ii>ni .OR. ii<1 .OR. jj>nj .OR. jj<1) THEN
!           PRINT *,"iijjbnd",ii,jj
!           stop
!         ENDIF
         IF (maskwater(ii,jj) == 0) THEN
           IF (sealev(ii,jj)-hb(ii,jj)-RHOIovS*h(ii,jj) > HWcut) THEN
             maskwater(ii,jj) = 1
             nlist = nlist + 1
             list(nlist) = ii + (jj-1)*ni
             hw(ii,jj) = sealev(ii,jj) - hb(ii,jj) - RHOIovS*h(ii,jj)
           ELSE
             hw(ii,jj) = 0.
           ENDIF
         ENDIF
       ENDDO
     ENDDO

   ENDIF
!t  PRINT *,"TESTF", SUM(hw)

!          Skip out if no new ocean points found
   IF (nlist == 0) EXIT

!          Copy current list to previous list
   nlistprev = nlist
   DO n=1,nlist
     listprev(n) = list(n)
   ENDDO
!----------
 ENDDO
!----------


!        Search for inland lakes, based on water thickness hw vs HWcut.
!        When find a new, not-already-found lake point (outer loop),
!        increment nlake, then iteratively search for adjacent lake
!        points, forming list indlake and setting maskwater,
!        and set npoilake when done.

 nlake = 1 ! ocean

 indlake=0
 npoilake=0


!     don't allow any non-ocean water (open lakes or sub-ice)
 DO j=1,nj
   DO i=1,ni
     IF (maskwater(i,j) /= 1) THEN
!            convert liquid water beneath ice to ice (or not..why?)
!            if (h(i,j).gt.HWcut) h(i,j) = h(i,j) + hw(i,j)/RHOIovS
       hw(i,j) = 0.
     ENDIF
   ENDDO
 ENDDO
!t  PRINT *,"TESTF2", SUM(hw)
 RETURN


!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 DO ja=1,nj
   DO ia=1,ni
     IF (maskwater(ia,ja) == 0 .AND. hw(ia,ja) > HWcut) THEN
!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

       nlake = nlake + 1
       IF (nlake > NlakeMAX) THEN
         WRITE (IOTERM, *)  &
            'Error (findwater): max # lakes exceeded, nlake=',nlake
         STOP
       ENDIF

!----------------------------
       DO iter = 1,ni*nj
!----------------------------

         nlist = 0

         IF (iter == 1) THEN

           nlist = 1
           list(1) = ia + (ja-1)*ni
           maskwater(ia,ja) = nlake
           npoi = 1
           indlake(npoi,nlake) = list(1)

         ELSE

!               normal iteration, look at adjacent pts for new lake pts
           DO n=1,nlistprev
             j = (listprev(n)-1) / ni + 1
             i =  listprev(n) - (j-1)*ni
             DO look=1,NLOOK1
               ii = MAX (1, MIN (ni, i + ISH1(look)))
               jj = MAX (1, MIN (nj, j + JSH1(look)))
               IF (maskwater(ii,jj) == 0 .AND. hw(ii,jj) > HWcut) THEN
                 maskwater(ii,jj) = nlake
                 nlist = nlist + 1
                 list(nlist) = ii + (jj-1)*ni
                 npoi = npoi + 1
                 IF (npoi > NPOIMAX) THEN
                   WRITE (IOTERM,'(/a/a,i6,a,i6)')  &
                      'Error (findwater): max lake size exceeded:',  &
                      '   npoi=',npoi,'  nlake=',nlake
                   STOP
                 ENDIF
                 indlake(npoi,nlake) = list(nlist)
               ELSE IF (maskwater(ii,jj) /= nlake) THEN
                 WRITE (IOTERM,'(/a/a,i6/a,i6/a,2i6/a,2i6)')  &
                    'Error (findwater): lake iteration error:',  &
                    '    maskwater=',maskwater(ii,jj), '    nlake    =',nlake,  &
                    '    ii,jj    =',ii,jj, '    ia,ja    =',ia,ja
                 STOP
               ENDIF
             ENDDO
           ENDDO

         ENDIF

!                Skip out if no new open-lake points found
         IF (nlist == 0) EXIT

!                Copy current list to previous list
         nlistprev = nlist
         DO n=1,nlist
           listprev(n) = list(n)
         ENDDO

!----------------
       ENDDO
!----------------
       npoilake(nlake) = npoi
!>>>>>>>>>>>>>>
     ENDIF
   ENDDO
 ENDDO
!>>>>>>>>>>>>>>

!        Finally, set all maskwater's within each lake to negative if
!        lake is closed (else maskwater remains positive, open lake)
 DO ilake = 2,nlake

   ifopen = 0
   DO n=1,npoilake(ilake)
     j = (indlake(n,ilake)-1) / ni + 1
     i =  indlake(n,ilake) - (j-1)*ni
     IF (h(i,j) == 0.) ifopen = 1
   ENDDO

   IF (ifopen == 0) THEN
     DO n=1,npoilake(ilake)
       j = (indlake(n,ilake)-1) / ni + 1
       i =  indlake(n,ilake) - (j-1)*ni
       maskwater(i,j) = -maskwater(i,j)
     ENDDO
   ENDIF

 ENDDO

 RETURN
END SUBROUTINE findwater

!-----------------------------------------------------------------------

SUBROUTINE adjustpres (maskwater, indlake, npoilake, nlake,  &
    h, hb, hw, hs, sealev, Icheck)  !hw INOUT, hs OUT, rest IN

!     Adjusts water thicknesses hw within each separate lake or ocean
!     to equalize hydrostatic water pressure on a commom reference
!     level. Then resets surface elevation hs.
!     Ignore consequent changes to maskwater, indlake, etc (i.e., if
!     change in hw would make ice ground or float, or if hw would
!     overflow sill of an open lake...wait till next call to findwater
!     and/or movewater).

!     maskwater = 0,1,+/-2... (grounded ice/ice-free land, ocean, lake#)
!     indlake   = (packed) i,j of each lake's points
!     npoilake  = number of points in each lake
!     nlake     = # of lakes (including ocean)
!     h         = ice thickness
!     hb        = bed+sed elevation
!     hw        = water thickness (modified)
!     hs        = surface elevation (modified)
!     sealev    = sea level

 USE modconstants
 USE modPHOMgrid
 USE modgridm, ONLY : ni, nj, darea, dxg, dyg
 USE modPHOMvarhydrol, ONLY : NPOIMAX,NlakeMAX

 IMPLICIT NONE

INTEGER, INTENT(IN)                      :: maskwater(NX,NY)
INTEGER, INTENT(IN)                      :: indlake(NPOIMAX,NlakeMAX)
INTEGER, INTENT(IN)                      :: npoilake(NlakeMAX)
INTEGER, INTENT(IN)                      :: nlake
REAL, INTENT(IN)                         :: h(0:NXp,0:NYp)
REAL, INTENT(IN)                         :: hb(0:NXp,0:NYp)
REAL, INTENT(IN OUT)                     :: hw(NX,NY)
REAL, INTENT(OUT)                        :: hs(0:NXp,0:NYp)
REAL, INTENT(IN)                         :: sealev(0:NXp,0:NYp)
INTEGER, INTENT(IN)      :: Icheck

REAL :: pres(NPOIMAX)
REAL :: tota, totp
INTEGER :: i, j, ilake, n

IF(Icheck /= 111) CALL outerrI("interface error on adjustpres call, Icheck:",Icheck)
!        Set hw and hs for land (no lake) and ocean
DO j=1,nj
  DO i=1,ni
    IF (maskwater(i,j) == 0) THEN
      hs(i,j) = hb(i,j) + hw(i,j) + h(i,j)
    ELSE IF (maskwater(i,j) == 1) THEN
      hw(i,j) = MAX (0., sealev(i,j) - hb(i,j) - RHOIovS*h(i,j))
      hs(i,j) = hb(i,j) + hw(i,j) + h(i,j)
    ENDIF
  ENDDO
ENDDO

!        Loop for each lake (open or sub-ice, but not ocean)
!-----------------------
DO ilake = 2,nlake
!-----------------------

!          Calculate pres = pressure/(RHOWSc*gEARTH) at "zero" elevation
!          for each lake point, and its areal average totp
  totp = 0.
  tota = 0.
  DO n=1,npoilake(ilake)
    j = (indlake(n,ilake)-1) / ni + 1
    i =  indlake(n,ilake) - (j-1)*ni
    pres(n) = RHOIovS*h(i,j) + (hb(i,j)+hw(i,j))
    totp = totp + pres(n)*darea(i,j)
    tota = tota +         darea(i,j)
  ENDDO
  IF(tota==0.)THEN
    totp = totp/1.e-40
  ELSE
    totp = totp/tota
  ENDIF
!          Subtract pres-totp from each point's water thickness,
!          to equalize new pres's for all points. This conserves
!          total water in lake, unless...if new hw would be negative
!          (newly grounded ice), just set hw to zero, ignore water
!          non-conservation, and wait till next call to findwater
!          to correct lake indices.
!          This workd both for closed lakes (ice rides above
!          hw, and open lakes (ice floats in hw).
  DO n=1,npoilake(ilake)
    j = (indlake(n,ilake)-1) / ni + 1
    i =  indlake(n,ilake) - (j-1)*ni
    hw(i,j) = MAX (0., hw(i,j) - (pres(n)-totp))
    hs(i,j) = hb(i,j) + hw(i,j) + h(i,j)
  ENDDO

!----------
ENDDO
!----------

RETURN
END SUBROUTINE adjustpres

!-----------------------------------------------------------------------

SUBROUTINE next2o(ifnexo, maskwater, h)

!        Set ifnexo=1 for ocean shelf points (h>0) adjacent to
!        open ocn pts, else = 0 (for setfhedge, and returned for
!        cliffmelt in icedyn

USE modPHOMgrid
USE modgridm, ONLY : ni, nj

IMPLICIT NONE

INTEGER, INTENT(OUT)                     :: ifnexo(NX,NY)
INTEGER, INTENT(IN)                      :: maskwater(NX,NY)
REAL, INTENT(IN)                         :: h(0:NXp,0:NYp)

INTEGER :: i,j,look,ii,jj
INTEGER, PARAMETER :: NLOOK = 4
INTEGER :: ish(NLOOK)=(/-1, 1,  0, 0 /), jsh(NLOOK)=(/ 0, 0, -1, 1 /)
!     E-W/N-S:
!DATA ish /-1, 1,  0, 0 /
!DATA jsh / 0, 0, -1, 1 /

ifnexo=0
DO j=1,nj
  DO i=1,ni
    IF (maskwater(i,j) == 1 .AND. h(i,j) == 0.) THEN
      ifnexo(i,j) = 1
    ELSE IF (maskwater(i,j) == 1 .AND. h(i,j) > 0.) THEN
      DO look=1,NLOOK
         ii = MAX (1, MIN (ni, i + ish(look)))
         jj = MAX (1, MIN (nj, j + jsh(look)))
         IF (maskwater(ii,jj) == 1 .AND. h(ii,jj) == 0.) ifnexo(i,j) = 1
      END DO
    END IF
  END DO
END DO

RETURN
END SUBROUTINE next2o

!-----------------------------------------------------------------------

SUBROUTINE setfhedge(h, hb, maskwater, ifnexo, fedge, hedge,  &
                     fracgu, fracgv, sealev, Icheck) !in

!        Set sub-grid ice fraction (fedge), and expected thickness of
!        shelf ice (hedge) , at shelf edges next to open ocean.
!        hedge = max of ice thicknesses in surrounding cells (zh,
!        allowing for "typical" downstream thinning).
!        Similar to sub-grid setting (1) for grounded ice in
!        climate_all, but only to reduce oceanmelt by fedge in icectl
!        (no sfc elev interps). Don't change fedge for grounded ice
!        (-DSUBGRIDEDGE  uses same array).

!        Also sets hedge>0 for ocean h=0 pts next to h>0,
!        to set calving in case small ice after icedyn.

USE modconstants, ONLY : RHOIovS
USE modPHOMgrid
USE modgridm, ONLY : ni, nj

IMPLICIT NONE

REAL, INTENT(IN OUT)                     :: h(0:NXp,0:NYp)
REAL, INTENT(IN)                         :: hb(0:NXp,0:NYp)
INTEGER, INTENT(IN)                      :: maskwater(NX,NY)
INTEGER, INTENT(IN)                      :: ifnexo(NX,NY)
REAL, INTENT(OUT)                        :: fedge(NX,NY)
REAL, INTENT(OUT)                        :: hedge(NX,NY)
REAL, INTENT(IN)                         :: fracgu(0:NX,0:NY)
REAL, INTENT(IN)                         :: fracgv(0:NX,0:NY)
REAL, INTENT(IN)                         :: sealev(0:NXp,0:NYp)
INTEGER, INTENT(IN)      :: Icheck

INTEGER, PARAMETER :: NLOOK = 4
INTEGER :: ish(NLOOK)=(/-1, 1,  0, 0 /), jsh(NLOOK)=(/ 0, 0, -1, 1 /)
!     E-W/N-S:

REAL :: zhav,zfgl,hbg,zh,zh2,zh3,zwei
INTEGER :: i,j,nav,look,ii,jj

IF(Icheck /= 111) CALL outerrI("interface error on setfhedge call, Icheck:",Icheck)

! BSL
DO j=1,nj
  DO i=1,ni
    IF (maskwater(i,j) == 0) fedge(i,j) = 1.
!     for now, hedge only different from h for shelf points (ocean)
    IF (maskwater(i,j)/=1) hedge(i,j) = h(i,j)
  enddo
enddo
! ^^^

!----

!----

DO j=1,nj
  DO i=1,ni
    IF (maskwater(i,j) == 1) THEN

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      IF (h(i,j) > 0 .AND. ifnexo(i,j) == 0) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        hedge(i,j) = h(i,j)
        fedge(i,j) = 1.
!~~~~~~~~~~~~~~~
      ELSE    ! do also for ocn h=0, to set hedge for calving,
! in case small ice after icedyn
!~~~~~~~~~~~~~~~
        zhav = 0.
        nav = 0
!............................
        DO look=1,NLOOK
!............................
          ii = MAX (1, MIN (ni, i + ish(look)))
          jj = MAX (1, MIN (nj, j + jsh(look)))
          IF (.NOT.(ii == i .AND. jj == j)) THEN    ! for flowline
            zh = 0.
            IF (maskwater(ii,jj) == 0 .AND. h(ii,jj) > 0.) THEN
              IF (j == jj) THEN
                zfgl = fracgu (MIN(i,ii), j)
              ELSE
                zfgl = fracgv (i, MIN(j,jj))
              END IF
              hbg = hb(ii,jj)*(1.-zfgl) + hb(i,j)*zfgl
              zh = (sealev(ii,jj)-hbg)/RHOIovS
            ELSE IF (maskwater(ii,jj) == 1 .AND. h(ii,jj) > 0.  &
                  .AND. ifnexo(ii,jj) == 0) THEN
              zh = h(ii,jj)
            END IF

            IF (zh > 0.) THEN
!unif               zh3 = zh  ! unif small thickness, 3 plcs
!                   "downstream thinning, typical shelf profiles"...:
              zh2 = zh * MAX (0.30, EXP(-dd0/100.e3))
!                   for small coverage, weight towards zh (ie SG margin close to adjacent thick ice), not zh2:
              zwei =  MIN (1., h(i,j)/zh2)
              zh3 = zh*(1.-zwei) + zh2*zwei
              zhav = MAX(zhav,zh3)          ! max, not average
!                   zhav = zhav + zh3             ! max, not average
              nav = nav + 1
            END IF
          END IF
!..................
        END DO
!..................
        IF (nav > 0) THEN
!               zhav = zhav/nav                   ! max, not average
!unif           zhav = min (zhav, 200.) ! unif small thickness, 3 plcs
          hedge(i,j) = MAX (zhav, h(i,j))
          fedge(i,j) = h(i,j)/hedge(i,j)       ! so fedge=0 if h=0, but hedge can still be > 0
        ELSE IF (h(i,j) > 0.) THEN ! should have downstream thinning here as well
          hedge(i,j) = h(i,j)
          fedge(i,j) = 1.
        ELSE IF (h(i,j) == 0.) THEN
          hedge(i,j) = 0.
          fedge(i,j) = 0.
        END IF
!~~~~~~~~~~~~~~~~
      END IF
!~~~~~~~~~~~~~~~~

!           impose minimum limit (1 m) on sub-grid ice thickness: 7/5/13
      IF (hedge(i,j) < 1.) THEN
        h(i,j) = 0.
        fedge(i,j) = 0.
        hedge(i,j) = 0.
      END IF

    END IF
  END DO
END DO

!----
# 6386

!----

RETURN
END SUBROUTINE setfhedge

!-----------------------------------------------------------------------

SUBROUTINE fracg(fracgu,fracgv,hgu,hgv,h,hb,hw,sealev)

! solve for:
! fracgu fracgv: fraction of a u/v-grid box that is grounded vs floating
! hgu hgv: height of the subgrid g.l. diagnostic

USE modconstants, ONLY : RHOIovS, HWcut
USE modPHOMgrid
USE modgridm, ONLY : ni, nj
USE modPHOMvardyn

IMPLICIT NONE

REAL, INTENT(IN)                         :: h(0:NXp,0:NYp)
REAL, INTENT(IN)                         :: hb(0:NXp,0:NYp)
REAL, INTENT(IN)                         :: hw(NX,NY)
REAL, INTENT(IN)                         :: sealev(0:NXp,0:NYp)

REAL, INTENT(OUT)                        :: fracgu(0:NX,0:NY)
REAL, INTENT(OUT)                        :: fracgv(0:NX,0:NY)
REAL, INTENT(OUT)                        :: hgu(0:NX,0:NY)
REAL, INTENT(OUT)                        :: hgv(0:NX,0:NY)

REAL ::zpot,zpota,zaf,zbf,zfgl,zhbg,zhg
INTEGER :: iloopuv,i,ish,j,jsh,nitmp,njtmp,ia,ja

fracgu=1.
fracgv=1.

!>>>>>>>>>>>>>>>>>>>>>
DO iloopuv = 1,2
!>>>>>>>>>>>>>>>>>>>>>

  IF (iloopuv == 1) THEN
    ish = 1
    jsh = 0
    nitmp = ni-1
    njtmp = nj
  ELSE
    ish = 0
    jsh = 1
    nitmp = ni
    njtmp = nj-1
  END IF

  DO j=1,njtmp
    DO i=1,nitmp
      ia = i + ish
      ja = j + jsh

!            If at grounding line, calculate areal fraction of this
!            u or v-grid cell that's grounded (fracg[u,v]). i.e.,
!            estimated sub-grid position of grounding line. Calculate
!            height-above-flotation for the two surrounding h-grid
!            points (one floating with h.a.f. < 0, one grounded with
!            h.a.f. > 0), and linearly interpolated zero-location
!            is the estimated g.l position. [This is equivalent to
!            linearly interpolating ice surface, ice base, and bed
!            elevations between the 2 h-points, and seeing where
!            flotation occurs]. fracg[u,v] is used in icedyn,
!            multiplying the basal coefficient coefbs[u,v].

!            Also calc g.l. ice thickness hg[u,v] (using fracg[u,v]).
!            Currently fracg[u,v] (and hg[u,v]) are diagnostic only
!            if schoofgl not defined. For schoofgl, fracg[u,v] are
!            used in calcgl to calculate hg[u,v] for schoof g.l. flux
!            (repeating hg[u,v] setting here, but within C loop).

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      IF (     (hw(i,j) > HWcut .AND. hw(ia,ja) <= HWcut)  &
            .OR. (hw(i,j) <= HWcut .AND. hw(ia,ja) > HWcut) ) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        zpot  = hb(i,j)   + hw(i,j)   + RHOIovS*h(i,j)
        zpota = hb(ia,ja) + hw(ia,ja) + RHOIovS*h(ia,ja)
        IF (hw(i,j) > HWcut) THEN
          zaf = zpota - zpot     ! ht above flot (liq m), grounded
          zbf = -hw(i,j)         ! ht above flot (liq m), floating
        ELSE  !IF (hw(ia,ja) > HWcut) THEN
          zaf = zpot - zpota
          zbf = -hw(ia,ja)
        END IF
        zfgl = zaf / MAX (zaf-zbf, 0.1)    ! for safety...should
        zfgl = MAX (0., MIN (1., zfgl))  ! have zaf>0, zbf<0
!           calc g.l. depth (lin.interp bed elev to u,v grid, vs. s.l.,
!           so only works for marine g.l.s, not inland lakes):
!           zhbg =       0.5*hb(i,j) +  0.5*hb(ia,ja)
        IF (hw(i,j) > HWcut) THEN
          zhbg = (1.-zfgl)*hb(ia,ja) + zfgl*hb(i,j)
        ELSE
          zhbg = (1.-zfgl)*hb(i,j)   + zfgl*hb(ia,ja)
        END IF
        zhg  = (sealev(i,j)-zhbg)/RHOIovS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ELSE IF (hw(i,j) > HWcut .AND. hw(ia,ja) > HWcut) THEN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        zfgl = 0.
        zhg  = 0.
!~~~~~~~~~~~~~
      ELSE
!~~~~~~~~~~~~~
        zfgl = 1.
        zhg  = 0.
!~~~~~~~~~~~~~~
      END IF
!~~~~~~~~~~~~~~

      IF (iloopuv == 1) THEN
        fracgu(i,j) = zfgl
        hgu(i,j)    = zhg
      ELSE
        fracgv(i,j) = zfgl
        hgv(i,j)    = zhg
      END IF

    ENDDO  ! i=1,nxtmp
  ENDDO    ! j=1,nytmp

!>>>>>>>>>>>>>>>>>>>>>
ENDDO
!>>>>>>>>>>>>>>>>>>>>>

RETURN
END SUBROUTINE fracg
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE  setArh(H,Tice)
! H dependence only via H dependence of Tpm
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm, ONLY : ni, nj, nip, njp
 USE moddim
 USE modPHOMgrid
 USE modicetherm, ONLY : Tmax
 USE modPHOMvartherm
 USE modconstants, ONLY : Betap
 USE modicerheology
 USE modinparm, ONLY : fnflow
 USE modclimforce, ONLY : glacIndex
 USE modPHOMvarhydrol, ONLY : maskwater

 IMPLICIT NONE

 REAL, INTENT(IN) :: H(0:MNAp,0:MNBp), Tice(0:MNAp,0:MNBp,0:NLEVp)

 REAL :: arhtmp, Arh0, fEnhSSA, fEnhShelf,fEnhStream
 INTEGER :: i, j, kz
 INTEGER, PARAMETER :: NLEVo2=NLEV/2, NLEVm1=NLEV-1, NLEVm2=NLEV-2
 INTEGER :: istart=1

!#if defined (HEINO) || defined (EISMINT00)
! fnflow  = 0.5*3.
! fnshelf = 0.5*3.
! REAL, PARAMETER :: FnSHELF = 0.55, FnSTREAM=0.55
!#else
! REAL, PARAMETER :: FnSHELF = 0.6, FnSTREAM=1.0 ! text and figs 3,4,6 maGRG.SIASSAenhancementFactorsFromAnisotropicISM.jglac10.pdf for shelf/stream
!#endif

!fEnhShelf=FnSHELF*fnflow/3.5
 fEnhShelf=0.48696/fnflow + 0.51304 ! from cathRitz discussion using using maGRG10 thermally coupled target: 0.6,5.6
! fEnhStream=FnSTREAM*fnflow/3.5
 fEnhStream= 0.5*(fEnhShelf+1.0)

IF(istart==1) THEN
  PRINT *,"fEnhShelf,fEnhStream: ",fEnhShelf,fEnhStream
!  PRINT *,"2*,1.5*,0.5* : arh: NLEV,NLEVm, 1:NLEVo2"

  PRINT *,"1.5*,0.5* : arh: NLEV, 1:NLEVo2"

!  PRINT *,"0.5* : arh:  1:NLEVo2"
!  PRINT *,"NO  ice hardening top half"
!  PRINT *,"NO basal flow enhancements"
  istart=0
ENDIF
 Arh0=Arrhen(nAexpmx)
 IF(ifArhReset==1) THEN
   arh(1:ni,1:nj,:)=Arh0
!   arhap(1:ni,1:nj)=(FnSHELF*Arh0)**POWIR
   arhap(1:ni,1:nj)=(fEnhShelf*Arh0)**POWIR
   ifArhReset=0
 ENDIF

 DO j=1,nj
   DO i=1,ni
     IF(H(i,j)>0.) THEN
       IF(maskwater(i,j)==1) THEN
         fEnhSSA=fEnhShelf
       ELSE
         fEnhSSA=fEnhStream
       ENDIF
       IF( Tice(i,j,1) > (Tmax+26.) ) THEN
         arh(i,j,:)=fnflow*Arh0
         arhap(i,j)=(fEnhSSA*Arh0)**POWIR
       ELSE
         arhap(i,j)=0.
         DO kz=1,NLEV
! if later merging vertical grids, remember grids are shifted: phom 1:NLEV = GSM 2:NTZ
           arhtmp=Arrhen( NINT( ( Tice(i,j,kz)  & 
              + Betap*H(i,j)*zeta(kz) -aeTmin )/dtA) ) 
           arh(i,j,kz)=fnflow*arhtmp
           arhap(i,j) = arhap(i,j)+ ((fEnhSSA*arhtmp)**POWIR)*dzeta(kz)
         ENDDO

         arh(i,j,NLEV)=1.5*arh(i,j,NLEV) ! bottom 1%

# 6601

!until sep 21/19         arh(i,j,NLEVm1)=1.5*arh(i,j,NLEVm1) ! next 2%

!         arh(i,j,NLEVm1:NLEV)=2.*arh(i,j,NLEVm1:NLEV)
!         arh(i,j,NLEVm2)=1.5*arh(i,j,NLEVm2)
       ENDIF
!       IF(glacIndex<0.5) arh(i,j,1:NLEVo2)=0.5*arh(i,j,1:NLEVo2) !interglacial ice hardening top half

       arh(i,j,1:NLEVo2)=0.5*arh(i,j,1:NLEVo2) !interglacial ice hardening top half

     ENDIF
   ENDDO
 ENDDO
!t PRINT *,"arh",arhtmp,arh(31,31,1),arhap(31,31)

RETURN
END SUBROUTINE setArh
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE phomCompUV(heati, u, v, &  !out
                      H, hs, budgall, baseMB, ub, vb, dt, ifcompw, Icheck) !in
!SUBROUTINE phomCompUV(heati, u, v, &  !out
!                      H, hs, ub, vb) !in
! NOTE: w, wa, budgall, baseMB, dt, ifcompw will be dumped after
! testing in favour of internal pGSMicetherm.F90/compwk
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! right now only computes u(1:ni-1,:,:)

!        ub,u,uh are on u-grid (staggered 1/2 box to right).
!        vb,v,vh are on v-grid (staggered 1/2 box to top).
!        (ie, an Arakawa "C" grid).

!        t,u,v,w(=dzeta/dt),wa(=dz/dt),uh,vh are on mid-levels
!        (0:nlevp). Extra levels k=0 (sfc) and k=nlevp (base) are
!        set for completeness, and are not prognostic.

!        Vertical integrals for u, uh, and heati (involving arh) are
!        done following Ritz et al.(1997), using:
!          s1a (at mid levels, for u,v)
!          s2a (at mid levels, for uh,vh and hence w)
!          s3a (integral over each layer, for heati).
!        and same for shelf softening in s1b, s2b, s3b.
!        Nb: treatment of shelf softening only valid for powi=3!
!        Top-level s2a0 and s2b0 are set for icedyn to compute
!        sheet-flow dynamics, and similarly s1a0,s1b0.

!        Calcs for u, uh use dbu, dbv (basal stress = driving stress
!        modified by shelf eqns left-hand sides, passed from icedyn.

USE modconstants, ONLY : HWcut
USE modPHOMgrid
USE modPHOMvardyn, ONLY : siu, siv, dbu, dbv, sih
USE modPHOMvartherm, ONLY : arh
USE modPHOMvarhydrol, ONLY : hw
USE modgridm, ONLY : ni, nj, darea, dxu, dyu, dxv, dyv

IMPLICIT NONE

REAL, DIMENSION(0:NXp,0:NYp), INTENT(IN) :: h, hs
REAL, DIMENSION(0:NX,0:NY), INTENT(IN) :: ub, vb
REAL , INTENT(IN) :: budgall(NX,NY), baseMB(NX,NY), dt
INTEGER, INTENT(IN) :: ifcompw
! if vert grids are merged, just use heati(NX,NY,NLEV)
REAL , INTENT(OUT) :: heati(0:NX,0:NY,0:NLEVp)
REAL , INTENT(OUT) :: u(0:NX,0:NY,0:NLEVp), v(0:NX,0:NY,0:NLEVp)
INTEGER, INTENT(IN)      :: Icheck
REAL :: w(NX,NY,0:NLEVp)

! for initial diagnostics:
REAL :: wa(NX,NY,0:NLEVp)

!     local:
REAL, DIMENSION(NX,NY,0:NLEVp) :: s1a, s2a, s1b, s2b
REAL :: s3a(NX,NY,NLEV), s3b(NX,NY,NLEV)
REAL :: uh(0:NX,0:NY,0:NLEVp), vh(0:NX,0:NY,0:NLEVp), dhdt(NX,NY)

REAL :: zh,zdbv2,zdb2,zs1a,zs1b,zs2a,zs2b,zdbu2,zuh,zsx,zhx,zsy,zhy,zus,zvs,wmax
INTEGER :: i,j,k,jm,jp,im,ip

!-----------------------------------------------------------------
!        Vertical integrals of ice rheology-temperature dependence
!        (on h-grid)
!-----------------------------------------------------------------

!        Calculate ice flow coefficients arh (for satable)
!        and arhap (for shelf flow and schoof g.l. flux, in icedyn)

! BSL set prior to call icedyn, update with newer h?
!CALL arrhenius (h, t, arh, arhap, tsurf, enhanceinv)

IF(Icheck /= 111) CALL outerrI("interface error on phomCompUV call, Icheck:",Icheck)


!        Calculate s1a, s2a, s3a (vertical integral terms) for
!        computation of u,uh,v,vh,heati
CALL satable_dyn (s1a, s2a, s3a, s1b, s2b, s3b, arh)

u=0.
v=0.
uh=0.
vh=0.
heati=0.
dhdt=0.
w=0.
wa=0.

!---------------------------------------------
!        Calculate eastward u, uh (on u-grid).
!        u and uh at layer mid levels.
!        Nb: treatment of shelf softening only valid for powi=3!
!---------------------------------------------


DO j = 1,nj
  DO i = 1,ni-1
    zh  = 0.5*(h(i,j) + h(i+1,j))
    IF (zh > 0.) THEN
      IF (.NOT. (hw(i,j) > HWcut .AND. hw(i+1,j) > HWcut) ) THEN
        zdbv2 = 0.25*(dbv(i,j)+dbv(i+1,j)+dbv(i,j-1)+dbv(i+1,j-1))
        zdb2 = dbu(i,j)**2 + zdbv2**2
        DO k=0,NLEVp
          zs1a = 0.5*(s1a(i,j,k) + s1a(i+1,j,k))
          zs1b = 0.5*(s1b(i,j,k) + s1b(i+1,j,k))
          zs2a = 0.5*(s2a(i,j,k) + s2a(i+1,j,k))
          zs2b = 0.5*(s2b(i,j,k) + s2b(i+1,j,k))
          u(i,j,k) = ub(i,j) - 2.*dbu(i,j)*zh  &
              * (zdb2*zs1a + siu(i,j)*zs1b) ! pollardC.modelDescript.gmd12.pdf eq 1 and using eq 3-7
! siu is SSA part of invariant, zdb2*zs1a is SIA part : sigma_xz ^2 + sigma_yz ^2
! dbu is sigma_xz /zeta (cf eq 3), presumably zs1a has zeta factor. zh from integral dz?
          
          uh(i,j,k) = ub(i,j)*zh*(1.-zeta(k)) - 2.*dbu(i,j)*(zh**2)  &
              * (zdb2*zs2a + siu(i,j)*zs2b)
        END DO
      ELSE
        DO k=0,NLEVp
          u (i,j,k) = ub(i,j)
          uh(i,j,k) = ub(i,j)*zh*(1.-zeta(k))
        END DO
      END IF
    END IF
  END DO
END DO

!----------------------------------------------
!        Calculate northward v, vh (on v-grid).
!        v and vh at layer mid levels.
!----------------------------------------------

DO j = 1,nj-1
  DO i = 1,ni
    zh  = 0.5*(h(i,j) + h(i,j+1))
    IF (zh > 0.) THEN
      IF (.NOT. (hw(i,j) > HWcut .AND. hw(i,j+1) > HWcut) ) THEN
        zdbu2 = 0.25*(dbu(i-1,j)+dbu(i,j)+dbu(i-1,j+1)+dbu(i,j+1))
        zdb2 = zdbu2**2 + dbv(i,j)**2
        DO k=0,NLEVp
          zs1a = 0.5*(s1a(i,j,k) + s1a(i,j+1,k))
          zs1b = 0.5*(s1b(i,j,k) + s1b(i,j+1,k))
          zs2a = 0.5*(s2a(i,j,k) + s2a(i,j+1,k))
          zs2b = 0.5*(s2b(i,j,k) + s2b(i,j+1,k))
          v(i,j,k) = vb(i,j) - 2.*dbv(i,j)*zh  &
              * (zdb2*zs1a + siv(i,j)*zs1b)
          
          vh(i,j,k) = vb(i,j)*zh*(1.-zeta(k)) - 2.*dbv(i,j)*(zh**2)  &
              * (zdb2*zs2a + siv(i,j)*zs2b)
        END DO
      ELSE
        DO k=0,NLEVp
          v (i,j,k) = vb(i,j)
          vh(i,j,k) = vb(i,j)*zh*(1.-zeta(k))
        END DO
      END IF
    END IF
  END DO
END DO

!---------------------------------------------------------------
!        Calculate heati (J/m3/y) over layers
!        Nb: treatment of shelf softening only valid for powi=3!
!---------------------------------------------------------------

DO j = 1,nj
  DO i = 1,ni
    IF (h(i,j) > 0. .AND..NOT. hw(i,j) > HWcut) THEN ! so for grounded only
      zdb2 = (0.5*(dbu(i,j)+dbu(i-1,j)))**2 + (0.5*(dbv(i,j)+dbv(i,j-1)))**2 ! basal stress **2
      DO k=1,NLEV
        heati(i,j,k) = 2. * zdb2 * (zdb2*s3a(i,j,k) + sih(i,j)*s3b(i,j,k))
      END DO
    END IF
  END DO
END DO


IF(ifcompw==1) THEN
!        Compute diagnostic vertical velocity wa (on h-grid mid levels)
!        (dz/dt, m/yr). For diagnostic output only.

!---------------------------------------------------
!        Calculate w (h*dzeta/dt) on h-grid.
!        (uh,vh and hence w are at layer mid points)
!---------------------------------------------------
  
  DO j=1,nj
    DO i=1,ni
      IF (h(i,j) > 0.) THEN
!           dhdt due to ice flow,sfc budget,basefrml,oceanmelt only
!           (kept in array just for wa below):
        dhdt(i,j) = budgall(i,j) 
        DO k=0,NLEVp
          zuh = (  uh(i,j,k)*dyu(i,j)-uh(i-1,j,k)*dyu(i-1,j)  &
             + vh(i,j,k)*dxv(i,j)-vh(i,j-1,k)*dxv(i,j-1) ) / darea(i,j)
          w(i,j,k) =  -baseMB(i,j) + dhdt(i,j)*(1.-zeta(k)) + zuh !+ downwArd, opposite direction of wk in pGSMicetherm
        END DO
      END IF
    END DO
  END DO

  DO j=1,nj
    jm = MAX (j-1,1)
    jp = MIN (j+1,nj)
    DO i=1,ni
      IF (h(i,j) > 0.) THEN
        im = MAX (i-1,1)
        ip = MIN (i+1,ni)
        zsx = (hs(ip,j)-hs(im,j)) / (dxu(i,j)+dxu(im,j))
        zhx = (h (ip,j)-h (im,j)) / (dxu(i,j)+dxu(im,j))
        zsy = (hs(i,jp)-hs(i,jm)) / (dyv(i,j)+dyv(i,jm))
        zhy = (h (i,jp)-h (i,jm)) / (dyv(i,j)+dyv(i,jm))
        DO k=0,NLEVp
          zus = 0.5*(u(i,j,k)+u(i-1,j,k))
          zvs = 0.5*(v(i,j,k)+v(i,j-1,k))
          wa(i,j,k) = -w(i,j,k) + dhdt(i,j)*(1.-zeta(k))  &
             + zus*(zsx-zeta(k)*zhx) + zvs*(zsy-zeta(k)*zhy)
        END DO
      END IF
    END DO
  END DO


!----------------------------
!        Convert w to dzeta/dt
!----------------------------
  
wmax = 0.5/dt
DO j=1,nj
  DO i=1,ni
    IF (h(i,j) > 0.) THEN
      zh = MAX (h(i,j),1.e-6)
      DO k=0,NLEVp
        w(i,j,k) = MAX ( -wmax, MIN ( wmax, w(i,j,k)/zh ))
      END DO
    END IF
  END DO
END DO

ENDIF !ifcompw


RETURN
END SUBROUTINE phomCompUV
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE GSMgrid2phom(Fout,Fin, Icheck)
! for Tc grid
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modPHOMgrid
 USE modgridm, ONLY : nip, njp
 USE moddim

 IMPLICIT NONE

 REAL, INTENT(IN) :: Fin(0:mNAp,0:mNBp,NCFZ)
 REAL, INTENT(OUT) :: Fout(0:mNAp,0:mNBp,0:NLEVp)
 INTEGER, INTENT(IN)      :: Icheck

 REAL :: wt(NLEV)=0., zgkzet
 INTEGER :: kzph, i, j, kzeta(NLEV)=0, istart=1

 IF(Icheck /= 111) CALL outerrI("interface error on GSMgrid2phom call, Icheck:",Icheck)

 IF(istart==1) THEN
   DO kzph=1,NLEV
     kzeta(kzph)=INT(zeta(kzph)/dzet)+1
     wt(kzph)= zeta(kzph)/dzet+1.0 - REAL(kzeta(kzph))
     IF(wt(kzph)>1.0 .OR.  wt(kzph)<0.0) THEN
       PRINT *, "wt error",wt(kzph),kzph
       CALL outerr('wt in GSMgrid2phom out of range')
     ENDIF
   ENDDO
!T   PRINT *,"kzeta",kzeta
   istart=0
 ENDIF

 Fout(0:nip,0:njp,0)=Fin(0:nip,0:njp,1)
 Fout(0:nip,0:njp,NLEVp)=Fin(0:nip,0:njp,NCZ)
 DO kzph=1,NLEV
   DO j=0,njp
     DO i=0,nip
       Fout(i,j,kzph)=(1.-wt(kzph))*Fin(i,j,kzeta(kzph)) + wt(kzph)*Fin(i,j,kzeta(kzph)+1)
     ENDDO
   ENDDO
 ENDDO

 RETURN
END SUBROUTINE GSMgrid2phom

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE phomGRID2GSM(Fout,Fin, Icheck)
! for U,V grids
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modPHOMgrid
 USE modgridm, ONLY : ni, nj
 USE moddim

 IMPLICIT NONE

 REAL, INTENT(IN) :: Fin(0:mNA,0:mNB,0:NLEVp)
 REAL, INTENT(OUT) :: Fout(0:mNA,0:mNB,NCZ)
 INTEGER, INTENT(IN)      :: Icheck

 REAL :: wt(NCZ)=0, zetag
 INTEGER :: kzgsm, i, j, kzetaph(NCZ)=0, istart=1, kzz

 
IF(Icheck /= 111) CALL outerrI("interface error on phomGRID2GSM call, Icheck:",Icheck)

 Fout(0:ni,0:nj,1)=Fin(0:ni,0:nj,0)
 Fout(0:ni,0:nj,NCZ)=Fin(0:ni,0:nj,NLEVp)

 IF(istart==1) THEN
   DO kzgsm=2,NCZ-1
     zetag=REAL(kzgsm-1)*dzet ! actual fractional depth of cell center
     DO kzz=1,NLEVp
       IF(zeta(kzz)>zetag) THEN
         kzetaph(kzgsm)=kzz-1
         EXIT
       ENDIF
     ENDDO 
     wt(kzgsm)= (zetag-zeta(kzetaph(kzgsm))) / ( zeta(kzetaph(kzgsm)+1) - zeta(kzetaph(kzgsm)) ) 
     IF(wt(kzgsm)>1.0 .OR.  wt(kzgsm)<0.0) THEN
       PRINT *, "wt error",wt(kzgsm), kzgsm
       PRINT *,"zeta+,zeta",zeta(kzetaph(kzgsm)+1),zeta(kzetaph(kzgsm)),zetag
       PRINT *,"zeta",zeta
       CALL outerr('wt in phomGRID2GSM out of range')
     ENDIF
  ENDDO
!T  PRINT *,"kzetaph",kzetaph
!T  PRINT *,"wt",wt
  istart=0
 ENDIF

 DO kzgsm=2,NCZ-1
   DO j=0,nj
     DO i=0,ni
       Fout(i,j,kzgsm)=(1.-wt(kzgsm))*Fin(i,j,kzetaph(kzgsm)) + wt(kzgsm)*Fin(i,j,kzetaph(kzgsm)+1)
     ENDDO
   ENDDO
 ENDDO

 RETURN
END SUBROUTINE PhomGRID2GSM
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

# 7210


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE basecoef (h, hb, hs, baseMB, TicePhom, Tsurfi, sealev, delt, Icheck)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!     Sets basal sliding coefficients crh[u,v] (u,v grids). Setting
!     starts from intrinsic crhmel (previously set in basecoef_init or
!     read from restart file), and modifies it due to basal temp,
!     water supply, etc. For floating/g.l. pts, uses unmodified crhmel.
!     (Actual values for ice-water interface are set in icedyn).

!     These calcs will be expanded when add explicit basal hydrology
!     (probably on h-grid), and effects on basal hydrologic flow.

!     Also set fracg[u,v] and hg[u,v] at grounding line.

! #if defined (PDfluxInvertCRH)
! to catch following changes: Tb warm/cold, float/GL/grounded.
! inversion uses |u/vbt|>5 as condition for update.
! so won't update during transition to |u/vbt|<=5 m/yr
! floating ice okay as fpin call in setcoefs updates crhu/v except
! this will then screw inversion


 USE modconstants, ONLY : RHOIovS, Betap, HWcut, RHOIc, gEARTH, RHOWSc
 USE modgridm, ONLY : ni, nj, kiceS
 USE modPHOMgrid
 USE modPHOMvardyn
 USE modPHOMvarhydrol
# 7242


# 7246

 USE modbeddrag, ONLY : POWb



# 7253

# 7257


 USE modGb, ONLY : rNeffC
 USE modinparm, ONLY : rNeffFact


USE modicerheology, ONLY : POWi
! USE modgridm, ONLY : randGen
USE modGb, ONLY : Gb
USE modbedtherm, ONLY : rLi

# 7270


# 7275


! sub ensembles
USE modinparm, ONLY : subEnszTrampF, subEnsWTbMN, subEnsCRHNOSb, subEnsExp

 IMPLICIT NONE

 REAL, INTENT(IN)                         :: h(0:NXp,0:NYp)
 REAL, INTENT(IN)                         :: hb(0:NXp,0:NYp)
 REAL, INTENT(IN)                         :: hs(0:NXp,0:NYp)
 REAL, INTENT(IN)                         :: baseMB(NX,NY) ! now basal massbal (to match Gb)
 REAL, INTENT(IN)                         :: TicePhom(0:NXp,0:NYp,0:NLEVp)
 REAL, INTENT(IN)                         :: Tsurfi(0:NXp,0:NYp)
 REAL, INTENT(IN)                         :: sealev(0:NXp,0:NYp), delt
 INTEGER, INTENT(IN)      :: Icheck

 REAL, DIMENSION(0:NX,0:NY) :: crhuS=0., crhvS=0

# 7305


! REAL :: CRHNOS=2.e-3 * (0.5e-5)**2
# 7310


REAL, PARAMETER :: UVupwindFULLTRAMP = 1000.0, UVupwindTRAMP=20.0
REAL, PARAMETER :: Hcut=10.0 ! m for counting Tb for interface
!REAL, PARAMETER :: WTbMN=0.5 ! weight of zTmn (adjacent minimum Tb) for basal sliding Tramp
!REAL, PARAMETER :: WTbMN=1.0 ! weight of zTmn (adjacent minimum Tb) for basal sliding Tramp
REAL, PARAMETER :: Ci0=2097.0 ! heat capacity of ice at 273.03K: 152.5 + 7.122*273.03= 273.03 K
REAL, PARAMETER :: TbpmGbCut = 0.5 !K allowance for subglacial meltwater to subgrid warm colder adjacent ice for interface
REAL, PARAMETER :: MINeffectP=1.e4 !Pa

REAL :: zcrhmel,zpot,zpota,zaf,zbf,zfgl,zhbg,zhg,zT,nzT,zcrh,fract,fracb, hI, CHRNOStill, zTmn
REAL :: zTramp, sealevfrac, zTmx, fractbnd, rTmp, rTsed, effectP !,crh_b,crh_t
INTEGER :: iloopuv,ish,jsh,nxtmp,nytmp,j,i,ia,ja, ifNoUpWind, ifTpm
INTEGER :: IcntDiagnosOUT=0, kiS=0

! sub ensembles
 REAL :: CRHNOSb,WTbMN,CRHNOS
 CRHNOSb=subEnsCRHNOSb
 WTbMN=subEnsWTbMN

!        Set basal sliding coefficients crhu (u grid, iloopuv=1), and
!        crhv (v grid, iloopuv=2), looping over u and v grids to avoid
!        duplicate code. Do for all points regardless of  water -
!        values will be re-set for water within icedyn iterations.
!        Also crh[u,v] will be adjusted in sedflow for deforming sed.

IF(Icheck /= 111) CALL outerrI("interface error on basecoef call, Icheck:",Icheck)


 fracgu = 1.
 fracgv = 1.
# 7344


# 7348



 CRHNOS=2.e-3 * (0.5e-5)**POWb



# 7359

!zTramp = REAL(subEnszTrampF)  * 3.125e3/50.e3 !
!zTramp = REAL(subEnszTrampF)  * 6.25e3/50.e3 !
!zTramp = REAL(subEnszTrampF) !
 zTramp = REAL(subEnszTrampF)  * dd0/50.e3 !
!zTramp = 0.001
!zTramp =2. * dd0/50.e3 !



! set crhu/vS, powbu/v
# 7374

 IF(kiS/=kiceS) CALL baseSPcoef() !only called at start and when change in active ice sheet
! INTENT(OUT): set crhuS/vS, powbu/v (crhuMN/X, crhvMN/X for -DPDfluxInvertCRH)
! CALL baseSPcoef() !use this for test of repeated baseSPcoef calls


# 7383


!>>>>>>>>>>>>>>>>>>>>>
 DO iloopuv = 1,2
!>>>>>>>>>>>>>>>>>>>>>

   IF (iloopuv == 1) THEN
     ish = 1
     jsh = 0
     nxtmp = ni-1
     nytmp = nj
   ELSE
     ish = 0
     jsh = 1
     nxtmp = ni
     nytmp = nj-1
   END IF

   DO j=1,nytmp
     DO i=1,nxtmp
       ia = i + ish
       ja = j + jsh

       IF (iloopuv==1) THEN
         zcrhmel=crhuS(i,j)
       ELSE
         zcrhmel=crhvS(i,j)
       ENDIF
       zTmx=-100.0

!            If at grounding line, calculate areal fraction of this
!            u or v-grid cell that's grounded (fracg[u,v]). i.e.,
!            estimated sub-grid position of grounding line. Calculate
!            height-above-flotation for the two surrounding h-grid
!            points (one floating with h.a.f. < 0, one grounded with
!            h.a.f. > 0), and linearly interpolated zero-location
!            is the estimated g.l position. [This is equivalent to
!            linearly interpolating ice surface, ice base, and bed
!            elevations between the 2 h-points, and seeing where
!            flotation occurs]. fracg[u,v] is used in icedyn,
!            multiplying the basal coefficient coefbs[u,v].

!            Also calc g.l. ice thickness hg[u,v] (using fracg[u,v]).
!            Currently fracg[u,v] (and hg[u,v]) are diagnostic only
!            if schoofgl not defined. For schoofgl, fracg[u,v] are
!            used in calcgl to calculate hg[u,v] for schoof g.l. flux
!            (repeating hg[u,v] setting here, but within C loop).

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
       IF (     (hw(i,j) > HWcut .AND. hw(ia,ja) <= HWcut)  &
             .OR. (hw(i,j) <= HWcut .AND. hw(ia,ja) > HWcut) ) THEN ! grounding line
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         zpot  = hb(i,j)   + hw(i,j)   + RHOIovS*h(i,j) ! elevation of surface with m water equiv
         zpota = hb(ia,ja) + hw(ia,ja) + RHOIovS*h(ia,ja)
         IF (hw(i,j) > HWcut) THEN
           zaf = zpota - zpot     ! ht of grounded surface above adjacent flot surface (liq m)
           zbf = -hw(i,j)         
         ELSE !IF (hw(ia,ja) > HWcut) THEN
           zaf = zpot - zpota
           zbf = -hw(ia,ja)
         END IF
         zfgl = zaf / MAX (zaf-zbf, 0.1)    ! for safety...should
         zfgl = MAX (0., MIN (1., zfgl))  ! have zaf>0, zbf<0
!zfgl = 0.5 ! 777

!           or as in Pattyn et al (2006) (doesn't work if h=0):
!if (hw(i,j).gt.HWcut) then
!  zf = (sealev(i,j) - hb(i,j))   / (RHOIovS*max (h(i,j),.1))
!  zg = (sealev(ia,ja) - hb(ia,ja)) / (RHOIovS*max (h(ia,ja),.1))
!else
!  zg = (sealev(i,j) - hb(i,j))   / (RHOIovS*max (h(i,j),.1))
!  zf = (sealev(ia,ja) - hb(ia,ja)) / (RHOIovS*max (h(ia,ja),.1))
!endif
!zfgl = (1.-zg) / max (zf-zg, .01) !grounded fraction at grounding line

!           calc g.l. depth (lin.interp bed elev to u,v grid, vs. s.l.,
!           so only works for marine g.l.s, not inland lakes):
!zhbg =       0.5*hb(i,j) +  0.5*hb(ia,ja)
!           slightly better in 100/300ka tests(?):
         IF (hw(i,j) > HWcut) THEN
           zhbg = (1.-zfgl)*hb(ia,ja) + zfgl*hb(i,j)
           sealevfrac= (1.-zfgl)*sealev(ia,ja) + zfgl*sealev(i,j) ! BSL HGU/V check
         ELSE
           zhbg = (1.-zfgl)*hb(i,j)   + zfgl*hb(ia,ja)
           sealevfrac= (1.-zfgl)*sealev(i,j) + zfgl*sealev(ia,ja) ! BSL HGU/V check
         END IF
!         zhg  = (sealev(i,j)-zhbg)/RHOIovS
         zhg  = (sealevfrac-zhbg)/RHOIovS

# 7487


# 7492

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
       ELSE IF (hw(i,j) > HWcut .AND. hw(ia,ja) > HWcut) THEN ! shelf or openo water
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         zfgl = 0.
         zhg  = 0.
!~~~~~~~~~~~~~
       ELSE ! fully grounded
!~~~~~~~~~~~~~
         zfgl = 1.
         zhg  = 0.
!~~~~~~~~~~~~~~
       END IF
!~~~~~~~~~~~~~~

       IF (iloopuv == 1) THEN
         fracgu(i,j) = zfgl
         hgu(i,j)    = zhg
# 7512

       ELSE
         fracgv(i,j) = zfgl
         hgv(i,j)    = zhg
# 7518

       END IF

!.... END of floating check logic ....................

# 7605

       ifTpm=0
       zT = 0.
       nzT = 0
       zTmn = 0.

       hI=0.5*(h(i,j)+h(ia,ja))

       IF (h(i,j) > Hcut) THEN
!         zT = zT + TicePhom(i,j,nlevp)-(273.15-BETAP*h(i,j))
# 7624

         zT = TicePhom(i,j,nlevp) -Gb(i,j) *(rLi/(Ci0)*(2/dzet)) /H(i,j) *delt ! additional temperature rise available from basal melt conversion
! (-Gb)* rLi / Ci /(dz) = K/yr warming of bed layer
!         IF(-Gb(i,j) *(rLi/(Ci0)*(2/dzet)) /H(i,j) *delt > 0.2) PRINT *,"dTGb",&
!            -Gb(i,j) *(rLi/(Ci0)*(2/dzet)) /H(i,j) *delt,Gb(i,j),H(i,j),(rLi/(Ci0)*(2/dzet))
# 7631


!         zT=MIN(TbpmGbCut,zT+BETAP*hI)
         zT= zT+BETAP*hI
         IF(zT>0.) zT= MIN(TbpmGbCut,0.5*zT)
# 7638

         nzT = 1
         zTmx=MAX(zT,zTmx)
         zTmn=zT
       ELSE IF (hw(i,j) > HWcut) THEN
         nzT = 1 ! wet => warm base
         zTmx = 0.
!         zTmn = 0.
       END IF

       IF (h(ia,ja) > Hcut) THEN
!         zT = zT + TicePhom(ia,ja,nlevp)-(273.15-BETAP*h(ia,ja))
# 7660

         rTmp = TicePhom(ia,ja,nlevp) -Gb(ia,ja) *(rLi/(Ci0)*(2/dzet)) /H(ia,ja) *delt ! additional T potential rise from basal melt conversion
! (-Gb)* rLi / Ci /(dz) = K/yr warming of bed layer
!         IF(-Gb(ia,ja) *(rLi/(Ci0)*(2/dzet)) /H(ia,ja) *delt > 0.2) PRINT *,"dTGb",&
!            -Gb(ia,ja) *(rLi/(Ci0)*(2/dzet)) /H(ia,ja) *delt,Gb(ia,ja),H(ia,ja),(rLi/(Ci0)*(2/dzet))
# 7667


!         rTmp=MIN(TbpmGbCut,rTmp+BETAP*hI)
         rTmp= rTmp+BETAP*hI
         IF(rTmp>0.) rTmp= MIN(TbpmGbCut,0.5*rTmp)
# 7674

         zT = zT + rTmp
         nzT = nzT + 1
         zTmx = MAX(rTmp,zTmx)
         zTmn = MIN(rTmp,zTmn)
       ELSE IF (hw(ia,ja) > HWcut) THEN
!         zT = zT + 0.
         nzT = nzT + 1
         zTmx = 0.
!         zTmn = MIN(0.,zTmn)
       END IF

       IF (nzT > 0) THEN
# 7689

         zT = MIN(0.,WTbMN*zTmn +(1.-WTbMN)*zT/REAL(nzT))
# 7696

       ELSE
!         zT = 0.5*(Tsurfi(i,j) + Tsurfi(ia,ja)) - 273.15
         zT = 0.5*(Tsurfi(i,j) + Tsurfi(ia,ja)) !Tsurfi already in celsius
         zTmx = zT
!         zTmn = zT
       END IF
# 7705



!....

!         influence of basal temperature, h2o on sliding:

!-----------------------------------------------------------------------

!     Sets current basal sliding coefficient (crh) and also
!     fractional influences of temp (fract, 0-1) and h2o (fracb, 0-inf),
!     given:
!       crhmel = intrinsic basal sliding coeff = zcrhmel
!       tb = basal temperature relative to pressure melt pt (deg C) = zT
!       bh2o = basal water supply (baseperc-basefrml, m/y) = 0.5*(  baseperc(i,j)  -basefrml(i,j) + baseperc(ia,ja)-basefrml(ia,ja))
!       hbsd = sub-grid std. dev. of bed topography (m) = 0.5*(hbsd(i,j)+hbsd(ia,ja))

!       zcrh = zcrhmel
!       fract = 1.
!       fracb = 1.
       
!     effect of basal temperature:
# 7745

!fract = MAX( 0., MIN(1.,(zT+zTramp)/zTramp) )**28  ! cf GSMdoc.org/hybrid coefficients/scaling for Tramp
! sub ensembles
       fract = MAX( 0., MIN(1.,(zT+zTramp)/zTramp) )**subEnsExp  ! cf GSMdoc.org/hybrid coefficients/scaling for Tramp
!IF (zT>=0.) THEN
!  fract = 1.
!ELSE
!  fract = 0.
!ENDIF
# 7756

# 7775

# 7782

       zcrh = MAX(fract*zcrhmel,CRHNOS)


# 7794

! LT
!       crh_t = 10. ** (fract*LOG10(zcrhmel) +  (1.-fract)*LOG10(CRHNOS))
!       crh_t = MAX(fract*zcrhmel,CRHNOS)
!       zcrh = 10. **( fract*LOG10(zcrhmel) +  (1.-fract)*LOG10(CRHNOS) )

! ^^^

!     effect of basal h2o supply (bh2o, m/yr):
!fracb =  -0.5*(baseMB(i,j) + baseMB(ia,ja)) * 1.e3 ! BSL
!crh_b = max (fracb*zcrhmel, CRHNOS)
!       fracb = 1. ! no bh2o effect
!       crh_b = 0. ! no bh2o effect

!+++
!     Runbasrcm4xp3:
!     fracb =  -0.5*(baseMB(i,j) + baseMB(ia,ja)) * 1.e2
!     crh_b = max (fracb*zcrhmel, CRHNOS)
!+++

!       zcrh = MAX (crh_t, crh_b)
!       zcrh = crh_t
!#if defined (HEINO) || defined (EISMINT00)
!       zcrh = MIN (zcrh, zcrhmel)
!#else
!       zcrh = MIN (zcrh, rmu*CRHSED)
!       zcrh = MIN (zcrh, CRHSED)
!#endif
!-----------------------------------------------------------------------

!         if floating or at grounding line, no influence:
       IF (zfgl < 1.) THEN
         fract = 1.
!         fracb = 1.
         zcrh = zcrhmel
       END IF

# 7833


! if not fully floating and at least partly warm based * grounded fraction * Neff term

# 7845

         IF (hw(i,j) > HWcut .OR. hw(ia,ja) > HWcut) THEN ! adjacent cell is marine (shelf or open water)
! sep 18/22 now taking MIN(adjaent marine hydrostatic, terrestrial treatment)
           effectP =  gEARTH * MIN( 0.5* (RHOIc*(h(i,j)+h(ia,ja)) + RHOWSc * &
              (hb(i,j)+hb(ia,ja) - (sealev(i,j)+sealev(ia,ja)))) , &
              RHOIc *( 0.5*(h(i,j)*rNeffC(i,j) + h(ia,ja)*rNeffC(ia,ja)) ) )
! assume fully connected to ocean
! if at grounding line with -DTsaiGL, then above will be recalced with subroutine coulomb
         ELSE
           effectP = gEARTH * RHOIc * 0.5* ( h(i,j)*rNeffC(i,j) + h(ia,ja)*rNeffC(ia,ja) )
!option to test:   effectP = gEARTH * RHOIc * 0.25* (h(i,j)+h(ia,ja)) * (rNeffC(i,j) + rNeffC(ia,ja))
!option breaks matching with standard basal hydrol treatment below, but might be otherwise more defensible? Need to test impact
         ENDIF
!         effectP = MAX(effectP, MINeffectP)
         effectP = MAX(effectP, 0.0)

# 7864

       IF (zfgl>0.001 .AND. fract>0.1) zcrh= zcrh*zfgl* MIN(10.,MAX(0.2, rNeffFact/(effectP+MINeffectP))) ! rNeffFact @= 2e4:10*5e5 (H=50) =5e6, must be >1e4

! end #if defined (1) !!!!!!!!!!!!!


# 7872

!#if defined (IDassess)
!     zcrh=zcrh*(1.+ randGen(0.05)) ! +/-2%
!#endif


       IF (iloopuv == 1) THEN
# 7885

         crhu(i,j) = zcrh
# 7890

       ELSE
# 7898

         crhv(i,j) = zcrh
# 7903

       END IF

     END DO  ! i=1,nxtmp
   END DO    ! j=1,nytmp

!>>>>>>>>>>
 END DO   !iloopuv loop
!>>>>>>>>>>

!IcntDiagnosOUT=IcntDiagnosOUT+1
! IF (IcntDiagnosOUT < 10 .OR. MOD(IcntDiagnosOUT,INT(1e6))==0) THEN
!   PRINT *, "max/min crhu:",MAXVAL(crhu(1:ni,1:nj)),MINVAL(crhu(1:ni,1:nj))
!   PRINT *, "max/min crhv:",MAXVAL(crhv(1:ni,1:nj)),MINVAL(crhv(1:ni,1:nj))
! ENDIF
# 7927


 kiS=kiceS
 RETURN

CONTAINS

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 SUBROUTINE baseSPcoef() ! INTENT(OUT): set crhu/vS, powbu/v (crhuMN/X, crhvMN/X for -DPDfluxInvertCRH)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
# 7939

 USE modbeddrag, ONLY : cbedDragStore, POWb

 USE modinparm, ONLY : rmu, fslid

!#if defined (ANcal) || defined (GRkiOnly) || defined(GCM)
# 7947

# 7950


 USE modflags, ONLY : ifHEAD

!#if defined (ANcal)
! USE modinparm, ONLY : sedcut
!#endif


 IMPLICIT NONE

# 7983

!REAL, PARAMETER :: CRHHARD =1.e-10  ! not used
!REAL, PARAMETER :: CRHSED  =1.e-6  ! not used
  REAL, PARAMETER :: SEDCUT=0.5 ! fsed= (sed fraction)**fbedpow if <1


# 7995


 REAL :: zcrhsed=0., zcrhslid=0., ztmp, zhbsd
 INTEGER :: istart=1

 IF(istart==1) THEN

   IF(ifHEAD==1) THEN

     PRINT *,"Neff0D local Neff(basal melt) approximation active"


     PRINT *,"-DNeffDRAG : basal sliding * MIN(10,MAX(0.2,rNeffFact/Neff))"

# 8014

     PRINT *,"weight of zTmn (adjacent minimum Tb) for basal sliding Tramp: WTbMN=", WTbMN

     PRINT *,"-DTbpmGbI limiting of TbpmI contributions to MIN(TbpmGbCut,0.5*Tbf,Tbf), Tbf=Tbadj+dT(Gb_adj)+BetaP*H_interf)"
     PRINT *,"Tbpm_interface=WTbMN*MIN(TbpmI_adj) + (1-WTbMN)*avg(TbpmI_adj)"
     PRINT *,"TbpmGbCut=",TbpmGbCut, "using interpolated H for Tb sliding activation"
# 8028


     PRINT *,"zTramp =",zTramp
# 8035

!     PRINT *,"basal sliding Tramp=1+MAX(2,0.02*hbsd), fract=MAX(0,MIN(1,(T+Tramp)/Tramp))**28"
!till mar22/22     PRINT *,"basal sliding Tramp=MAX(2,0.02*hbsd), fract=MAX(0,MIN(1,(T+Tramp)/Tramp))**28"
# 8040

     PRINT *,"basal sliding Tramp=MAX(1,0.02*hbsd), fract=MAX(0,MIN(1,(T+Tramp)/Tramp))**28"


   ENDIF

# 8093

   zcrhsed =rmu*3.e3/(3.e4)**POWb ![3 km/yr / 30kPa**POWb]
   zcrhslid =fslid*200./(1.e5)**POWb ! 200 m/yr / (100kPa)^POWb
   PRINT *,"single basal power law, icedyn basal sliding exponent:",POWb
   WRITE(*,*)'hGSM hard bed 100kPa sliding speed: ',zcrhslid*(100.e3)**(POWb) ,' m/yr'
   WRITE(*,*)'hGSM till bed 10kPa sliding speed: ',zcrhsed*(10.e3)**(POWb) ,' m/yr'
   PRINT *,"smooth hard to soft transition: Ub= (Usoft/Uhard)**SedFraction * Uhard"


# 8114



# 8119

  PRINT *,"SEDCUT: ",SEDCUT
ENDIF


! basal drag coefficient initialized
! crhmel=CRHNOS !ghosting cells covered
!  fsed=0. ! ghosting cells covered
!  fsed(1:ni,1:nj)=cbedDragStore(1:ni,1:nj,kiceS)
   fsed=cbedDragStore(:,:,kiceS) ! basal soft bed fraction
# 8132

! powbu=powb
! powbv=powb

  crhu = CRHNOS
  crhv = CRHNOS

!>>>>>>>>>>>>>>>>>>>>>
  DO iloopuv = 1,2
!>>>>>>>>>>>>>>>>>>>>>

   IF (iloopuv == 1) THEN
     ish = 1
     jsh = 0
     nxtmp = ni-1
     nytmp = nj
   ELSE
     ish = 0
     jsh = 1
     nxtmp = ni
     nytmp = nj-1
   END IF

   DO j=1,nytmp
     DO i=1,nxtmp
       ia = i + ish
       ja = j + jsh

!@@@ HEINO @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# 8299

! single POWb, continuous hard -> soft transition
       ztmp = SQRT( 1.e-20 + fsed(i,j)*fsed(ia,ja) ) 
       zcrhmel=  (zcrhsed/zcrhslid)**ztmp * zcrhslid
!@@@

!@@@

# 8319


       IF (iloopuv==1) THEN
         crhuS(i,j)=zcrhmel
       ELSE
         crhvS(i,j)=zcrhmel
       ENDIF
       
     END DO  ! i=1,nxtmp
   END DO    ! j=1,nytmp

!>>>>>>>>>>
 END DO   !iloopuv loop
!>>>>>>>>>>

# 8340


 istart=0
 END SUBROUTINE baseSPcoef
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


 END SUBROUTINE basecoef
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
