# 1 "modPHOM.F90"

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modPHOMgrid
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
USE moddim
IMPLICIT NONE
SAVE

INTEGER, PARAMETER :: NX=mNA, NY=mNB
INTEGER, PARAMETER :: NXp=NX+1,  NYp=NY+1
INTEGER, PARAMETER :: NLEV=12, NLEVp=NLEV+1
!INTEGER, PARAMETER :: NLEV=14, NLEVp=NLEV+1
!INTEGER, PARAMETER :: NLEV=30, NLEVp=NLEV+1

!REAL, PARAMETER :: dx0=40.e3, dy0=dx0,   dd0=dx0  ! INTENT(IN) minimally in icedyn ! 40x40km
!REAL, PARAMETER :: dx0=20.e3, dy0=dx0,  dd0=dx0  ! 20x20km
!REAL, PARAMETER :: dx0=25.e3, dy0=dx0,   dd0=dx0  ! INTENT(IN) minimally in icedyn ! 40x40km
# 22

REAL, PARAMETER :: dx0=20.e3, dy0=dx0  ! for PSgrid only, INTENT(IN) minimally in icedyn ! 20x20km
# 26



REAL :: dd0=dx0 ! reset in pGSMinit.F90/seticegrid for LL and 1

! minimum limits to handle small velocities in icedyn
! REAL, PARAMETER :: gmin   = (0.01/dd0)**2    ! INTENT(IN)
REAL :: gmin   = (0.01/dx0)**2 ! this just for PSgrid, otherwise  reset in pGSMinit for LL or SQ grid

REAL :: dzeta(NLEV)
REAL :: zeta(0:NLEVp)  ! INTENT(IN), zeta IS DEPTH OF CENTER of kz CELL
REAL :: zetah(0:NLEV)  ! INTENT(IN) only used in icetherm, satable: DEPTH OF LOWER EDGE OF kz CELL

REAL ::  zlambda

CONTAINS


!=======================================================================
SUBROUTINE setPHOMkzGrid()
!=======================================================================

!        Set vertical ice grid

!     nb: dzeta..., dzsed... have to add up to 1:

 SAVE
 
 INTEGER :: kz

 REAL :: dzeta10(10)=(/.05, .10, .15, .15, .20, .15, .10, .05, .03, .02/)
! REAL :: dzeta12(12)=(/.05, .10, .10, .15, .20, .143, .093, .062, .047, .031, .016, .008/)
 REAL :: dzeta12(12)=(/.05, .10, .10, .15, .20, .15, .10, .0522, .04, .03, .02, .0078/) !dzeta12K basal cell @match for NCZ=65, exact is 0.0078125
!REAL :: dzeta12(12)=(/.05, .10, .10, .15, .20, .15, .10, .052, .04, .03, .02, .008/) !old dzeta12K
! REAL :: dzeta12(12)=(/.05, .10, .10, .15, .20, .15, .10, .05, .04, .03, .02, .01/) ! daveP orig unit Nov/21
 REAL :: dzeta14(14)=(/.05, .10, .10, .15, .20, .15, .10, .045, .03, .025, .02, .015, .01, .005/)

 IF (NLEV == 10) THEN
   dzeta(1:10)=dzeta10(1:10)
 ELSE IF (NLEV == 12) THEN
   dzeta(1:NLEV)=dzeta12(1:NLEV)
 ELSE IF (NLEV == 14) THEN
   dzeta(1:NLEV)=dzeta14(1:NLEV)
 ELSE IF (NLEV == 30) THEN
   DO kz=1,10
     dzeta(kz) = 0.1/10.
   ENDDO
   DO kz=11,20
     dzeta(kz) = 0.8/10.
   ENDDO
   DO kz=21,30
     dzeta(kz) = 0.1/10.
   ENDDO
 ELSE
   DO kz=1,NLEV
     dzeta(kz) = 1./NLEV
   ENDDO
!       write (ioterm,*) 'Error: unknown NLEV=',NLEV
!       stop
 ENDIF

 zeta(0) = 0.
 zeta(1) = 0.5*dzeta(1)
 DO kz=2,NLEV
   zeta(kz) = zeta(kz-1) + 0.5*(dzeta(kz-1)+dzeta(kz))
 ENDDO
 zeta(NLEVp) = 1.

! DO k=0,NLEV
!   dzetah(kz) = zeta(kz+1) - zeta(kz)
! ENDDO

 zetah(0) = 0.
 DO kz=1,NLEV
   zetah(kz) = zetah(kz-1) + dzeta(kz)
 ENDDO
RETURN
END SUBROUTINE setPHOMkzGrid
!=======================================================================

END MODULE modPHOMgrid
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modPHOMvardyn
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modPHOMgrid

 IMPLICIT NONE

! Variable info is found in the README

!.........................
! Passed in call statement
!.........................
!      REAL  ::  h(NX,NY)
!      REAL  ::  hs(NX,NY)
!      REAL  ::  hb(NX,NY) ! INTENT(IN) [m]:ice thickness
!      REAL  ::  budgall(NX,NY)
!      REAL  ::  ua(0:NX,0:NY), va(0:NX,0:NY)
!      REAL  ::  ub(0:NX,0:NY), vb(0:NX,0:NY)
!      REAL  ::  timeice, dt
!      INTEGER ::  ifrest, iffirse, iffirsr

!.........................
! ModPHOMvardyn global variables

!REAL :: hbs(0:NXp,0:NYp,NICE)
!.........................
! coupling variables
 REAL :: hbsd(NX,NY) ! INTENT(IN) [m]:subgrid std deviation of bed topography
 REAL :: fedge(NX,NY) ! INTENT(IN) []:dynamic mask (flux edge) for no flow out of grid boxes with fedge < 1
! static setting as phomCompUV and thermo not in Hybridicestep scope
 REAL :: heath(NX,NY)=0. ! INTENT(OUT) [J/yr/m^3]:heating due to shelfy stream dynamics
 REAL :: heatb(NX,NY)=0. ! INTENT(OUT) [J/yr/m^2]:heating due to basal shear stress
 REAL :: sih(NX,NY)=0. ! INTENT(OUT) :shelf-stress contribution on sheet rheology on h grid
 REAL :: siu(0:NX,0:NY)=0., siv(0:NX,0:NY)=0. ! INTENT(OUT) :shelf-stress contribution on sheet rheology on u/v-grid
 REAL :: dbu(0:NX,0:NY)=0., dbv(0:NX,0:NY)=0. ! INTENT(OUT) [N m-2]: basal drag
! REAL :: visc(NX,NY) ! INTENT(OUT) [Pa s]:effective viscosity

 REAL :: budgCons(NX,NY)=0. ! INTENT(OUT) : mass-conserved budget (for agda setting)

!#if defined (dualpowb)
 REAL :: fsed(0:NXp,0:NYp)=0.
 INTEGER :: powbu(0:NX,0:NY)=0, powbv(0:NX,0:NY)=0
! REAL :: fpinu(0:NX,0:NY)=0, fpinv(0:NX,0:NY)=0, hBw(NX,NY)
!#endif

! convergence failure
!INTEGER :: iconvfail ! INTENT(IN OUT)
 INTEGER :: idieA,idieS,idieC,iletfail ! INTENT(IN)

 REAL, PARAMETER :: ubmin  =  0.30            ! INTENT(IN) ! 0.01
 INTEGER :: ifSPSdiagSUMout=0

 INTEGER :: muind(0:NX,0:NY), mvind(0:NX,0:NY) ! INTENT(OUT)
 INTEGER :: masku(0:NX,0:NY), maskv(0:NX,0:NY) ! INTENT(OUT) diagnostic

!.......................................................................

END MODULE modPHOMvardyn
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modPHOMvarhydrol
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modPHOMgrid

 IMPLICIT NONE

! basal conditions - hydrology, drag, sediment
! Variable info is found in the README

!.........................
! ModPHOMvarhydrol global variables
!.........................
! coupling variables

 REAL :: crhu(0:NX,0:NY)=0.,   crhv(0:NX,0:NY)=0. ! INTENT(IN) [m a-1 Pa-n]:coefficent in non-linear basal sliding law
! REAL :: crhu(0:NX,0:NY),  crhv(0:NX,0:NY) ! INTENT(IN) [m a-1 Pa-n]:coefficent in non-linear basal sliding law
# 195


# 200


# 204


 REAL :: uw(0:NX,0:NY)=0.,     vw(0:NX,0:NY)=0. ! INTENT(IN) [m a-1]:liquid water horizontal velocities (0 if basal hydrology is turned off by NOMOVEW)
 REAL :: fsedu(0:NX,0:NY)=0.,  fsedv(0:NX,0:NY)=0. ! INTENT(IN) []:areal fraction of sediments (weighting), only for deformation work heat source from subglac till deformation
 REAL :: fracgu(0:NX,0:NY), fracgv(0:NX,0:NY) ! INTENT(IN) []:fraction of a u/v-grid box that is grounded (vs floatng)
 REAL :: hgu(0:NX,0:NY),    hgv(0:NX,0:NY) ! INTENT(OUT) [m]:height of the subgrid g.l. diagnostic

 INTEGER, PARAMETER ::  NPOIMAX = MAX(1,(NX*NY)/5) ! max # of pts in one lake
 INTEGER, PARAMETER ::  NlakeMAX = 50              ! max # of lakes

! set by findwater **************************
 INTEGER :: indlake(NPOIMAX,NlakeMAX) ! INTENT(OUT):individual lakes across the grid
 INTEGER :: npoilake(NlakeMAX), nlake ! INTENT(OUT):number of points in each lake and number of lakes which includes the ocean
 INTEGER :: maskwater(NX,NY)=0 !INTENT (OUT):water mask, 0 if bare ground or grounded ice, 1 if ocean (with or without ice shelf, 2 or 3 if lake)

! set by findwater and reset by adjustpres *********************
 REAL :: hw(NX,NY)=0. ! INTENT(IN OUT) [m]:ocean water column thickness, need static for phomCompUV out of Hybridicestep scope
 REAL :: hedge(mNA,mNB) ! profiled adjusted H_edge for floating terminus
 INTEGER :: ifnexo(mNA,mNB)

END MODULE modPHOMvarhydrol
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modPHOMvartherm
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modPHOMgrid

! Variable info is found in the README
! Renaming variables example 'USE MODULE, SubroutineVar=>ModuleVar'

!.........................
! ModPHOMvartherm global variables
!.........................
! coupling variables, arh static as phomCompUV is not in scope of Hybridicestep
 REAL ::  arhap(NX,NY)=0. ! INTENT(IN) [s-1 Pa-3]:temperature-dependent flow coefficients based on vertically integrated arrhenius to the power of 1/n
 REAL ::  arh(NX,NY,NLEV)=0. ! INTENT(IN) [s-1 Pa-3]:arrhenius coefficient and yields s1a0, s1b0, s2a0, s2b0
 INTEGER :: ifArhReset=1

END MODULE modPHOMvartherm
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modPHOMvarocean
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modPHOMgrid

! Variable info is found in the README
! Renaming variables example 'USE MODULE, SubroutineVar=>ModuleVar'

!.........................
! ModPHOMvarocean global variables
!.........................
! coupling variables: CURRENTLY NOT ACTIVE SINCE CALVING INCLUDED IN AGD
 REAL :: cliffmelt(NX,NY) ! INTENT(OUT) [m a-1]:cliff failure for h > ~100m vertical exposed tidewater cliff walls
 REAL :: calvice(NX,NY) ! INTENT(IN) [m a-1]:calving rate #@ RIGHT NOW NOT USED (bypassed since agd already has calv)
 REAL :: crevliq(NX,NY) ! INTENT(IN)[m]:additional crevasse depth due to water in surface crevasses,#@ ONLY USED #if defined (CLIFFMELT)
!.......................................................................

END MODULE modPHOMvarocean
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!|||||||||||||||||||||||||||||||||||||||||||||||||
# 291

!|||||
