# 1 "pGSMicedynSGhyps.F90"
!@GCM: this contains the top level ISM code

! -DHybGSMcompUVtest generate GSM SIA velocities (uGSM, vGSM) with icedyn active
! -DgsmUV2Therm  for use with HybGSMcompUVtest, has thermo use GSM SIA velocities with icedyn active

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE icedynGSM(ntherm0,ibcx,ibcy,ki,tym) !legacy name ifield)
! ice dynamics/SMB/thermo loop over time dlong, starting at time=tym, internal timestep delt
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! icedynGSM(initial thermal/ism timestep ratio, boundary condition type,
!     ice sheet#,time), all IN variables
! ntherm0=0 => isthermi=0, ie no thermal coupling

 USE modicelegacy
 USE moddiagno
 USE modtopo
 USE modflags
 USE modinparm
 USE modicestore
 USE modlake
 USE modGb
 USE modicetherm, ONLY : Tc => Tit
 USE modMBpass
 USE modbeddrag
 USE modiceUV
 USE modconstants, ONLY : RHCONV, BETAP, Re, DEG2RAD, gEARTH, RHOIc
 USE modcorevar, ONLY : newrun, mbv, sealev, delt, sealevSc, newrunSG
 USE modnewdelt
 USE modTsurf
 USE modicedyn
 USE modStation
 USE modIU
 USE modIOflags
 USE modiceMASK, ONLY : imGrnlnd


 USE modPHOMgrid, ONLY : NLEVp, NLEV
 USE modicetherm, ONLY : heati
 USE modPHOMvartherm, ONLY : ifArhReset
! USE modPHOMvardyn, ONLY : budgCons
 USE modPHOMvardyn, ONLY : budgCons, ifSPSdiagSUMout, fedge
 USE  modPHOMvarhydrol, ONLY : hedge, ifnexo
! USE modiceUV, ONLY : uak, vak
 USE  modPHOMvarhydrol, ONLY : hw,fracgu,fracgv,hgu,hgv,maskwater,indlake,npoilake,nlake,crhu,crhv,hedge


# 51

# 54


# 58


# 62


# 67



# 74


# 78


# 82


# 88


# 92


IMPLICIT NONE

! thermo stuff

 INTEGER, INTENT(IN)             :: ntherm0
 INTEGER, INTENT(IN)             :: ibcx(NICE),ibcy(NICE)
 INTEGER, INTENT(IN)             :: ki
 REAL, INTENT(IN)                :: tym
!ccccccccccccccccccccccccccccccc
! field defs:
! ** hp is current negative earth surface elevation
! ** hpb is current negative ice base elevation (ie !=hp when floating)
! ** hd is H-hp (no bouyancy accounting)
! ** hdn is surface elevation of ice(Hn1), ie accounts for floating
!ccccccccccccccccccccccccccccccc

 INTEGER, PARAMETER :: MNNA=-1*mNA

! This is to ensure zero flux if there is ice in grid cell but no ice upslope
 REAL, PARAMETER :: HdiffMIN=20.0 !upslope diffusion coefficient cutoff
 REAL, PARAMETER :: HDdmn=20.0 !upslope diffusion coefficient hd diff cutoff

!COMMON/itime/t            !must also copy to ifield for age model


 REAL :: agdb(mNA,mNB) ! set to initial agd for each call

 REAL, DIMENSION(0:mNAp,0:mNBp) :: hdn, Hn1, hpb, hde ! all set in calctop

 REAL :: dvs(0:mNA,0:mNB) ! meridional basal velocities from duvs->compdedge
 REAL :: dus(0:mNA,0:mNB) ! zonal basal velocities from duvs->compdedge
 REAL, DIMENSION(0:mNAp,0:mNBp) :: duvs, ddc ! must be fully ghosted to interp to du/vs and d?, from compdm
 REAL :: uold(0:mNA,0:mNB,NCZ),vold(0:mNA,0:mNB,NCZ) ! unstored from u/volds then set for each loop
 REAL :: Btv(0:mNA,0:mNB),Bzv(0:mNA,0:mNB,NCZ) !double and single integrals from Bint
 REAL :: Btu(0:mNA,0:mNB),Bzu(0:mNA,0:mNB,NCZ) !double and single integrals
!************************************

! REAL, PARAMETER :: FSHRHCONV=0.99*RHCONV, FSHLF=10./(1.-1./RHCONV)
 REAL, PARAMETER :: FSHRHCONV=0.99*RHCONV, FSHLF=1./(1.-1./RHCONV)

!************************************
 REAL, PARAMETER :: hzero=1.e-18

 INTEGER :: istart=1 !for thermoinit
!g only needed for itop=4, ie greenland

!Surface Drainage calc**********************************************
 REAL :: agda(mNA,mNB,3) ! set to 0 at call start
 REAL :: sumdelt ! ""
# 145

 REAL, PARAMETER :: TYM0SurfDrain=-115001.0, TYM0AGDASurfDrain=TYM0SurfDrain-100.0

! REAL, PARAMETER :: TYM0SurfDrain=-121001.0, TYM0AGDASurfDrain=TYM0SurfDrain-100.0
 REAL ::  hfilla(mNA,mNB) !hfill* in m water,  reset from hfillb at each call start

!**********************************************

!**********************************************

!SGhyps#################################################################
# 160
!SGhyps#################################################################

# 164

  INTEGER :: istartHE=1

# 172

# 175


 INTEGER :: ish0

 REAL :: rHatill=3., rHbtill=1.5, rHftill=3. !Hbay, Hbay mouth, Foxe basin

 REAL :: stress(mNA,mNB) ! from compdm

 INTEGER :: irescnt=0


! block for resolution finer than 40 km or 1*0.5 degree
# 193

 REAL, PARAMETER :: tstmpMIN=0.0125 !# minimum timestep enforced to > tstmpMIN
# 197

# 200


 INTEGER ::  i, j, ix, jy, ij, nt, l

 REAL :: umx(NICE)=(/(1000.,i=1,NICE)/), vmx(NICE)=(/(1000.,i=1,NICE)/)

 INTEGER :: ieq, ifReloop, imx, jmx ! all set at call start
 REAL :: x, y, crit, tnew, tstmp, tbeginstep, deltatMB, tnewb
 INTEGER :: inobadexit, nthermr, ivmb, ith, itout
 REAL :: rHEtill1, rHEtill2, dtTh
 INTEGER :: iht, ih5k, j1, i1, j2, i2, kz
 REAL :: vavg, vmn

! Greenland Ice reset to hold constant:
 INTEGER :: i1GR=1, j1GR=1, j2GR=1
 REAL :: HfixGR(mNAP,mNB)=0.
 INTEGER, PARAMETER :: ifGRfixH=0 !set to 1 if want to forcing constant GR H
 INTEGER :: ifoutFD, iugcm !, ifGCMb=1

# 226


# 235




REAL, DIMENSION(0:mNA,0:mNB) :: uia, via, uit, vit ! NICE saved/reset, IN/OUT for icedyn
REAL, DIMENSION(0:mNA,0:mNB) :: ubt, vbt ! OUT from icedyn
! if vert grids are merged, just use heati(NX,NY,NLEV)
REAL, DIMENSION(0:mNA,0:mNB,0:NLEVp) :: uph, vph, heatiPH ! OUT from phomCompUV
REAL :: Tphom(0:mNAp,0:mNBp,0:NLEVp) ! from Tc->GSMgrid2phom
INTEGER :: nim, njm, ifchkHmx
REAL ::  chkHmxOld

REAL, DIMENSION(0:mNAp,0:mNBp) :: hb

INTEGER :: Icnttstepmin=0 , icntMinStepFail=0
INTEGER ::  ifTiceNew, Icheck=111, nThrmEquil

# 256



 REAL :: HOthr=20., timeLastOut=-1.e7
 REAL :: timeInit=-180000. ! start of Heinrich diagnostics
 INTEGER :: ifLISsq10yrOutput=0 ! 1= 10 year output, 0 = regular output #@ setup CHECK



!*******************************************************************
!     for NICE>1
! REAL :: agdsk(mNA,mNB,NICE)=0 !can't be in accumr since need flux cut
!*******************************************************************

 INTEGER ::  ifIceFail


# 275

 REAL, PARAMETER :: pConsCorr=1.0


# 286


# 290


!#########non-PHOM SIA ######################################################
# 328

!#########non-PHOM SIA ######################################################

 ieq=0
 imx=ni/2
 jmx=nj/2

 ifSPSdiagSUMout=1


# 340

!      ieq=1
!mar3/20 IF(tym<-254000.) ieq=1

# 346

 crit=1.
 CALL setbed(rk1,rk2,rk3,rk4,tau,delt,itop,ieq,ki)
!****************************

 h=0.
 h(0:nip,0:njp)=hs(0:nip,0:njp,ki)
 hp(0:nip,0:njp)=hps(0:nip,0:njp,ki)
 hd(0:nip,0:njp)=h(0:nip,0:njp)-hp(0:nip,0:njp)
 hb=hp*(-1.0)

!#if defined (basalHydrol) && ! defined (pyDriver) && ! defined(standAloneBasal)
! IF(istartHyd==1) THEN
!   PRINT *,' calling readbasalparms for basal hydro ensemble parameters'
!   CALL readbasalparms()
!   istartHyd=0
! ENDIF
!#endif

# 371


!######Porog#########
# 403

!######end Porog#########

# 408


!SGhyps#################################################################
!needs hp set, must come before CALL unStoreGSMicedyn
# 418

!SGhyps#################################################################

!*******************set ice sheet (ki) dependent GENITER matrix solver params********************
# 430

!*******************END set GENITER matrix solver params********************

 ifReloop=0
 CALL unStoreGSMicedyn(ifReloop)

!****************auto time step adjust****************************-
 CALL setIceTimeStep() !calls newdelt, sets tstmp
! PRINT *,"DELT TEST1",delt,tym,INT(tym/100.  - 0.1)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# 450

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% restartDEBUG%%%%%%%%%%%%%%%

# 456

533 CONTINUE                  ! TARGET FOR CONVERGENCE RETRY******
!******do not allow bad exits for minimum timesteps******
 IF(stpfact(ki) < stpfmin .OR. tstmp< 0.2) THEN
   inobadexit=1
 ELSE
   inobadexit=0
 END IF
 IF(inoexbad == 1) inobadexit=1
!********************************************************

!**GENITER****************************************************
# 476


!t******* set thermo timestep*********************************
 IF(ntherm0>0) THEN
   isthermi=1
 ELSE
   isthermi=0
   nthermr=1000
 END IF

 IF(isthermi == 1) CALL setThermoTimeStep(dtTh,nthermr) !both are INTENT(OUT)
!t************************************************************

 CALL setPHOMmasks(H)

!.....MAIN time loop prestart.............................................
!***************************************************************************************

 agda=0.
 sumdelt=0

# 505



 ifArhReset=1
 ifTiceNew=1
 ifchkHmx=0


# 515

# 518


# 524


 DO  nt=1,ntice ! ********** MAIN TIME STEPPING LOOP *****************
# 530

   tbeginstep=tym+(nt-1)*delt
   tnew=tbeginstep + delt




!*****************EA Fjord sediment control*********************
# 552

!*****************EA Fjord sediment control END*********************

!*****************Heinrich events control*********************
# 559


!************step mass-balance **********************-
   ivmb= mbv
   IF(INT(REAL(ivmb)*delt +0.01) > 20) ivmb=INT(20./delt +0.1)
!   IF((REAL(ivmb)*delt +0.01) < 5.0) ivmb=2*ivmb


   IF((REAL(ivmb)*delt +0.001) < 1.0) ivmb=NINT(1.0/delt) ! min of 1 yr calls
# 570

   IF(ivmb == 1 .OR. MOD(nt,ivmb) == 1) THEN ! accumr must be called for nt==1
     deltatMB=delt*REAL(ivmb)
!!$   IF(ivmb == 1 .OR. MOD(nt,ivmb) == 1 .OR. nt == ntice) THEN
!!$     deltatMB=delt*REAL(ivmb)
!!$     IF( (ntice-nt)<ivmb ) deltatMB=delt*REAL(ntice-nt)
!!$     IF(nt==ntice) deltatMB=delt
     tnewb=tnew
# 580

# 590


!@GCM: accumr (file pn35.f) is the surface mass-balance (SMB) calc including calving
! OUTPUT: accumulation: saccu, melt: smelt, netmass-balance(m): agd(mNA,mNB)=agdm
! OUTPUT: annual mean surface temperature: Tsurf, Tsumsea, precyrt
! OUTPUT: cagd, iagd, precip, precrunoff, IF(itypesealev(i,j,ki)==3): sealevel
! INPUT: hd, hfilla, sealev, climate fields
! SGhyps OUTPUT: Tsurfhyps, agdhyps, maskhyps, lunfilledhyps
! SGhyps UPDATE: Hhyps,hdhyps (maskyps 0->1); H,hd (maskhyps 1 -> 0)
! SGhyps **CAVEAT**: Tsurf(maskhyps==1) invalid until reset in fluxhyps except for newrun==1 and istart(ki)==1
!CALL setPHOMmasks(H) ! needed here to have hedge up to date if -DhedgeActive
      CALL accumr(ki,tnewb,hfilla,deltatMB)
!PRINT *,"accumr tnew",tnew
!CALL outTVNEA(tnew,123,ki)
!IF(nt==1) CALL outF4test(H(1:mNA,1:mNB),agd,saccu,smelt,"H,agd,saccu,smelt",244)
!CALL outHF3(tnew,hd,h,u,v,121,ki) !idat121/131 detailed output
!PRINT *,"accumr agdedgesum",SUM(agdm(ni-1,1:nj)/nj),SUM(agd(ni-1,1:nj)/nj)

# 618

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% restartDEBUG%%%%%%%%%%%%%%%

# 640

   END IF !IF(ivmb == 1 .OR. MOD(nt,ivmb) == 1 .OR. nt == ntice)


# 673


# 698



!****************init bed therm after mass-balance call**************-
   IF( istart == 1 .AND. nt == 1 ) THEN
     CALL initicedyn(ki) ! istart set to 0 at ki=NICE

!************GENITER NSPCG matrix solver global INIT******************
! NOTE: runs were different if this block is in initicedyn(ki), likely NSPCG memory leak
!###############################################
# 723

!###############################################
!******************************
   ENDIF !istart==1 and nt==1
!*******************************************************************


!SGhyps#################################################################
# 751
!SGhyps#################################################################

# 761

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% restartDEBUG%%%%%%%%%%%%%%%

! ********initialize variables (in case of non-linear iteration)****************
   CALL calctop(h,Hn1) !set iffloat, hd, hdn, hde, hpb, must come after fluxhyps since this can change H


!**thermo** compute integrals
! Bint vertically integrates the temperature dependent rheology coefficient
! INPUTS: h, Tc, and hd for upwinding, CONSTANT INPUTS: Aexp()
# 776

   IF (istart==1 .AND. nt == 1) THEN ! ie for 1 start to get U, V -> W to equilibrate thermo
     CALL Bint(Btv,Bzv,h,hdn,1)
     CALL Bint(Btu,Bzu,h,hdn,0)
   ENDIF


!************
   IF (istart==1 .AND. nt == 1) THEN  !compute old velocities and rest of thermo init, only active for newrun(ki) /= 0
!**********-compute diffusion coefficients first.******************-
!           d3 = qt, d4= qb
     IF(newrun(ki) > 0 .AND. ifColdStartIce(ki)==1) THEN !compute old velocities
!###########################################
!#if defined (1)
       u=0.
       v=0.
       uold=0.
       vold=0.
       dbedgl=0.0 ! compdm initial called for coldstart, want to inhibit streaming

! only during init, so skip HybGSMcompUVtest option
!#endif
!#if defined (HybGSMcompUVtest) || ! defined (1)
!#else
       IF(mtype == 1) THEN
!         CALL compd3rm(ki)
!         CALL compd4rm(ki)
       ELSE IF(mtype == 3) THEN
# 807

       ELSE
         CALL compdm(h,hde,hp,sealev,duvs,ddc,stress,dss,ki) ! compute D_i j from last timestep with updated H,hde
         CALL compdedge(0) ! computes d3,d4,dus,dvs and if ifBtFact==1 d1,d2
       END IF
# 814

!**thermo** compute old velocities ******************************-
! compuv now uses raw ddc interped to interface = d3/d4
! vold output, rest is all inputs
       CALL compuv(vold,dvs,d3,h,hdn,Bzv,ki,1)
       CALL compuv(uold,dus,d4,h,hdn,Bzu,ki,0)
       uold(0:ni,0:nj,:)=MAX(-2.e3,MIN(2.e3,uold(0:ni,0:nj,:))) ! m/yr, needed to avoid crazy velocities from griddedObsH0
       vold(0:ni,0:nj,:)=MAX(-2.e3,MIN(2.e3,vold(0:ni,0:nj,:)))
!#endif
!###########################################
!**************THERMO equilibrate for newrun(ki)>0************************************
       CALL thermEquil(uold,vold,1)
     ENDIF !IF(newrun(ki) > 0 .AND. ifColdStartIce(ki)==1)

   ENDIF !IF (istart==1 .AND.  nt == 1)

!**************max out mass-balance for convergence****************
   ish0=1
   DO  j=1,nj
     DO  i=1,ni
!            if((agdm(i,j)+Gb(i,j))<0.0) then
!               agd(i,j)=max((agdm(i,j)+Gb(i,j)),-h(i,j)/delt)
!            else
       agd(i,j)=agdm(i,j)+Gb(i,j)
       agdb(i,j)=agd(i,j)
!            endif

!************set up kill flag and 0 check
       IF(h(i,j) <hzero .AND. agd(i,j)<=0.)THEN
         IF(        h(i-1,j)<hzero .AND. h(i,j-1)<hzero  &
            .AND. h(i,j+1)<hzero .AND. h(i+1,j)<hzero) THEN
# 848

           h(i,j)=0.
         END IF
       ELSE
         IF(h(i,j)>0.02 .OR. agd(i,j)>0.01) ish0=0
       END IF
!************
     END DO
   END DO

!############ basal hydrology ####################################
# 893

!############ basal hydrology ####################################



!****************************************************************-
!        compute s1a  (need to be computed only once for each
!                          time step)
!CALL comps(s1a,hdn,H,agd,d1,d2,a1,a3,delt,ki)

!........START nonlinear solution ITERATIONS ++++++++++++++++++++++++++++++++++++++++++++++

   l=0

!           ************************************************************
!           simple relaxation GIA, usually not used
   IF(nt == 1 .AND. ifLocalGIA == 1) CALL relaxGIAstep() 
!           ************************************************************

   IF(ish0==1) THEN
     Hn1=0.
     agd=0.
     ifIceFail=0
     PRINT *,"ish0==1 no ice, skipping icedyn at tnew: ",tnew
     GO TO 933
   END IF
!   tnew=0 ! skip icedyn for PD diagnostic test
!   exit
!#################################################################

# 937
!SGhyps!******************************

# 971

   CALL Hybridicestep(ifIceFail,ki)


!###########################################
# 989

!#################################################################

933 CONTINUE                  !target for 0 ice (ish0==1) ice dyn bypass

   IF(ifIceFail==0) THEN
     iyconv(ki)=iyconv(ki)+1
     ntiter(ki)=niter+ntiter(ki)
   ELSEIF(ifIceFail > 0) THEN
! try again
     
!********************-NEWDELT RELOOP************************************
!-upon convergence failure, halve timestep and redo time loop
     istp(ki)=0
     IF(stpfact(ki) > stpfmin) THEN
       stpfact(ki)=stpfact(ki)/2.
       tstmp=tstepnw(ktcur(ki),ki)*stpfact(ki)
       IF(tstmp< tstmpMIN) THEN !minimum timestep
         IF(ifIceFail==2) THEN
# 1011

           CALL outerr('minimum timestep failure with NaN, exiting run')
         ENDIF
         stpfact(ki)=stpfact(ki)*2.
         Icnttstepmin=Icnttstepmin+1
         IF (Icnttstepmin < 100 .OR. MOD(Icnttstepmin,100)==0) PRINT *,'tstepmin',tstmp,tym,ki
         GO TO 535
       END IF
       CALL newdelt(tstmp,tym,ki,1)

       ifReloop=1

       CALL unStoreGSMicedyn(ifReloop)

# 1029

       GO TO 533            ! go back to redo loop
     ELSE
       icntMinStepFail=icntMinStepFail+1
       IF(icntMinStepFail>100) CALL outerr(">100 icedyn convergence failures at min timestep")
     END IF
535  CONTINUE
!**************-end of NEWDELT stuff**************

!************-convergence book-keeping**************-
!        store diverging cases
     noconv(ki)=noconv(ki)+1
     IF(noconv(ki)<100) THEN
       iti(ki)=iti(ki)+1
       times(iti(ki),ki)= tnew
     END IF
   END IF
   IF(l == nitp.AND.noconv(ki) <= 2) THEN
!        write out data of first five non-converging cases
     CALL outresF(tnew,hd,h,u,v,resid,d1,d2,80+ki,ki)
   END IF
!**********************************************************

   CALL couran(ki) !legacy cfl diagnostic using diffusion coefficients
!**************FINAL ASSIGNMENTS************************-
!     assign final result
   CALL calctop(Hn1,H) !set iffloat, hd, hdn, hde, hpb
!   ifReloop=0

# 1066

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% restartDEBUG%%%%%%%%%%%%%%%

!**main ice thermodynamics step********************************************************
 IF(MOD(nt,nthermr) == 0 .AND. isthermi == 1) CALL stepThermo()
!** END main ice thermodynamics step********************************************************

 IF (istart>0 .AND. nt==(3*ntice)/4) THEN
   IF(newrun(ki)==1 .AND. ifColdStartIce(ki)==1) CALL thermEquil(u,v,1)
 ENDIF

# 1079


!   CALL conser(tnew,ki)

!******special short time step output***********************
   IF(tnew< -12999. .AND. tnew>-60000. .AND. MOD(nt,nthermr) == 0) THEN
!         ih1=int(tnew)
!         i2=nio(1,2)
!         j2=njo(1,2)
!         BETAP=0.00087
!N         call outVF2(tnew,73,i2,j2,hd,H,u,Tc,agd,agdb,resid)
!         Tsb=Tc(i2,j2,NCZ)+BETAP*H(i,j)
!         write(74,142)tnew,hd(i2,j2),Tsb,u(i2,j2,1),u(i2,j2,NCZ)
! 142  format(f9.1,x,f7.1,1x,f6.2,1x,2(f7.1,1x))
   END IF

!*******agda stuff for accumulating surface runoff for surface drainage calc******************
   IF(((tnew>TYM0AGDASurfDrain .AND. tnew<-1999.) .OR. ifGCM==1) .AND. IFMWPext==1) THEN
!      if(tnew>-117002. .and. tnew<-1999..and. IFMWPext>0) then
     IF(ish0==0) THEN

       nim=ni-1
       njm=nj-1
       agda(2:nim,2:njm,2)=agda(2:nim,2:njm,2)+MIN(budgCons(2:nim,2:njm),0.)*delt
       agda(2:nim,2:njm,3)=agda(2:nim,2:njm,3)+MIN(MAX(budgCons(2:nim,2:njm),-1.*cagd(2:nim,2:njm)),0.)*delt
# 1128

     END IF

     IF(ki==kNA) THEN
       i1=ilx(-169.9)
!       i2=ix(-50.1)
!       j2=jy(39.1)
       i2=ni-1
       j2=nj
     ELSE
       i1=2
       i2=ni-1
!     j2=min(jy(39.1),nj)
       j2=nj-1
     END IF

# 1148


     DO j=2,j2
       DO i=i1,i2
         IF(agd(i,j)<0.) THEN
!assumes precyrt=P-E (also works for precyrt=max(P-E,0) )
! ie, f agd>0 => accumulation, so runoff=precip-evap-agd
! if agd<0 => runoff=precip-evap + |net mass loss = agd|
! smelt includes snowmelt but that just balances accumulation
!

# 1161

!since budgCons is flux balanced and non-freezing rain doesn't contribute to budgCons
! agda must be <=0, ie mass loss for mwpext.f90
           agda(i,j,1)= agda(i,j,1) + MIN(budgCons(i,j)-pConsCorr*precyrt(i,j),0.) *delt

!old but same           agda(i,j,1)= agda(i,j,1)-( MAX(precyrt(i,j)-budgCons(i,j),0.) &
!              + MIN(precyrt(i,j),0.) )*delt !since budgCons is flux balanced and non-freezing rain - evap doesn't contribute to budgCons
# 1170

         END IF
       END DO
     END DO

     sumdelt=sumdelt+delt
   END IF
!*******end agda stuff******************


!#######################################################################
# 1216

!########### basalSed #######################################

# 1226


# 1236



 IF(tnew>timeInit) THEN   ! calculate amplitudes only after initialisation phase
   IF (MOD(NINT(1000.*tnew),1000)==0) CALL diagnoLISsq(tnew,tf,timeInit,h,u,0,HOthr)
 ENDIF
 IF(tnew<=timeInit) THEN   ! calculate threshold only during initialisation phase
   IF (MOD(NINT(1000.*tnew),1000)==0) CALL diagnoLISsqThr(tnew,timeInit,h,u,HOthr)
 ENDIF

 IF (ifLISsq10yrOutput==1) THEN
   IF(MOD(NINT(100*tnew),1000)==0 .AND. (tnew>timeLastOut)) THEN
     timeLastOut=tnew
     CALL icedynout(tnew,tf,ki)
     CALL outVF(tnew,njo(ki,2),hd,H,u,v,82,ki)
   ENDIF
 ENDIF


!.....END nt=1,ntice timestep loop ....................................................
 END DO


 IF (istart>0) THEN
   IF(newrun(ki)==1 .AND. ifColdStartIce(ki)==1) THEN
     IF(ki==kANT) THEN
       CALL thermEquil(u,v,4)
    ELSE
       CALL thermEquil(u,v,3)
     ENDIF
!   ELSEIF(newrun(ki)==-1) THEN ! removed Wed Jun 26 13:04:45 NDT 2019
!     CALL thermEquil(u,v,1)
   ENDIF
 ENDIF

! dtCFL set with uvmx(ki) from previous icedynGSM call
 CALL vstats(vavg,umx(ki),vmn,u,imx,jmx,h,ki)
 CALL vstats(vavg,vmx(ki),vmn,v,imx,jmx,h,ki)

! iht=10*nint(tnew/10.)
 iht=NINT(tnew) ! still needed for compSurfaceDrainage

# 1280


! INPUTS: agda, hd, iH, hfill, sealev(i,j,ki); CONSTANT: hposPD
! OUTPUT: hfill*
# 1287



 IF (ifLISsq10yrOutput==0) THEN
   CALL icedynout(tnew,tf,ki)
 endif
# 1295


!*****************************************************************
!.....store working fields

 hs(0:nip,0:njp,ki)=h(0:nip,0:njp)
! hps(0:nip,0:njp,ki)=hp(0:nip,0:njp) ! hp/hps is only updated in full GIA calc done outside of icedynGSM
 IF(ifLocalGIA == 1) hps(0:nip,0:njp,ki)=hp(0:nip,0:njp)




 uolds(0:ni,0:nj,:,ki)=uold(0:ni,0:nj,:)
 volds(0:ni,0:nj,:,ki)=vold(0:ni,0:nj,:)
 Gbs(1:ni,1:nj,ki)=Gb(1:ni,1:nj)
 CALL storeT(ki)

 uak(0:ni,0:nj,ki)=uat(0:ni,0:nj) 
 vak(0:ni,0:nj,ki)=vat(0:ni,0:nj) 
 uiak(0:ni,0:nj,ki)=uia(0:ni,0:nj) 
 viak(0:ni,0:nj,ki)=via(0:ni,0:nj) 
 uitk(0:ni,0:nj,ki)=uit(0:ni,0:nj) 
 vitk(0:ni,0:nj,ki)=vit(0:ni,0:nj) 

# 1323

# 1326



!*****************************************************************

!SGhyps#################################################################
# 1352

!#################################################################

 IF(ih5k == 0.)THEN
   CALL FLUSH(20+ki)  ! idat21 iu
   CALL FLUSH(123+10*(ki-1)) !idat123 iu
   CALL FLUSH(103)
 END IF

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# 1374

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% restartDEBUG%%%%%%%%%%%%%%%

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# 1389

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% TESTfields %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# 1404

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CHECKfields %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# 1409


 IF(ki == NICE) istart=0

! agdsk(:,:,ki)=agd
 RETURN

CONTAINS
!*******************************************************************

!*******************************************************************
 SUBROUTINE initicedyn(ki)
!*******************************************************************
  INTEGER, INTENT(IN) :: ki
  
  IF(ki==kNA .AND. ifGRfixH==1) THEN
    PRINT *,"setting GR 49.99W:nip, 84N:58.1N Hconst"
    i1GR=ilx(-49.99)
    j1GR=jly(-49.99)
    j2GR=jly(-42.5)
    HfixGR(i1GR:nip,j1GR:j2GR)=H(i1GR:nip,j1GR:j2GR)
  ENDIF

  iVersionCompdm=2

  IF(ki == 1  .AND. ifHEAD==1) THEN

    PRINT *,"using HybridIceDyn hybrid SIA/SSA ice dynamics with Pollard core"
# 1440

# 1443


# 1447

# 1450


# 1458


# 1484

! time, H, hd, station#, iunit0, k
  CALL outSite1(tnew,h,hd,1,124,ki)
  END IF

! initThrm also has internal trap to prevent double calls in
! case of convergence failure at beginning
! init Tc and Gb
  CALL initThrm(newrun,tym,m,rm,mtype,ki) ! init ice thermodynamics, needs Tsurf,sealev for coldstart

  RETURN

 END SUBROUTINE initicedyn
!*****************************************************************

# 1682


!*******************************************************************
 SUBROUTINE unStoreGSMicedyn(ifReloop)
!*******************************************************************
  IMPLICIT NONE

  INTEGER, INTENT(IN) :: ifReloop

!.....unstore h and hp
  IF(ifReloop==1) THEN ! otherwise needs to be done earlier than call
    h(0:nip,0:njp)=hs(0:nip,0:njp,ki)
    hp(0:nip,0:njp)=hps(0:nip,0:njp,ki)
    hd(0:nip,0:njp)=h(0:nip,0:njp)-hp(0:nip,0:njp)
    IF(ifLocalGIA == 1) hb(0:nip,0:njp)=hp(0:nip,0:njp)*(-1.0)
ENDIF

  uold(0:ni,0:nj,:)=uolds(0:ni,0:nj,:,ki)
  vold(0:ni,0:nj,:)=volds(0:ni,0:nj,:,ki)
  Gb(1:ni,1:nj)=Gbs(1:ni,1:nj,ki)

  uat=uak(:,:,ki)
  vat=vak(:,:,ki)
  uia=uiak(:,:,ki)
  via=viak(:,:,ki)
  uit=uitk(:,:,ki)
  vit=vitk(:,:,ki)
!  CALL setmaskwater(ifReloop) ! needed for maskCalvSSM in accumr/icecalvSSM, when reloop and each icedynGSM call start if NICE>1 (and for very first call)


# 1715



# 1722



# 1727


  hfilla(1:ni,1:nj)=hfillb(1:ni,1:nj,ki)

!SGhyps#################################################################
# 1747
!SGhyps#################################################################


!***************************************

! Hold in thermo is set to H for thermo istart(ki)==1, but still need
! set Hold in thermo to H (i11chck flag=10) here given timestepping for newrun<1
! MOVED here as accumr updates H for SGhyps active
  IF(( istart==1 .AND. newrun(ki)<1) .OR. ifReloop==1) &
     CALL thermo(hp,h,hdn,u,v,uold,vold,dtTh,hfilla,sealev,ki,itout,1,ieq,10)
! only resets Hold in thermo (i11chck flag=10)

  IF(ifReloop==1 .AND. istart==1 .AND. newrun(ki) > 0 .AND. ifColdStartIce(ki)==1) THEN
    IF(ki == NICE) THEN
      istart=2
      PRINT *,"istart=1 convergence failure reloop, retains completed thermo init"
      RETURN
    ELSE
      PRINT *,"istart==1 .AND. newrun(ki) > 0 .AND. ifColdStartIce(ki)==1 convergence failure"
      PRINT *," for ki<NICE, code can't handle this, so exiting, resubmit with smaller first icedyn timestep for ki ",ki
      STOP 1
    ENDIF
  ENDIF

  CALL unstoreT(ki)
!bedT
! CALL unbstoreT(ki)

  RETURN
 END SUBROUTINE unStoreGSMicedyn
!*******************************************************************


!*****************************************************************
 SUBROUTINE thermEquil(ut,vt,kTiterFac)
!*****************************************************************
  REAL, INTENT(IN) :: ut(0:mNA,0:mNB,NCZ),vt(0:mNA,0:mNB,NCZ)
  INTEGER, INTENT(IN) :: kTiterFac
  REAL, PARAMETER :: dtThEquil=50.
  REAL :: Tcool
# 1791

  CALL thermo(hp,h,hdn,ut,vt,ut,vt,dtTh,  &
     hfilla,sealev,ki,1,1,ieq,11)
!                    ,ki,itout,icompw,ieq,i11chck)
!     ieq =1 => C_rock =C_rock/1000 for equilibrium computation
!  CALL FLUSH(6)
  ieq=1

  IF(ki==kGR) THEN
    nThrmEquil=20 ! initial ice
    Tcool=4.0
  ELSEIF(ki==kNA) THEN
    nThrmEquil=2 ! bit initial ice
    Tcool=4.0
  ELSEIF(ki==kANT) THEN
    nThrmEquil=30
    Tcool=2.0
# 1811

  ELSE
    nThrmEquil=2
    Tcool=6.0
  ENDIF


  IF(newrun(ki)<0) Tcool=0. ! Holocene

  
# 1823


  nThrmEquil=nThrmEquil*kTiterFac
  Tsurf=Tsurf-Tcool

  IF(ifHEAD==1) THEN
    PRINT *,'T coldstart Ice init: run for ',NINT(dtThEquil)*nThrmEquil,' years with iequilib=1 for bedtherm init'
!#if defined (1)
!  PRINT *,"U, V, = 0 for 1 thermo init on first call then nonzero"
!#else
    PRINT *,"U, V nonzero (computed with initThrm T0) from SIA GSM for thermEquil on first call"
!#endif
    PRINT *,"equilibrating T_ice with Tsurf-",Tcool,"C and dtThEquil:",dtThEquil, " at time:", tnew
  ENDIF

  DO ith=1,nThrmEquil
    IF(MOD(ith,50) == 0.) THEN
      itout=1
    ELSE
      itout=0
    END IF
    CALL thermo(hp,h,hdn,ut,vt,ut,vt,  &
       dtThEquil,hfilla,sealev,ki,itout,0,ieq,11)
  END DO
!  ieq=0
  Tsurf=Tsurf+Tcool
# 1853

  RETURN
 END SUBROUTINE thermEquil
!*****************************************************************

# 1949


!*****************************************************************
 SUBROUTINE setIceTimeStep()
!*****************************************************************
! timestep reduction only limited by # of reductions (stpfmin) and minimum timestep (tstmpMIN)

  INTEGER :: itcyr, icfl, kt, istartCFL=1
  REAL :: dtCFL
# 1964

  REAL, PARAMETER :: CFLfactor=0.85 !rechecked for nn5303 and hiRes nn27001 GRcal may/20
# 1968


# 1973



  IF(ktcur(ki)>ntchng(ki)) THEN
    PRINT *,"warning: ktcur>ktchng for ki:",ki
    PRINT *,"=> restart file from run with different tstep input"
    PRINT *,"ktcur(ki) reset to ntchng(ki)"
    ktcur(ki)=ntchng(ki)
    stpfact(ki)=1.0
    istp(ki)=1
  ENDIF

  istp(ki)=istp(ki)+1
  IF(istp(ki)>istmx .AND. stpfact(ki)<0.8)THEN
    stpfact(ki)=stpfact(ki)*2.
    istp(ki)=0
    tstmp=tstepnw(ktcur(ki),ki)*stpfact(ki)
!t          write(*,*)'newdelt 1'
    CALL newdelt(tstmp,tym,ki,1)
  END IF

!......adjust time step from tstep.* file
  itcyr=NINT(tym/100.)
  IF(ntchng(ki)>0) THEN
    DO kt=1,ntchng(ki)
      IF(itcyr == itchng(kt,ki)) THEN
# 2001

        CALL  newdelt(tstepnw(kt,ki),tym,ki,1)
        ktcur(ki)=kt
        tstmp=tstepnw(ktcur(ki),ki)
        stpfact(ki)=1.0
        RETURN
      END IF
    END DO
  END IF

!...adjust time step down to current step
  IF(stpfact(ki)< 0.98)THEN
    tstmp=tstepnw(ktcur(ki),ki)*stpfact(ki)
  ELSE
    tstmp=tstepnw(ktcur(ki),ki) ! tstepnw(0,ki) set to input delt in SUBROUTINE iconst
  END IF
# 2020

  CALL newdelt(tstmp,tym,ki,0)

!***CFL timestep control on delt*******************************
  icfl=0
  umx(ki)=MAX(umx(ki),10.)
  vmx(ki)=MAX(vmx(ki),10.)

# 2030

  dtCFL=CFLfactor*MIN(dyv(imx,jmx)/vmx(ki),dxu(imx,jmx)/umx(ki)) ! proper

  IF(istartCFL==1) THEN
    PRINT *,"dtCFL reduction factor is ",CFLfactor
    istartCFL=0
  ENDIF
# 2039

!  dtCFL=0.85*Re*MIN(dthe/vmx(ki),s(jmx,ki)*dphi/umx(ki))
!      if( sqrt(umx**2+vmx**2)*delt/(Re*dthe)>0.9) then

  DO WHILE ( tstmp>(0.99*dtCFL))
    IF(stpfact(ki) > stpfmin) THEN
      stpfact(ki)=stpfact(ki)/2.
      tstmp=tstepnw(ktcur(ki),ki)*stpfact(ki)
      IF(tstmp< tstmpMIN) THEN !minimum timestep, eg 0.15625
        stpfact(ki)=stpfact(ki)*2.
        PRINT *,'CFL tstepmin',tstmp,tym,ki, dtCFL
        EXIT
      END IF
    ELSE
      PRINT *,'stpfact(ki) limit on CFL'
      EXIT
    END IF
    icfl=1
  END DO

532 CONTINUE

  IF(icfl == 1) THEN
    PRINT *,'CFL tstmp,dtCFL ',tstmp, dtCFL,umx(ki),vmx(ki) !imx,jmx not yet set except for NICE=1 with save
    CALL newdelt(tstmp,tym,ki,1)
  END IF
  IF(delt<0.001) CALL outerr('delt is too small in setIceTimeStep')
  RETURN
 END SUBROUTINE setIceTimeStep
!*****************************************************************

!*****************************************************************
 SUBROUTINE setThermoTimeStep(dtTh,nthermr)
!*****************************************************************


 REAL, PARAMETER :: dtThmax=4.0
 REAL, PARAMETER :: dtThmin=1.0
# 2080


! REAL, PARAMETER :: dtThmin=2.5
! above must mod into delt of the ISM
! problem: nthermr (delt thermo/delt ISM) must be >1
!     therefore must use 5.0, 2.5, 1.25,.. to get decent factors

  REAL, INTENT(OUT) :: dtTh
  INTEGER, INTENT(OUT) :: nthermr
!  INTEGER :: itt


  dtTh=REAL(ntherm0)*delt
  IF(dtTh<dtThmin) THEN
    nthermr=NINT(dtThmin/delt)
  ELSE IF(dtTh>dtThmax) THEN
    nthermr=NINT(dtThmax/delt)
  ELSE
    nthermr=ntherm0
  END IF

  dtTh=delt*REAL(nthermr)

  IF(MOD(ntice,nthermr) /= 0) CALL outerrI2('setThermoTimeStep: nthermr does not mod into ntice',ntice,nthermr)
!  IF(MOD(ntice,nthermr) /= 0) THEN
!    nthermr=nthermr+1
!    DO itt=1,10
!      IF(MOD(ntice,nthermr) == 0) EXIT
!      nthermr=nthermr-1
!    END DO
!  END IF


! will crash if nthermr=1
!  IF(nthermr <2)THEN
!    PRINT *,'delt,ntherm0,ntice,dtThmax,dtTh,nthermr ',delt,ntherm0,  &
!       ntice,dtThmax,dtTh, nthermr
!    CALL outerr('cant mod thermo into ISM, nthermr=1,  &
!       try delt=25/2**n')
!  END IF
  RETURN
 END SUBROUTINE setThermoTimeStep
!*****************************************************************

!*****************************************************************
 SUBROUTINE stepThermo()
!*****************************************************************
! recompute d3,d4,dvs
!###########################################
# 2131


  INTEGER :: ib, jb


  CALL setArh(H,Tphom) !in
  CALL phomCompUV(heatiPH, uph, vph, &  !out
     H, hdn, agd, Gb, ubt, vbt, delt, 0, Icheck) !in, ifcompw=0 : Gb and agd not used for this call
  CALL phomGRID2GSM(u,uph, Icheck) !kz=2:NCZ-1
  CALL phomGRID2GSM(v,vph, Icheck)
  CALL phomGRID2GSM(heati,heatiPH, Icheck)
  heati(0:ni,0:nj,NCZ)=heatiPH(0:ni,0:nj,NLEV) ! heatiPH(:,:,NLEVp)=0 since interface, but NCZ is bottom cell
! BSL
!     uPHOM=u
!     vPHOM=v
! ^^^
!!$     PRINT *,"v",u(31,46,:)
!!$     PRINT *,"vph",uph(31,46,:)
!!$     PRINT *,"heati",heati(31,46,:)
!!$     PRINT *,"heatiPH",heatiPH(31,46,:)
# 2156


# 2163

!###########################################

# 2170

!     dtTh=delt*REAL(nthermr)
  IF(nt == ntice .AND. ki<3) THEN
    IF(MOD(nint(tnew),500) == 0) THEN
      itout=1
    ELSE
      itout=0
    END IF
  ELSE
    itout=0
  END IF


! INPUTS: all interface variables, modTsurf; CONSTANT INPUTS: modbedtherm
! OUTPUTS: Tc, Gb
# 2193

  u=MIN(1.e5,MAX(-1.e5,u))
  v=MIN(1.e5,MAX(-1.e5,v))
  CALL thermo(hp,h,hdn,u,v,uold,vold,dtTh,  &
     hfilla,sealev,ki,itout,1,ieq,11) ! compute Temp(H(tnew),tnew,uv(H(tnew),T(told))
  ifTiceNew=1
  uold=u
  vold=v


# 2207


  RETURN
 END SUBROUTINE stepThermo
!*****************************************************************

# 2345
!   NOT defined (1)

!*****************************************************************
 SUBROUTINE calctop(hin,hout) !set iffloat, hd, hdn, hde, hpb
!*****************************************************************

  REAL, INTENT(IN) :: hin(0:mNAp,0:mNBp)
  REAL, INTENT(OUT) :: hout(0:mNAp,0:mNBp)
  REAL :: bouy1

   DO  j=0,njp
     DO  i=0,nip
       hout(i,j)=hin(i,j)
       hd(i,j)=hout(i,j)-hp(i,j)
       bouy1=RHCONV*(hp(i,j)+sealev(i,j,ki))
       IF(bouy1 > hout(i,j)) THEN
         hpb(i,j)=hout(i,j)/RHCONV  - sealev(i,j,ki)
         iffloat(i,j)=1
       ELSE
         hpb(i,j) = hp(i,j)
         iffloat(i,j)=0
       ENDIF
       hdn(i,j)=hout(i,j)-hpb(i,j)
!       IF(ki==kEA) THEN
!         hde(i,j)=0.5*(hdn(i,j)+hd(i,j))
         hde(i,j)=0.01*(99.*hdn(i,j)+hd(i,j)) !adhoc fix for ice island problem
!       ELSE
!         hde(i,j)=hdn(i,j)
!       ENDIF
     END DO
   END DO
   RETURN
!*****************************************************************
  END SUBROUTINE calctop
!*****************************************************************

# 2414


!*****************************************************************
 SUBROUTINE icedynout(tnew,tf,ki)
!*****************************************************************
!************-WRITE OUTPUT**********************************-

!t  USE modPHOMvarhydrol, ONLY : coulbu, coulbv, maskcoulbu, maskcoulbv

  IMPLICIT NONE
  
  REAL, INTENT(IN) :: tnew, tf
  INTEGER, INTENT(IN) :: ki
  INTEGER :: ihts, ih2d, ihrest, ih5h, ihk, ihe2d, iceSheetID, iub, i, j
  INTEGER :: iwarmbase(mNA,mNB)
  INTEGER :: jlatgrp, ilongrp, iht


  iht=NINT(tnew)

  IF(tnew> -21000.0) THEN
    WHERE (h(1:ni,1:nj)>101. .AND. (Tc(1:ni,1:nj,NCZ)+h(1:ni,1:nj)*BETAP) >-0.01) 
      iwarmbase(1:ni,1:nj)=1
    ELSEWHERE
      iwarmbase(1:ni,1:nj)=0
    END WHERE
    IF( ABS(tnew-tf) < 5.0) THEN ! idwrm output
      iub=105+ki
      WRITE(iub,'("# iwarmbase (if warm basal ice anytime over) last 21kyr ",i3,1x,i3)') ni,nj
      DO j=nj,1,-1
        WRITE(iub,'(i1)')(iwarmbase(i,j),i=1,ni)
      END DO
    END IF
  END IF

!     jlatgrp=int((ylatma(ki)+dthed*0.5-72.58)/dthed +0.5)
  IF(ijouttran == 1) THEN
    jlatgrp=njo(ki,2)
    iub=90+ki
    CALL jouttran(jlatgrp,hd,h,u,v,iub,ki)

    iub=92
    CALL outtran(nio(ki,2),hd,h,u,v,iub,ki)
    CALL diagnoLISsq(tnew,tf,timeInit,h,u,1,HOthr)

  END IF
! PRINT *,"TESToutjoutt1 uold :",INT(tnew),SUM(uold(1:ni,1:nj,:)),k
  IF(iVF2out == 1) THEN
    i2=nio(1,2)
    j2=njo(1,2)
    IF(tnew>-60000.) CALL outvF2(tnew,i2,j2,hd,h,u,Tc,agd,agdb,resid,122,ki)
  END IF

!     nrec is in years
  ihts=MOD(iht,nrec2)        !integer output interval(year) for time series
  ih2d=MOD(iht,nrec)         !integer output interval(year) for 2-d ice field
  IF(iht>-14690 .AND. iht<-13510) ihts=0
!  IF(iht>-39500.0 .AND. iht<-33000.0) ihts=0

  ihrest=MOD(iht,25000)
  ih5k=MOD(iht,5000)
  ih5h=MOD(iht,500)
  ihk=MOD(iht,1000)

!      if(iht.eq. -30000) call outHF(tnew,hd,h,u,v,delt,97,ki)


  IF(ki==kEA) THEN
    ih2d=1
    IF(iht == -85000) ih2d=0
    IF(iht == -65000) ih2d=0
    IF(iht == -19000) ih2d=0
  ENDIF


  ihe2d=1
  IF(iht == itym2dout) ihe2d=0
!      if(ih5k.eq.0) ihE2d=0
!g      if(iht.eq.-261000) ihE2d=0

!*************** EISMINT OUTPUT**********
!      if(ihk==0 .and. iht>-21001.0) call outHFs4(tnew,hp,H,u,v,145,ki)
!      if(ih5h==0 .and. iht>-26001.0 .and. iht<-5990.)
! #@ KEY dt=500 year idHhpuv output frequency setting
# 2501

!  IF( ih5k==0 .AND. iht>-120001.0 .AND.  (iht<-7990. .OR. (ki==kNA .AND. iht<-4990.) .OR. ki==kGRonly .OR. ki==kANT) )  &
!  IF( ih5k==0 .AND.  (iht<-7990. .OR. (ki==kNA .AND. iht<-4990.) .OR. ki==kGRonly .OR. ki==kANT) )  &
  IF( ih2d==0 .AND.  ((ki==kEA .AND. iht<-7990) .OR. (ki==kNA .AND. iht<-4990) .OR. (ki/=kEA .AND. ki/=kNA)) )  THEN
# 2507

    CALL outHFs5(tnew,hp,h,u,v,127,ki) !idHhpuv, use this unit# for 32 to 26.5ka 500 year slices

  ENDIF

# 2514




# 2520


# 2527


  IF(ki==kANT) THEN
    IF( iht >= -130000 .AND. iht <= -125000 .AND. MOD(iht,1000) == 0 ) ih2d=0
    IF(iht == -205000) ih2d=0
    IF(iht == -115000) ih2d=0
    IF(iht == -110000) ih2d=0
    IF( iht >= -25000 .AND. iht <= 0 .AND. MOD(iht,1000) == 0 ) ih2d=0
  ENDIF

# 2539


# 2547

  IF( MOD(iht,20000)==0  .AND.iht>-62000 .AND. iht< -11000)  CALL outHF3(tnew,hd,h,u,v,121,ki) !idat121/131 detailed output
  IF(iht==0 .AND. (ki==kGRonly .OR. ki==kANT)) CALL outHF3(tnew,hd,h,u,v,121,ki)



! PRINT *,"TESToutHF3 uold :",INT(tnew),SUM(uold(1:ni,1:nj,:)),k

  IF(ihe2d == 0) THEN
!    CALL outHF3(tnew,hd,h,u,v,81,ki) !idat121/131 detailed output

    IF(ijouttran==2) THEN
      ilongrp=nio(ki,1)
      iub=90+ki
      CALL outtran(ilongrp,hd,h,u,v,iub,ki)
    END IF
!         call outVF(tnew,njo(ki,1),hd,H,u,v,82,ki)
!         call outVFA(tnew,njo(ki,1),hd,H,u,v,82,ki)


!$$$c****
!$$$         ihdmax=0
!$$$         jhdmax=0
!$$$         do j=1,nj
!$$$            do i=1,ni
!$$$               if(hd(i,j)>hdmax .and. h(i,j)>0.1) then
!$$$                  hdmax= hd(i,j)
!$$$                  ihdmax=i
!$$$                  jhdmax=j
!$$$               endif
!$$$            enddo
!$$$         enddo
!$$$         call outBiTz(tnew,ihdmax,jhdmax,H,85,ki)
!**-
!         call outBiTz(tnew,nio(ki,1),njo(ki,1)H,85,ki)
  END IF

# 2589

  IF(ihts == 0) THEN
    IF(ki==kEA .OR. ki==kNA) CALL streamflux(u,v,h,ki)
! PRINT *,"TESTstreamflux uold :",INT(tnew),SUM(uold(1:ni,1:nj,:)),k
    IF(ki==kGRonly .OR. ki==kANT) THEN
      CALL outTVNEA(tnew,123,ki)
# 2597

      IF(ki==kGRonly) THEN
        iub=103
        iceSheetID=2 !iceland
        IF(xlonma(ki)>-12.0) CALL outTVsubIce(tnew,iceSheetID,iub,ki) !iu not modified by k
      ENDIF
    ELSEIF(ki==kNA) THEN
      CALL outTVNAh(tnew,h,hd,123,ki)
      iub=103
      iceSheetID=1 !GR
      IF(xlonma(ki)>-25.0) CALL outTVsubIce(tnew,iceSheetID,iub,ki) !iu not modified by k
# 2610

    ELSEIF(ki==kEA) THEN
      CALL outTVEAh(tnew,h,hd,123,ki)
!      CALL outTBsites(tnew,h,259,ki)
    ELSE
      CALL outTV(tnew,123,ki)
# 2618

    ENDIF
  END IF
! PRINT *,"TESTpresite1 uold :",INT(tnew),SUM(uold(1:ni,1:nj,:)),k
  IF(ihts == 0) CALL outSite1(tnew,h,hd,1,124,ki)


!*****OLD OUTPUT: station**************************
  iub=20+ki
  IF(ihts == 0) CALL output2(tnew,u,v,iub,ki)
!  CALL output2b(iunits(3,ki),ki)
!*****************************************************************
!   WRITE(iunits(3,ki),*)'time(y),suml,sumr,sumd1,per'
!      write(iunit(3,ki),*)'time(y),deltm1,imax1,jmax1,deltm2,imax2,jmax2'


# 2644



!#if defined (ANT) || defined (ANcal)
# 2652


# 2659


# 2663


# 2667


! !!!!!! for testing/tracking of wt
!  CALL UVWout(H)
!  STOP
! !!!!!!



!t  IF(ABS(tnew+236.0e3)<1) THEN
!t    CALL outG3test(hwb(1:mNA,1:mNB),rNeffC(1:mNA,1:mNB),REAL(maskcoulbu(1:mNA,1:mNB)),"hwb, rNeffC,maskcoulbu" ,241)
!t    CALL outG3test(coulbu(1:mNA,1:mNB),coulbv(1:mNA,1:mNB),REAL(maskcoulbv(1:mNA,1:mNB)),"coulbu, coulbv,maskcoulbv" ,242)
!t    CALL outHFs5(tnew,hp,h,u,v,127,ki) !idHhpuv, use this unit# for 32 to 26.5ka 500 year slices
!t    STOP
!t  ENDIF

  RETURN
 END SUBROUTINE icedynout
!*****************************************************************

!*****************************************************************
 SUBROUTINE relaxGIAstep()
!*****************************************************************
!         RHCONV=1025./920.
  INTEGER :: rks
  REAL :: hh, hsea0, hsea

  rks=(rk2+rk3)*0.5

!**old form, does not properly take into account water load****-
!**hpiso for Greenland****************-
  IF(itop(ki)>3 .OR. itop(ki)<1) THEN
    DO  j=0,njp
      DO  i=0,nip
        hp(i,j)=rk1*hp(i,j)+rk2*Hn1(i,j)+rk3*h(i,j) +rk4*hpiso(i,j,ki)
!     protect protect protect protect
        IF(ABS(hp(i,j))<hzero) hp(i,j)=0.0
      END DO
    END DO

!**take into account water load********************-
  ELSE 
    DO  j=0,njp
      DO  i=0,nip
!  rk2=rk3
! assumes 0 sealev(i,j,ki) and present day isostasy at t0
        hsea0=hpo(i,j,ki)*RHCONV
        hsea=(hp(i,j)+sealev(i,j,ki))*RHCONV
        IF(hsea0 < 1.) THEN !not originally marine
!                     hh=h(i,j)+Hn1(i,j) !hh is the netload - load at t0
          hh=MAX(h(i,j)+Hn1(i,j),2.*hsea)
        ELSE IF( hsea < 0.5*(Hn1(i,j)+h(i,j)) )THEN !grounded
          hh=h(i,j)+Hn1(i,j)-2.*hsea0
        ELSE          !floating
          hh=2.*(hsea-hsea0)
        END IF

        hp(i,j)=rk1*hp(i,j)+rks*hh +rk4*hpo(i,j,ki)

!                 protect protect protect protect
        IF(ABS(hp(i,j))<hzero)hp(i,j)=0.0
      END DO
    END DO
    hb=hp*(-1.0)
  END IF

  RETURN
 END SUBROUTINE relaxGIAstep
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!###############################################

!###############################################

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 SUBROUTINE Hybridicestep(ifConvFail,ki)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  USE modPHOMvardyn, ONLY : idieA,idieS,idieC,hbsd,muind,mvind,heatb,heath,siu,siv,sih,iletfail
# 2747

  USE modPHOMgrid
!#t  USE modiceW
# 2754

# 2757


# 2765


# 2770


  IMPLICIT NONE

  INTEGER, INTENT(IN) :: ki
  INTEGER, INTENT(OUT) :: ifConvFail

# 2783


!#t  REAL :: Tb
  INTEGER :: kt
  REAL :: swaterflux, deltb
  INTEGER ::  Icheck=111, istartHyb=1,istartkiHyb(NICE)=1

! for now, set maskwater, indlake, npoilake, hw via findwater call in icedyn

! ifRestart>0 restart
! #@
!  PRINT *,"TEST end"
!  stop

  IF(istartHyb==1) THEN
    CALL setPHOMkzGrid()
    WRITE(*,*)'dd0 =',dd0
!    istartHyb=0
  ENDIF
 
  idieA = 0
  idieS = 1
  idieC = 0

  IF(tstmp< 2.*tstmpMIN)  THEN
    iletfail = 0
  ELSE
    iletfail = 1
  ENDIF

!  PRINT *,"flags",ifRestart, istartHyb(ki),ifConvFail
!  hb=hp*(-1.0)
  hbsd(1:ni,1:nj)=hbstd(1:ni,1:nj,ki)

  IF(ifTiceNew==1) THEN
!d    deltb=MINVAL(Tc(1:ni,1:nj,1:NCZ))
!d    IF(deltb<-60) PRINT *,"TCminERROR: ",deltb
!d    PRINT *,"TCcheck"
    CALL GSMgrid2phom(Tphom,Tc,Icheck)
    ifTiceNew=0
  ENDIF


# 2838


  CALL findwater(hw, maskwater, indlake, npoilake, nlake, Hn1, hb, sealev(:,:,ki), Icheck)  ! sets up maskwater, indlake, npoilake, nlake, hw
  CALL adjustpres(maskwater, indlake, npoilake, nlake, Hn1, hb, hw, hdn, sealev(:,:,ki),Icheck) ! adjusts hw; outputs hdn
  CALL setArh(Hn1,Tphom) !now Arh depends on maskwater, other Ichecks will catch Hn1 Tphom dimension errors
!  CALL basecoef(Hn1,hb,hdn,Gb,Tphom,Tphom(:,:,0),sealev(:,:,ki),Icheck) ! sets crhu/v fracgu/v hgu/v in modicevarhydrol and replaces fracg
  CALL basecoef(Hn1,hb,hdn,Gb,Tphom,Tphom(:,:,0),sealev(:,:,ki),delt,Icheck) ! sets crhu/v fracgu/v hgu/v in modicevarhydrol and replaces fracg
  CALL next2o(ifnexo, maskwater, Hn1) ! sets up ifnexo
  CALL setfhedge(Hn1, hb, maskwater, ifnexo, fedge, hedge, fracgu, fracgv, sealev(:,:,ki), Icheck) ! sets up fedge, hedge and adjusts h


!#################################################################
# 2905

!#################################################################

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# 2949

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% restartDEBUG%%%%%%%%%%%%%%%
# 2954


  IF(istartkiHyb(ki)==1) THEN
! don't want this output for restarted runs
    IF(newrun(ki)==1) CALL icedynout(tnew-delt,tf,ki)
# 2968

    istartHyb=0
    istartkiHyb(ki)=0
  ENDIF

 
  CALL icedyn(Hn1, hdn, uat, vat, ubt, vbt, uia, via, uit, vit, hb, sealev(:,:,ki), agd, &
     tnew-delt, delt,ifConvFail, Icheck)
  CALL phomConvCheck(uat,vat,ifConvFail)
# 2979

  
  u(0:ni,0:nj,NCZ)=ubt(0:ni,0:nj)
  v(0:ni,0:nj,NCZ)=vbt(0:ni,0:nj)
  u(0:ni,0:nj,2)=uat(0:ni,0:nj) ! what gets output to jtran.* transect output
  v(0:ni,0:nj,2)=vat(0:ni,0:nj)

!!$   j=41
!!$   DO i=69,73
!!$     Tb=Tc(i,j,NCZ)+Betap*H(i,j)
!!$     WRITE(242,142)i,hd(i,j),H(i,j),hp(i,j),Tb, &
!!$      Tsurf(i,j),u(i,j,2),v(i,j,2),w(i,j,2),Gb(i,j), agdm(i,j)
!!$   ENDDO
!!$   WRITE(242,*)"*********************",tnew
!!$142 FORMAT((i3,x),3(f7.1,x),2(f7.3,x), &
!!$    2(f10.4,x),f8.4,x,f6.2,6(x,f7.2))

# 3009


  RETURN
 END SUBROUTINE Hybridicestep
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!*******************************************************************
  SUBROUTINE phomConvCheck(uat,vat,ifConvFail)
! In the event of a conv fail on first GSMicedyn call - added iconvfail check
!*******************************************************************

   USE ieee_arithmetic ! is_nan check is critical for catching catastrophic convergence failures

   USE modPHOMvardyn, ONLY : muind,mvind
# 3026

   REAL, PARAMETER :: HsumMAX=REAL(mNA*mNB)*3000.
   REAL, PARAMETER :: HmxBND=6500.
  REAL, DIMENSION(0:mNA,0:mNB), INTENT(IN) :: uat, vat
  INTEGER, INTENT(INOUT) :: ifConvFail
  REAL :: chkHmx,  rninj
  INTEGER :: icntNCwrite=0
 
!  INTEGER :: IcntHighUV=0

   IF (ifConvFail == 0) THEN

     chkHmx=MAXVAL(Hn1(1:ni,1:nj))
# 3071


     IF (ieee_is_nan(SUM(Hn1(1:ni,1:nj)))) THEN ! ifort
       PRINT *, 'Hn1 contains NaN(s): exit at time = ', tnew
       ifConvFail = 2
     ELSEIF(chkHmx>HmxBND) THEN
       PRINT *,'MAXVAL(Hn1)>HmxBND: exit at time = ', tnew
       PRINT *,"Hn1 Max value at i,j:",MAXLOC(Hn1(1:ni,1:nj)), chkHmx
       ifConvFail = 2
     ELSEIF(ifchkHmx==1 .AND. (chkHmx-chkHmxOld)>300.) THEN
       PRINT *,'Hmx Jump > 300m: exit at time = ', tnew
       PRINT *,'Hmx, old Hmx:',chkHmx,chkHmxOld
       PRINT *,"Hn1 Max value at i,j:",MAXLOC(Hn1(1:ni,1:nj))
       ifConvFail = 2
     ELSEIF (ieee_is_nan(SUM(uat(1:ni,1:nj)))) THEN
       PRINT *, 'uat contains NaN(s): exit at time = ', tnew
       ifConvFail = 2
     ELSEIF (ieee_is_nan(SUM(vat(1:ni,1:nj)))) THEN
       PRINT *, 'vat contains NaN(s): exit at time = ', tnew
       ifConvFail = 2
# 3111

     ENDIF
     chkHmxOld=chkHmx
     ifchkHmx=1
   ENDIF

# 3122


# 3126

   IF ((ifConvFail == 0) .OR. (delt < 3.*stpfmin)) THEN
     IF (MOD(tnew,100.) == 0.) THEN
       PRINT *,' '
       PRINT *,'timeice, dtimeice:',tnew, delt
       PRINT *,'SSA grid points - muind, mvind, ki: ', MAXVAL(muind),MAXVAL(mvind), ki
       PRINT *,'Variable |   Mean |           Max |',&
          '           Min |           Sum |'
       rninj=ni*nj
       WRITE(6,855)'Hn1',&
          SUM(Hn1(1:ni,1:nj))/rninj,   MAXVAL(Hn1(1:ni,1:nj)),  MINVAL(Hn1(1:ni,1:nj)),  SUM(Hn1(1:ni,1:nj))
       WRITE(6,855)'hdn',&
          SUM(hdn(1:ni,1:nj))/rninj, MAXVAL(hdn(1:ni,1:nj)), MINVAL(hdn(1:ni,1:nj)), SUM(hdn(1:ni,1:nj))
       WRITE(6,855)'T',&
          SUM(Tphom(1:ni,1:nj,:))/SIZE(Tphom(1:ni,1:nj,:)), &
          MAXVAL(Tphom(1:ni,1:nj,:)),  MINVAL(Tphom(1:ni,1:nj,:)),  SUM(Tphom(1:ni,1:nj,:))
       WRITE(6,855)'uat',&
          SUM(ABS(uat(1:ni,1:nj)))/rninj,MAXVAL(uat(1:ni,1:nj)),MINVAL(uat(1:ni,1:nj)), SUM(uat(1:ni,1:nj))
       WRITE(6,855)'vat',&
          SUM(ABS(vat(1:ni,1:nj)))/rninj,MAXVAL(vat(1:ni,1:nj)),MINVAL(vat(1:ni,1:nj)), SUM(vat(1:ni,1:nj))
# 3165

       FLUSH(6)
     ENDIF
855  FORMAT (a4 ,4f16.2)
856  FORMAT (a4 ,4E16.2)

!     IF(MOD(tnew,10.)==0.) CALL diagtran(tnew,31,h,93,ki)
   ENDIF
# 3175


   RETURN
  END SUBROUTINE phomConvCheck
!*******************************************************************

!*******************************************************************
 SUBROUTINE setPHOMmasks(Hnm)
!*******************************************************************

 REAL, INTENT(INOUT) :: Hnm(0:mNAp,0:mNBp) 
 INTEGER ::  Icheck=111

  CALL findwater(hw, maskwater, indlake, npoilake, nlake, Hnm, hb, sealev(:,:,ki), Icheck)  ! sets up maskwater, indlake, npoilake, nlake, hw
# 3194

  RETURN
 END SUBROUTINE setPHOMmasks
!*******************************************************************

!###############################################
# 3432

!###############################################


!#if defined (HybGSMcompUVtest) || ! defined (1)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE  compdedge(ifBtFact) ! computes d3,d4,dus,dvs and if ifBtFact==1 d1,d2
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! 1) prevents flux down bare cliff/slope,
! 2) applies shelf enhancement factor (is this duplicating dsb?)
! 3) upwinds at marine ice margin (be it shelf edge or GL)
! 4) interpolative weighting at grounding line

 IMPLICIT NONE

 INTEGER, INTENT(IN) :: ifBtFact

 REAL, PARAMETER :: Hmin=5.0 
 REAL :: ddct, hh, xgl

   DO j=1,nj
     DO i=0,ni
       hh=0.5*(H(i,j)+H(i+1,j))
!       d4(i,j)=0.5*(ddc(i,j)+ddc(i+1,j))
       ddct=0.5*(ddc(i,j)+ddc(i+1,j))
       d4(i,j)=0.5*(ddc(i,j)*ddc(i+1,j))/(ddct+1.e-60) + 0.5*ddct !avg(harmonic,mean)
!       dus(i,j)=0.5*(duvs(i,j)+duvs(i+1,j)) !duvs is cell centered from compdm
!       dus(i,j)=0.5*(duvs(i,j)+duvs(i+1,j))*hh !duvs is cell centered from compdm
       ddct=0.5*(duvs(i,j)+duvs(i+1,j))
!       dus(i,j)=hh*( 0.5*duvs(i,j)*duvs(i+1,j)/(ddct+1.e-60) + 0.5*ddct ) !duvs is cell centered from compdm, avg(harmonic,mean)
       dus(i,j)=hh*duvs(i,j)*duvs(i+1,j)/(ddct+1.e-60) !duvs is cell centered from compdm, harmonic mean
! since mass con matrix (td?,tb?) uses hdn, use hdn here
       IF( (h(i,j)<HdiffMIN .AND. hdn(i,j)>(HDdmn+hdn(i+1,j))) .OR.   &
          (h(i+1,j)<HdiffMIN .AND. hdn(i+1,j)>(HDdmn+hdn(i,j))) ) THEN
         d4(i,j)=0.
         dus(i,j)=0.
       ELSE ! impose shelf enhancement or grounding line treatment if applicable
         IF( h(i,j)>Hmin) THEN
           IF(h(i+1,j)>Hmin) THEN 
             IF(iffloat(i,j)==1) THEN
               IF(iffloat(i+1,j)==1) THEN
                 dus(i,j)=FSHLF*dus(i,j) !not shelf edge, so use interpolated mean
               ELSE !iffloat(i+1,j)==0 && iffloat(i,j)==1 , ie grounding line, i+1,j grounded
                 xgl=MIN(1.0,(H(i+1,j)-RHCONV*(sealev(i+1,j,ki)+hp(i+1,j))) / & !H0-depth0
                    ( (H(i+1,j)-H(i,j)) - RHCONV*(sealev(i+1,j,ki)-sealev(i,j,ki) + hp(i+1,j)-hp(i,j)) ))
                 xgl=MAX(xgl,0.)
                 dus(i,j)=xgl*duvs(i+1,j)*H(i+1,j)+(1.-xgl)*FSHLF*duvs(i,j)*H(i,j)
               ENDIF
             ELSEIF(iffloat(i+1,j)==1) THEN ! && iffloat(i,j)==0 , ie grounding line, i,j grounded
               xgl=MIN(1.0,(H(i,j)-RHCONV*(sealev(i,j,ki)+hp(i,j))) / & !H0-depth0
                  ( (H(i,j)-H(i+1,j)) + RHCONV*(sealev(i+1,j,ki)-sealev(i,j,ki) + hp(i+1,j)-hp(i,j)) ))
               xgl=MAX(xgl,0.)
               dus(i,j)=xgl*duvs(i,j)*H(i,j)+(1.-xgl)*FSHLF*duvs(i+1,j)*H(i+1,j)
             ENDIF
           ELSEIF(iffloat(i,j)==1) THEN ! && H(i+1,j)<=Hmin
             dus(i,j)=FSHLF*duvs(i,j)*H(i,j) !upwind at shelf edge
!             d4(i,j)=ddc(i,j)
           ELSE  ! iffloat(i,j)==0 && H(i+1,j)<=Hmin
             dus(i,j)=duvs(i,j)*H(i,j) !upwind grounded ice margin (could include GL)
!             dus(i,j)=duvs(i,j)*hh !upwind grounded ice margin (could include GL)
             d4(i,j)=ddc(i,j)
           ENDIF
         ELSEIF(h(i+1,j)>Hmin) THEN ! && H(i,j)<=Hmin
           IF(iffloat(i+1,j)==1) THEN 
             dus(i,j)=FSHLF*duvs(i+1,j)*H(i+1,j) !upwind at shelf edge
!             d4(i,j)=ddc(i+1,j)
           ELSE !ice edge, iffloat(i+1,j)==0 && H(i,j)<=Hmin, upwind to grounded ice
             dus(i,j)=duvs(i+1,j)*H(i+1,j) !upwind grounded ice margin (could include GL)
!             dus(i,j)=duvs(i+1,j)*hh !upwind grounded ice margin (could include GL)
             d4(i,j)=ddc(i+1,j)
           ENDIF
         ENDIF
       ENDIF
       IF(ifBtFact==1) d2(i,j)=Btu(i,j)*d4(i,j)+dus(i,j)*hh
     END DO
   END DO
!   IF(ifBtFact==1) d2(0:ni,1:nj)=Btu(0:ni,1:nj)*d4(0:ni,1:nj)+dus(0:ni,1:nj)*hh

   DO j=0,nj
     DO i=1,ni
       hh=0.5*(H(i,j)+H(i,j+1))
!       d3(i,j)=0.5*(ddc(i,j)+ddc(i,j+1))
       ddct=0.5*(ddc(i,j)+ddc(i,j+1))
       d3(i,j)=0.5*(ddc(i,j)*ddc(i,j+1))/(ddct+1.e-60) +0.5*ddct !avg(harmonic,mean)
!       dvs(i,j)=0.5*(duvs(i,j)+duvs(i,j+1))*hh
       ddct=0.5*(duvs(i,j)+duvs(i,j+1))
!       dvs(i,j)=hh*(0.5*(duvs(i,j)*duvs(i,j+1))/(ddct+1.e-60) + 0.5*ddct) !duvs is cell centered from compdm, avg(harmonic,mean)
       dvs(i,j)=hh*duvs(i,j)*duvs(i,j+1)/(ddct+1.e-60) !duvs is cell centered from compdm, harmonic mean

       IF( (h(i,j)<HdiffMIN .AND. hdn(i,j)>(HDdmn+hdn(i,j+1))) .OR.   &
          (h(i,j+1)<HdiffMIN .AND. hdn(i,j+1)>(HDdmn+hdn(i,j))) ) THEN
         d3(i,j)=0.
         dvs(i,j)=0.
       ELSE ! impose shelf enhancement or grounding line treatment if applicable

         IF( h(i,j)>Hmin) THEN
           IF(h(i,j+1)>Hmin) THEN 
             IF(iffloat(i,j)==1) THEN
               IF(iffloat(i,j+1)==1) THEN
                 dvs(i,j)=FSHLF*dvs(i,j) !not shelf edge, so use interpolated mean
               ELSE !iffloat(i,j+1)==0 && iffloat(i,j)==1 , ie grounding line, i,j+1 grounded
                 xgl=MIN(1.0,(H(i,j+1)-RHCONV*(sealev(i,j+1,ki)+hp(i,j+1))) / & !H0-depth0
                    ( (H(i,j+1)-H(i,j)) - RHCONV*(sealev(i,j+1,ki)-sealev(i,j,ki) + hp(i,j+1)-hp(i,j)) ))
                 xgl=MAX(xgl,0.)
                 dvs(i,j)=xgl*duvs(i,j+1)*H(i,j+1) + (1.-xgl)*FSHLF*duvs(i,j)*H(i,j)
               ENDIF
             ELSEIF(iffloat(i,j+1)==1) THEN ! && iffloat(i,j)==0 , ie grounding line, i,j grounded
               xgl=MIN(1.0,(H(i,j)-RHCONV*(sealev(i,j,ki)+hp(i,j))) / & !H0-depth0
                  ( (H(i,j)-H(i,j+1)) + RHCONV*(sealev(i,j+1,ki)-sealev(i,j,ki) + hp(i,j+1)-hp(i,j)) ))
               xgl=MAX(xgl,0.)
               dvs(i,j)=xgl*duvs(i,j)*H(i,j)+(1.-xgl)*FSHLF*duvs(i,j+1)*H(i,j+1)
             ENDIF
           ELSEIF(iffloat(i,j)==1) THEN ! && H(i,j+1)<Hmin
             dvs(i,j)=FSHLF*duvs(i,j)*H(i,j) !upwind at shelf edge
!             d3(i,j)=ddc(i,j)
           ELSE ! H(i,j+1)<Hmin && iffloat(i,j)==0
             dvs(i,j)=duvs(i,j)*H(i,j) !upwind grounded ice margin (could include GL)
!             dvs(i,j)=duvs(i,j)*hh !upwind grounded ice margin (could include GL)
             d3(i,j)=ddc(i,j)
           ENDIF
         ELSEIF(h(i,j+1)>Hmin) THEN ! && H(i,j)<=Hmin
           IF(iffloat(i,j+1)==1) THEN ! &&iffloat(i,j)=0, grounding line, i,j grounded
             dvs(i,j)=FSHLF*duvs(i,j+1)*H(i,j+1) !upwind at shelf edge
!             d3(i,j)=ddc(i,j+1)
           ELSE !ice edge, h(i,j+1)>Hmin && h(i,j)<=Hmin && iffloat(i,j+1)==0, upwind to grounded ice dvs(i,j)=duvs(i,j)
             dvs(i,j)=duvs(i,j+1)*H(i,j+1) !upwind grounded ice margin (could include GL)
!             dvs(i,j)=duvs(i,j+1)*hh !upwind grounded ice margin (could include GL)
             d3(i,j)=ddc(i,j+1)
           ENDIF
         ENDIF
       ENDIF
       IF(ifBtFact==1) d1(i,j)=Btv(i,j)*d3(i,j)+dvs(i,j)*hh
     END DO
   END DO
!   IF(ifBtFact==1) d1(1:ni,0:nj)=Btv(1:ni,0:nj)*d3(1:ni,0:nj)+dvs(1:ni,0:nj)*hh

 RETURN
END SUBROUTINE compdedge
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!#################if defined (HybGSMcompUVtest) || ! defined (1)###############################
!#endif
!###############################################


# 3696



!###############################################
# 3970


END SUBROUTINE icedynGSM
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

# 4011


