# 1 "pGSMicetherm.F90"
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE thermo(hp,H,hdn,u,v,uold,vold,dtt, &
   hfilla,sealev,ki,itout,icompw,ieq,i11chck)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     i11chck=call check. stop with error if i11chck != 11 or 10, (10 ->
!     reset Hold)
!     i11chck to ensure that subroutine is called correctly (ie not with
!     old call)
!     ieq =1 => C_rock =C_rock/1000 for equilibrium computation
!      !  Tsurf will have to be COMMONed
!     THERMODYNAMIC BLOCK
!     computes ice Temperature field (deg C): Tc and grids to zeta ISM grid
!     also computes basal net massbalance: Gb(i,j)
!     Horizontal BCs: endpoints are equated to interior points,
!     but corner points are neither used nor set
!     Hold local
!     INPUTS: all interface variables, modTsurf; CONSTANT INPUTS: modbedtherm
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! flow chart:

! compute w field
! compute full dth, ntnh
!1 loop ntnh

!2 loop j
!2 save old T(i,kz)

!3 loop i
!3 compute inner nested time dtzt,ntn

!4 loop itn to ntn
!4     compute matrix coeficients
!4     solve matrix equation

!3 if itnh  =  =  1, compute thin ice terms
!3 if itnh  =  =  ntnh, compute Gb basal mass balance

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm
 USE modbedtherm !inputs, grid stuff, physical parameters, Tmtrx?
 USE modicetherm ! Tit , Tmin, Tmax
 USE modconstants
 USE modGb !output
 USE modTsurf !input
 USE modiceW
 USE modicerheology, ONLY : Tmaxsafe
 USE modTmtrx, ONLY : Tmtrx1,Tmtrx2,Tmtrx3,Tmtr13

   USE ieee_arithmetic ! is_nan check

# 57



 IMPLICIT NONE

! INTEGER, PARAMETER :: NTZp = NTZ+1
 REAL, PARAMETER :: Hmin = 50., Betapd = 1.11*Betap
 REAL, PARAMETER :: ToceanDEEP=0.

!      REAL, PARAMETER :: Tbed = -1.)  ! for interpolating T ie assume net accum at Tbed
 REAL, PARAMETER :: dzoL = dzet/rLi
!      REAL, PARAMETER :: Tmax = 4.9) !  maximum allowable ice temperature
 REAL, PARAMETER :: CmF=Cm/1000., RhoCmF = rhor*CmF
!     passed variables-----------------------:
! ----------     INPUT ------------
!     REAL ui(0:mNA,0:mNB,NCZ), vi(0:mNA,0:mNB,NCZ)   ! velocities, ice grid
 REAL,DIMENSION(0:mNAp,0:mNBp),INTENT(IN) :: hp,H,hdn
 REAL, INTENT(IN) :: u(0:mNA,0:mNB,NTZ),v(0:mNA,0:mNB,NTZ)  ! interface velocities, T grid vertically
 REAL, INTENT(IN) :: uold(0:mNA,0:mNB,NTZ),vold(0:mNA,0:mNB,NTZ)
 REAL,INTENT(IN) ::  hfilla(mNA,mNB), sealev(0:mNAP,0:mNBP,NICE), dtt
 INTEGER, INTENT(IN) :: ki,itout,icompw,ieq,i11chck

!   COMMONed OUT*****************
! COMMON/cGb/Gb

!     COMMONed IN/OUT************
! COMMON/Atherm/Tit


!     following are IN only*****
! COMMON/cGf/GHF(mNA,mNB,NICE)
! COMMON/ctsurf/Tsurf(mNA,mNB)
! COMMON/cThbound/Tmin,Tmax
!      COMMON/Itime/t            !must also copy to ifield for age model

!     local*****************
 REAL :: Tsurfo(mNA,mNB)
 REAL :: wk(NTZp)
!     REAL tit(mNAp,mNBp,NTZ), titnw(mNAp,mNBp,NTZ)
 REAL :: Tmtrx4(NTFZ),dtzt
 REAL :: Tdmtrx2(NTFZ)
 REAL :: Tbds(0:mNAp,0:mNBp)

 INTEGER :: kz, i, j, itn, ntnh, itnh, ifloat, kTpm, ntn, ic, jc, ntnr, itnr, ktz, imx, jmx
 REAL :: rdhocdt,rhocdt, Ap0, dth, ucocf, vcocf, Qb, Tbd, bouy, Tsurfn, vavg, vmx, vmn
 REAL :: Tpm, ft, ftTs, qbar1, Gb1, ttsurf, ttsurfb, hdm, wdepth, Tavg, Tstatmx, Tstatmn
 REAL :: Tbavg, Tr2avg, Gbtmp

!!! NNTOP3 local variables
REAL :: Tpmr, Tpc
INTEGER :: kbz

 REAL :: Hold(0:mNAp,0:mNBp,NICE)=0.
 INTEGER :: istart(NICE)=1, icntTpmerror=0, icnwarn
 REAL, PARAMETER :: Tminp=Tmin+1
! statement functions::::::::
 REAL :: Ci, tt, tki, tkapi


!----------------Statement Fns-------------------------
# 120

 tki(tt) = 9.828*EXP(-0.0057*(tt+273.15))  ! Thermal conductivity, W m^-1 K^-1
 Ci(tt) = 152.5 + 7.122*(tt+273.15)


 tkapi(tt) = SECYR*tki(tt)/(RHOIc*Ci(tt))  ! m^2/a


 IF(i11chck > 12 .OR. i11chck < 10) &
    CALL outerr('i11chck error in thermo(), call list bad')

! reset of Hold under convergence failure, or under restart
 IF(i11chck == 10) THEN
   Hold(1:ni,1:nj,ki) = H(1:ni,1:nj)
   istart(ki)=0
   RETURN
 ENDIF


 IF(istart(ki)==1) THEN
   Hold(1:ni,1:nj,ki) = H(1:ni,1:nj)
   istart(ki)=0
 ENDIF


!************thin or no ice bedtherm matrix terms**********
 IF(ieq == 1) THEN
   rdhocdt = RhoCmF/dtt
 ELSE
   rdhocdt = RhoCm/dtt
 ENDIF


 DO kz = NCZp,NTFZ ! this is only for itnh=1 Tbed only calc (ie no ice cover)
   Ap0 = rdhocdt*dsz(kz-NCZ)
   Tdmtrx2(kz) = Ap0+Tmtr13(kz)
 ENDDO

!**********************************************


!**********************************************


 rHeatBedCellSrc=0.


!      tnew=t+dtt
 Tbds(0:nip,0:njp)=tit(0:nip,0:njp,NTZ)

!     CALL igrd2tgrd(u,ui,ni,nj,nkz)
!     CALL igrd2tgrd(v,vi,ni,nj,nkz)

!     determine w(zet)
!      IF (icompw.eq.1) CALL compw(w,u,v,H,hp,hpold,dtt,ki)
 IF (icompw == 1) CALL compwk(wadv,u,v,H,Hold(:,:,ki),dtt)

 wadv=MIN(1.e2,MAX(-1.e2,wadv))
 icnwarn=0

!***********nested full thermo for advection term****
 ntnh = 1
 dth = dtt

 DO j = 1,nj
   DO i = 1,ni

     ucocf = MAX(0.001,ABS(u(i,j,2)),ABS(u(i-1,j,2)),ABS(uold(i,j,2)),ABS(uold(i-1,j,2)))/dxg(i,j)
     vcocf = MAX(0.001,ABS(v(i,j,2)),ABS(v(i,j-1,2)),ABS(vold(i,j,2)),ABS(vold(i,j-1,2)))/dyg(i,j)
     vcocf = MAX(.0001,ABS(vcocf))
# 192

     dth = MIN(dth,1./ucocf)

     dth = MIN(dth,1./vcocf)

     ntnh = MIN(MAX( ntnh, INT(0.99999 + dtt/dth) ),100)
!            ntnh = max( ntnh, int(0.99 + dtt/dth) )

   ENDDO
 ENDDO
 dth = dtt/REAL(ntnh)

!***********nested full thermo for advection term****
 DO itnh = 1,ntnh  ! full nested time loops
   ft = REAL(itnh)/REAL(ntnh)

!     loop i,j, determine tstep
   DO j = 1,nj 

!  save old T's for jrow
     CALL compTcoef(Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4,Qb,tit, &
        hdn,H,Hold(1,j,ki),u,v,uold,vold,wk,dtt,0,0 &
        ,ntn,1,j,ki,ifloat,Tbd)
     DO i = 1,ni
       wk=wadv(i,j,:)

! this version needs sealev(i,j,ki) and hfilla inputs from subroutine ifield
! problem if bouy>5.0 for landlocked depression
       bouy=RHCONV*(hp(i,j)+sealev(i,j,ki))-H(i,j)
       IF(bouy >5.0 .AND. hfilla(i,j)<1.0) THEN
         ifloat=1
         Tbd=-1.0
       ELSEIF ( (hfilla(i,j)/0.91) > MAX(H(i,j),1.0) ) THEN
         ifloat=1
         Tbd=0.
       ELSE
         ifloat=0
       ENDIF

!     catch Hmin for linear T profile, and continue on H = 0
       IF (H(i,j) > Hmin) THEN
!*************full therm computation

         IF (itnh == 1) Tsurfo(i,j) = tit(i,j,1)

!               IF (Tsurf(i,j)>Tmax)write(91,*)'Tsurf too large,
!     $              Tsurf, i,j: ',Tsurf(i,j),i,j
         IF (Tsurf(i,j)>Tmax) Tsurf(i,j) = Tmax

         IF (Tsurf(i,j) <= Tmin) THEN 
!           WRITE(*,*)'Tsurf too cold, Tsurf, i,j, H: ',Tsurf(i,j),i,j, H(i,j)
           Tsurf(i,j)=Tminp
         ENDIF
!     check for/determine nested dtt
         IF (ABS(wadv(i,j,2))<1.e-20) THEN
           dtzt = ((H(i,j)*dzet)**2)/tkapi(tit(i,j,NTZ))
         ELSE
           dtzt = MIN( H(i,j)*dzet/ABS(wadv(i,j,2)), &
              ((H(i,j)*dzet)**2)/tkapi(tit(i,j,NTZ)))
         ENDIF

         dtzt = 10.* dtzt ! implicit solution below, so don't need to meet CFL

!$$$               dtzt = min(dth,10.* dtzt)
!$$$            ucocf = 0.5*(uold(i,j,2)+uold(i-1,j,2))/(s(j,ki)*Re*dphi)
!$$$            ucocf = max(.0001,abs(ucocf))
!$$$            vcocf = 0.5*(vold(i,j,2)+vold(i,j-1,2))/(Re*dthe)
!$$$            vcocf = max(.0001,abs(vcocf))
!$$$            dtzt = min(dtzt,1./ucocf)
!$$$            dtzt = min(dtzt,1./vcocf)
!$$$               dtzt = 10.* dtzt

         ntn = MAX( INT(0.5 + dth/dtzt), 1)

!               IF (ntn>1) THEN
!                  write(*,*)'nested ntn,ntnh,dtzth: ',
!     $                 ntn,ntnh,dth,i,j
!               ENDIF
!               ntn = 1

! ug
!               IF (ntn>200) THEN
!         IF (ntn>150) THEN
!         IF (ntn>1000) &
         IF (ntn>100) THEN
           dtzt = dth/100.0
           ntn = 100
           IF(icnwarn<2 .AND. H(i,j)>100.0) THEN ! chose 120 to avoid numerous ntn >100 < 120 for NHgcm startup with H just bit > 50 m
             WRITE(*,*)'ntn too big: ntn,H,w,u,v,T3,Tb: ', &
              ntn,H(i,j),wadv(i,j,2),u(i,j,2),v(i,j,2), &
              tit(i,j,3),tit(i,j,NTZ),i,j,icnwarn
             icnwarn=icnwarn+1
           ENDIF
         ENDIF

!--------------------------------------
         IF (ntn == 1) THEN
           itn = 1

!     calc coefficients
           CALL compTcoef(Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4,Qb,tit, &
              hdn,H,Hold(i,j,ki),u,v,uold,vold,wk,dtt,itn,itnh, &
              ntnh,i,j,ki,ifloat,Tbd)


!     surface BC
           Tsurfn = ft*Tsurf(i,j) + (1.-ft)*Tsurfo(i,j)
           tmtrx4(2) = tmtrx4(2)-tmtrx1(2)*Tsurfn  ! tmtrx1(2) is not used in solvmtrx

           Tpm = -(ft*H(i,j) + (1.-ft)*Hold(i,j,ki))*Betap

!************bedtherm matrix terms**********
           IF(ieq == 1) THEN 
             rhocdt = RhoCmF/dth
           ELSE
             rhocdt = RhoCm/dth 
           ENDIF ! rhocdt is reset below for subloops

           DO kz = NCZp,NTFZ
             kbz=kz-NCZ
!######################### bedfrost ###################################
# 322

             Ap0 = rhocdt*dsz(kbz)

!######################### bedfrost ###################################
             Tmtrx2(kz) = Ap0+Tmtr13(kz)
             Tmtrx4(kz) = tit(i,j,kz)*Ap0
           ENDDO
           Tmtrx4(NTFZ) = tit(i,j,NTFZ)*Ap0 + GHF(i,j,ki) ! [GHF]=J/yr
!**********************************************

           CALL solvmtrx(tit,kTpm,Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4, &
              Tpm,i,j,NTZ)  ! solve matrix


!                  CALL chkmtrx(idump,tit,Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4,
!     $                 Tpm,i,j,NTZ,kTpm,0)  ! solve matrix

           tit(i,j,1) = Tsurfn
! Basal melting:
           IF (itnh == ntnh) THEN

             IF (kTpm>1 .AND. kTpm<NTZp) THEN
!               Qbar1 = -tkbs*(Tit(i,j,NTZ)-Tit(i,j,NTZp))/dzbf

!$$$      IF (Qbar1> 1.0) THEN
!$$$         write(155,*)Qbar1,Tit(i,j,NTZ),Tit(i,j,NTZ+1),
!$$$     Tit(i,j,NTZ+2),i,j,H(i,j)
!$$$c         write(155,*)Qbar1,Tit(i,j,NTZ),Tit(i,j,NTZ+1),i,j,H(i,j)
!$$$         Qbar1 = 1.0
!$$$      ELSE IF ( Qbar1< -0.5) THEN
!$$$         write(154,*)Qbar1,Tit(i,j,NTZ),Tit(i,j,NTZ+1),
!$$$     Tit(i,j,NTZ+2),i,j,H(i,j)
!$$$         Qbar1 = -0.5
!$$$      ENDIF

!               Qb = Qb+Qbar1

               IF (tit(i,j,kTpm)< -5. .AND. icntTpmerror<100) THEN
                 IF (tit(i,j,kTpm)< -Betap*H(i,j)) THEN
                   WRITE(*,*)'Tice(kTpm)<Tpm error ',kTpm,tit(i,j,kTpm), H(i,j), tit(i,j,NTZp)
                   icntTpmerror=icntTpmerror+1
                 ENDIF
             ENDIF


               Gb1 =  0.

               IF(kTpm<NTZ) THEN
                 DO ktz = kTpm,NTZ-1
                   Gb1 = Gb1+ Ci(tit(i,j,ktz))*( &
                      Tmtrx1(ktz)*tit(i,j,ktz-1) &
                      +Tmtrx2(ktz)*tit(i,j,ktz) &
                      +Tmtrx3(ktz)*tit(i,j,ktz+1) &
                      -Tmtrx4(ktz) )
                 ENDDO
               ENDIF

               Gb(i,j) = dzoL*H(i,j)*( Gb1 &
                  + Ci(tit(i,j,NTZ))*0.5*( &
                  Tmtrx1(NTZ)*tit(i,j,NTZ-1) &
                  +Tmtrx2(NTZ)*tit(i,j,NTZ) &
                  +Tmtrx3(NTZ)*tit(i,j,NTZ+1) &
                  -Tmtrx4(NTZ) ) )

               IF (tit(i,j,NTZ)<-Betap*H(i,j) .AND. Gb(i,j)<0.) tit(i,j,NTZ)=-Betap*H(i,j) ! above already reduced Gb for latent Heat cons

# 402

             ELSE
               Gb(i,j) = 0.
             ENDIF

           ENDIF          ! itnh = ntnh

!-----------case ntn > 1 , nested loops--------------
         ELSE

!                  Tsurfo = tit(i,j,1)

           ntnr = ntnh*ntn
           DO itn = 1,ntn
!     calc coefficients
             itnr = (itnh-1)*ntn + itn
             CALL compTcoef(Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4,Qb,tit, &
                hdn,H,Hold(i,j,ki),u,v,uold,vold,wk,dtt,itn,itnr, &
                ntnr,i,j,ki,ifloat,Tbd)

!     surface BC
             ftTs = REAL(itnr)/REAL(ntnr)
             Tsurfn = ftTs*Tsurf(i,j) + (1.-ftTs)*Tsurfo(i,j)
             tmtrx4(2) = tmtrx4(2)-tmtrx1(2)*Tsurfn  ! tmtrx1(2) is not used
             dtzt = dtt/REAL(ntnr) ! not extraneous

             Tpm = -(ftTs*H(i,j) + (1.-ftTs)*Hold(i,j,ki))*Betap

!************bedtherm matrix terms**********
             IF(ieq == 1) THEN
               rhocdt = RhoCmF/dtzt
             ELSE
               rhocdt = RhoCm/dtzt
             ENDIF

             DO kz = NCZp,NTFZ
               kbz=kz-NCZ
!######################### bedfrost ###################################
# 449

             Ap0 = rhocdt*dsz(kbz)

!######################### bedfrost ###################################
               Tmtrx2(kz) = Ap0+Tmtr13(kz)
               Tmtrx4(kz) = tit(i,j,kz)*Ap0
             ENDDO
             Tmtrx4(NTFZ) = tit(i,j,NTFZ)*Ap0 + GHF(i,j,ki)
!**********************************************

             CALL solvmtrx(tit,kTpm,Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4, &
                Tpm,i,j,NTZ) ! solve matrix


           ENDDO
           tit(i,j,1) = Tsurf(i,j)

! Basal melting on last ntnh loop:
           IF (itnh == ntnh) THEN
             IF (kTpm>1 .AND. kTpm<NTZp) THEN
!               Qbar1 = -tkbs*(Tit(i,j,NTZ)-Tit(i,j,NTZp))/dzbf

!               Qb = Qb+Qbar1 !#@ WHY ??? NOT USED, diagnostic??
               IF (tit(i,j,kTpm)< -5. .AND. icntTpmerror<100) THEN
                 IF (tit(i,j,kTpm)< -Betap*H(i,j)) THEN
                   WRITE(*,*)'Nested Tice(kTpm)<Tpm error ',kTpm,tit(i,j,kTpm), H(i,j), tit(i,j,NTZp)
                   icntTpmerror=icntTpmerror+1
                 ENDIF
               ENDIF


               Gb1 =  0.
               IF(kTpm<NTZ) THEN
                 DO ktz = kTpm,NTZ-1
                   Gb1 = Gb1+ Ci(tit(i,j,ktz))*( &
                      Tmtrx1(ktz)*tit(i,j,ktz-1) &
                      +Tmtrx2(ktz)*tit(i,j,ktz) &
                      +Tmtrx3(ktz)*tit(i,j,ktz+1) &
                      -Tmtrx4(ktz) )
                 ENDDO
               ENDIF

! 0.5 factor for dzeta(bottom)=0.5*dzet
               Gb(i,j) = dzoL*H(i,j)*( Gb1 &
                  + Ci(tit(i,j,NTZ))*0.5*( &
                  Tmtrx1(NTZ)*tit(i,j,NTZ-1) &
                  +Tmtrx2(NTZ)*tit(i,j,NTZ) &
                  +Tmtrx3(NTZ)*tit(i,j,NTZ+1) &
                  -Tmtrx4(NTZ) ) )

               IF (tit(i,j,NTZ)<-Betap*H(i,j) .AND. Gb(i,j)<0.) tit(i,j,NTZ)=-Betap*H(i,j) ! above already reduced Gb for latent Heat cons

# 515

             ELSE
               Gb(i,j) = 0.
             ENDIF

           ENDIF         ! end of Gb comp for itnh=ntnh


         ENDIF
!*************end full therm computation***********

!-----------do H < Hmin comp--------

!H< Hmin over-ride for water covered ground
! this version needs sealev(i,j,ki) and hfilla inputs from subroutine ifield
! problem if bouy>5.0 for landlocked depression
       ELSEIF (itnh == 1) THEN
         ttsurf=Tsurf(i,j)
         Gb(i,j) = 0.

         IF (ifloat==1) THEN ! marine or lacustrine
           IF (H(i,j) < 0.1) THEN
             tit(i,j,1:NTZ-1) = 0.
             ttsurfb=MAX(ttsurf,Tbd) ! no ice,  can be >0
             wdepth=MAX(sealev(i,j,ki)+hp(i,j),0.)
             IF(wdepth > 500.) THEN
               tit(i,j,NTZ) = ToceanDEEP
             ELSE
               tit(i,j,NTZ) = MAX(ttsurfb,ToceanDEEP)*(500.-wdepth)/500. &
                  & +ToceanDEEP*wdepth/500. !=0 if H>0.1 for ToceanDEEP=0
             ENDIF

           ELSE
             ttsurfb=MIN(ttsurf,Tmax) !ice, </= Tmax=0
             Tbd=MAX(ttsurfb,Tbd)
             DO kz = 1,NTZ-1
!     linear interpolate T from MIN(Tsurf,Tmax) to water freeze point Tbd
               tit(i,j,kz) = (Tbd*REAL(kz-1) + &
                  ttsurfb*REAL(NTZ-kz))/REAL(NTZ-1)
               tit(i,j,NTZ) = ToceanDEEP
             ENDDO
           ENDIF

         ELSEIF (H(i,j) < 1.0) THEN !non-floating, TTOP
!     no limit on Tsurf
!  TTOP(T_air) parameterization from plot of Smith and Riseborough, 2002
!  with mods for -17C and +4.118C bounds
           DO kz = 1,NTZ-1
             tit(i,j,kz) = ttsurf
           ENDDO
           IF(IbedT(i,j,ki)==1) THEN
             IF(ttsurf>4.118) THEN
               tit(i,j,NTZ) = ttsurf-1.0
             ELSEIF(ttsurf<-17.0) THEN
               tit(i,j,NTZ) = ttsurf
             ELSE
               tit(i,j,NTZ)=1.2 +0.583269*ttsurf &
                  -0.0286538*(ttsurf**2)
             ENDIF
           ELSE            !assume no thermal offset, just surface offset due to snow
             IF(ttsurf>7.375) THEN
               tit(i,j,NTZ) = ttsurf
             ELSEIF(ttsurf<-17.14) THEN
               tit(i,j,NTZ) = ttsurf
             ELSE
               tit(i,j,NTZ)=3.51751 +0.72829*ttsurf &
                  -0.0278275*(ttsurf**2)
             ENDIF
           ENDIF

         ELSE ! Hmin>H>=1.0, non-floating
           ttsurf=MIN(ttsurf,Tmax)

!--------find basal temperature of highest iced neighbour

           Tbd = ttsurf
           Hdm = hdn(i,j)
           DO jc = -1,1
             DO ic = -1,1
               IF ( hdn(i+ic,j+jc)>Hdm .AND. &
                  H(i+ic,j+jc)>Hmin ) THEN
                 Tbd = MIN(Tbds(i+ic,j+jc),0.)
                 Hdm = hdn(i+ic,j+jc)
               ENDIF
             ENDDO
           ENDDO

!                  Tbd = 0.5*(ttsurf + Tbd)
!                  Tbd = 0.25*(ttsurf + 3.*Tbd)
!      Tbd = 0.1*(ttsurf + 9.*Tbd)
!      IF (Tbd>Tbed)Tbd = Tbed
!      IF (Tbd<-8.)Tbd = -8.


           DO kz = 1,NTZ
!     linear interpolate T from surface to bedrock
             tit(i,j,kz) = (Tbd*REAL(kz-1) + &
                ttsurf*REAL(NTZ-kz))/REAL(NTZ-1)

           ENDDO

         ENDIF             ! end of thin ice

!************thin or no ice bedtherm ********** ! only active for itnh == 1
         Tpm = -(ft*H(i,j) + (1.-ft)*Hold(i,j,ki))*Betap
         DO kz = NCZp,NTFZ
           kbz=kz-NCZ
!######################### bedfrost ###################################
# 633

           Ap0 = rdhocdt*dsz(kz-NCZ)

!######################### bedfrost ###################################
           Tmtrx4(kz) = tit(i,j,kz)*Ap0
         ENDDO
         Tmtrx4(NTFZ) = tit(i,j,NTFZ)*Ap0 + GHF(i,j,ki)

         CALL solvbmtrx(Tit,Tmtrx1,Tdmtrx2,Tmtrx3,Tmtrx4,i,j)
!**********************************************

       ENDIF               ! outer if ( H>Hmin else )


     ENDDO
   ENDDO
!     end of main loop

!--------------------------------------------------
!     grid  BC's, corners not set and not used:
!     grid ends are set to interior points for flow law calc
! Tmaxsafe is 1C to allow for Betap*H temperature adjustment
   tit(1:ni,1:nj,1:NTZ)=MIN(MAX(tit(1:ni,1:nj,1:NTZ),Tmin), Tmaxsafe) !safety cut


     IF (ieee_is_nan(SUM(tit(1:ni,1:nj,1:NTZ)))) THEN ! ifort only
       PRINT *, 'Tit contains NaN(s), resetting'
       tit(1:ni,1:nj,1:NTZ)=MIN(MAX(Tmin,tit(1:ni,1:nj,1:NTZ)),Tmaxsafe) !safety cut
       IF (ieee_is_nan(SUM(tit(1:ni,1:nj,1:NTZ)))) &
          PRINT *, 'reset failed, Tit still contains NaN(s)'
     ENDIF


   DO kz = 1,NTZ
     DO j = 1,nj
       tit(nip,j,kz) = tit(ni,j,kz)
       tit(0,j,kz) = tit(1,j,kz)

     ENDDO
     DO i = 1,ni
       tit(i,njp,kz) = tit(i,nj,kz)
       tit(i,0,kz) = tit(i,1,kz)

     ENDDO
   ENDDO

 ENDDO   ! end of ntnh loop

 Gb=MIN(Gb,0.)

!-------------------------set H old-----------------------

 IF(i11chck == 11) Hold(1:ni,1:nj,ki) = H(1:ni,1:nj)


!     periodic save of T field
!     IF () CALL outTemp(tit)

!     CALL tgrd2igrd(Tc,tit,ni,nj,nkz)

! -------------------output---------------------------
 IF (itout == 1) THEN
!         CALL chkmtrx(idump,tit,Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4,
!     $        Tpm,1,1,NTZ,kTpm,1)  ! solve matrix

!***only place where explic bedtherm is different from nonbedt**
   CALL Tstats(Tavg,Tstatmx,Tstatmn,Tbavg,Tr2avg,H,ki)
   WRITE(*,199)Tavg,Tstatmx,Tstatmn,Tbavg,Tr2avg
199 FORMAT('Tavg,Tstatmx,Tstatmn,Tbavg,Tr2avg : ',5(f6.2,1x))
!***************************************************************
   CALL wstats(vavg,vmx,vmn,wadv(:,:,1:NTZ),H,ki)
   WRITE(*,*)'wavg,wmx,wmn ',vavg,vmx,vmn
   CALL vstats(vavg,vmx,vmn,v,imx,jmx,H,ki)
   WRITE(*,201)vavg,vmx,vmn,imx,jmx
201 FORMAT('vavg,vmx,vmn:',3(1x,f7.0),2(1x,i3))
   CALL vstats(vavg,vmx,vmn,u,imx,jmx,H,ki)
   WRITE(*,202)vavg,vmx,vmn,imx,jmx
202 FORMAT('uavg,umx,umn: ',3(1x,f7.0),2(1x,i3))
 ENDIF

 RETURN
END SUBROUTINE thermo
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE initThrm(newrun,tym,m,rm,mtype,ki)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     all passed PARAMETERs are IN
!     OUT are       COMMON/Alookup/Aexp(0:nAexpmx),aeTmin,dtA
!                   COMMON/cThbound/Tmin,Tmax

!     inits thermal model and connected stuff
!     inits Tc and Aexp lookup table for Bint
!     Tc horizontal grid ends are set  =  to interior neighbours
!     Tc does not have horizontal corners defined
!     *Need previous accumr CALL to get Tsurf*
! SETS ALL: modicetherm[Tc/Tit, Tmin/max, Tcs], modGb[Gb], modicerheology:[Aexp(0:nAexpmx),aeTmin,dtA]
! INPUTS: general fnflow
! INPUTS: coldstart: hs, Tsurf
! calls bedTinit -> SET : modbedtherm variables:
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modinparm, ONLY : fnflow, kANT, kGR, kNA
 USE modflags
 USE modicestore, ONLY : hs
 USE modicetherm, Tc => Tit 
! USE modbedtherm, ONLY : GHF
 USE modconstants
 USE modGb
 USE modicerheology
 USE modTsurf
 USE modIU
 USE modgridm, ONLY : ni, nip, nj, njp
 USE modgridm, ONLY : randGen
 IMPLICIT NONE

 INTEGER, INTENT(IN) :: newrun(NICE), m, mtype, ki
 REAL, INTENT(IN) :: tym, rm

!-----Glen till march/16------------
! REAL, PARAMETER :: Q = 60000., Q2 = 139000., QoR = Q/Rgas, Q2oR = Q2/Rgas
! REAL, PARAMETER :: A1 = 1.14e-5, A2 = 5.47e+10 !Pa^-3 yr^-1

# 758

!-----Glen from CuffeyPaterson.PhysicsOfGlaciers.3ed.2010.pdf, cf GSMcodemods.org ------------
 REAL, PARAMETER :: Q = 60000., Q2 = 115000., QoR = Q/Rgas, Q2oR = Q2/Rgas
 REAL, PARAMETER :: A1 = 8.9836e-6, A2 = 7.43377e+5 !Pa^-3 yr^-1


! ayne      PARAMETER(A1 = 1.1392e-5, A2 = 5.4593e+10) !Pa^-3 yr^-1
 REAL, PARAMETER :: AEiso = 1.e-16 !Pa^-3 yr^-1
# 768


!------GBS----------
 REAL,  PARAMETER :: QN = 49000., QN2 = 197000., QNoR = QN/Rgas, QN2oR = QN2/Rgas
 REAL,  PARAMETER :: AN1 = SECYR*3.9e-3 , AN2 = SECYR*5.9e+27 !  in MPa..
 REAL,  PARAMETER :: dgrain = 0.001  ! grain size in m

!      PARAMETER(Tmax = 4.9) !  maximum allowable ice temperature


!      PARAMETER(nabe = na*mNB*NICE)

 REAL :: aeTminK, A1c, A2c, flowfac, rz, Tice, Aterm, Tbm1, TBsetGRnoise  !, GHFHok
 INTEGER :: itt, kz, i, j, iTrset, nAtmax
 REAL, PARAMETER :: Tcool=-4.0 !degC
# 785

 REAL, PARAMETER :: TBsetGR=-6.0

# 790

 REAL, PARAMETER :: TBsetAN=-4.0 

 INTEGER, PARAMETER :: NCZov2=NCZ/2
 INTEGER :: istart(NICE)=1, nirst,njrst,krst

 IF (istart(ki) == 1) THEN

   IF (ABS(dzet-(1./REAL(NCZ-1))) > 0.00001) THEN
     WRITE(*,*)'fix dzet in modcore.F90 to 1./REAL(NCZ-1)'
     CALL outerr('pGSMicetherm/initThrm, reset dzet in modcore ')
   ENDIF

   Tmax = 0.0                   !  maximum allowable ice temperature
   dtA = (0.1+Tmaxrng-aeTmin)/REAL(nAexpmx)
   aeTminK = aeTmin+273.15


!     init Bint lookup table
!     all flow factors put into Aexp

!     mahaf  as1 = (2.*(RHOIcc*g)**m/(amah**m*(m+2)*recm**(m-1)))*conma*flowfac
!     mahaf  ds = as1/re**2
!      A0c = 2.*(RHOIcc*gEARTH)**m *A0*Re**(-(m+1))

! ug
   IF(ifHEAD==1 .AND. ki ==1) THEN
     PRINT *,"compTcoef now also vertically interpolates T, u, v for basal cell horizontal advection"
     PRINT *,"compwk now computes wadv at vertical cell interface"
     PRINT *,"apr4/22: fx:Tijmo/m2o save"
# 822

# 825

     PRINT *,"using PHOM deformation heating for ice thermodynamics"

# 830

     PRINT *,"using PHOM basal fricting heating"


     PRINT *,"using upwinded divergence form of horizontal T advection -DdivAdvectT or -DoldTadvect"

# 841

     PRINT *,"using new Patankar consistent Finite Vol 3 pt upwind horizontal T advect -DnewFVCadvectT"
# 850


     PRINT *,"now using analytic profile for T_ice init, (z/H)**2 for ANT, **5 for rest"
     PRINT *,"ANT: Tice init Tsurf Tcool=-1.0C, GR: -5C, rest= -4C"
     PRINT *,"TB0 set for ANT:",TBsetAN,", for rest:",TBsetGR
     PRINT *,"Tbd= -1.0 (-1.8) except still = -1.8 for init (assume less chance of perennial sea ice), Feb8/18"
     PRINT *,"pGSMicetherm.F90 sep24.17A"
     PRINT *,'INITTHERM,ieq. aeTmin,dtA,dzet,NTZ: ',aeTmin,dtA,dzet,NTZ
!     PRINT *,'H>2 Tbed0 = -4'
!     PRINT *,"apr7/17 New Tice init for newrun < 0, with ice: NOW: Tice_t0=  MIN(-3.0,"
!     PRINT *, "avg( old lin interp Tsurf:Tbed=-5C, analytic vert diffusion: dT=GHF(i,j,ki)*dz/k_ice ))"
     PRINT *,'apr30/16 added full powerlaw vertical advect to kz=NTZ'
     PRINT *,'fx Gb jul30/09, before was non-zero for floating thiNICE'
     PRINT *,'apr26/09: floating Tb=marine/lake T for all ice'
     PRINT *,'ieq acceleration for THERMO, THERMO call is longer'
     PRINT *,'tit(NTZ)=TTOP(T_air) from plot of SmithandRiseborough02'
     PRINT *,'version NNTTOP2:  + IbedT'
     PRINT *,'version NNTTOP2c: + (hfill/0.91)/sea thin/0 ice bypasses'
     PRINT *,"    plus symmetric thin ice thermo"
     PRINT *,'THERMO, version NN, fx:Tijm2o, solvmtrx>=-55. cut'
     PRINT *,'dtzt = 10*dtzt in ntn loop set, ntnmx=1000'
     PRINT *,'True horizontal upw, 2nd order: xbiTAR2.NNHc'
     PRINT *,' compwk, relative w, + upwards'
   ENDIF

   IF (ki == 1) THEN
     IF(ifHEAD==1) THEN
       IF( mtype == 2) THEN
!            print *,'AR2xbi1TG,10dtw,upwTbd,hmin50,nominT_b,pwr3pt adv'
         PRINT *,'AR2xbi1TG,10dtw,upwTbd,fxpwr3pt adv,hybrbasl, Za'
         WRITE(2,*)'AR2xbi1TG,10dtw,upwTbd,fxpwr3pt adv,hybrbasl, Za'
       ELSE
         PRINT *,'xbi1TG,10dtw,upwTbd,hmin50,nominT_bed,PN rheo'
         WRITE(2,*)'xbi1TG,10dtw,upwTbd,hn50,nominT_bed,PN rheo'
       ENDIF
     ENDIF

! n         read(*,*) flowfac
     flowfac=fnflow
! so         read(*,*) flowfac,Tiso
     IF(ifHEAD==1) WRITE(*,*)'flowfactor  =  ',flowfac
! so         write(*,*)'Tiso  =  ',Tiso
!         A0 = 4.1294e-5*flowfac


     IF( mtype == 2) THEN   ! GLEN -----------------

       A1c = 2.*flowfac*(RHOIc*gEARTH)**m *A1*Re**(-(m+1)) ![L^-(2m+1)/yr]
       A2c = 2.*flowfac*(RHOIc*gEARTH)**m *A2*Re**(-(m+1))
       Aterm=2.*flowfac*(RHOIc*gEARTH)**m *Re**(-(m+1))

!            Aec = 2.*flowfac*(RHOIc*gEARTH)**m *AEiso*Re**(-(m+1))
       IF(ifHEAD==1) PRINT *,'A1c,A2c,nAexpmx ',A1c,A2c,nAexpmx
       nAtmax = NINT((Tmax-aeTmin)/dtA)

! iso
! iso            write(*,*) 'EISMINT ISOthermal ice, AEiso= ',AEiso
# 909

       DO itt = 1,nAtmax
!     Aexp(itt) = A0c*exp(-QoR/(aeTminK+REAL(itt)*dtA))
         Tice = MIN(aeTminK+REAL(itt)*dtA,274.0)
# 915

         IF (Tice<263.15) THEN
           Aexp(itt) = A1c*EXP(-QoR/Tice)
         ELSE
           Aexp(itt) = A2c*EXP(-Q2oR/Tice)
         ENDIF
         
! iso
! iso               Aexp(itt) = Aec

       ENDDO

     ELSE                   ! GBS --------------------

       call outerr('HybridIceDyn only setup for Glen flow law (mtype=2)')


       A1c = 2.*flowfac*(RHOIc*gEARTH)**rm *AN1*Re**(-(rm+1.))* &
          dgrain**(-1.4)*(1.e-6)**rm  ! Dicks uses MPa
       A2c = 2.*flowfac*(RHOIc*gEARTH)**rm *AN2*Re**(-(rm+1.))* &
          dgrain**(-1.4)*(1.e-6)**rm
       IF(ifHEAD==1) PRINT *,'A1c,A2c,nAexpmx ',A1c,A2c,nAexpmx
       nAtmax = INT((Tmax+0.1-aeTmin)/dtA)

       DO itt = 1,nAtmax
!     Aexp(itt) = A0c*exp(-QoR/(aeTminK+REAL(itt)*dtA))
         Tice = MIN(aeTminK+REAL(itt)*dtA,274.0)
! so            T = Tiso
         IF (Tice<257.) THEN
           Aexp(itt) = A1c*EXP(-QNoR/Tice)
         ELSE
           Aexp(itt) = A2c*EXP(-QN2oR/Tice)
         ENDIF
       ENDDO


     ENDIF !mtype


     DO itt = nAtmax,nAexpmx
       Aexp(itt) = Aexp(nAtmax)
     ENDDO 

     IF(mtype /= 2) CALL outerr("initThrm: HybridIceDyn only set for mtype==2")
     Arrhen= Aexp/Aterm

   ENDIF

!------------INIT Tc & Tr-----------------------------
   IF (newrun(ki) > 0) THEN
     Gb=0.
     
     DO j = 1,nj
       DO i = 1,ni
         IF(hs(i,j,ki)>2.) THEN
# 972

!             GHFHok=GHF(i,j,ki)*hs(i,j,ki)/(2.2*SECYR) !2.2=tki(-10.56C),  [GHF]=J/yr
!             Tice=Tsurf(i,j)+GHFHok*dzet  : EDC -54.5 + 54e-3*3.13e3/2.2 =  22.327 >> 0 !
!Tbm1= -1.0-Betap*hs(i,j,ki)
           IF(ki==kANT) THEN 
!             Tice=MIN(Tsurf(i,j)-2.0,0.)
!             Tice=MIN(Tsurf(i,j),0.)
             Tice=MIN(Tsurf(i,j)-1.0,0.)
# 982

             DO kz = 1,NCZ 
               rz=REAL(kz-1)*dzet +1.e-6
               Tc(i,j,kz) = (TBsetAN-Tice) * rz**1.6  + Tice ! @ fit to idBTz.nn1501 with Tice=MIN(Tsurf(i,j),0.)
!               Tc(i,j,kz) = (TBsetAN-Tice) * rz**2  + Tice ! bit colder than above to compensate for warm bias of run
             ENDDO
           ELSE
             Tice=MIN(Tsurf(i,j)-5.0,0.)
# 992

             DO kz = 1,NCZ ! @ fit to GRIP nn10041
               IF(kz<=NCZov2) THEN
                 rz=REAL(kz-1)*dzet*2
                 Tc(i,j,kz) = -5.0*rz  + Tice
               ELSE
                 rz=REAL(kz-1)*dzet
# 1001

                 Tc(i,j,kz) = (TBsetGR-Tice) * rz**5   + Tice

               ENDIF 
             ENDDO
           ENDIF
         ELSE
           DO kz = 1,NCZ
             Tc(i,j,kz) = MIN(Tsurf(i,j)-Tcool,0.)
           ENDDO
         ENDIF
       ENDDO
       DO kz = 1,NCZ
         Tc(0,j,kz) = Tc(1,j,kz)
         Tc(nip,j,kz) = Tc(ni,j,kz)
       ENDDO
     ENDDO
     DO i = 1,ni
       DO kz = 1,NCZ
         Tc(i,0,kz) = Tc(i,1,kz)
         Tc(i,njp,kz) = Tc(i,nj,kz)
       ENDDO
     ENDDO

     iTrset=1
   ELSE !newrun<1, ensure Tice<=T_pressure_melting
     iTrset=0     
     IF (newrun(ki) ==-1) THEN
       DO j = 1,nj !no ice in ghost cells, so non-ghost range only
         DO i = 1,ni
           IF(hs(i,j,ki)>0.1) THEN
             DO kz=1,NCZ
               rz=REAL(kz-1)*dzet 
!               IF(Tc(i,j,kz)>-hs(i,j,ki)*rz*Betap) Tc(i,j,kz)= -1.0 -hs(i,j,ki)*rz*Betap
               Tc(i,j,kz)=MIN(Tc(i,j,kz),-hs(i,j,ki)*rz*Betap)
             ENDDO
           ENDIF
         ENDDO
       ENDDO
     ENDIF
   ENDIF ! newrun > 0


   CALL bedTinit(ki,iTrset)           ! bedtherm init

   IF(iinTbed>0 .AND. newrun(ki) == 1) THEN 
     CALL inTbed0(ki) ! reset Tr to input TR?A.dat
   ELSE
     READ(*,*)
   ENDIF

   IF (newrun(ki)/=0) CALL storeT(ki) !for newrun=0 iTrset=0 so Tbed not updated here

 ENDIF
 istart(ki) = 0

 RETURN
END SUBROUTINE initThrm
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE compTcoef(Tmtrx1,Tmtrx2,Tmtrx3,Tmtrx4,Qb, Ti, &
   hd,H,Hol,u,v,uold,vold,wk,dtt,itn,itnr &
   ,ntnr,i,j,ki,ifloat,Tbd)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     Tmtrx? are OUT, rest are IN
!     assumes wk is + upwards
!     for: Tmtrx1*Tkz-1 + Tmtrx2*Tkz + Tmtrx3*Tkz+1 = Tmtrx4

!     calc coefficients for thermo matrix
!     must be CALLed with itn = 0 when i = 1 to store Tijo,...
!     only saves old T for itn = 0,

!     note uin,vin,wk are at the cell interfaces, as are the zeta points
!      T is solved at cell centers. wk is + upwards

!     must be CALLed by i inner loop as old T field is saved by
!      neighbouring lines along i

! [ matrix equation terms are in Celsius/year = [Tmtrx4] = [T*Tmtrx2] ]
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm
 USE modconstants
 USE modbedtherm, ONLY : tkb,dzbf


 USE modPHOMvardyn, ONLY : heatb, heath
 USE modicetherm, ONLY : heati

 USE modicetherm, ONLY : rHeatBedCellSrc



# 1096


 IMPLICIT NONE

!      PARAMETER(Ci = 2009.)  ! J kg^-1 K^-1
!      PARAMETER(Qbar0 = 0.05*SEC/YR)  ! J/yr/m^2
 REAL, PARAMETER :: PECRIT = 10.0  ! turn on point for upwinding

 REAL, INTENT(IN) :: Ti(0:mNAp,0:mNBp,NTFZ)
 REAL :: Tijmo(0:mNAp,NTZ),Tijo(-1:mNAp+1,NTZ),Tijpo(0:mNAp,NTZ)  ! to save old temps
 REAL :: Tijm2o(0:mNAp,NTZ),Tijp2o(0:mNAp,NTZ)  ! to save old temps
 REAL :: Tmtrx1(NTFZ),Tmtrx2(NTFZ),Tmtrx3(NTFZ),Tmtrx4(NTFZ)
 REAL, INTENT(IN) :: H(0:mNAp,0:mNBp),hd(0:mNAp,0:mNBp)
! wk is positive upwards
 REAL, INTENT(IN) :: wk(NTZ+1),u(0:mNA,0:mNB,NTZ),v(0:mNA,0:mNB,NTZ)
 REAL, INTENT(IN) :: uold(0:mNA,0:mNB,NTZ),vold(0:mNA,0:mNB,NTZ)
 REAL, INTENT(IN) :: Hol, Tbd
 REAL, INTENT(OUT) :: Qb
 INTEGER, INTENT(IN) :: itn,itnr,ntnr,i,j,ki,ifloat

 REAL :: ut(NTZ),vst(NTZ), utm(NTZ),vstm(NTZ)
!      REAL uc(NTZ),vsc(NTZ)
!     bg = int (sin) /dthe, b2 = int (sin^{-1}) /dphi, s = sin, s2 = sin(the+)
 REAL :: defHeatc(NTZ)
 REAL :: tkpi(NTZ)  !  effective diffusivity for kz- interface
 REAL :: Cirydz2(NTZ)  !  RHOIc*Ci(kz)*dz**2 ; [Ci]= J kg^-1 K^-1; [Cirydz2]= J m^-1 K^-1
 REAL :: gradip, gradim, gradjp, gradjm, dgrdip, dgrdjp, &
    dgrdim, dgrdjm, qbtauoh ! gradient terms
 REAL :: dz, dx, dy, zeta, dtzt, dtt, ft, fto, ftm, ftmo, Hij, tkiP, tkiM, fpl, rhoiCidz
 REAL :: Fadvz1, Fadvz3, gam1, gam3, Pecg1, Pecg3
 INTEGER :: nip2=0, ii, kz
 REAL :: tki, Ci, tt

!**************************
 SAVE Tijm2o,Tijmo,Tijo,Tijpo,Tijp2o,defHeatc,qbtauoh
 SAVE ut,utm,vst,vstm  !  for repeated nested advection otherwise remove

!***STATEMENT FNS****************************************************
# 1137

 tki(tt) = 9.828*EXP(-0.0057*(tt+273.15))*SECYR  ! Thermal conductivity, J/yr m^-1 K^-1
 Ci(tt) = 152.5 + 7.122*(tt+273.15)

!      tki(tt) =2.1
!      Ci(tt) = 2009
!      tkapi(tt) = SECYR*tki(tt)/(RHOIc*Ci(tt))  ! m^2/a
!      pec(ww,tt,hh) = abs(ww)*hh/(REAL(NTZ-1)*tkapi(tt))
!      zetm(kz) = 1.- REAL(kz-1)/REAL(NTZ-1)  !   =  1-zeta
# 1148


!     save local old temperatures, Tijo(i,kz) = Ti(i,jcurrent,kz)
!     grid ends are set  =  to interior points
!     grid ends are only needed for inwind advection, setting them
!     to 0 kills the advection, maybe Tijo ends should be set so that
!     outwind advection is calculated


! nnnnnnnnnnn init before nested loops
 IF (itn == 0) THEN
!   heati(:,:,:)=0.
!   heatb=0.
   IF ( i>1 ) CALL outerr('itn = 0, but i>1')
   IF (j>2) THEN
     Tijm2o(0:nip,1:NTZ) = Tijmo(0:nip,1:NTZ)
     Tijmo(0:nip,1:NTZ)  = Tijo(0:nip,1:NTZ)
   ELSEIF (j==2) THEN
     Tijm2o(0:nip,1:NTZ) = Tijo(0:nip,1:NTZ)
     Tijmo(0:nip,1:NTZ)  = Tijo(0:nip,1:NTZ)
   ELSE ! j=1
     Tijmo(1:ni,1:NTZ)  = Ti(1:ni,j,1:NTZ)
     Tijmo(0,1:NTZ) = Tijmo(1,1:NTZ)
     Tijmo(nip,1:NTZ) = Tijmo(ni,1:NTZ)
     Tijmo(0:nip,NTZ)= 0.75*Tijmo(0:nip,NTZ) + 0.25*Tijmo(0:nip,NTZ-1)    ! interpolating for basal cell horizontal advection
     Tijm2o(0:nip,1:NTZ) = Tijmo(0:nip,1:NTZ)
   ENDIF

   Tijo(0:nip,1:NTZ)   = Ti(0:nip,j,1:NTZ)
   Tijpo(0:nip,1:NTZ)  = Ti(0:nip,MIN(j+1,nj),1:NTZ)
   Tijp2o(0:nip,1:NTZ) = Ti(0:nip,MIN(j+2,nj),1:NTZ)
   
   Tijo(-1,1:NTZ) = Ti(1,j,1:NTZ)
   Tijo(0,1:NTZ) = Ti(1,j,1:NTZ)
   Tijo(nip,1:NTZ) = Ti(ni,j,1:NTZ)
   Tijo(nip+1,1:NTZ) = Ti(ni,j,1:NTZ)

! interpolating for basal cell horizontal advection
   Tijo(0:nip,NTZ)= 0.75*Tijo(0:nip,NTZ) + 0.25*Tijo(0:nip,NTZ-1)
   Tijpo(0:nip,NTZ)= 0.75*Tijpo(0:nip,NTZ) + 0.25*Tijpo(0:nip,NTZ-1)
   Tijp2o(0:nip,NTZ)= 0.75*Tijp2o(0:nip,NTZ) + 0.25*Tijp2o(0:nip,NTZ-1)
   RETURN
 ENDIF

!-----------------------------------------------

 dtzt = dtt/REAL(ntnr) ! timestep for call
 ft = REAL(itnr)/REAL(ntnr) ! nested interpolation factor
 ftm = 1.-ft
 fto = REAL(itnr-itn)/REAL(ntnr) ! (itnh-1)/ntnh ->
 ftmo = 1.-fto                   ! full nested interpolation factor
 Hij = ft*H(i,j) + ftm*Hol
 dz = Hij*dzet
 dx=dxg(i,j)

 dy=dysg(i,j)
# 1206



!**initialize effective interface diffusivities
 DO kz = 2,NTZ
   tkiP =  tki(Ti(i,j,kz))
   tkiM =  tki(Ti(i,j,kz-1))
   tkpi(kz) =   2.*(tkiP*tkiM) / (tkiP+tkiM)
   Cirydz2(kz) = RHOIc*Ci(Ti(i,j,kz))*dz**2
 ENDDO


!--------------------first nested loop init-----------------
 IF (itn == 1) THEN

   DO kz = 2,NTZ
!         uc(kz) = (u(i-1,j,kz) + u(i,j,kz))*0.5
!         vsc(kz) = (v(i,j-1,kz)*s2(j-1,ki) + v(i,j,kz)*sinp(j))*0.5
!o         uco(kz) = (uold(i-1,j,kz) + uold(i,j,kz))*0.5
!o         vsco(kz) = (vold(i,j-1,kz)*s2(j-1,ki) + vold(i,j,kz)*sinp(j))*0.5
     utm(kz) = ftmo*uold(i-1,j,kz) + fto*u(i-1,j,kz)
     ut(kz) =  ftmo*uold(i,j,kz) + fto*u(i,j,kz)


     vstm(kz) =  (ftmo*vold(i,j-1,kz) + fto*v(i,j-1,kz)) *sinp(j-1) !sinp =1 if not lat/long grid
     vst(kz) = (ftmo*vold(i,j,kz) + fto*v(i,j,kz)) *sinp(j)
# 1235

   ENDDO
!interpolate to basal cell vertical center (u,v(NTZ) are basal velocity)
   ut(NTZ)=0.75*ut(NTZ)+0.25*ut(NTZ-1)
   utm(NTZ)=0.75*utm(NTZ)+0.25*utm(NTZ-1)
   vst(NTZ)=0.75*vst(NTZ)+0.25*vst(NTZ-1)
   vstm(NTZ)=0.75*vstm(NTZ)+0.25*vstm(NTZ-1)

! adjust gradj for int vs, ie stick in extra bg(j,ki):
!         gradHi = ( H(i+1,j)-H(i-1,j) )/(bg(j,ki)*Re*dphi*2.)
!         gradHj = ( H(i,j+1)-H(i,j-1) )/(bg(j,ki)*Re*dthe*2.)
!         grdhpi = ( hp(i+1,j)-hp(i-1,j) )/(bg(j,ki)*Re*dphi*2.)
!         grdhpj = ( hp(i,j+1)-hp(i,j-1) )/(bg(j,ki)*Re*dthe*2.)

# 1301

# 1313


!$$$      kz = NTZ
!$$$c      zetm2 = zetm(kz)*0.5=0.
!$$$      ddzet0= -(hp(i,j)-hpol)/dtt
!$$$c     following should follow upwind convention for Pe>1 ie derivative
!$$$c     should be at cell interface
!$$$         ddzetuv =  (u(i,j,kz) + u(i-1,j,kz) )
!$$$     $        * (-grdhpi)
!$$$     $        +  (v(i,j,kz)*sinp(j) + v(i,j-1,kz)*sinp(j-1) )
!$$$     $        * (-grdhpj)
!$$$
!$$$         ddzet(NTZ) = ddzet0 + 0.5*ddzetuv  !  = ddzet

!$$$      DO kz = 2,NTZ
!$$$         IF (defHeatc(kz)< -1.e-15) THEN
!$$$            write(172,*)kz,defHeatc(kz),Ti(i,j,kz),Hij,i,j
!$$$     $           ,pec(wk(kz),Ti(i,j,kz),Hij)
!$$$            CALL flush(172)
!$$$            defHeatc(kz) = 0.
!$$$         ENDIF
!$$$      ENDDO

 ENDIF
!  end first nest loop only calcs
!-------------------------------start nested steps

 DO kz = 2,NTZ
!         Fadvz1 = (wk(kz) - ddzet(kz))/dz
!         Fadvz3 = (wk(kz+1) - ddzet(kz))/dz
!         Fadvz1 = (wk(kz) - 0.5*(ddzet(kz-1)+ddzet(kz)) )/dz
!         Fadvz3 = (wk(kz+1) - 0.5*(ddzet(kz)+ddzet(kz+1)) )/dz

   IF(kz<NTZ) THEN
     Fadvz1 = -wk(kz)/dz ! since wk is + up, but zeta grid is + down
     Fadvz3 = -wk(kz+1)/dz

!     compute vertical advection implicit/diffusion
     gam1=tkpi(kz)/Cirydz2(kz) ! = D_e/(rho*C*DeltaX) Patankar
     gam3=tkpi(kz+1)/Cirydz2(kz)
     Pecg1 = ABS(Fadvz1)/gam1
     Pecg3 = ABS(Fadvz3)/gam3

     IF (Pecg1<PECRIT) THEN
       fpl=(1. - 0.1*Pecg1)**5 ! = A(|P|) Patankar power law
       Tmtrx1(kz) = - fpl*gam1 - MAX(Fadvz1,0.) ! Pantankar a_W eq5.57b *-1 (Tmtrx1 on other side of eq) and W+up -> -MAX(-Fw,0)=MIN(Fw,0)
     ELSE
       Tmtrx1(kz) = - MAX(Fadvz1,0.) ! ie Patankar a_w with reverse w: -MAX(-F,0), = MIN(Fadvz1,0)
     ENDIF

     IF (Pecg3<PECRIT) THEN
       fpl=(1. - 0.1*Pecg3)**5
       Tmtrx3(kz) = - fpl*gam3 - MAX(-Fadvz3,0.)
     ELSE
       Tmtrx3(kz) =  - MAX(-Fadvz3,0.) ! ie Patankar a_e with reverse w: -MAX(F,0), = MIN(-Fadvz3,0)
     ENDIF

# 1372

     Tmtrx2(kz) = 1./dtzt -Tmtrx1(kz)-Tmtrx3(kz)


     Tmtrx4(kz) = Ti(i,j,kz)/dtzt &

# 1380

     + (heati(i,j,kz)+heath(i,j)) /(RHOIc*Ci(Ti(i,j,kz))) !heati is dU/dz (SIA) part of deformation heating, heath is SSA

# 1385



   ELSEIF(ifloat==0) THEN ! kz==NTZ
     rhoiCidz = Cirydz2(NTZ)/dz
   

     Qb = heatb(i,j)  ! heatb is J/m^2/yr
# 1396


! Interpolating to cell center for : d/dt cell energy and  energy from work
     Tmtrx4(NTZ) = (0.25*Ti(i,j,NTZ-1)+0.75*Ti(i,j,NTZ))/dtzt &
        + Qb*2./rhoiCidz & ! = heatb/(rho*C*dz/2), again because basal cell depth is dz/2, heatb: J/m^2/y

# 1404

     + (heati(i,j,NTZ) + heath(i,j))/(RHOIc*Ci(Ti(i,j,NTZ))) ! heati/h: J/m3/y
!heati(0:ni,0:nj,NCZ)=heatiPH(0:ni,0:nj,NLEV), @ matched vertical position for dzeta12K in modPHOMgrid for NTZ=65

     rHeatBedCellSrc(i,j)=  Qb + (heath(i,j)+heati(i,j,NTZ))*dz/2.0 ! J/m^2/y


# 1413


   ENDIF ! IF(kz<NTZ)


# 1420


!     add horizontal advection:
!TODO Discretization -> divergence form, vst has sin factor for divergence, upwind interpolation of T to vgrid



# 1510

! new (aug 9/19) interface consistent 3 pt upwinding from pg 19(13) of goodFiniteVolumeDiscretizationChap.pdf (Second Order Upwind Scheme),
! imposed apriori mass conservation so that Sum_i(a_i)=0+source+dt terms (ie dumped (Fe-Fw+Fn-Fs) term))
! this matches Patankar 2D (eq 5.57g) and 3D discretizations eq 5.61:3
! nov/21 sign check: consistent with goodFiniteVolumeDiscretizationChap.pdf (a_(i+2)>0 here and a_ee<0 in pdf but has term on other side of eq)
   IF (ut(kz)<0) THEN
     Tmtrx4(kz) = Tmtrx4(kz) - ut(kz)*(1.5*Tijo(i+1,kz)-0.5*Tijo(i+2,kz)-Tijo(i,kz))/dx !consistent with (newTadvect with no split ut/tum)
   ELSE
     Tmtrx4(kz) = Tmtrx4(kz) - ut(kz)*(0.5*Tijo(i,kz)-0.5*Tijo(i-1,kz))/dx
   ENDIF
   IF (utm(kz)<0) THEN
     Tmtrx4(kz) = Tmtrx4(kz) + utm(kz)*(0.5*Tijo(i,kz)-0.5*Tijo(i+1,kz))/dx
   ELSE
     Tmtrx4(kz) = Tmtrx4(kz) + utm(kz)*(1.5*Tijo(i-1,kz)-0.5*Tijo(i-2,kz)-Tijo(i,kz))/dx
   ENDIF
   IF (vst(kz)<0) THEN
     Tmtrx4(kz) = Tmtrx4(kz) - vst(kz)*(1.5*Tijpo(i,kz)-0.5*Tijp2o(i,kz)-Tijo(i,kz))/dy
   ELSE
     Tmtrx4(kz) = Tmtrx4(kz) - vst(kz)*(0.5*Tijo(i,kz)-0.5*Tijmo(i,kz))/dy
   ENDIF
   IF (vstm(kz)<0) THEN
     Tmtrx4(kz) = Tmtrx4(kz) + vstm(kz)*(0.5*Tijo(i,kz)-0.5*Tijpo(i,kz))/dy
   ELSE
     Tmtrx4(kz) = Tmtrx4(kz) + vstm(kz)*(1.5*Tijmo(i,kz)-0.5*Tijm2o(i,kz)-Tijo(i,kz))/dy
   ENDIF



 ENDDO

! cccccccccccccccccc
!  rest of lower BC

 IF(ifloat==0) THEN
   kz = NTZ

! remember have divided through by cell thickness=dz/2
!      Fadvz1 =2.* (wk(kz) - 0.5*(ddzet(kz-1)+ddzet(kz)) )/dz
   Fadvz1 = 2.*Fadvz3 !Tb cell height is 0.5* rest of ice cells
   Fadvz3 = -2.*wk(kz+1)/dz

!     compute vertical advection implicit/diffusion
   gam1=2.*tkpi(kz)/Cirydz2(kz) !Tb cell height is 0.5* rest of ice cells
   gam3=2.*tkb/(dzbf*rhoiCidz)  ! Tb is at cell interface, so use tkb (follows from Patankar interface formula)
!      Pecg1 = Fadvz1/gam1

   Pecg1 = ABS(Fadvz1)/gam1
   Pecg3 = ABS(Fadvz3)/gam3

   IF (Pecg1<PECRIT) THEN
     fpl=(1. - 0.1*Pecg1)**5
     Tmtrx1(kz) = - fpl*gam1 - MAX(Fadvz1,0.)
   ELSE
     Tmtrx1(kz) = - MAX(Fadvz1,0.) ! ie Patankar a_w with reverse w: -MAX(-F,0), = MIN(Fadvz1,0)
   ENDIF

   IF (Pecg3<PECRIT) THEN ! CHECKED, and always satisified for NEA105 run
     fpl=(1. - 0.1*Pecg3)**5
       Tmtrx3(kz) = - fpl*gam3 ! - MAX(-Fadvz3,0.)
   ELSE
     Tmtrx3(kz) = 0. !  - MAX(-Fadvz3,0.) ! ie Patankar a_e with reverse w: -MAX(F,0), = MIN(-Fadvz3,0)
   ENDIF

# 1575

   Tmtrx2(NTZ) = 0.75/dtzt -Tmtrx1(NTZ) -Tmtrx3(NTZ)

   Tmtrx1(NTZ) = Tmtrx1(NTZ) + 0.25/dtzt ! breaks Patankar rule

 ELSE !ifloat == 1
   Tmtrx3(NTZ) = 0.
   Tmtrx1(NTZ) = 0.
   Tmtrx2(NTZ) = 1.
   Tmtrx4(NTZ) = Tbd
   Qb=0.
 ENDIF

# 1590


!TEST
!!$ IF(j==31 .AND. i==47) THEN
!!$   kz=NCZ-1
!!$   PRINT *,"phom/defheat",(heati(i,j,kz)+heath(i,j)) /(RHOIc*Ci(Ti(i,j,kz)))&
!!$      ,defHeatc(kz)  ! deformation heating
!!$   PRINT *,'phom/qb',heatb(i,j), - qbtauoh*Hij*SECYR      ! implict in U and H, [Qb]= J/yr /m^2
!!$   kz=NCZ
!!$   PRINT *,"phom/defheat",(heati(i,j,kz)+heath(i,j)) /(RHOIc*Ci(Ti(i,j,kz)))&
!!$      ,defHeatc(kz)  ! deformation heating
!!$   PRINT *,'phom/qb',heatb(i,j), - qbtauoh*Hij*SECYR      ! implict in U and H, [Qb]= J/yr /m^2
!!$ ENDIF

 RETURN
END SUBROUTINE compTcoef
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE solvmtrx(xx,kTpm,x1,x2,x3,x4,Tpm,i,j,nk)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! xx, kTpm are OUT, rest are IN
! modified for zeta(1:N) grid, ie starts at k = 2
! enforces T_zetN < =  Tpm
! kTpm is highest level at pressure melting point

! solves Tk- *x1 + Tk *x2 + Tk+ *x3  =  x4 for k = 2,N
! with x2(1) and x3(N) not used (tridiag)
! RETURNs Tk in xx

! problem is that eqn 1 is T = Tsurf, then eqn2,..
! then eqnN-1, then second order shawn thing, so
! REALly need to do solve from eqn2 to eqnN, with
! eqn2 using x1 = 0 and x4  = x4-x1*T1, x1 is not used in first
! TDMA anyway, so this should be modified for this special case,
! but at same time keep copy of general solver for future use.
! Bottom equation is centred one above base. Actually could place it
! at bottom
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modconstants, ONLY : Betap
 USE moddim
 USE modicetherm, ONLY : Tmin
 IMPLICIT NONE

 REAL, PARAMETER :: Tmnrng = -70.
 INTEGER, PARAMETER :: nm = NTFZ

 REAL :: x1(nm),x2(nm),x3(nm),x4(nm),xx(0:mNAp,0:mNBp,nm)
 REAL :: wk1(nm),wk2(nm),wkb(nm)
 REAL, INTENT(IN) :: Tpm
 INTEGER, INTENT(IN) :: i, j, nk
 INTEGER, INTENT(OUT) :: kTpm
 REAL :: Tmn, Tpmz

 INTEGER :: iwarn=0, nkp, kz

!     solve for xx
 Tmn = 10.
 kTpm = 0 

 wk1(2) = x3(2)/x2(2)
 wk2(2) = x4(2)/x2(2)
 DO  kz = 2,NTFZ-2
   wkb(kz) = (x2(kz+1)-x1(kz+1)*wk1(kz))
   wk1(kz+1) = x3(kz+1)/wkb(kz)
   wk2(kz+1) = (x4(kz+1)-x1(kz+1)*wk2(kz))/wkb(kz)
 END DO
 wk2(NTFZ) = (x4(NTFZ)-x1(NTFZ)*wk2(NTFZ-1))/(x2(NTFZ)- &
    x1(NTFZ)*wk1(NTFZ-1)) 

 DO kz = NTFZ-1,nk,-1
   wk2(kz) =  MAX(Tmnrng,wk2(kz)-wk1(kz)*wk2(kz+1))
 ENDDO


! trap pressure melting point
! assumes that Ti(?) = Tpm  = > T(nk) = Tpm
 IF (wk2(nk) > Tpm) THEN
   wk2(nk) = Tpm
   kTpm = nk
!**recompute Tr*****
   nkp = nk+1
   x4(nkp) = x4(nkp)-x1(nkp)*Tpm
   wk1(nkp) = x3(nkp)/x2(nkp)
   wk2(nkp) = x4(nkp)/x2(nkp)
   DO  kz = nkp,NTFZ-2
     wkb(kz) = (x2(kz+1)-x1(kz+1)*wk1(kz))
     wk1(kz+1) = x3(kz+1)/wkb(kz)
     wk2(kz+1) = (x4(kz+1)-x1(kz+1)*wk2(kz))/wkb(kz)
   ENDDO
   wk2(NTFZ) =(x4(NTFZ)-x1(NTFZ)*wk2(NTFZ-1))/(x2(NTFZ)- &
      x1(NTFZ)*wk1(NTFZ-1))

   DO kz = NTFZ-1,nk+1,-1
     wk2(kz) = wk2(kz)-wk1(kz)*wk2(kz+1)
   ENDDO

!**********
 ENDIF

! back sub, wk2 was Q_i, becomes T
! wk1 = P_i
 DO kz = nk-1,2,-1
   wk2(kz) = wk2(kz)-wk1(kz)*wk2(kz+1)
   Tpmz = Tpm*REAL(kz-1)/REAL(nk-1)
   IF (wk2(kz)>Tpmz) THEN
     wk2(kz) = Tpmz
     kTpm = kz
   ELSEIF (wk2(kz)<Tmnrng) THEN
     Tmn=wk2(kz)
     wk2(kz) = Tmnrng
   ENDIF

 ENDDO

 xx(i,j,1:NTFZ) = wk2(1:NTFZ)

 IF(Tmn<Tmin .AND. MOD(iwarn,100) == 0)WRITE(*,*) &
    "**Warning: Tmn(solvmtrx),i,j,kTpm,H: ",Tmn,i,j,kTpm, &
    -1.*Tpm/Betap
 IF(iwarn>1000.) THEN
   iwarn = iwarn+1
 ELSE
   iwarn = iwarn+10
 ENDIF

 IF(iwarn>1000000) iwarn=1001

 RETURN
END SUBROUTINE solvmtrx
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE bedTinit(ki,iTrset)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     Tr is OUT
!     bed surface is at kz = 0
!     dz(kz) is distance between z(kz-1) and z(kz)
!     PURPOSE: reads in bed params, makes bed grid,
!       intialises Tr (constant flux soln), computes constant
!       matrix terms
! SETS all of modbedtherm except for GHF (which is an input for iTrset=1,2)
! SETS bedT constant matrix terms via comptbrmtrx(dz,tkb)
! for iTrset=1,2 (newrun .NE.1) also SETS Tit(:,:,NTZp,NTFZ)
! INPUTS(cold start: iTrset=1,2) H, hp, sealev, GHF
! NO external field INPUTS for restart iTrset=0, aside from file reads
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modbedtherm
 USE modicetherm, ONLY : Tit
 USE modconstants, ONLY : SECYR, RHOW, RHCONV
 USE modgridm, ONLY : ni,nj
 USE modcorevar, ONLY : sealev
 USE modicefields, ONLY : H, hp
 IMPLICIT NONE

 INTEGER, INTENT(IN) :: ki, iTrset

 REAL :: scale, sum, dTmax, Tz, ttime, Tbd, bouy
 INTEGER :: i,j,kz, kbz
 INTEGER :: icheck, inew, icode

# 1756


 INTEGER :: istart(NICE)=1

! ccccccccccccc
 IF(ifHEAD==1) WRITE(*,*)'initializing T_rock for ki =  ',ki,' using 3meg rhoc'

 IF (istart(1) == 1) THEN
   READ(*,*)tkbs
   IF(ifHEAD==1) THEN
     PRINT *,"Now using TTOP2 for bedT init IF iTrset == 1"
     WRITE(*,*)'Bed thermal conductivity =  ',tkbs,' in W /m /K'
     PRINT *,'now Tbs Tsurf to Tbd scaled(200 m) 4 ice-free marine'
     PRINT *,'now using abs(Pec) in advection: ie Patankar A(|Pec|)'
     WRITE(*,*)'new dsz(1),fixed dsz(ntbz), Tpass, storeTr'
     WRITE(2,*)'new dsz(1),fixed dsz(ntbz), Tpass, storeTr'
     WRITE(*,*)'C_m =  ',Cm
   ENDIF
 ENDIF

 tkb = tkbs*SECYR

 IF (istart(ki) == 1) THEN
   IF(ifSETthermBedType<0) THEN 
     CALL askofil(37,'thermo Bed type map for TTOP, 1 where sediment, else 0')
!     CALL inFI(IbedT,37,ttime,ki,1,mNA,mNB,NICE,ni,nj)
     icheck=1888
     inew=1
     CALL readGenMaskijcheck(37,ni,nj,IbedT(1:ni,1:nj,ki),inew,icheck)
   ELSE ! only for -DSQgrid, then value is read in from rsi
     READ(*,*)
     IbedT=ifSETthermBedType
   ENDIF
!######################### bedfrost ###################################
# 1799

   READ(*,*)
   READ(*,*)
   IF(ifHEAD==1) PRINT *,"TTOP2 thermo model, NO BED permafrost"
!######################### bedfrost ###################################

!######################### bedfrost ###################################
 ENDIF

 IF(istart(1)==1) THEN
!-------------------make bed grid--------------------
!   dz(1) = 0.5 !\delta x, ie cell center distances, since kz=NCZ is Tb and cell interface, dz1 should be < 0.5*dz
!   dz(2)= RbedSCALE**2 !**2 to have dsz(1)<=dsz(2)/RbedSCALE  *RbedSCALE keeps dsz(2)>dsz(1)
!   sum = dz(1)+dz(2)
!   DO kz = 3,ntbz
!     dz(kz) = RbedSCALE*dz(kz-1)
!     sum = sum+dz(kz)
!   ENDDO
   dz(1)=1.5
   sum=dz(1)
   DO kz = 2,ntbz
     dz(kz) = RbedSCALE**kz + REAL(kz)
     sum = sum+dz(kz)
   ENDDO

   scale = BEDTdepth/sum
   dz(ntbz) = scale*dz(ntbz)
   DO kz = ntbz-1,1,-1
     dz(kz) = scale*dz(kz)
     dsz(kz) = 0.5*(dz(kz)+dz(kz+1)) ! \Delta x, ie thickness of grid cells
   ENDDO
   dzbf = dz(1)
   dsz(ntbz) = dz(ntbz)
   dsz(1) = dz(1)+0.5*dz(2) !since kz=ntz is at interface

   IF(ifHEAD==1) THEN
     WRITE(*,*) "dz,dsz::::ntbz: ",ntbz
     DO kz=1,ntbz
       WRITE(165,*)dz(kz),dsz(kz)
       WRITE(*,*)dz(kz),dsz(kz)
     ENDDO
   ENDIF

   dTmax=RhoCm/tkb *dz(1)**2
   IF(ifHEAD==1) PRINT *,"**bed dtThmax= ",dTmax," for dzmin for bed: ",dz(1)

!######################### bedfrost ###################################
# 1881

ENDIF !IF(istart(1)==1) THEN
!######################### bedfrost ###################################

!######################### bedfrost ###################################


! cccccccccccc initialize Tr--------------------
!-----assume equilibrium soln with constant flux GHF

 IF(iTrset == 1) THEN
   DO j = 1,nj
     DO i = 1,ni
       bouy=RHCONV*(hp(i,j)+sealev(i,j,ki))-H(i,j)
       if(bouy>3.0) then !floating/marine
         Tz=-1.8
       ELSEIF(H(i,j)<1.0) THEN
         Tbd=Tit(i,j,NCZ)  !Tit is initialized to Tsurf or restart prior to call of this routine
!  TTOP(T_air) parameterization from plot of Smith and Riseborough, 2002
!  with mods for -17C and +4.118C bounds
         if(IbedT(i,j,ki)==1) then
           if(Tbd>4.118) then
             Tz = Tbd-1.0
           elseif(Tbd<-17.0) then
             Tz = Tbd
           else
             Tz=1.2 +0.583269*Tbd &
                -0.0286538*(Tbd**2)
           endif
         else              !assume no thermal offset, just surface offset due to snow
           if(Tbd>7.375) then
             Tz = Tbd
           elseif(Tbd<-17.14) then
             Tz = Tbd
           else
             Tz=3.51751 +0.72829*Tbd &
                -0.0278275*(Tbd**2)
           endif
         endif
       ELSE ! H>1
         Tz=Tit(i,j,NCZ)
       endif

       IF (Tz< -50.)Tz = -50.
!            z = 0.
       DO kz = NCZp,ncfz
         kbz = kz-NCZ
         Tz = Tz+dz(kbz)*GHF(i,j,ki)/tkb ! [GHF]=J/yr
         Tit(i,j,kz) = Tz
       ENDDO
     ENDDO
   ENDDO

 ELSEIF(iTrset == 2) THEN
   DO j = 1,nj
     DO i = 1,ni
       Tz = Tit(i,j,ntz)
       IF (Tz< -20.)Tz = -20.
!            z = 0.
       DO kz = NCZp,ncfz
         kbz = kz-NCZ
         Tz = Tz+dz(kbz)*GHF(i,j,ki)/tkb 
         Tit(i,j,kz) = Tz
       ENDDO
     ENDDO
   ENDDO
 ENDIF ! IF(iTrset == 1)

 CALL compTbrmtrx(dz,tkb)

 istart(ki) = 0

 RETURN
END SUBROUTINE bedTinit
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE compTbrmtrx(dz,tkb)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     Bmtrx? and dtb are IN, Tr is OUT, Bmtrx2 is IN/OUT

!     computes constant matrix elements for bed
!     designed to conserve fluxes alas Patankar
!     ignoring latent heat contribution for now
!     assumes bottom layer is melted
!  [ matrix equation terms are in J year^-1 m^-2 = [Tmtrx4] = [T*Tmtrx2] ]
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE moddim
 USE modTmtrx, ONLY : Tmtrx1,Tmtrx2,Tmtrx3,Tmtr13
 IMPLICIT NONE

 REAL, INTENT(IN) :: dz(ntbz), tkb
 INTEGER :: kz, kbz, kbzp


! ccccccccccc


 DO kz = NCZ+1,NTFZ-1

   kbz = kz-NCZ
   kbzp = kz+1-NCZ
   Tmtrx3(kz) =  -tkb/dz(kbzp)
   Tmtrx1(kz) =  -tkb/dz(kbz)
   Tmtr13(kz) = -Tmtrx1(kz)-Tmtrx3(kz)
!b         write(180,*)'Tmtrx1,3,13 ',Tmtrx1(kz),Tmtrx3(kz),Tmtr13(kz)
 ENDDO

!b      CALL flush(180)
!*****warm base BC

 Tmtrx1(NTFZ) =  - tkb/dz(ntbz)
 Tmtr13(NTFZ) = -Tmtrx1(NTFZ)

 RETURN
END SUBROUTINE compTbrmtrx
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE solvbmtrx(Tit,x1,x2,x3,x4,i,j)

!     solves bed matrix
!     used for init and thin/noice points
!     otherwise solvmtrx solves whole matrix
!     eq: x1*T(kz-1)+x2*T(kz)+x3*T(kz+1) = x4
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE moddim
 IMPLICIT NONE

 INTEGER, PARAMETER :: nm = NTFZ

 REAL, INTENT(OUT) :: Tit(0:mNAp,0:mNBp,nm)
 INTEGER, INTENT(IN) :: i, j
 REAL :: x1(nm),x2(nm),x3(nm),x4(nm)

 REAL :: wk1(nm),wk2(nm),wkb(nm)
 INTEGER :: kz


!     NCZp = NCZ+1
!***bring in x1(NCZp) term since it's start of tridiag matrix
 x4(NCZp) = x4(NCZp)-x1(NCZp)*Tit(i,j,NCZ)
 wk1(NCZp) = x3(NCZp)/x2(NCZp)
 wk2(NCZp) = x4(NCZp)/x2(NCZp)

 DO  kz = NCZp,NTFZ-1
   wkb(kz) = (x2(kz+1)-x1(kz+1)*wk1(kz))
   wk1(kz+1) = x3(kz+1)/wkb(kz)
   wk2(kz+1) = (x4(kz+1)-x1(kz+1)*wk2(kz))/wkb(kz)
 ENDDO

! wk2(NTFZ) = (x4(NTFZ)-x1(NTFZ)*wk2(NTFZ-1))/(x2(NTFZ)- &
!    x1(NTFZ)*wk1(NTFZ-1))
 Tit(i,j,NTFZ) = wk2(NTFZ)

! BACK SUB****************
 DO kz = NTFZ-1,NCZ+1,-1
   wk2(kz) = wk2(kz)-wk1(kz)*wk2(kz+1)
   Tit(i,j,kz) = wk2(kz)
 ENDDO

 RETURN
END SUBROUTINE solvbmtrx
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE inTbed0(ki)
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modicetherm, Tc => Tit
 USE modgridm, ONLY : nip, njp
 USE modbedtherm, ONLY : BEDTdepth, RbedSCALE
 IMPLICIT NONE

 INTEGER, INTENT(IN) :: ki
 INTEGER, PARAMETER :: IbedTversion=5
 INTEGER :: nip2,njp2,nin,njn, ninNTBZ, inIbedTversion
 REAL :: rinbedscale, rinBEDTdepth
 INTEGER :: iu, i, j, kz

! CALL cdim(ki)
 iu = 37
 CALL askofilunf(iu,'T_rock input file')
 
 READ(iu) ninNTBZ, rinbedscale, rinBEDTdepth
 READ(iu)nin,njn
 nip2=nip+1
 njp2=njp+1
 IF(nin /= nip2 .OR. njn /= njp2 .OR. ninNTBZ /=NTBZ .OR. &
    rinBEDTdepth /= BEDTdepth .OR. rinbedscale /= RbedSCALE) THEN
   WRITE(*,*)"nip2,nin,njp2,njn: ",nip2,nin,njp2,njn
   PRINT *,"ninNTBZ,NTBZ: ",ninNTBZ,NTBZ
   PRINT *,"rinbedscale,RbedSCALE:",rinbedscale,RbedSCALE
   PRINT *,"rinBEDTdepth,BEDTdepth:",rinBEDTdepth,BEDTdepth
   CALL outerr('ni,nj or BED mismatch for GF??.dat')
 ENDIF
 IF(ifHEAD==1) PRINT *,"T_rock input passes dimension, bed scale check"

 READ(iu) Tcs(0:nip,0:njp,NCZp:NCFZ,ki)
 Tc(0:nip,0:njp,NCZp:NCFZ) = Tcs(0:nip,0:njp,NCZp:NCFZ,ki)
 READ(iu) inIbedTversion
 CLOSE(iu)

 IF(inIbedTversion /= IbedTversion) &
    CALL outerr('IbedTversion mismatch, binary read error in inTbed0')

 IF(ifHEAD==1) WRITE(*,*)'Tr read in complete for ki= ',ki

 RETURN
END SUBROUTINE inTbed0
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE outTbed0(tym,iu0)
! save basal temperature field for future runs
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modicetherm, ONLY : Tcs
 USE modgridm
 USE moddim
 USE modbedtherm, ONLY : BEDTdepth, RbedSCALE
 IMPLICIT NONE

 REAL, INTENT(IN)                  :: tym
 INTEGER, INTENT(IN)               :: iu0

 INTEGER, PARAMETER :: IbedTversion=5
 INTEGER :: iu, ki, i, j, kz


 
 DO ki = 1,NICE
   CALL cdim(ki)
   iu = iu0 -1+ki
   WRITE(iu)  NTBZ, RbedSCALE, BEDTdepth
   WRITE(iu) nip+1,njp+1,xlonmi(ki),xlonma(ki),ylatmi(ki),ylatma(ki),dphid,dthed
   WRITE(iu) Tcs(0:nip,0:njp,NCZp:NCFZ,ki)
   WRITE(iu) IbedTversion
   IF(ifHEAD==1) WRITE(*,*)'T_rock written for ki= ',ki, 'at time', tym
 END DO

 RETURN
END SUBROUTINE outTbed0
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE compwk(wk,u,v,H,Hold,dt)
! wk is OUT, rest are IN
! wk is + upward on vertical interface. ie wk(:,:,NTZp) is basal interface.
! using integrated sin =bg format
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     v(i,j) is v(i,j+) following deblonde
!     wk= Gb -(1-zeta)*dH/dt - \div \dot H*/int_b^kz U(i,j,kz) dzeta
!     = -w_t of ritzFL.GrISManal.climdyn97.pdf, cf GSM w_t notes
! wk(i,j,kz) is value of wk at kz cell interface
! assumes kz=NTZp is ice/bed interface, kz=1 is top ice surface
! wk(i,j,1) not used, so not calced
 USE modgridm
 USE modconstants, ONLY : Re
 USE modGb

 IMPLICIT NONE

 REAL, PARAMETER :: dzeto2s=dzet/2.

 REAL, PARAMETER :: hmin=50.0
 REAL, PARAMETER :: hmin2=2.*hmin

 REAL, INTENT(OUT) :: wk(mNA,mNB,NTZp)
 REAL, INTENT(IN) :: u(0:mNA,0:mNB,NTZ),v(0:mNA,0:mNB,NTZ)
 REAL, INTENT(IN) :: H(0:mNAp,0:mNBp),Hold(0:mNAp,0:mNBp)
 REAL, INTENT(IN) :: dt

 REAL :: vk(0:MAX(mNA,mNB),NTZp),dHodt, Hc2, zetm, dzeto2(NTZ)
 INTEGER :: i,j,kz

 zetm(kz)=1.- REAL(kz-1.5)/REAL(NTZ-1) ! = 1-zeta, accounting for half thick cell on bottom, only good for kz=1:NTZ


 vk(0:MAX(ni,nj),NTZp)=0

 dzeto2(1:NTZ-1)=dzeto2s
 dzeto2(NTZ)=dzeto2s/2.

 DO j=1,nj

   DO kz=NTZ,2,-1
     DO i=0,ni
       Hc2=(H(i,j)+H(i+1,j))
       IF(Hc2>hmin2) THEN 
         vk(i,kz)=u(i,j,kz) * &
            Hc2*dzeto2(kz)  + vk(i,kz+1) !H*integral(U)dzeta to vertical cell interface ( ie SUM: delta H * VertAvg(u) )
       ELSE
         vk(i,kz)=0.
       ENDIF
     ENDDO
   ENDDO


   DO kz=NTZ,2,-1
     DO i=1,ni
       IF(H(i,j)>hmin) THEN
         wk(i,j,kz)= (vk(i,kz)-vk(i-1,kz))/dxg(i,j) ! grad_i at horizontal cell center, vertical cell interface
       ELSE
         wk(i,j,kz)=0.
       ENDIF
     ENDDO
   ENDDO
 ENDDO


 DO i=1,ni

   DO kz=NTZ,2,-1
     DO j=0,nj
       Hc2=(H(i,j)+H(i,j+1))
       IF(Hc2>hmin2) THEN
         vk(j,kz)=v(i,j,kz) * Hc2*dzeto2(kz)  + vk(j,kz+1)
       ELSE
         vk(j,kz)=0.
       ENDIF
     ENDDO
   ENDDO

   DO j=1,nj
     IF(H(i,j)>hmin) THEN
       dHodt=(H(i,j)-Hold(i,j))/dt
       wk(i,j,NTZp)=Gb(i,j) ! => wk>0 is upward
       DO kz=NTZ,2,-1
         wk(i,j,kz)=Gb(i,j) - zetm(kz)*dHodt - &
             (vk(j,kz)*sinp(j) - vk(j-1,kz)*sinp(j-1))/dysg(i,j) &
            - wk(i,j,kz)
       ENDDO
!t       IF(ABS(wk(i,j,2))>100.0) THEN
!t         PRINT *,"Gb,H,vk",Gb(i,j),H(i,j),vk(j,2),zetm(kz)*dHodt,wk(i,j,2),dysg(i,j),dxg(i,j)
!t        PRINT *,"Re,dthe,dphi",Re,dthe,dphi,dysgKi(i,j,ki),dxgKi(i,j,ki)
!t       ENDIF
     ENDIF
!!$     IF((i==33 .OR. i==43 .OR. i==51) .AND. j==32) THEN
!!$       PRINT *,"i,j",i,j
!!$       PRINT *,"Gb",Gb(i,j)
!!$       DO kz=1,NTZp
!!$         WRITE(*,'(G11.4,1x,i2)'),wk(i,j,kz),kz
!!$       ENDDO
!!$       kz=NTZ
!!$       PRINT *,zetm(kz)*dHodt,(vk(j,kz)*sinp(j) - vk(j-1,kz)*sinp(j-1))/dysg(i,j),&
!!$          wk(i,j,kz)-(Gb(i,j) - zetm(kz)*dHodt - (vk(j,kz)*sinp(j) - vk(j-1,kz)*sinp(j-1))/dysg(i,j))
!!$       PRINT *,"H:",H(i,j),Hold(i,j),dt,zetm(NTZ)
!!$       IF(i==51) STOP
!!$     ENDIF
   ENDDO

 ENDDO

 RETURN
END SUBROUTINE compwk
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE unstoreT(ki)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modicetherm, ONLY : Tc => Tit, Tcs
 USE modgridm
 IMPLICIT NONE

 INTEGER, INTENT(IN)   :: ki

 Tc(0:nip,0:njp,1: NCFZ)=Tcs(0:nip,0:njp,1:NCFZ,ki)

 RETURN
END SUBROUTINE unstoreT
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

SUBROUTINE storeT(ki)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modicetherm, ONLY : Tc => Tit, Tcs
 USE modgridm
 IMPLICIT NONE

 INTEGER, INTENT(IN)   :: ki

 Tcs(0:nip,0:njp,1:NCFZ,ki)=Tc(0:nip,0:njp,1:NCFZ)

 RETURN
END SUBROUTINE storeT
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


SUBROUTINE readTinit(tnew,ki,iu)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     must be called by initThrm
!     compression: for jrow if hs.eq.0 then leave
!     and use value from initThrm
!      else if point has T(i,j,1)=T(i,j,NCZ) then put in code
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modicetherm, ONLY : Tc => Tit
 USE modgridm

 REAL, INTENT(IN)    :: tnew
 INTEGER, INTENT(IN)                     :: ki
 INTEGER, INTENT(IN)    :: iu

 READ(iu,60)tipet
60 FORMAT(2X,13X,f14.1)
 READ(iu,*)

 WRITE(6,*) 'tipet= ',tipet,' ki= ',ki
!        if(abs(tnew-tipet)> 0.1) then
!          write(6,*) 'wrong initial time '
!          stop
!        endif

 DO  j=0,njp
   READ(iu,110)j1,ich
   IF(j1 /= j)CALL outerr('wrong restart T_ice field, j')
110 FORMAT(2X,i4,i4)

   IF(ich == 1) THEN
     DO i=0,nip
       READ(iu,110)i1,kch
       IF(i1 /= i)CALL outerr('wrong restart T_ic field, i')

       IF(kch == 1) THEN
         READ(iu,40)(Tc(i,j,kz),kz=1,NCZ)
       END IF
     END DO
   END IF

 END DO

40 FORMAT(1PE13.6,9E13.6)
 RETURN
END SUBROUTINE readTinit
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE outBiTz(tnew,i,j,H,iunit,ki)
! rewinds, so last output per unit
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm
 USE modicetherm, Tc => Tit
 IMPLICIT NONE

 INTEGER, INTENT(IN) :: i, j, iunit, ki
 REAL :: H(0:mNAp,0:mNBp)
! REAL :: Bi(0:mNAp,0:mNBp,NTZ)
! COMMON/cBdate/Bi
 REAL :: zeta,z,xlon,ylat,tnew
 INTEGER :: kz

 REWIND(iunit)
 zeta=H(i,j)/REAL(NCZ-1)
 xlon = xlonmi(ki) + dphid*(REAL(i) -0.5)
 ylat = ylatma(ki) + dthed*(0.5 -REAL(j))
 WRITE(iunit,144)zeta,i,j,xlon,ylat

144 FORMAT('#depth, T(d), dzeta: ',f7.3,2(1x,i3),2(1x,f8.3))
! 144 FORMAT('#depth, T(d), Bidate, dzeta: ',f7.3,2(1x,i3),2(1x,f8.3))

 WRITE(iunit,60)tnew
60 FORMAT('#',1x,'time(years)= ',f14.1,' ************')

142 FORMAT(f6.1,1x,f7.2,1x,f11.2)
 DO kz=1,NTZ
   zeta=REAL(kz-1)/REAL(NCZ-1)
   z=zeta*H(i,j)
   WRITE(iunit,142)z,Tc(i,j,kz)!,Bi(i,j,kz)
 ENDDO
! CALL flush(iunit)
 RETURN
END SUBROUTINE outBiTz
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
