# 1 "pGSMinit.F90"
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE seticegrid()
! SETS modgridm

! interface indices refer to previous grid cell center, eg dxu(i,j) is on i+1/2
! (0:ni,0:nj) for interface differentials, (1:ni,1:nj) for centered
! dxu, dyu at u interface, j center,
! so dxu(i,j)=x(i+1,j)-x(i,j);  dyu=length of interface
! dxv, dyv at v interface, i center
! dxc, dyc centered around i+0.5,j+0.5 corner
! dxg, dyg cell centered (cell width)
! dysg is for divergence: for spherical: grid cell integral[SIN(theta) dthe dphi] *Re /dphi

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modgridm
 USE modflags, ONLY : ifHEAD
 USE modconstants, ONLY : DEG2RAD, Re
# 23



 USE modPHOMgrid, ONLY : setPHOMkzGrid, dd0, gmin


 IMPLICIT NONE

 REAL :: theno, conv, phiw, u1, u2
 INTEGER :: i,j,ki, istart=1

!.....define dimensions for ice sheets (used by subroutine cdim(ki))
 DO  ki=1,NICE
   IF(ifHEAD==1) PRINT *, 'what is xlmi,xlma,ylmi,ylatm for ice sheet ',ki
   READ(5,*)xlonmi(ki),xlonma(ki),ylatmi(ki),ylatma(ki)
   xlonLLmi(ki)=xlonmi(ki)
   xlonLLma(ki)=xlonma(ki)
   ylatLLmi(ki)=ylatmi(ki)
   ylatLLma(ki)=ylatma(ki)
 ENDDO

 IF(ifHEAD==1) PRINT *, 'what is dphid,dthed in degrees? '
 READ(5,*) dphid,dthed
 IF(ifHEAD==1) PRINT *, 'dphid,dthed:',dphid,dthed
 IF(dphid<0.001 .OR. dthed < 0.001) CALL outerr("seticegrid: dphid or dthed too small")

 dphi=dphid*DEG2RAD
 dthe=dthed*DEG2RAD
 s=1.0
 s2=1.0
 bg=1.0
 b2=1.0
 the=1.0

# 74


 DO  ki=1,NICE
   xcglonLLmi(ki)=xlonLLmi(ki)-0.5*dphid !ghost cell center
   xcglonLLma(ki)=xlonLLma(ki)+0.5*dphid !ghost cell center
   ycglatLLmi(ki)=ylatLLmi(ki)-0.5*dthed
   ycglatLLma(ki)=ylatLLma(ki)+0.5*dthed
   
!   PRINT *,"xlonma(ki),ylatmi(ki)",xlonma(ki),ylatmi(ki)
!   PRINT *,"dthed,dphid","dthed,dphid"
   nxi(ki)=NINT( (xlonma(ki)-xlonmi(ki))/dphid )
   nyi(ki)=NINT( (ylatma(ki)-ylatmi(ki))/dthed )
   nxiLL(ki)=nxi(ki)
   nyjLL(ki)=nyi(ki)

   IF(IgridType(ki)<2) THEN 
     IF( nxi(ki)>mNA .OR. nyi(ki)>mNB .OR. nxi(ki)<5 .OR. nyi(ki)<5 ) THEN
       PRINT *,"xlonmi(ki),xlonma(ki)",xlonmi(ki),xlonma(ki)
       PRINT *,"ylatmi(ki),ylatma(ki) ",ylatmi(ki),ylatma(ki)
       PRINT *, 'active grid dimensions too small'
       PRINT *, 'mNA, mNB: ',mNA,mNB
       PRINT *, 'nxi ',nxi,' nyi ',nyi,' ki: ',ki
       CALL outerrI('pGSMinit/seticegrid, mNA or mNB not large enough, reset in modcore.F90, ki:',ki)
     END IF
   ENDIF

   IF(IgridType(ki)==1) THEN !lat/long regular grid
     CALL cdim(ki)
!........define the(nj,NICE) and phi(ni,NICE) ! in radians
     theno=(90.0-ylatma(ki))*DEG2RAD
     the(1,ki)=theno+(dthe/2.0)
     DO  j=2,nj
       the(j,ki)=the(j-1,ki)+dthe
     END DO

     phiw=xlonmi(ki)*DEG2RAD
     phi(1,ki)=phiw+(dphi/2.0)
     DO  i=2,ni
       phi(i,ki)=phi(i-1,ki)+dphi
     END DO

!........define bg(nj),s(0:njp),s2(nj),b2(nj) i.e(j+1/2)
     DO  j=1,nj
       u2=the(j,ki)+dthe/2.
       u1=the(j,ki)-dthe/2.
       bg(j,ki)=-(COS(u2)-COS(u1))/dthe ! ( grid cell integral of sin(theta) )/dtheta
       b2(j,ki)=(LOG(TAN(u2/2.))-LOG(TAN(u1/2.)))/dthe
       s(j,ki)=SIN(the(j,ki)) !cell centered
       s2(j,ki)=SIN(the(j,ki)+dthe/2.0) ! cell interface
       dxgKi(1:ni,j,ki)= bg(j,ki)*Re*dphi ! grid cell integral[SIN(theta) dthe dphi] *Re /dthe
       dysgKi(1:ni,j,ki)= bg(j,ki)*Re*dthe ! for divergence: grid cell integral[SIN(theta) dthe dphi] *Re /dphi
!       dygKi(1:ni,j,ki)= bg(j,ki)*Re*dthe/s(j,ki) ! Re* grid cell integral of SIN(theta) /SIN(theta), fixed jan 16/18
       dygKi(1:ni,j,ki)= Re*dthe ! for advective gradient
       dareaKi(1:ni,j,ki)= bg(j,ki)*dthe*dphi*Re**2
     END DO
     s2(0,ki)=SIN(theno)
     s(0,ki)=SIN(the(1,ki)-dthe)
     s(nj+1,ki)=SIN(the(nj,ki)+dthe)
     dyuKi(0:ni,0:nj,ki)= Re*dthe !interface
     dyvKi(0:ni,0:nj,ki)= Re*dthe 

     DO j=0,nj
       dxvKi(0:ni,j,ki)=s2(j,ki)*Re*dphi
       dxuKi(0:ni,j,ki)=s(j,ki)*Re*dphi
     ENDDO


     dd0=Re*dthe
     gmin=(0.01/dd0)**2
     dxcKi(0:ni,0:nj,ki)=dxvKi(0:ni,0:nj,ki)
     dycKi(0:ni,0:nj,ki)=dyuKi(0:ni,0:nj,ki)
     IF(istart==1) THEN
       CALL setPHOMkzGrid()
       istart=0
     ENDIF



   ELSEIF(IgridType(ki)==0) THEN !regular dx,dy grid
     CALL cdim(ki)

     PRINT *,"IgridType(ki)==0 and defined SQgrid: dx= 10.0 km *dphid"

     dxSQ=Re*dphi ! Re=1.e4*180.0/PI ! makes dx= 10.0 km *dphid
     dySQ=Re*dthe
     PRINT *,"dx,dy:",dxSQ,dySQ
     dxvKi(0:ni,0:nj,ki)=dxSQ
     dyuKi(0:ni,0:nj,ki)= dySQ
     dxgKi(1:ni,1:nj,ki)= dxSQ
     dysgKi(1:ni,1:nj,ki)= dySQ
     dygKi(1:ni,1:nj,ki)= dySQ
     dareaKi(1:ni,1:nj,ki)= dxSQ*dySQ
     dxuKi(0:ni,0:nj,ki)=dxSQ
     dyvKi(0:ni,0:nj,ki)=dySQ


     dd0=dxSQ
     gmin=(0.01/dd0)**2
     dxcKi(0:ni,0:nj,ki)=dxSQ
     dycKi(0:ni,0:nj,ki)=dySQ
     IF(istart==1) THEN
       CALL setPHOMkzGrid()
       istart=0
     ENDIF



# 204

   ELSE
     CALL outerrI2('seticegrid: comp flags are not correct for: ki, IgridType(ki)',ki,IgridType(ki))
   ENDIF !IgridType
   
   IF(ifHEAD==1) PRINT *,"for ki=",ki,"GSM icedyn grid ni, nj:",nxi(ki),nyi(ki)

   IF (nxiLL(ki)>mLLNA .OR. nyjLL(ki)>mLLNB .OR. nxiLL(ki)<5 .OR. nyjLL(ki)<5) THEN
     PRINT *, 'Lat/Lon LL dimensions too small '
     PRINT *, 'mLLNA, mLLNB: ',mLLNA,mLLNB
     PRINT *, 'nxiLL ',nxiLL,' nyjLL ',nyjLL,' ki: ',ki
     CALL outerrI('pGSMinit/seticegrid, mLLNA or mLLNB not large enough, reset in modcore.F90, ki:',ki)
   ELSE
     IF(ifHEAD==1) PRINT *,"for ki=",ki,"ni_LL, nj_LL:",nxiLL(ki),nyjLL(ki)
   END IF

 ENDDO !ki=1,NICE

 RETURN
END SUBROUTINE seticegrid
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE inFGhostSet(f,iu,xlon1,xlon2,ylat1,ylat2,ki,inew,ni,nj)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     call inFGhostSet(bedgl,37,xlon1,xlon2,ylat1,ylat2,ki,1,ni,nj)
!     Hs (ghosted,NICE) grid input routine, reads non-ghosted input, and sets ghost bounds

! REVERSES j order
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm, ONLY : dthed,dphid
 USE moddim, ONLY : mNAp, mNBp, NICE
 USE modflags
 IMPLICIT NONE

 REAL, INTENT(OUT)         :: f(0:mNAp,0:mNBp,NICE)
 INTEGER, INTENT(IN)    :: iu
! REAL, INTENT(OUT)      :: tym
 REAL, INTENT(OUT)      :: xlon1
 REAL, INTENT(OUT)      :: xlon2
 REAL, INTENT(OUT)      :: ylat1
 REAL, INTENT(OUT)      :: ylat2
 INTEGER, INTENT(IN)    :: ki
 INTEGER, INTENT(IN)    :: inew
 INTEGER, INTENT(IN)    :: ni
 INTEGER, INTENT(IN)    :: nj

 REAL :: scalefact,dthedin,dphidin
 INTEGER :: niin=0, njin=0, i, j, iformat=0
 CHARACTER (LEN=80) :: fdescript

 scalefact=0.


 IF(inew==1) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   READ(iu,*)iformat, scalefact

   IF(ifHEAD==1) THEN  
     IF(scalefact /= 0.0) THEN
       WRITE(*,*)'scaling input field by ',scalefact
     END IF
   ENDIF

! long/lats are grid boundaries, ie not centre of boundary grid cells
   READ(iu,*) niin,njin,xlon1,xlon2,ylat1,ylat2,dphidin,dthedin
! 3       format(2(i4,x),4(f11.6,x),2(f7.5,x))
!         write(6,*)'xlonmi,xlonma,ylatmi,ylatma,dphid,dthed'
!         write(6,*) xlonmi,xlonma,ylatmi,ylatma,dphid,dthed
   IF(ifHEAD==1) THEN 
     PRINT *,'xlon1,xlon2,ylat1,ylat2,dphidin,dthedin'
     PRINT *, xlon1,xlon2,ylat1,ylat2,dphidin,dthedin
     IF(dthed /= dthedin .OR. dphid /= dphidin) THEN
       WRITE(*,*)'WRONG grid size on input field'
       WRITE(*,*)'dphid,dphidin,dthed,dthedin: ',dphid,dphidin, dthed,dthedin
     END IF
   ENDIF
   IF(ni /= niin .OR. nj /= njin) THEN
     WRITE(*,*)'wrong # of points in input field: '
     WRITE(*,*)'ni,niin,nj,njin: ',ni,niin,nj,njin
     CALL outerr('pGSMinit/inF2, input dimensions do not match active dimensions')
   END IF
   READ(iu,*) !extra line for future additions, eg. unique run #
 END IF
 READ(iu,*)
! READ(iu,60)tym
!60 FORMAT(15X,f14.1)

 DO j=njin,1,-1
   READ(iu,*)(f(i,j,ki),i=1,niin)
   IF(scalefact /= 0.0) THEN
     DO i=1,niin
       f(i,j,ki)=scalefact*f(i,j,ki)
     END DO
   END IF
 END DO
 f(0,1:nj,ki)=f(1,1:nj,ki)
 f(ni+1,1:nj,ki)=f(ni,1:nj,ki)
 f(1:ni,0,ki)=f(1:ni,1,ki)
 f(1:ni,nj+1,ki)=f(1:ni,nj,ki)

 RETURN
END SUBROUTINE inFGhostSet
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE inF2(f,iu,tym,xlon1,xlon2,ylat1,ylat2,  &
   ki,inew,ifldcode,nd1,nd2,nd3,ni,nj)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     call inF2(bedgl,37,ttime,xlon1,xlon2,ylat1,ylat2,
!     ki,1,4,mNA,mNB,NICE,ni,nj)
!     nd? are physical dimensions
!     n? are logical dimensions
!     general input routine, c range is true range of field c
! ifldcode:
! solid: 1 H  2 hp  3 hpo  4 bedtype 5 hd
! velocities 11 |v|_0 12 |v|_s 13 U_0 14 U_s 15 V_0 16 V_s 17 w
! mass-balance 21 precip 21 accum 23 ablat 24 Gb
! thermodyn: 31 T_0 32 T_b 33 T_b-1 34 GHF

!     iformcode: format type:
!     1 hd/hp/V: f7.1
!     2 T/ablat: f6.2
!     3 H: f6.1
!     4 w/Gb: f5.2
!     5 accum: f4.2
!     6 bedgl: i2
!     7 general: e15.7
!     else *

! REVERSES j order
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm, ONLY : dthed,dphid
 use modflags
 IMPLICIT NONE

 REAL, INTENT(OUT)         :: f(nd1,nd2,nd3)
 INTEGER, INTENT(IN)    :: iu
 REAL, INTENT(OUT)      :: tym
 REAL, INTENT(OUT)      :: xlon1
 REAL, INTENT(OUT)      :: xlon2
 REAL, INTENT(OUT)      :: ylat1
 REAL, INTENT(OUT)      :: ylat2
 INTEGER, INTENT(IN)    :: ki
 INTEGER, INTENT(IN)    :: inew
 INTEGER, INTENT(IN)    :: ifldcode
 INTEGER, INTENT(IN)    :: nd1
 INTEGER, INTENT(IN)    :: nd2
 INTEGER, INTENT(IN)    :: nd3
 INTEGER, INTENT(IN)    :: ni
 INTEGER, INTENT(IN)        :: nj

 REAL :: scalefact,dthedin,dphidin
 INTEGER :: niin=0, njin=0, i, j, iformat=0,ifldc1=0
 CHARACTER (LEN=80) :: fdescript
 CHARACTER (LEN=6) :: chfieldtype(9,4)
 DATA chfieldtype/'H','hp','hpo','bedtyp','h_surf',' ',' ',' ',' ',  &
    '|v|_0','|v|_sl','U_0','U_sl','V_0','V_sl','w',' ',' ',  &
    'precip','accum','ablat','Gb',' ',' ',' ',' ',' ',  &
    'T_0','T_b','T_b-1','GHF',' ',' ',' ',' ',' '/


 scalefact=0.


 IF(inew==1) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   READ(iu,*)ifldc1,iformat, scalefact
! 2       format(2(i2,x),e9.1)

   IF(ifHEAD==1) THEN  
     IF(ifldc1>99) THEN
       WRITE(*,*) "Data has grid-centered region def in header"
       ifldc1=ifldc1-100
     END IF

     IF(scalefact /= 0.0) THEN
       WRITE(*,*)'scaling input field by ',scalefact
     END IF

     IF(MOD(ifldcode,10)==0) THEN
       WRITE(*,*)'field type is undefined'
     ELSE IF(ifldcode<10) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode,1)
     ELSE IF(ifldcode<20) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode-10,2)
     ELSE IF(ifldcode<30) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode-20,3)
     ELSE IF(ifldcode<40) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode-30,4)
     ELSE
       WRITE(*,*)'field type is undefined'
     END IF

     IF(ifldc1 /= ifldcode) THEN
       WRITE(*,*)'**warning : ifieldcode in, ifldcode called: ', ifldc1,ifldcode
!         call outerr('wrong field code')
     END IF
   ENDIF

! long/lats are grid boundaries, ie not centre of boundary grid cells
   READ(iu,*) niin,njin,xlon1,xlon2,ylat1,ylat2,dphidin,dthedin
! 3       format(2(i4,x),4(f11.6,x),2(f7.5,x))
!         write(6,*)'xlonmi,xlonma,ylatmi,ylatma,dphid,dthed'
!         write(6,*) xlonmi,xlonma,ylatmi,ylatma,dphid,dthed
   IF(ifHEAD==1) THEN 
     PRINT *,'xlon1,xlon2,ylat1,ylat2,dphidin,dthedin'
     PRINT *, xlon1,xlon2,ylat1,ylat2,dphidin,dthedin
     IF(dthed /= dthedin .OR. dphid /= dphidin) THEN
       WRITE(*,*)'WRONG grid size on input field'
       WRITE(*,*)'dphid,dphidin,dthed,dthedin: ',dphid,dphidin, dthed,dthedin
     END IF
   ENDIF
   IF(ni /= niin .OR. nj /= njin) THEN
     WRITE(*,*)'wrong # of points in input field: '
     WRITE(*,*)'ni,niin,nj,njin: ',ni,niin,nj,njin
     CALL outerr('pGSMinit/inF2, input dimensions do not match active dimensions')
   END IF
   READ(iu,*) !extra line for future additions, eg. unique run #
 END IF
 READ(iu,60)tym
60 FORMAT(15X,f14.1)

 DO j=njin,1,-1
   READ(iu,*)(f(i,j,ki),i=1,niin)
   IF(scalefact /= 0.0) THEN
     DO i=1,niin
       f(i,j,ki)=scalefact*f(i,j,ki)
     END DO
   END IF
 END DO

 RETURN
END SUBROUTINE inF2
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE in2F2(f,f2,iu,tym,xlon1,xlon2,ylat1,ylat2,  &
   inew,ifldcode,nd1,nd2,ni,nj,nij0)

!         call in2F2(Hs,hps,iuin,tym,xlon1,xlon2,ylat1,ylat2,
!     $        inew,ifldcode,NAP2,NBP2,ni,nj,nij0)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     2 field version
!     call inF2(bedgl,37,ttime,xlon1,xlon2,ylat1,ylat2,
!     ki,1,4,mNA,mNB,NICE,ni,nj,nij0)
!     nd? are physical dimensions
!     n? are logical dimensions
!     general input routine, c range is true range of field c
! ifldcode:
! solid: 1 H  2 hp  3 hpo  4 bedtype 5 hd 6 H hp
! velocities 11 |v|_0 12 |v|_s 13 U_0 14 U_s 15 V_0 16 V_s 17 w
! mass-balance 21 precip 21 accum 23 ablat 24 Gb
! thermodyn: 31 T_0 32 T_b 33 T_b-1 34 GHF

!     iformcode: format type:
!     1 hd/hp/V: f7.1,x,f7.1
!     2 T/ablat: f6.2
!     3 H: f6.1,x,f7.1
!     4 w/Gb: f5.2
!     5 accum: f4.2
!     6 bedgl: i2
!     7 general: e15.7
!     else *

! REVERSES j order
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm, ONLY : dthed,dphid
 use modflags
 IMPLICIT NONE

 REAL, INTENT(OUT)      :: f(nd1,nd2)
 REAL, INTENT(OUT)     :: f2(nd1,nd2)
 INTEGER, INTENT(IN)    :: iu
 REAL, INTENT(OUT)      :: tym
 REAL, INTENT(OUT)      :: xlon1
 REAL, INTENT(OUT)      :: xlon2
 REAL, INTENT(OUT)      :: ylat1
 REAL, INTENT(OUT)      :: ylat2
 INTEGER, INTENT(IN)    :: inew
 INTEGER, INTENT(IN)    :: ifldcode
 INTEGER, INTENT(IN)    :: nd1
 INTEGER, INTENT(IN)    :: nd2
 INTEGER, INTENT(IN) :: ni
 INTEGER, INTENT(IN) :: nj
 INTEGER, INTENT(IN) :: nij0

 REAL :: scalefact,dthedin,dphidin
 INTEGER :: niin, njin, i, j, iformat,ifldc1, nip, njp, ierrLine
 CHARACTER (LEN=60) :: fdescript
! CHARACTER (LEN=16) :: ctimefill
 CHARACTER (LEN=6) :: chfieldtype(9,4)
 SAVE niin,njin,iformat
 DATA chfieldtype/'H','hp','hpo','bedtyp','h_surf','H,hp',' ', ' ',' ',  &
    '|v|_0','|v|_sl','U_0','U_sl','V_0','V_sl','w',' ',' ',  &
    'precip','accum','ablat','Gb',' ',' ',' ',' ',' ',  &
    'T_0','T_b','T_b-1','GHF',' ',' ',' ',' ',' '/



 IF(inew==1) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   READ(iu,*,IOSTAT=ierrLine)ifldc1,iformat, scalefact
!2  FORMAT(2(i2,x),e9.1)
   IF(ierrLine /=0) &
      CALL outerrI('SUBROUTINE in2F2 has header line# 2 IOSTAT=',ierrLine)

   IF(ifHEAD==1) THEN
     IF(MOD(ifldcode,10)==0) THEN
       WRITE(*,*)'field type is undefined'
     ELSE IF(ifldcode<10) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode,1)
     ELSE IF(ifldcode<20) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode-10,2)
     ELSE IF(ifldcode<30) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode-20,3)
     ELSE IF(ifldcode<40) THEN
       WRITE(*,*)'field type is ',chfieldtype(ifldcode-30,4)
     ELSE
       WRITE(*,*)'field type is undefined'
     END IF
   ENDIF
   IF(ifldc1 /= ifldcode) THEN
     WRITE(*,*)'ifieldcode in, ifldcode called: ',ifldc1,ifldcode
     CALL outerr('wrong field code')
   END IF

! long/lats are grid boundaries, ie not centre of boundary grid cells
   READ(iu,*,IOSTAT=ierrLine) &
      niin,njin,xlon1,xlon2,ylat1,ylat2,dphidin,dthedin
   IF(ierrLine /=0) &
      CALL outerrI('SUBROUTINE in2F2 has header line# 3 IOSTAT=',ierrLine)

!3  FORMAT(2(i4,x),4(f11.6,x),2(f7.5,x))
!         write(6,*)'xlonmi,xlonma,ylatmi,ylatma,dphid,dthed'
!         write(6,*) xlonmi,xlonma,ylatmi,ylatma,dphid,dthed
   IF(ifHEAD==1) THEN  
     PRINT *,'xlon1,xlon2,ylat1,ylat2,dphidin,dthedin'
     PRINT *, xlon1,xlon2,ylat1,ylat2,dphidin,dthedin

     IF(dthed /= dthedin .OR. dphid /= dphidin) THEN
       WRITE(*,*)'WRONG grid size on input field'
       WRITE(*,*)'dphid,dphidin,dthed,dthedin: ',dphid,dphidin, dthed,dthedin
     END IF
   ENDIF
   IF(ni /= niin .OR. nj /= njin) THEN
     WRITE(*,*)'wrong # of points in input field: '
     WRITE(*,*)'ni,niin,nj,njin: ',ni,niin,nj,njin
     CALL outerr('pGSMinit/in2F2, input dimensions do not match active dimensions')
   END IF
   READ(iu,*) !extra line for future additions, eg. unique run #
 END IF
! READ(iu,*,IOSTAT=ierrLine) ctimefill, tym
 READ(iu,60,IOSTAT=ierrLine) tym
60 FORMAT(15X,f14.1)
   IF(ierrLine /=0) &
      CALL outerrI('SUBROUTINE in2F2 has header line# 5 IOSTAT=',ierrLine)

 njp=nj+nij0-1
 nip=ni+nij0-1

 IF (nip>nd1 .OR. njp>nd2) THEN
   PRINT *,"overdimension error in in2F2"
   PRINT *,"nd1, nip, nd2, njp, nij0 ",nd1, nip, nd2, njp, nij0
   CALL outerr('pGSMinit/in2F2, dimension error in subroutine call')
 ENDIF

 DO j=njp,nij0,-1
   READ(iu,*)(f(i,j),f2(i,j),i=nij0,nip)
 END DO

 RETURN
END SUBROUTINE in2F2
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE restunf(tnew,hs,hps,uolds,volds,iu,ki)
! write binary restart file (all fields except GIA)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! unformatted restart save

 USE modgridm
 USE modinparm, ONLY : IFMWPext
 USE modlake
 USE modGb
 USE modicetherm, ONLY : Tcs
 USE modflags, ONLY : isthermi
 USE modnewdelt
 USE modcorevar, ONLY : sealevSc

 USE modiceUV, ONLY : uak, vak, uiak, viak, uitk, vitk
 USE modicestore, ONLY : hpLLs

# 602


# 610


# 614


 IMPLICIT NONE

 REAL, INTENT(IN)    :: tnew
 REAL, INTENT(IN)    :: Hs(0:mNAp,0:mNBp,NICE), hps(0:mNAp,0:mNBp,NICE)
 REAL, INTENT(IN)    :: uolds(0:mNA,0:mNB,NCZ,NICE),volds(0:mNA,0:mNB,NCZ,NICE)
 INTEGER, INTENT(IN)    :: iu
 INTEGER, INTENT(IN)   :: ki
 INTEGER, PARAMETER :: Iversion=3, ITversion=2, IUVversion=7

 WRITE(iu) tnew
 WRITE(iu) ni,nj
 WRITE(iu) hs(0:nip,0:njp,ki)
 WRITE(iu) hps(0:nip,0:njp,ki)
 WRITE(iu) sealevSc
 WRITE(iu) stpfact(ki), istp(ki),  ktcur(ki)
 IF(isthermi == 1) THEN
   WRITE(iu) NCZ,IUVversion
   WRITE(iu) uolds(0:ni,0:nj,:,ki)
   WRITE(iu) volds(0:ni,0:nj,:,ki)

   WRITE(iu) uak(0:ni,0:nj,ki)
   WRITE(iu) vak(0:ni,0:nj,ki)
   WRITE(iu) uiak(0:ni,0:nj,ki)
   WRITE(iu) viak(0:ni,0:nj,ki)
   WRITE(iu) uitk(0:ni,0:nj,ki)
   WRITE(iu) vitk(0:ni,0:nj,ki)

 ENDIF

 IF(isthermi == 1) THEN
   WRITE(iu) isthermi,ITversion
   WRITE(iu) NCFZ
   WRITE(iu) Tcs(0:nip,0:njp,1:NCFZ,ki)
   WRITE(iu) Gbs(1:ni,1:nj,ki)
   WRITE(iu) ni,nj,ki
 END IF

# 658


# 668


 IF(IFMWPext==1) THEN
   WRITE(iu) IFMWPext, Iversion
   WRITE(iu) hfillb(0:nip,0:njp,ki)
   WRITE(iu) ni,nj,ki
 END IF

! need to add basal hydrology restart writes here

# 686


 RETURN
END SUBROUTINE restunf
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE initice(ihpt0,ki) !legacy name initia(ki)
! read in topos
! initialize : modicestore[hs,hps] ,/chpiso/hpiso, full restart readin (except for option basal hydro/sed)
! (or read from restart file along with convergence failure reloop variables)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modIU
 USE modflags
 USE modicestore
 USE modtopo
 USE modicefields
 USE modicelegacy
 USE modcorevar, ONLY : newrun, sealevSc
 USE modnewdelt
 USE modconstants, ONLY : RHOIc,RHOearth
 USE modgridm
 USE modicetherm, ONLY : Tcs
 USE modGb

 USE modiceUV, ONLY : uak, vak, uiak, viak, uitk, vitk
 USE modinparm, ONLY : kEA, kNA
# 717



 
 IMPLICIT NONE

 INTEGER, INTENT(IN)          :: ki, ihpt0(NICE)
 INTEGER, PARAMETER :: IUVversion=7
 INTEGER :: nczfrst, iuvversionrst, ificethermrest, iTversion
 INTEGER :: i,j


 REAL :: rdep1(0:mNAp,0:mNBp)
 REAL :: rhor, xlon1, xlon2, ylat1, ylat2, tymRestartFile
 INTEGER :: nirst, njrst, krst, ncfzrst

!     initialize h field.
!     initialize hp field, knowing that hp(t=0)=hpo.

 rhor=RHOIc/RHOearth

   IF(itop(ki)==0) THEN
     hpo(:,:,ki)=0.
     hposPD(:,:,ki)=0.
     hs0(:,:,ki)=0.
     READ(*,*) ! skip hp t0 input
     IF(ifHEAD==1) PRINT *, 'itop=0, topo (ie hpo, hposPD) and H set to 0.'
   ELSE
     CALL readTopoH0(ki) ! standard format basal (hpo) and surface topo field and hs0 input or set
     CALL adjustTopo(ihpt0(ki),ki) ! Do GIA correction for t0 topo hpo
   ENDIF



 IF(newrun(ki) == 1) THEN
   hps(0:nip,0:njp,ki)=hpo(0:nip,0:njp,ki)
   hs(0:nip,0:njp,ki)=hs0(0:nip,0:njp,ki)
!   uolds=0.
!   volds=0.

 END IF !newrun==1



!************restart section*********************
 IF(newrun(ki) <= 0) THEN

   READ(iurestrt(ki)) tymRestartFile
   PRINT *, 'restart time from restart file: tymRestartFile= ',tymRestartFile,' ki= ',ki
   IF(t0 /= tymRestartFile) PRINT *, 'NOTE: t0,t0in for restart are different: ',t0,tymRestartFile

   READ(iurestrt(ki)) nirst,njrst
   IF(nirst/=ni .OR. njrst/=nj) THEN 
     PRINT *,"nirst,ni,njrst,nj: ",nirst,ni,njrst,nj
     CALL outerr("restart file has ni,nj mismatch")
   ENDIF
   READ(iurestrt(ki)) hs(0:nip,0:njp,ki)
   READ(iurestrt(ki)) hps(0:nip,0:njp,ki)
   READ(iurestrt(ki)) sealevSc
   PRINT *,"restart file sealevSc is ",sealevSc
   READ(iurestrt(ki)) stpfact(ki), istp(ki), ktcur(ki)
   IF(isthermi == 1) THEN
     READ(iurestrt(ki)) nczfrst, iuvversionrst
     IF((nczfrst /= NCZ) .OR. (iuvversionrst /= IUVversion)) THEN
       PRINT *,"initice: restart file check error for NCZ/in or IUVversion/in: "&
          ,NCZ,nczfrst,IUVversion,iuvversionrst
       CALL outerr("restart file has version mismatch, likely dimension mismatch")
     ENDIF
     READ(iurestrt(ki)) uolds(0:ni,0:nj,:,ki)
     READ(iurestrt(ki)) volds(0:ni,0:nj,:,ki)

     READ(iurestrt(ki)) uak(0:ni,0:nj,ki)
     READ(iurestrt(ki)) vak(0:ni,0:nj,ki)
     READ(iurestrt(ki)) uiak(0:ni,0:nj,ki)
     READ(iurestrt(ki)) viak(0:ni,0:nj,ki)
     READ(iurestrt(ki)) uitk(0:ni,0:nj,ki)
     READ(iurestrt(ki)) vitk(0:ni,0:nj,ki)


     WRITE(*,*)"Tc restart about to read in Tcs and Gbs"
     READ(iurestrt(ki)) ificethermrest, iTversion
     IF(ificethermrest /= 1 .OR. iTversion /= 2) THEN
       PRINT *,"Thermo restart error: ificethermrest, iTversion: ",ificethermrest, iTversion
       CALL outerr("restart file has version mismatch, likely dimension mismatch")
     ENDIF
     READ(iurestrt(ki)) ncfzrst
     IF(ncfzrst /= NCFZ) THEN
       PRINT *,"ncfzrst, NCFZ:",ncfzrst, NCFZ
       CALL outerr("restart file has NCFZ mismatch")
     ENDIF

     READ(iurestrt(ki)) Tcs(0:nip,0:njp,1:NCFZ,ki)
     READ(iurestrt(ki)) Gbs(1:ni,1:nj,ki)
     READ(iurestrt(ki)) nirst,njrst,krst
     IF(nirst/=ni .OR. njrst/=nj) THEN 
       PRINT *,"therm restart tail nirst,ni,njrst,nj,krst,ki: ",nirst,ni,njrst,nj,krst,ki
       CALL outerr("therm restart file tail has ni or nj mismatch")
     ENDIF

   ENDIF
 END IF !newrun==0

 hpiso(0:nip,0:njp,ki)= hpo(0:nip,0:njp,ki)-rhor*hs(0:nip,0:njp,ki)

 IF(newrun(ki) == -1) THEN
   PRINT *,"newrun(ki)==-1, resetting hps/hs to topo inputs"
   hps(0:nip,0:njp,ki)=hpo(0:nip,0:njp,ki)
   hs(0:nip,0:njp,ki)=hs0(0:nip,0:njp,ki)
!   uolds=0.
!   volds=0.
 END IF 

  
# 852

   hpLLs(0:nip,0:njp,ki)=hps(0:nip,0:njp,ki)
   hLLs(0:nip,0:njp,ki)=hs(0:nip,0:njp,ki) 


!************END restart section*********************

! Legacy diagnostics for non-linear SIA solution

!     initialize maximum of cfl and ddim

 cflm(ki)=-1.e20
 ddimm(ki)=-1.e20

!     initialize variables for check of cons. of mass

 sumtr(ki)=0.0
 sumtl(ki)=0.0
 perma(ki)=0.0

!     initialize variables for convergence tests
 noconv(ki)=0
 iyconv(ki)=0
 ntiter(ki)=0
!     initialize divergence times
 times(1:ntimes,ki)=0.0

# 881



 RETURN
END SUBROUTINE initice
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

# 911


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGHF()
! read in deep geothermal heat flux map(s)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     CALL readGHF()
!     Qb = GHF(i,j,ki) - qbtauoh*Hij !implict in U and H
!     reads GHF?A.dat
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modconstants
 USE modtopo
 USE modbedtherm, ONLY : GHF
 USE modgridm
 USE moddim
 USE modinparm, ONLY : kNA, wGF1
 USE modStation
 IMPLICIT NONE

 INTEGER :: idatform,ki,i,j,iu, kin, iINPUTENDcheck
 CHARACTER (LEN=80) :: fdescript
 REAL :: GHFp, GHFbayf, x, y, GHFin(mNA,mNB,2)
 INTEGER :: ios, nin, njn
 REAL :: xlonmiin,xlonmain,ylatmiin,ylatmain

 DO ki = 1,NICE
   CALL cdim(ki)
   iu = 37

   DO kin= 1,2
     CALL askofil(iu,'GHFlux file')

     READ(iu,'(A)')fdescript
     IF(ifHEAD==1) THEN
       WRITE(*,*)ki,fdescript
       WRITE(2,*)ki,fdescript
     ENDIF
     READ(iu,*,IOSTAT=ios) idatform
     IF  (ios/=0 .OR. idatform > 4) THEN
       idatform=1
       BACKSPACE(iu)
     END IF
     IF(ifHEAD==1) WRITE(*,*) "GHFdat format type ",idatform

     IF(idatform>1) THEN
       READ(iu,*)nin,njn, xlonmiin,xlonmain,ylatmiin,ylatmain
       IF(nin /= ni .OR. njn /= nj) THEN
         WRITE(*,*)"ni,nin,nj,njn: ",ni,nin,nj,njn
         CALL outerr('ni,nj mismatch for GF??.dat')
       ELSEIF(ABS(xlonmiin-xlonmi(ki))>0.0001 .OR. ABS(xlonmain-xlonma(ki))>0.0001 .OR. &
          ABS(ylatmiin-ylatmi(ki))>0.0001 .OR. ABS(ylatmain-ylatma(ki))>0.0001) THEN
         WRITE(*,*)" xlonmiin,xlonmain,ylatmiin,ylatmain:",xlonmiin,xlonmain,ylatmiin,ylatmain
         PRINT *,"should be ",xlonmi(ki), xlonma(ki), ylatmi(ki), ylatma(ki)
         CALL outerr('grid bounds mismatch for GF??.dat')
       END IF
       READ(iu,*)
     END IF

     IF(idatform>2) THEN
!            read(*,*)GHFp
!            write(*,*)"GHFlux increased by:(mW/m2) ",GHFp
       READ(iu,*)
       DO j = nj,1,-1
         READ(iu,*)(GHFin(i,j,kin),i= 1,ni)
       END DO
     READ(iu,*) iINPUTENDcheck
     IF(iINPUTENDcheck/=777) &
        CALL outerrI('pGSMinit.F90/GHFlux end of infile error =>header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
     ELSE

       DO j = 1,nj
         READ(iu,2)(GHFin(i,j,kin),i= 1,ni)
2        FORMAT(8F8.3)
       END DO
     READ(iu,*) iINPUTENDcheck
     IF(iINPUTENDcheck/=777) &
        CALL outerrI('pGSMinit.F90/GHFlux end of infile error =>header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
     END IF
     CLOSE(iu)
   ENDDO

# 995

   GHF(1:ni,1:nj,ki)=(wGF1*GHFin(1:ni,1:nj,1)+(1.-wGF1)*GHFin(1:ni,1:nj,2)) *0.001*SECYR


# 1009


! hpo not set yet
!$$$         write(*,*) "GHF+GHFbayf: G of Boothia mask"
!$$$         ia=ix(-92.5)
!$$$         ib=ix(-86.5)
!$$$         ja=jy(74.65)
!$$$         jb=jy(67.3)
!$$$         WHERE(hpo(ia:ib,ja:jb,ki)>50.)
!$$$            GHF(ia:ib,ja:jb,ki)=GHF(ia:ib,ja:jb,ki) + GHFbayf
!$$$         END WHERE

   IF(ifHEAD==1) THEN
     WRITE(*,*)'GHF read in for ki= ',ki
     WRITE(*,*)'GHF at stn1: ',GHF(nio(ki,1),njo(ki,1),ki)*1000. /SECYR
   ENDIF

 END DO
 RETURN
END SUBROUTINE readGHF
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE setGHF()
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! set GHF(,) to Qbar0
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modconstants
 USE modbedtherm, ONLY : GHF
 IMPLICIT NONE
!INCLUDE 'sit.inc.f'

!      PARAMETER(Qbar0 = 0.05)  !W/m^2, 0.05 for Greenland, else 0.045
!      PARAMETER(Qbar0 = 0.047)    !W/m^2, 0.05 for Greenland, else 0.045
!      PARAMETER(Qbar0 = 0.055)    !W/m^2, 0.05 for Greenland, else 0.045
!      PARAMETER(Qb0yr=Qbar0*SECYR)
 REAL :: qbar0, qb0yr

 WRITE(*,*)'ENTER QBAR0 W/m^2'
 READ(*,*)qbar0
 qb0yr=qbar0*SECYR
 GHF=qb0yr
 WRITE(*,*)'GHF set to Qbar0 = ',qbar0*1.e3, 'mW/m^2'

 RETURN
END SUBROUTINE setGHF
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE setpre(precip)
! set precip to sum of vpre*water ice conversion
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modconstants, ONLY : sconv
 USE modclimforce, ONLY : vpre
 USE modgridm, ONLY : ni, nj, cdim
 USE moddim
 IMPLICIT NONE

 REAL, INTENT(OUT)    :: precip(mNA,mNB,NICE)
 REAL :: spre
 INTEGER ki, i, j, it

 DO ki=1,NICE
   CALL cdim(ki)
   DO j=1,nj
     DO i=1,ni
       spre=0.
       DO it=1,Nmonth
         spre = spre + vpre(i,j,ki,it)
       END DO
!               precip(i,j,ki)=sconv*spre !readvpre now does sconv*
       precip(i,j,ki)=SCONV*spre
     END DO
   END DO
 END DO


 RETURN
END SUBROUTINE setpre
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readTopoH0(ki)
! standard format basal topo field input
! reads modern topo : hpo (basal), hposPD (surface)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modIOnames
 USE modtopo
 USE modgridm
 USE modicestore, ONLY : hs0
 USE modinparm, ONLY : kNA, kGRonly, kANT, kEA
 USE modconstants, ONLY : RHCONV, RfloatCONV
 USE modgengrid, ONLY : xy2ij

 IMPLICIT NONE

! REAL, INTENT(OUT)              :: hposPD(0:mNAp,0:mNBp,NICE)
 INTEGER, INTENT(IN)                      :: ki
 INTEGER :: iu, i, j, nip2, njp2, iINPUTENDcheck, ib, jb
 REAL ttime, ylat1, ylat2, xlon1, xlon2

!      ix(x) = nint((x-xlonmi(ki) + dphid/2.)/dphid)
!      jy(y) = nint((ylatma(ki) + dthed/2. - y)/dthed)

 iu=37
 OPEN (UNIT=37,FILE=topofn(ki),STATUS='old')
 IF(ifHEAD==1) THEN
   PRINT *, 'ki= ',ki
   PRINT *, 'ni,nj,nip ',ni,nj,nip
 ENDIF

!     initialize fields to zero

 njp2=njp+1
 nip2=nip+1
 IF(ifHEAD==1) PRINT *,"about to open basal topo file"
!#if  defined (CAPcal) || defined (1)
# 1129

 IF( IgridType(ki)<2) THEN
   CALL inF2(hpo,iu,ttime,xlon1,xlon2,  &
      ylat1,ylat2,ki,1,3,mNAp+1,mNBp+1,NICE,nip2,njp2)
 ELSE
   CALL inFGhostSet(hpo,iu,xlon1,xlon2,ylat1,ylat2,ki,1,ni,nj)
 ENDIF


   READ(iu,*) iINPUTENDcheck
   IF(iINPUTENDcheck/=111) CALL &
      outerrI('pGSMinit/readTopoH0 hpo endFile Check Error => header/format/dimension mismatch, iINPUTENDcheck:',iINPUTENDcheck)

 CLOSE(iu)

# 1146

IF(ki==kGRonly) THEN !remove Scotland traces
  CALL xy2ij(-14.9,63.1,ib,jb)
  hpo(ib:nip,jb:njp,ki)=MAX(300.,hpo(ib:nip,jb:njp,ki))
ENDIF


IF(ki==kEA) THEN ! remove NE GR traces
  CALL xy2ij(0.0,70.0,ib,jb)
  hpo(0:ib,0:jb,ki)=MAX(600.,hpo(0:ib,0:jb,ki))
ENDIF


 IF(ki==kNA .OR. ki==kGRonly .OR. ki==kANT .OR. ki==kIC) THEN
!   CALL GRtopfix(hpo(:,:,ki),ki)
   hpo0PD(:,:,ki)=hpo(:,:,ki)
 ENDIF

 IF(ki==kNA .OR. ki==kGRonly .OR. ki==kIC) THEN
   CALL askofil(iu,'hposPD topo with embedded PD ice')
# 1168

   CALL inF2(hposPD,iu,ttime,xlon1,xlon2,  &
      ylat1,ylat2,ki,1,3,mNAp+1,mNBp+1,NICE,nip2,njp2)


   READ(iu,*) iINPUTENDcheck
   IF(iINPUTENDcheck/=111) CALL &
      outerrI('pGSMinit/readTopoH0 hposPD endFile Check Error => header/format/dimension mismatch, iINPUTENDcheck:',iINPUTENDcheck)

   CLOSE(iu)

   IF(ki==kGRonly) hposPD(ib:nip,jb:njp,ki)=MAX(300.,hposPD(ib:nip,jb:njp,ki)) !remove Scotland traces

!      else if(xlonmi(ki)>-16.0) then  !fix possible blow up point
!        hpo(1:ix(-3.1),jy(82.2):jy(72.2),ki)
!     $        =max(700.0,hpo(1:ix(-3.1),jy(82.2):jy(72.2),ki))
!        hposPD(:,:,ki)=hpo(:,:,ki)
   WRITE(*,*)'initializing H to hpo0PD-hposPD, ki=',ki
   hs0(0:nip,0:njp,ki)= MAX(hpo0PD(0:nip,0:njp,ki)-hposPD(0:nip,0:njp,ki),0.)
   ifColdStartIce(ki)=1
 ELSE IF(ki==kANT) THEN
   CALL askofil(iu,'hs0 PD ice thickness')
   CALL inFGhostSet(hs0,iu,xlon1,xlon2,ylat1,ylat2,ki,1,ni,nj)
!   CALL inF2(hs0,iu,ttime,xlon1,xlon2,  &
!      ylat1,ylat2,ki,1,3,mNAp+1,mNBp+1,NICE,nip2,njp2)

   READ(iu,*) iINPUTENDcheck
   IF(iINPUTENDcheck/=111) CALL &
      outerrI('pGSMinit/readTopoH0 hs0 endFile Check Error => header/format/dimension mismatch, iINPUTENDcheck:',iINPUTENDcheck)

   CLOSE(iu)
!   hposPD(0:nip,0:njp,ki)= MIN(hpo(0:nip,0:njp,ki),hs0(0:nip,0:njp,ki)/RHCONV) - hs0(0:nip,0:njp,ki)
   hposPD(0:nip,0:njp,ki)= hpo(0:nip,0:njp,ki) - hs0(0:nip,0:njp,ki)
   ifColdStartIce(ki)=1
 ELSE
   hposPD(:,:,ki)=hpo(:,:,ki)
   hs0(:,:,ki)=0.
   ifColdStartIce(ki)=0
 END IF

 DO j=0,njp
   DO i=0,nip
     IF(RHCONV*hpo0PD(i,j,ki)>Hs0(i,j,ki)) THEN
       IF(Hs0(i,j,ki)>1.0) THEN
         hSurfPD(i,j,ki)=RfloatCONV*Hs0(i,j,ki) !floating PD ice
       ELSE
         hSurfPD(i,j,ki)=-hpo0PD(i,j,ki) !no PD ice (= hposPD for this case)
       ENDIF
     ELSE
       hSurfPD(i,j,ki)=-hposPD(i,j,ki)
     ENDIF
   ENDDO
 ENDDO

 RETURN
END SUBROUTINE readTopoH0
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE adjustTopo(ihpadj0t,ki)
! impose ihpadj0t Eemian GIA correction and any other topo adjustments
! call after initice
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modtopo
 USE modgridm
 USE modinparm, ONLY : kNA

 IMPLICIT NONE

 INTEGER, INTENT(IN)                  :: ihpadj0t
 INTEGER, INTENT(IN)                      :: ki
 INTEGER :: ihp0order
 REAL, DIMENSION(mNA,mNB) :: hadj0t,hpadj0t
 INTEGER :: iu, i, j, nij0
 REAL ttime, ylat1, ylat2, xlon1, xlon2

 iu=37
 nij0=1
 IF(ihpadj0t==1) THEN
   CALL askofil(iu,'hp0 correction topo')
   READ(iu,*) ihp0order
   IF(ifHEAD==1) WRITE(*,*)"hp0 correction order: ",ihp0order
   CALL in2F2(hadj0t,hpadj0t,iu,ttime,xlon1,xlon2,ylat1, ylat2,1,6,mNA,mNB,ni,nj,nij0)
   CLOSE(iu)
   IF(ifHEAD==1)  WRITE(*,*)"hp0 correction time ",ttime
 ELSE
   READ(*,*)
   IF(ifHEAD==1) WRITE(*,*)"NO hpadj0t correction"
 END IF


 IF(ki==kNA) THEN
   CALL hudStraitNAtopfix(hpo(:,:,ki),ki)

 IF(ifHEAD==1)  WRITE(*,*)"Nov5/16, Now no 650m cut on hpo for NA"

 END IF

 IF(ihpadj0t==1) THEN
!     $        hpo(1:ni,1:nj,ki)=(1.+ihp0order) *hpo(1:ni,1:nj,ki)
!     $        -hpadj0t(1:ni,1:nj)

# 1285

     hpo(1:ni,1:nj,ki)=(1.+ihp0order) *hpo(1:ni,1:nj,ki) -hpadj0t(1:ni,1:nj)
! ie hpo corrected = hpotopo + sum_1^ihp0order (hpotopo-hp0run_iorder)
! = hpo previous order -(hp0run_previousresult - hpotopo)
! hpadj0t_iorder=hpadj0t_{iorder-1} + hp0run_iorder

 END IF

 RETURN
END SUBROUTINE adjustTopo
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE inittildef(ifTillDef,hpoS,rmu)
! initialize basal drag coefficients map: dbedgl
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modconstants
 USE modgridm
 USE modbeddrag
 USE modinparm, ONLY : kNA, fbedpow, kEA
 USE modgengrid
 USE modicestore, ONLY : hs0
 USE modtopo, ONLY : hpo0PD

 IMPLICIT NONE
!INCLUDE 'sit.inc.f'

 INTEGER, INTENT(IN)                  :: ifTillDef
 REAL, INTENT(IN)                     :: hpoS(0:mNAp,0:mNBp,NICE)
 REAL, INTENT(IN)                     :: rmu

 REAL :: ftillscale


 REAL, PARAMETER :: CsHEINO  = 500.0 !yr-1


 REAL, PARAMETER :: rmtl=1.25, rmtlp=1.+rmtl, rmtlp2=2.+rmtl

 REAL, PARAMETER :: D0  = 7.9e-7,  & !sec-1
    rhot=2.39e+3,    & !kg/m^3
    rhoef=rhot-rhow, &
    rmu0=5.2e9,    & !Pa sec
    phif=22.        !friction angle ^o

 REAL, PARAMETER :: rhoIovE=RHOIc/RHOearth
 REAL :: x, y, dbg
!REAL :: tanphi, dtlldef
 INTEGER ix, jy, i, j, ki, ia, ib, ja, jb, i1, i2, j1, j2, iINPUTENDcheck
 REAL ttime, ylat1, ylat2, xlon1, xlon2 , hbUnload

!tanphi=TAN(phif*pi/180.)

!nn      read(*,*)rmu

!dtlldef= (2.*d0)**(1.-rmtl) *(RHOIc*gE)**rmtlp *re**(-rmtlp2) *SECYR  &
!    /( rmtlp *rhoef*gE*tanphi* rmu**rmtl) ![L^(-2.rmtl)/yr]
# 1346

   ftillscale=1.e4/1.e5**BEDGEXP !scale so that tilldef(100kPa stress)=10km/yr *rmu


 IF(ifTillDef==1 .AND. ifHEAD==1) THEN
!###################################################
# 1363

!####################### NOT EAcal #######################

# 1368

   PRINT *,'hw=0 : bed**fbedpow, fbedpow=',fbedpow
   PRINT *,'hw>0 && dbedgl>0.25 -> min(1.5,2.*dbedg)'

!   PRINT *,'hpoS<-10 : bed**fbedpow, fbedpow=',fbedpow
!   PRINT *,'hpoS>5 -> min(1.5,2.*dbedg(i,j)), >150: MAX(dbedgl,0.2)'
!##############################################

!#################### EAcal ##########################
# 1379


   PRINT *,'hpoS<-2 : Svalbard(hd0>350)=0'
   PRINT *,"tillmodN4e3: further 1.5*bedg:Booth:95:89W,67.8:73.8N"
   PRINT *,"tillmodN4e2: 1.5* Arctic core tillenhancement"

   PRINT *,'TILLDEF parameter: ',rmu
# 1388

 END IF

 IF(ifTillDef==1) THEN
   DO ki=1,NICE
     CALL cdim(ki)
     CALL askofil(37,'Bed till map')
     CALL inF2(dbedgl,37,ttime,xlon1,xlon2, ylat1,ylat2,1,1,4,mNA,mNB,1,ni,nj)
     READ(37,*) iINPUTENDcheck

     IF(iINPUTENDcheck/=888) &
        CALL outerrI('pGSMinit.F90/dbedgl end of infile error =>header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
     CLOSE(37)

!###################################################
# 1433

!###################################################
# 1451

! leave dbedgl as is
# 1488

!#################### EAcal ##########################


     IF(ki==kEA) THEN !Svaalbard fix:
       ia=ilx(10.1)
       ib=ilx(29.9)
       jb=jly(76.1)
       ja=jly(80.6)

       DO j=ja,jb
         DO i=ia,ib
           IF(hpoS(i,j,ki) < -350.) dbedgl(i,j)=0.
         ENDDO
       ENDDO


     ELSEIF(ki==kNA) THEN !NA Arctic archipelago marine enhancements
       ia=ilx(-104.5)
       ib=ilx(-85.5)
       jb=jly(67.2)
       ja=jly(79.2)

       WHERE(hpoS(ia:ib,ja:jb,ki)>1.)
         dbedgl(ia:ib,ja:jb)=1.5 *dbedgl(ia:ib,ja:jb)
       END WHERE

       ia=ilx(-94.5)
       ib=ilx(-89.5)
       jb=jly(67.8)
       ja=jly(73.8)

       WHERE(hpoS(ia:ib,ja:jb,ki)>1.)
         dbedgl(ia:ib,ja:jb)=1.5 *dbedgl(ia:ib,ja:jb)
       END WHERE

       ia=ilx(-88.5)
       ib=ilx(-86.5)
       jb=jly(68.2)
       ja=jly(70.8)

       WHERE(hpoS(ia:ib,ja:jb,ki)>1.)
         dbedgl(ia:ib,ja:jb)=1.5 *dbedgl(ia:ib,ja:jb)
       END WHERE
     END IF
     cbedDragStore(1:ni,1:nj,ki)=dbedgl(1:ni,1:nj)  ! till cover fraction
   END DO !ki=1:NICE
 END IF  ! IF(ifTillDef==1)



 IF(ifTillDef==2) THEN
   IF(ifHEAD==1) PRINT *,"ISMIP HEINO ni*nj bedtill input"
!   dbedgl=2.0 ! to get sediment on ghost ends
!   IF(ABS(BEDGEXP-1.0)>0.001) CALL outerr('ABS(BEDGEXP)>0.001, code set for linear soft bed law')
   DO ki=1,NICE
     CALL cdim(ki)
     CALL askofil(37,'Bed till map, HEINO format')
     IF(mNA >321) CALL outerr('mNA >321, HEINO tilldef ibedmsk read will need format change in code')
     DO i1=1,6
       READ(37,*)
     ENDDO
     DO j=1,nj
       READ(37,'(321i1)') (ibedmsk(i,j), i=1,ni) ! 2=till, 1=hard bed, 0 = marine
       DO i=1,ni
         IF(ibedmsk(i,j)==1) THEN
           dbedgl(i,j)=0.
         ELSEIF(ibedmsk(i,j)==2) THEN
           dbedgl(i,j)=1.
         ELSE ! marine
           dbedgl(i,j)=2.
         ENDIF
       ENDDO
     ENDDO
     cbedDragStore(1:ni,1:nj,ki)=dbedgl(1:ni,1:nj)  ! till cover fraction
     READ(37,*) iINPUTENDcheck
     IF(iINPUTENDcheck/=888) &
        CALL outerrI('pGSMinit.F90/dbedgl end of infile error =>header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
     CLOSE(37)
   ENDDO
   ftillscale=CsHEINO/(RHOIc*gEARTH)
   PRINT *,'rmu TILLDEF parameter: ',rmu
   PRINT *,'10kPa linear tilldef velocity is: ',  &
      1.0E4 *rmu*ftillscale,' m/yr'
 ELSEIF(ifTillDef==3) THEN
   IF(ifHEAD==1) PRINT *,"ISMIP EISMINT linear sliding via tilldef"
   READ(*,*) ! skip tilldef input file
   dbedgl=1.0 
   cbedDragStore=1.0
   ftillscale=1.e-3
   IF(ABS(BEDGEXP-1.0)>0.001) CALL outerr('ABS(BEDGEXP)>0.001, code set for linear soft bed law')
   PRINT *,'EISMINT SLIDING(TILLDEF) rmu parameter: ',rmu
   PRINT *,'10kPa linear tilldef velocity is: ',  &
      1.0E4*rmu*ftillscale,' m/yr'
 ENDIF  ! IF(ifTillDef==2 or 3)
! #if defined (1)

 IF(ifTillDef<1 .OR. ifTillDef>3) THEN
   IF(ifHEAD==1) PRINT *,"NO TillDef"
   dbedgl=0.
!   cbedDragStore=0.  (already static init to 0)
   READ(*,*)
 ELSE ! tilldef active
# 1598

!#else
! 1 debdgl is pure effective sediment fraction only
!   IF(ifHEAD==1) PRINT *, "HybridIceDyn: dbedg not scaled for compdm sliding"
!   WHERE (dbedgl(1:ni,1:nj,:) >0. .AND. dbedgl(1:ni,1:nj,:)<1.0)
!     dbedgl(1:ni,1:nj,:)=dbedgl(1:ni,1:nj,:)**2
!   END WHERE
!#endif
 ENDIF 


# 1632

 hbstd=50.


 RETURN
END SUBROUTINE inittildef
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE precnr(ki)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!     define precip(1:ni,1:nj,NICE)

 USE modIOnames
 USE modMBshare, ONLY : precip
 USE modgridm
 IMPLICIT NONE

 INTEGER, INTENT(IN)                      :: ki
 REAL :: sconv 
 REAL :: ttime, xlon1, xlon2, ylat1, ylat2
 INTEGER :: iu,i,j

 iu=37
 OPEN (UNIT=iu,FILE=precfn(ki),STATUS='old')
 PRINT *, 'ki= ',ki
 PRINT *, 'ni,nj ',ni,nj

 CALL inF2(precip,iu,ttime,xlon1,xlon2,  &
    ylat1,ylat2,ki,1,21,mNA,mNB,NICE,ni,nj)

 CLOSE(iu)

 sconv=1.0/0.910 !rhow_rain/RHOIc
 WRITE(*,*)"New precip adjusted to ice equivalent"
 DO j=1,nj
   DO i=1,ni
     precip(i,j,ki)=sconv*precip(i,j,ki)
   END DO
 END DO

 RETURN
END SUBROUTINE precnr
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE inithfill(newrun)
! init hfill (lake depths) field
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modIU
 USE modlake
 USE modgridm
!INCLUDE 'sit.inc.f'
 IMPLICIT NONE

 INTEGER :: newrun(NICE), iu, ki, Iversion, IFMWPextin
 INTEGER :: nirst,njrst,krst, iINPUTENDcheck, icode

 hfillb=0
 DO ki=1,NICE
   CALL cdim(ki)
   IF(newrun(ki)==0) THEN
     READ(*,*)
     READ(iurestrt(ki)) IFMWPextin, Iversion
     IF(IFMWPextin /= 1 .OR. Iversion /= 3) THEN
       PRINT *,"error: IFMWPextin, Iversion: ",IFMWPextin, Iversion, "for ki=",ki
       CALL outerr("restart file has version mismatch, likely dim mismatch or orbital forcing flag not =10 if -DEBMadd")
     ENDIF
     READ(iurestrt(ki)) hfillb(0:nip,0:njp,ki)
     READ(iurestrt(ki)) nirst,njrst,krst
     IF(nirst/=ni .OR. njrst/=nj) THEN 
       PRINT *,"inithfill restart tail nirst,ni,njrst,nj,krst,k: ",nirst,ni,njrst,nj,krst,ki
       CALL outerr("inithfill restart file tail has ni,nj mismatch")
     ENDIF
   ELSE
     CALL askfil(-37,"hfill0 correction")
     icode=-1
     CALL readGenFij(37,ni,nj,hfillb(1:ni,1:nj,ki),1,icode,1666)
   END IF !IF(newrun(ki)==0)
 END DO

 RETURN
END SUBROUTINE inithfill
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGenFij(iu,ni,nj,f,inew,icode,icheckEnd)
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! inew == 1 : check header, ==0 just read timeslice
! inew == 2 : 5 line header
! icheckEnd>0  : icheckEnd end of file and close
! icode=0 just output 2nd line icode, if < 0, skip 2nd line, else check if icode matches
! NOTE, icode is INTENT(INOUT) so must pass a variable, not constant
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modflags
 IMPLICIT NONE

 INTEGER, INTENT(IN)                  :: iu, inew, icheckEnd
 INTEGER, INTENT(IN)                      :: ni
 INTEGER, INTENT(IN)                      :: nj
 INTEGER, INTENT(INOUT)                   :: icode
 REAL, INTENT(OUT)                     :: f(ni,nj)
 INTEGER ::  nin, njn, i, j, iINPUTENDcheck, icodeIN
 CHARACTER (LEN=80) :: fdescript

 IF(inew >0) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   IF(icode<0) THEN 
     READ(iu,*) 
   ELSEIF(icode==0) THEN
     READ(iu,*) icode
   ELSE
     READ(iu,*) icodeIn
     IF(icodeIn/=icode) THEN
       PRINT *,"icode, icodeIn:",icode,icodeIN
       CALL outerrS("Terminating run due to icode mismatch in input file:",fdescript)
     ENDIF
     icode=icodeIn
   ENDIF
   READ(iu,*) nin,njn
   IF(ni/=nin .OR. nj/=njn) THEN
     PRINT *,"ni,nin,nj,njn:",ni,nin,nj,njn
     CALL outerrS("Terminating run due to ni or nj mismatch in input file:",fdescript)
   ENDIF
   READ(iu,*)
   IF(inew >1) READ(iu,*) ! to keep 5 line header standard
 ENDIF

 DO j=nj,1,-1
   DO i=1,ni
     READ(iu,*) f(i,j)
   END DO
 END DO

 IF(icheckEnd >0 ) THEN !END of file check

   READ(iu,*) iINPUTENDcheck
   IF(iINPUTENDcheck/=icheckEnd) THEN
     PRINT *,"for ",fdescript
     PRINT *,"end of file IcheckEnd code should be",icheckEnd,"but is",iINPUTENDcheck
     CALL outerrI('end of infile error => header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
   ENDIF

   CLOSE(iu)
 ENDIF


 PRINT *,"readGenFij completed for",fdescript


 RETURN
END SUBROUTINE readGenFij
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGenMaskijcheck(iu,ni,nj,msk,inew,icheckEnd)
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! inew == 1 : check header, ==0 just read timeslice
! icheckEnd >0  icheckEnd end of file and close
! read in integer field
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modflags
 IMPLICIT NONE

 INTEGER, INTENT(IN)                  :: iu, inew, icheckEnd
 INTEGER, INTENT(IN)                      :: ni
 INTEGER, INTENT(IN)                      :: nj
 INTEGER, INTENT(OUT)                     :: msk(ni,nj)
 INTEGER ::  nin, njn, i, j, iINPUTENDcheck

 CHARACTER (LEN=80) :: fdescript

 IF(inew == 1) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   READ(iu,*) 
   READ(iu,*) nin,njn
   IF(ni/=nin .OR. nj/=njn) THEN
     PRINT *,"ni,nin,nj,njn:",ni,nin,nj,njn
     CALL outerr("ni or nj mismatch in inithfill")
   ENDIF
   READ(iu,*)
   READ(iu,*)
 ENDIF

 DO j=nj,1,-1
   DO i=1,ni
     READ(iu,*) msk(i,j)
   END DO
 END DO
! CLOSE(iu)
 IF(icheckEnd>0) THEN !END of file check
   READ(iu,*) iINPUTENDcheck
   CLOSE(iu)
   IF(iINPUTENDcheck/=icheckEnd) THEN
     PRINT *,"for ",fdescript
     CALL outerrI('end of infile error => header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
   ENDIF
 ENDIF


 PRINT *,"readGenMaskijcheck completed for",fdescript


 RETURN
END SUBROUTINE readGenMaskijcheck
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!$$$cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!$$$      subroutine mskhs(hs,hms,ki)
!$$$cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!$$$c     MASKS OUT ELLESMERE
!$$$      USE modicelegacy
!$$$      real hs(0:mNAp,0:mNBp,NICE),hms(0:mNAp,0:mNBp)
!$$$      common/rconv2pal/dthed,dphid
!$$$      integer istart, i, j
!$$$
!$$$      do j=1,nj
!$$$         do i=1,ni
!$$$            if(imGrnlnd(i,j)==1) then
!$$$               hms(i,j)=hs(i,j,ki)
!$$$            else
!$$$               hms(i,j)=0.
!$$$            endif
!$$$         enddo
!$$$      enddo
!$$$
!$$$
!$$$ 999        return
!$$$            end
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE hudStraitNAtopfix(hpo,ki)
!Hudson Strait topographic fix routine
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modgridm
 USE modflags, ONLY : ifHEAD
 USE modinparm, ONLY : kNA
!INCLUDE 'sit.inc.f'
 IMPLICIT NONE

 REAL, INTENT(INOUT)             :: hpo(0:mNAp,0:mNBp)
 INTEGER, INTENT(IN)           :: ki
 INTEGER :: ia, ib, i, j

 ia=ilx(-67.5)
 ib=ilx(-65.5)

 IF(ki .NE. kNA) THEN
   PRINT *,"warning: hudStraitNAtopfix called with ki .NE. kNA., bypassing"
   RETURN
 ENDIF
 IF(ia<1 .OR. ib>ni .OR. jly(61.25)<1 .OR. jly(60.75)>nj) &
    CALL outerr('hudStraitNAtopfix bounds violation, kNA grid does not cover Hstrait')

 DO j=jly(61.25),jly(60.75)
   DO i=ia,ib
     hpo(i,j)=MIN(hpo(i,j),599.0)
   END DO
 END DO

 IF(ifHEAD==1) WRITE(*,*)"hpo minned to 599 at Hstrait"

 RETURN
END SUBROUTINE hudStraitNAtopfix
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE GRtopfix(hpo,ki)
!Nares Strait topographic fix routine
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modgridm
 USE modflags, ONLY : ifHEAD
 USE modgengrid

 IMPLICIT NONE

 REAL, INTENT(INOUT)             :: hpo(0:mNAp,0:mNBp)
 INTEGER, INTENT(IN)           :: ki
 INTEGER :: i, j

 CALL xy2ij(-64.8,80.9,i,j)
 hpo(i,j)=55.0
 CALL xy2ij(-66.3,80.6,i,j)
 hpo(i,j)=55.0
 CALL xy2ij(-67.3,80.4,i,j)
 hpo(i,j)=55.0


 IF(dphid<0.49) CALL outerr('GRtopfix needs to be reset for dphid<0.49')


 IF(ifHEAD==1) WRITE(*,*)"hpo mods for Nares"

 RETURN
END SUBROUTINE GRtopfix
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readRdot(xrdot,yRdot,nRdotin)
! Rdot site coordinates
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE modRdot
 USE moddim
!INCLUDE 'sit.inc.f'
 IMPLICIT NONE

 REAL, INTENT(OUT) ::  xRdot(NRDOT,NICE),yRdot(NRDOT,NICE)
 INTEGER, INTENT(OUT) :: nRdotin(NICE)
 CHARACTER (LEN=3) :: sitename
 CHARACTER (LEN=88) :: fdescript
 INTEGER :: ist, ki,iu

 DO ki = 1,NICE
   iu = 37
   CALL askofil(iu,'Rdot file')
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) THEN
     WRITE(*,*)ki,fdescript
     WRITE(2,*)ki,fdescript
   ENDIF
   READ(iu,*) nRdotin(ki)
   IF(ifHEAD==1) WRITE(*,*)nRdotin(ki)," Rdot sites"
   IF(nRdotin(ki)>NRDOT) CALL outerr('increase size of NRDOT to match Rdot data file')
   DO ist=1,nRdotin(ki)
     READ(iu,*) sitename,yRdot(ist,ki),xRdot(ist,ki)
     
   END DO
   CLOSE(iu)
 END DO
 RETURN
END SUBROUTINE readRdot
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!##############################################
# 2047
! #if defined (LatLonGRID)
!##############################################


!=======================================================================
 SUBROUTINE setGRmask()
!**   MASK OUT GRIS******************
!* okay to retain even with combined full NA/GR grid
!=======================================================================
USE modgridm
USE modflags, ONLY : ifHEAD
USE modinparm, ONLY : kNA, kGRonly
USE modiceMASK, ONLY : imGrnlnd
USE modgengrid
IMPLICIT NONE

INTEGER :: ia, ja, ib, jb, istart=1, ki, ii, jj

!**   define GR/NA mask******************


 IF(istart == 1 .AND. MAX(kNA,kGRonly)>0) THEN ! can't have both kNA and kGRonly > 0
   IF (kNA>0) THEN
     ki=kNA
   ELSE
     ki=kGRonly
   ENDIF
   call cdim(ki)
   IF(ifHEAD==1) WRITE(*,*)'mskdiagnoM: mask out Greenland, updated Dec7/21'
   imGrnlnd(1:ni,1:nj)=1

   CALL xy2ijB(-60.8,81.82,ib,jb)
   DO jj=1,jb
     DO ii=1,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO
   CALL xy2ijB(-150.0,81.81,ia,ja) ! xy2ijB automatically bounds to active grid range
   CALL xy2ijB(-64.6,81.49,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     enddo
   enddo
   CALL xy2ijB(-150.0,81.49,ia,ja)
   CALL xy2ijB(-66.2,80.99,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,80.98,ia,ja)
   CALL xy2ijB(-67.3,80.52,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,80.51,ia,ja)
   CALL xy2ijB(-68.2,79.92,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,79.9,ia,ja)
   CALL xy2ijB(-70.2,79.51,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,79.51,ia,ja)
   CALL xy2ijB(-72.2,78.51,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,78.5,ia,ja)
   CALL xy2ijB(-74.4,75.51,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,75.4,ia,ja)
   CALL xy2ijB(-68.0,73.4,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,73.25,ia,ja)
   CALL xy2ijB(-64.0,70.1,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,69.9,ia,ja)
   CALL xy2ijB(-56.0,58.9,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   CALL xy2ijB(-150.0,57.1,ia,ja)
   CALL xy2ijB(-48.0,30.1,ib,jb)
   DO jj=ja,jb
     DO ii=ia,ib
       imGrnlnd(ii,jj)=0
     ENDDO
   ENDDO

   IF(ki==kNA) THEN
     CALL xy2ijB(-150.0,57.1,ia,ja)
     CALL xy2ijB(-48.0,30.1,ib,jb)
     DO jj=ja,jb
       DO ii=ia,ib
         imGrnlnd(ii,jj)=0
       ENDDO
     ENDDO
   ELSEIF(ki==kGRonly) THEN !iceland
     CALL xy2ijB(-25.4,67.49,ia,ja)
     CALL xy2ijB(-4.0,61.4,ib,jb)
     DO jj=ja,jb
       DO ii=ia,ib
         imGrnlnd(ii,jj)=2
       ENDDO
     ENDDO
   ENDIF
 ENDIF
 istart=0

RETURN
END SUBROUTINE setGRmask
!=======================================================================

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGenHead(iu, ni,nj,icode)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modgridm, ONLY : dthed, dphid
 USE modflags, ONLY : ifHEAD
 IMPLICIT NONE

 INTEGER, INTENT(IN) :: iu, ni, nj
 INTEGER, INTENT(INOUT) :: icode
 INTEGER :: niin, njin, icodeIn
 REAL :: dphidin, dthedin, xlon1, xlon2, ylat1, ylat2
 CHARACTER (LEN=120) :: fdescript

 READ(iu,'(A)')fdescript
 IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
 READ(iu,*) icodeIn
 READ(iu,*) niin,njin,xlon1,xlon2,ylat1,ylat2,dphidin,dthedin
 IF(dthed /= dthedin .OR. dphid /= dphidin) THEN
   WRITE(*,*)'WRONG grid size on input field'
   WRITE(*,*)'dphid,dphidin,dthed,dthedin: ',dphid,dphidin, dthed,dthedin
 END IF

 IF(icode /=0 .AND. icodeIn/=icode) CALL outerrI2('header line #2 icode mismatch, terminating run, icode,icodeIN:',icode,icodeIN)
 icode=icodeIn
 IF(ni /= niin .OR. nj /= njin) THEN
   WRITE(*,*)'wrong # of points in input field: '
   WRITE(*,*)'ni,niin,nj,njin: ',ni,niin,nj,njin
   CALL outerr('pGSMinit/readGenHead, input dimensions do not match active dimensions')
 END IF
 READ(iu,*) !extra line for future additions, eg. unique run #

 RETURN
END SUBROUTINE readGenHead
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


# 2288


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGen2Fij(iu,ni,nj,f1,f2,inew,icode,icheckEnd)
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! inew == 1 : check header, ==0 just read timeslice
! inew == 2 : 5 line header
! icheckEnd>0  : icheckEnd end of file and close
! icode=0 just output 2nd line icode, if < 0, skip 2nd line, else check if icode matches
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modflags
 IMPLICIT NONE

 INTEGER, INTENT(IN)                  :: iu, inew, icheckEnd
 INTEGER, INTENT(IN)                      :: ni
 INTEGER, INTENT(IN)                      :: nj
 INTEGER, INTENT(INOUT)                      :: icode
 REAL, INTENT(OUT)                     :: f1(ni,nj),f2(ni,nj)
 INTEGER ::  nin, njn, i, j, iINPUTENDcheck, icodeIN
 CHARACTER (LEN=80) :: fdescript

 IF(inew >0) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   IF(icode<0) THEN 
     READ(iu,*) 
   ELSEIF(icode==0) THEN
     READ(iu,*) icode
   ELSE
     READ(iu,*) icodeIn
     IF(icodeIn/=icode) THEN
       PRINT *,"icode, icodeIn:",icode,icodeIN
       CALL outerrS("Terminating run due to icode mismatch in input file:",fdescript)
     ENDIF
     icode=icodeIn
   ENDIF
   READ(iu,*) nin,njn
   IF(ni/=nin .OR. nj/=njn) THEN
     PRINT *,"ni,nin,nj,njn:",ni,nin,nj,njn
     CALL outerrS("Terminating run due to ni or nj mismatch in input file:",fdescript)
   ENDIF
   READ(iu,*)
   IF(inew >1) READ(iu,*) ! to keep 5 line header standard
 ENDIF

 DO j=nj,1,-1
   DO i=1,ni
     READ(iu,*) f1(i,j),f2(i,j)
   END DO
 END DO

 IF(icheckEnd >0 ) THEN !END of file check

   READ(iu,*) iINPUTENDcheck
   IF(iINPUTENDcheck/=icheckEnd) THEN
     PRINT *,"for ",fdescript
     PRINT *,"end of file IcheckEnd code should be",icheckEnd,"but is",iINPUTENDcheck
     CALL outerrI('end of infile error => header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
   ENDIF

   CLOSE(iu)
 ENDIF


 PRINT *,"readGen2Fij completed for",fdescript


 RETURN
END SUBROUTINE readGen2Fij
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGenFijz(iu,ni,nj,nkz,f,inew,icode,icheckEnd)
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! inew == 1 : check header, ==0 just read timeslice
! inew == 2 : 5 line header
! icheckEnd>0  : icheckEnd end of file and close
! icode=0 just output 2nd line icode, if < 0, skip 2nd line, else check if icode matches
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 USE modflags
 IMPLICIT NONE

 INTEGER, INTENT(IN)                  :: iu, inew, icheckEnd
 INTEGER, INTENT(IN)                  :: ni
 INTEGER, INTENT(IN)                  :: nj
 INTEGER, INTENT(IN)                  :: nkz
 INTEGER, INTENT(INOUT)               :: icode
 REAL, INTENT(OUT)                    :: f(ni,nj,nkz)
 INTEGER ::  nin, njn, i, j, iINPUTENDcheck, icodeIN, nkzn, kz
 CHARACTER (LEN=80) :: fdescript

 IF(inew >0) THEN
   READ(iu,'(A)')fdescript
   IF(ifHEAD==1) WRITE(*,*)'opening ',fdescript
   IF(icode<0) THEN 
     READ(iu,*) 
   ELSEIF(icode==0) THEN
     READ(iu,*) icode
   ELSE
     READ(iu,*) icodeIn
     IF(icodeIn/=icode) THEN
       PRINT *,"icode, icodeIn:",icode,icodeIN
       CALL outerrS("Terminating run due to icode mismatch in input file:",fdescript)
     ENDIF
     icode=icodeIn
   ENDIF
   READ(iu,*) nin,njn,nkzn
   IF(ni/=nin .OR. nj/=njn) THEN
     PRINT *,"ni,nin,nj,njn:",ni,nin,nj,njn
     CALL outerrS("Terminating run due to ni or nj mismatch in input file:",fdescript)
   ENDIF
   IF(nkz>nkzn) THEN
     PRINT *,"nkz,nkzn:",nkz,nkzn
     CALL outerrS("Terminating run due to nkzIn < nkz for input file:",fdescript)
   ENDIF
   READ(iu,*)
   IF(inew >1) READ(iu,*) ! to keep 5 line header standard
 ENDIF

 DO j=nj,1,-1
   DO i=1,ni
     READ(iu,*) (f(i,j,kz),kz=1,nkz)
   END DO
 END DO

 IF(icheckEnd >0 ) THEN !END of file check

   READ(iu,*) iINPUTENDcheck
   IF(iINPUTENDcheck/=icheckEnd) THEN
     PRINT *,"for ",fdescript
     PRINT *,"end of file IcheckEnd code should be",icheckEnd,"but is",iINPUTENDcheck
     CALL outerrI('end of infile error => header/format/dim mismatch, iINPUTENDcheck:',iINPUTENDcheck)
   ENDIF

   CLOSE(iu)
 ENDIF


 PRINT *,"readGenFij completed for",fdescript


 RETURN
END SUBROUTINE readGenFijz
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
